// *****************************************************************************
//
// source:    AC_RICH_evnt.C
//
// type:      source code
//
// created:   03 Nov 1999
//
// author:    Andreas Gellrich
//            HERA-B Collaboration
//            DESY Zeuthen
//            Platanenallee 6
//            D-15738 Zeuthen
//            Germany
//            email: Andreas.Gellrich@desy.de
//
// symbol:    AC_RICH_evnt()
//
// called by: AC_evnt()
//
// hints:     see $HBROOT/ARTE/dev/include/dq/DQ.h for COMP_XYZ and LVL_ABC
//
// changes:
// 03 Nov 1999 (AG) further developments
// 19 Nov 1999 (AG) cna -> ac (alca)
// 22 Nov 1999 (AG) includes in alca
// 02 Feb 2000 (DD) beta=1 histogram, rings/event
// 03 Feb 2000 (AG) fixed function name
// 06 Apr 2000 (DD) rich reco w/o RSEG filling, fading radius histograms
// 11 Apr 2000 (DD) original SN restored, fill radius histo using database
// 16 Sep 2000 (AG) introduced ARTE_mark()
//
// *****************************************************************************

// standard includes
#include <iostream.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>

// ONLINE includes
#include "rhp.h"
#include "event.h"
#include "grr.h"

// Arte includes
#include "arte/arte.hh"
#include "arte/RSEG_cmp.hh"
#include "dq/DQ.h"
#include "farm/farm.hh"
#include "rich/rich.h"
#include "rise/RingSearchARTE.h"
#include "alca/AC_RICH.hh"

// externals
extern int FAGRR;

// prototypes
int AC_RICH_rec_beta1(float richtx, float richty);


//////////////// Fill RICH AC histograms /////////////////////////////
int 
AC_RICH_evnt() 
{
  ARTE_mark( "AC_RICH_evnt 0", 0 ); // helps to trace back problems

  static float minPlotRadius=.045;
  static float maxPlotRadius=.060;
  static int   maxOccupancy=750;
  
  // -- occupancy cut
  int n0=RichRun::gede->hit1.index();
  int n1=RichRun::gede->hitl.index();
  if (n1-n0>maxOccupancy) return 1;


  // -- reconstruct rings, do not fill RSEG
  if (!RICH_Search) RICH_Search=new RingSearchARTE;
  RingSearchTrack::exist     = false;
  RingSearchAttrib::reuse    = true;
  RingSearchAttrib::bookrseg = false;
  RingSearchAttrib::bookhitb = false;
  float defaultSN=RingSearchAttrib::SNcut;
  RingSearchAttrib::SNcut    = 4;
  RICH_Search->getrings(); // db
  //RICH_Search->getrings(35,55,.5,6); //up
  //RICH_Search->getrings(45,60,-1,6); //down
  RingSearchAttrib::SNcut    = defaultSN;
	

  for(int i=0; i<RICH_Search->numrings(); i++) {
    Ring &ring=RICH_Search->getRing(i);

    float rad=ring.thetaC();
    if (rad<minPlotRadius) continue;
    if (rad>maxPlotRadius) continue;

    // -- check directions
    float tx=ring.phi();
    float ty=ring.lambda();
    rhp_hfill(AC_ID(COMP_RICH, LVL_REC, 201),tx,ty,1);

    // -- fading radius
    rhp_mult(AC_ID(COMP_RICH, LVL_REC,2002),.9999);
    rhp_hf1(AC_ID(COMP_RICH, LVL_REC,2002),rad,1);
    // -- accumulated radius
    rhp_hf1(AC_ID(COMP_RICH, LVL_REC, 202),rad,1);

    if (!AC_RICH_rec_beta1(ring.tx(), ring.ty())) {
      // -- biased fading radius
      rhp_mult(AC_ID(COMP_RICH, LVL_REC,2003),.9999);
      rhp_hf1(AC_ID(COMP_RICH, LVL_REC,2003),rad,1);
      // -- biased accumulated radius
      rhp_hf1(AC_ID(COMP_RICH, LVL_REC, 203),rad,1);
    }
  }
  rhp_hf1(AC_ID(COMP_RICH,LVL_REC, 200),RICH_Search->numrings(),1);

  ARTE_mark( "AC_RICH_evnt 1", 0 ); // helps to trace back problems
  return 0; // return error code (0 = okay)
}



//////////////  Select RICH Beta=1 w/ ECAL ////////////////////////
int AC_RICH_rec_beta1(float richtx, float richty) {
  static float max_dty=3e-3;  // angle match
  static float min_esum=8;    // minimum cluster energy
  static float max_p=200;     // maximum p
  static float min_p=10;      // minimum p
  for (ArteTable<RCCL>::iterator irccl=ArteTable<RCCL>::begin();
       irccl!=ArteTable<RCCL>::end(); ++irccl) {
    if ((*irccl)->esum<min_esum) continue;
    if (fabs((*irccl)->ty - richty)>max_dty) continue;
    float p=288/fabs((*irccl)->x - richtx*(*irccl)->z);
    if (p<min_p) continue;
    if (p>max_p) continue;
    return 0; // cluster found
  }

  return 1; // cluster not found
}
