
// ********************************************************************
//
//  Routine:      acwrite_
//  Author:       T.Oest
//
//  Description:  interface routine for alignment/calibration
//                and data quality
//
//  Modification:
//   data quality added                            27/09/99 T.Oest
//   calibartion & alignment added                 19 Nov 1999 (AG)
//   AC_init() is moved to acinit                  22 Nov 1999 (AG)
//   use ARTE_mark()                               25 Apr 2000 (AG)
//   if online & L4RecoMask=0 do not run AC_evnt   10 Oct 2001 (JH)
//   Do rhp publication and reset here for DQ & AC 19 Dec 2001 (JH)
//   publish if RHPPUB>0 (new case RHPPUB<0)       17 Jan 2002 (JH)
//   Do AC if L4HitMask !=0                        28 Jan 2002 (JH)
//   Use faReco() and faHit() to get masks         09 Mar 2002 (JH)
//
// ********************************************************************

// standard includes
#include <iostream.h>

// Arte includes
#include "arte/arte.hh"
#include "farm/farm.hh"
#include "farm/fa.hh"

// externals
int extern FADEB;
int extern RHPPUB;

// prototypes
int DQ_evnt();
int AC_evnt();

//*********************************************************************
// function
//*********************************************************************
extern "C" void
acwrite_()
{ 
  // show up
  //cout << "acwrite_()" << endl;

  // inits
  int rc = 0;
  
  // call the data quality routine
  ARTE_mark( "DQ_evnt", 0 );
  rc = DQ_evnt();

  // If online but both RecoMask and HitMask are 0 don't do AC 
  if(faReco(0xFFFF) || faHit(0xFFFF)) {
    // alignment/calibration interface
    ARTE_mark( "AC_evnt", 0 );
    rc = AC_evnt();
  }

  // publish rhp objects to subscribers (gatherers) and reset them after publishing
  if (RHPPUB > 0) faCheckRhp( FADEB ? 1 : 0 );  
}
