*-- Author :   

      SUBROUTINE ALQJET

************************************************************************
*
* Description: Selection of PYTHIA fragments from heavy quarks
*              as preparation of generation of underlying event (FRITIOF)
* 
*              Steered by parameter  modefrag  in  common /luopts/
*
*    modefrag = 1:
*              Keep only the single resonance/particle originating
*              directly from the heavy (anti)quark.
*    modefrag = 2:
*              Clusters containing a heavy (anti)quark are kept with 
*              all decay fragments.
*              Color strings spanned by heavy (anti)quarks are first
*              boosted into their rest system and then all daughter 
*              particles moving into the heavy (anti)quark hemisphere are
*              selected. Ohter particles are discarded. Then
*              the event is boosted back to the lab frame
*              Other strings and clusters, not connected to heavy 
*              (anti)quarks, are discarded.
*    modefrag = 3:
*              Boost parton level event into rest frame of colliding 
*              partons.
*              Select the heavy quark and antiquark, discard all
*              other partons.
*              Apply independent fragmentation to create heavy
*              (anti)quark jets.
*              Boost the event back to the lab frame.
*    modefrag = 4:
*              Clusters containing a heavy (anti)quark are kept with
*              all deacy fragments.
*              Color strings spanned by heavy (anti)quarks are kept
*              with all deacy fragments.
*              Other strings and clusters, not connected to heavy
*              (anti)quarks, are discarded.
*    modefrag = 5:
*              Keep the complete hard proton nucleon interaction
*              in which a heavy quark-antiquark pair is produced.
*              Ignore nuclear energy loss and multiple scattering.
*
*
*-- Authors :  J. Ivarsson,  T. Lohse    
*
*   
* Called from: ALPYTH
*
* External references: LUND, PYTHIA
*
* Arguments: none
*
************************************************************************

      IMPLICIT NONE

#include "arte/Arte.fi"
#include "arte/ArteFlag.fi"
#include "jets/LUDAT1.fi"
#include "jets/LUDAT3.fi"
#include "jets/LUJETS.fi"
#include "pyth/PYSUBS.fi"
#include "pyth/PYPARS.fi"
#include "arte/luopts.fi"

      INTEGER NSRC, I, j, im, LUCOMP, mq(2), jet(2), mclu(2)
      integer mess_id_1, mess_id_2
      REAL betax, betay, betaz, prod
      save mess_id_1, mess_id_2

#include "arte/Arte.ff"

c     Identify the line number in buffer LUJETS of the heavy quarks
c     MSEL=4 for charm and MSEL=5 for beauty.

      mq(1) = 0
      mq(2) = 0

      do i=1, N
         if( K(i,1).ne.0 .and. (K(i,1).le.20.or.K(i,1).ge.31) ) then
            if(K(i,2).eq.MSEL)  mq(1) = i
            if(K(i,2).eq.-MSEL) mq(2) = i
         endif
      enddo

      if(mq(1).eq.0.or.mq(2).eq.0) then
         call MEERRC(mess_id_1,'ALQJET: No heavy quark pair found')
         goto 900
      endif

c     check the various fragmentation options
c
      if (modefrag.eq.3) then
c check whether heavy quarks sit in clusters or in strings
        do  j = 1, 2
          mclu(j) = -1
          if (K(mq(j),4).ne.0) then
            if (K(K(mq(j),4),2).eq.91) then
              mclu(j) = K(mq(j),4)
            endif
          endif
        enddo
c independent fragmentation of heavy quarks (keep clusters)
c
c flag heavy quarks which are not sitting in clusters as unfragmented
        do  j = 1, 2
          if (mclu(j).eq.-1) K(mq(j),1) = 1
        enddo
c discard all particles but the heavy quarks, their clusters
c and fragments of their clusters
        do  j = 1, N
          if (K(j,1).ne.0.and.K(j,1).lt.21) then
            if (j.ne.mq(1).and.j.ne.mq(2).and.
     +          j.ne.mclu(1).and.j.ne.mclu(2)) then
              im = j
 2            continue
              im = K(im,3)
              if (im.eq.mclu(1).or.im.eq.mclu(2)) goto 3
              if (im.ne.0)                        goto 2
              K(j,1)=0
 3            continue
            endif
          endif
        enddo
c compute velocity of Q-Qbar system
        betax = (P(mq(1),1)+P(mq(2),1))/
     +          (P(mq(1),4)+P(mq(2),4))
        betay = (P(mq(1),2)+P(mq(2),2))/
     +          (P(mq(1),4)+P(mq(2),4))
        betaz = (P(mq(1),3)+P(mq(2),3))/
     +          (P(mq(1),4)+P(mq(2),4))
c boost into the rest system
        call lurobo(0.,0., -betax, -betay, -betaz)
c apply independent fragmentation, request flavour conservation,
C allow energy violation
        MSTJ(1) = 2
        MSTJ(3) = 5
        call LUEXEC
        MSTJ(1) = 1
        MSTJ(3) = 0
c boost back into the lab system
        call lurobo(0.,0.,betax, betay, betaz)

      elseif (modefrag.lt.5) then

c     Identify the line number in buffer LUJETS of the strings/clusters
c     containing the heavy quarks.

        jet(1) = K(mq(1),4)
        jet(2) = K(mq(2),4)

        if((K(jet(1),2).ne.91.and.K(jet(1),2).ne.92).or.
     +    (K(jet(2),2).ne.91.and.K(jet(2),2).ne.92)) then
          call MEERRC(mess_id_2,
     +     'ALQJET: Heavy quarks do not connect to cluster/string')
          goto 900
        endif

        If (modefrag.eq.2) then
          do j = 1,2

c     Keep complete clusters and keep half strings pointing towards the 
c     heavy quarks.  
       
            if(K(jet(j),2).eq.92) then

c     Velocity of string/cluster j.

              betax = P(jet(j),1)/P(jet(j),4)
              betay = P(jet(j),2)/P(jet(j),4)
              betaz = P(jet(j),3)/P(jet(j),4)
            
c     Boost event into the rest system of the string/cluster.

              call lurobo(0.,0.,-betax, -betay, -betaz)      

c     Loop over daughters of string/cluster and discard those not pointing
c     into the hemisphere of the heavy quark, by setting status flag, K(i,1), 
c     to 0 (so called empty line).
            
              do i= K(jet(j),4), K(jet(j),5)
                 prod = P(i,1)*P(mq(j),1) + P(i,2)*P(mq(j),2) + 
     +                  P(i,3)*P(mq(j),3) 
                 if(prod.lt.0) then
                    K(i,1) = 0
                 endif
              enddo

c     Boost event back into the lab system

              call lurobo(0.,0.,betax, betay, betaz)
            endif

          enddo
        endif

c     Loop over all lines in LUJETS....

        do i=1,N
c...     except empty lines, comment lines, the heavy quarks
c        and the strings/clusters
          if( K(i,1).ne.0 .and. K(i,1).le.20 .and.
     +        i.ne. mq(1).and.i.ne. mq(2). and.
     +        i.ne.jet(1).and.i.ne.jet(2) ) then

c        Check the family tree of the line. 
c        modefrag = 2,4:   Discard particles which do not
c          have one of the selected strings/clusters as an ancestor. 
c        modefrag = 1:     Discard light particles which do not
c          have a heavy flavour hadron as an ancestor
            im = i
 10         continue
            if(im.eq.0.or.K(im,1).eq.0.or.K(im,1).ge.21) then 
              K(i,1) = 0
              goto 20
            else if(modefrag.eq.2.or.modefrag.eq.4) then
              if (im.eq.jet(1).or.im.eq.jet(2))         goto 20
            else if(modefrag.eq.1) then
              if (mod(abs(K(im,2)),1000)/100.eq.MSEL.or.
     +            mod(abs(K(im,2)),10000)/1000.eq.MSEL) goto 20
            endif

            im = K(im,3)
            goto 10    

 20         continue
            
          endif
        enddo
      endif

c     remove lines with status flag =0. Update event history information.
      call luedit(12)      

c     remove comment lines. Update event history information.
      call luedit(15)

 900  END

************************************************************************
* End of ALQJET
************************************************************************


