//
// $Source: /afs/desy.de/group/hera-b/repository/ARTE/arte/include/PointerBase.hh,v $
// $Revision: 1.16 $
// $Author: irisha $
// $Date: 2001/07/23 13:07:44 $
//
// Created: Jan 1996,   C. Borgmeier
//
//
// This file is part of the C++ interface for Arte
// the HERA-B reconstruction and analysis tool
//
// This is the definition of the ArtePointerBase class.
//
// ArtePointerBase is the base class of the ArtePointer
// template class.
//
//
// .SS Usage // A Pointer to an GESL is declared as
//
// .ft B
//      ArtePointer<GESL> geslPoint;
//
//
// .SS See Also
//
// ArtePointer.hh ArteStructures.hh
//
// Author: Christoph Borgmeier, Hartwig Albrecht
//

#ifndef ARTE_POINTER_BASE_HH_
#define ARTE_POINTER_BASE_HH_


#ifndef __CINT__
#include <iosfwd>
#else
#include <iostream>
#endif


#ifdef USE_GATE
class TBuffer;
#endif


//template <class T> class ArteTable;

class ArtePointerBase
{
#if !(__GNUC__==2 && __GNUC_MINOR__<91 )
  template <class T> friend class ArtePointer;
#endif
#ifdef USE_GATE
  friend inline TBuffer& operator<<( TBuffer&, ArtePointerBase );
  friend inline TBuffer& operator>>( TBuffer&, ArtePointerBase& );
#endif

private:
  unsigned int idx;

public:
  inline ArtePointerBase( const ArtePointerBase& i ): idx( i.idx ) {}
#if !defined(ARTE_CXX_COMPAT_MODE) || (ARTE_CXX_COMPAT_MODE==1)
public:
#else
protected:
#endif
  inline ArtePointerBase( unsigned int i );
  inline unsigned int index() const { return idx; }

  int operator==( const ArtePointerBase & arg ) const;
  int operator!=( const ArtePointerBase & arg ) const;

  //protected:
  inline unsigned int & indexReference();
  // Returns the private data.

  void increment() { ++idx; }
  void decrement() { --idx; }

  ArtePointerBase& operator+=( int offset ) { idx+=offset; return *this; }
  
public:
#ifndef __CINT__
  void print( std::ostream& os ) const;
#endif

  const ArtePointerBase& operator=( const ArtePointerBase& rhs )
  { idx = rhs.idx; return *this; }
};


#ifndef __CINT__
inline std::ostream& operator<<( std::ostream & os,
				 const ArtePointerBase & ap );
#endif


#include "PointerBase.icc"

#endif
