*-- Author :    Hartwig Albrecht   23/03/95
*   changed by  Siegmund Nowak     25/01/00  (call of dggeomread for MC)  
      SUBROUTINE MMREVT (IOS)

************************************************************************
*
* Description: read next event record
* 
* Called from: USEXEC
*
* External references: MMREAD, FAREAD, dggeomread
*
* Arguments: in/out description
*   IOS      out    flag: -1: end of data
*                          0: no event input file given
*
* Changes:
* 08 Nov 2000 (AG) introduced acreadmc
* 06 Mar 2001 (SE) moved MMSTATISTICS from MMWIPE where it was confusing
* 01 Aug 2001 (PK,JH) call acread also for MC (acreadmc not used any more)
* 01 Oct 2001 (JH) call FAMSG for MC to check RPM messages in FARM mode
* 12 Dec 2001 (DG) small corretions to the sequential read from one unit
*
************************************************************************

      IMPLICIT NONE

#include "arte/Arte.fi"
#include "arte/ArteFlag.fi"
#include "arte/ZIOCOM.fi"
#include "auto/MCIN.fi"
#include "auto/MEIO.fi"
#include "auto/NGHD.fi"
#include "auto/GEHD.fi"
#include "auto/MEMO.fi"

      INTEGER I, IARR(NAMELEN), IN, IOS, FAREAD, FARM, LOCF
      CHARACTER CNAME*80

#include "arte/Arte.ff"
#include "auto/MCIN.ff"
#include "auto/MEIO.ff"
#include "auto/NGHD.ff"
#include "auto/GEHD.ff"
#include "auto/MEMO.ff" 

C exactly one event input unit given ?
      
      IN = 0
      IF (IMEIO_FLAG(LDSTI) .GT. 0)  THEN
        IN = LDSTI
      END IF
      
      IF (IMEIO_FLAG(LMINII) .GT. 0)  THEN
        IF (IN .NE. 0)  GO TO 10
        IN = LMINII
      END IF
      
      IF (IMEIO_FLAG(LDSTIE) .GT. 0)  THEN
        IF (IN .NE. 0)  GO TO 10
        IN = LDSTIE
      END IF
      
      IF (IMEIO_FLAG(LMINIIE) .GT. 0)  THEN
        IF (IN .NE. 0)  GO TO 10
        IN = LMINIIE
      END IF
      
      IF (IMEIO_FLAG(LEVI) .GT. 0)  THEN
        IF (IN .NE. 0)  GO TO 10
        IN = LEVI
      END IF

C to mix or not to mix

      IF (FIOMIX(5) .NE. 0.)  GO TO 40

      IF (IMEIO_FLAG(LEVI1) .GT. 0)  THEN
        IF (IN .NE. 0)  GO TO 10
        IN = LEVI1
      END IF

      GO TO 20

 10   CONTINUE
      CALL MEERRC (76534,
     +  'MMREVT event input: more than one unit specified')
      
 20   IF (IMEIO_FLAG(LNOTEI) .GT. 0)  GO TO 210
      IF (IMEIO_FLAG(LEDIRI) .GT. 0)  GO TO 220

C check that event input file 2, 3, and 4 are not given

      DO IOS = 2, 4
        IF (IMEIO_FLAG(LEVIN(IOS)) .GT. 0)  THEN
          CALL MEERRC (653434,
     +      'MMREVT: file EVIN2..4 given but no IOMIX command')
        END IF
      END DO

      IF (IN .EQ. 0)  THEN
        IF (INOUTFARM .EQ. 0)  THEN
          CALL MEERRC (6346345,
     +      'MMREVT: no event input file given')
          IOS = -1
          GO TO 900
        END IF
      ELSE
        IF (INOUTFARM .NE. 0)  THEN
          CALL MEERRC (789563,
     +          'MMREVT: no event input file in farm mode allowed')
          IOS = -1
          GO TO 900
        END IF
      END IF

C sequential read from one unit.

      IF (INOUTFARM .NE. 0)  GO TO 50
C For a header as EVHD with IMEMO_SUP .EQ. 0, IOA and IOD are equal
      IF (BITTEST(BMEMO_IOD(LOCF(JEVHD)-LARLN1), 1) .EQ. 0 .OR.
     +    BITTEST(BMEMO_IOD(LOCF(JEVHD)-LARLN1),HMEIO_BIT(IN)) .EQ. 0)
     +  THEN
        C78 = 'MMREVT: EVHD not marked for input on'
        CALL CHHTOC (IQ(HQMEIO_STREAM(1,IN)), CNAME)
        CALL CH78BC (CNAME)
        CALL CH78BC ('- an event input stream!')
        CALL ME78EC (534888)
        IOS = -1
        GO TO 900
      END IF

  30  CALL MMREAD (IN, CNAME)

C end of data ?

      IF (CNAME .EQ. 'EOD')  THEN
        IOS = -1
        GO TO 900
      END IF

C skip event ?

      IF (CNAME .EQ. 'SKP')  GO TO 30

C event record ?

      IF (CNAME .EQ. 'EVHD')  GO TO 100

C special record (all unknown tables were skipped yet in mmrtab)
      CALL CHCTOH (CNAME, IARR, NAMELEN)
      DO I = 1, NARLNK
        IF (IARR(1) .EQ. HMEMO_NAME(1,I))  GO TO 35
      END DO

 35   CALL USRSPC (IN, JLINK(I))
      GO TO 30

C mix events
      
 40   IF (INOUTFARM .NE. 0)  THEN
        CALL MEERRC (976457,
     +        'MMREVT: mixing in farm mode is nonsense')
        IOS = -1
        GO TO 900
      END IF

      IF (IN .NE. 0)  CALL MEERRC (97845,
     + 'Read mixed events: only units evin1,2,3,4 accepted')
      CALL MMRMIX (IOS)

C end of data ?

      IF (IOS .LT. 0)  GO TO 900
      GO TO 100

C farm input

 50   FARM = FAREAD ()

C end of data

      IF (FARM .GT. 0)  THEN
        IOS = -1
        GO TO 900
      END IF

C error condition

      IF (FARM .LT. 0)  THEN

        C78 = 'MMREVT: read error in farm mode. FARM ='
        CALL CH78BI (FARM)
        CALL ME78PC (13645, 30)
        CALL USRERR (0)
        GO TO 50
      END IF

 100  CONTINUE

C get the correct geometrical constants

c      IF (JNGHD .EQ. 0)  CALL ATCONS

      IF (JMCIN .EQ. 0)  THEN

C real data. Read CnA constants and RPM messages
         CALL FAMSG 
         CALL ACREAD
      ELSE

C MC data with geometry
         IF(FMCIN_VGEO(1) .NE. 0.) THEN 
            CALL DGGEOMREAD
	    CALL FAMSG
            CALL ACREAD
         ENDIF 

      END IF
      GO TO 900

 210  IF (IN .NE. 0)  CALL MEERRC (113425,
     +  'MMREVT notebook input: other units specified')
      CALL MEERRC (53445,
     +  'MMREVT: notebook input not yet possible')
      GO TO 900

 220  IF (IN .NE. 0)  CALL MEERRC (87634,
     +  'MMREVT Edir input: other units specified')
      CALL MEERRC (53445,
     +  'MMREVT: Edir input not yet possible')

 900  CONTINUE

C fill statistics
      DO I = 1, NMEMO()
          IF (IQ(IQMEMO_DIV(I)) .EQ. IXEVNT) THEN
            IF (JLINK(I) .NE. 0) THEN
              CALL MMSTATISTICS (I)
            END IF
          END IF
      END DO

      END

************************************************************************
* End of MMREVT
************************************************************************
