//                              -*- Mode: C++ -*-
//: ECStrategyFactory.C
//
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 12/6/97
// Last Update ........
//

#define ESEM
#include "care/ECStrategyFactory.hh"
#include "care/ESPeakPos.hh"
#include "care/ESCorCOGPos.hh"
#include "care/ESCTCValley.hh"
#include "care/ESPID04.hh"
#include "care/ESPIDEnergy3cell.hh"
#include "care/ESLightYieldCor.hh"
#ifdef ESEM
#  include "care/ESEMSPos.hh"
#endif
/*
CLASS
  ECStrategyFactory


*/

ESPosStrategy* peakpos = 0;
ESPosStrategy* cogpos = 0;
ESPosStrategy* corcogpos = 0;
ESCTCDisStrat* valstrat = 0;
ESPIDStrategy* pid04 = 0;
ESPIDStrategy* pid3c = 0;
ESLightYieldCor* lightyield = 0;
ESEnCorStrategy* encor = 0;

///////////////////  static data definition  //////////////////////
// initial value for curren data
ECStrategyFactory::StratTypes ECStrategyFactory::currdata = 
                     {COG, Valley, PeakDis, Pid3Cell};

// default values
ECStrategyFactory::StratTypes ECStrategyFactory::defValues =
                     {COG, Valley, PeakDis, Pid3Cell};

/////////////////// member function definitions /////////////////////
// constructor and destructor
ECStrategyFactory::ECStrategyFactory(){}
ECStrategyFactory::~ECStrategyFactory(){}

 //  Set the defaults
void ECStrategyFactory::setDefaults(struct StratTypes & newdef){
  defValues = newdef;
}


 //--
  //   Get a brand new strategy for position
ESPosStrategy* ECStrategyFactory::PositionStrategy() const{
  return PositionStrategy(currdata.ptype);
}
//--
//   Get a new PositionStrategy of type pos
ESPosStrategy* ECStrategyFactory::PositionStrategy(PosTyp pos) const {
    ESPosStrategy* posstrat = 0;
    switch(pos){
    case Peak:
      if( peakpos==0 ) peakpos = new ESPeakPos();
      posstrat = peakpos;
      break;
    case COG:
      if( cogpos==0 ) cogpos = new ESCOGPos();
      posstrat =  cogpos;
      break;
    case E_CorCOG:
      if( corcogpos==0){
#     ifndef ESEM
        corcogpos =  new ESCorCOGPos();
#     else
        corcogpos =  new ESEMSPos();
#     endif
      }
      posstrat = corcogpos;
      break;
    default:
      if( peakpos==0 ) peakpos = new ESPeakPos();
      posstrat = peakpos;
      break;
    }
    return posstrat;
}

//--
//   Get a brand new strategy for Clus-To-Clus dist
ESCTCDisStrat* ECStrategyFactory::CTCStrategy() const {
  return CTCStrategy(currdata.ctctype);
}

//--
//   Get a new strategy for Clus-To-Clus dist of type ctcs
ESCTCDisStrat* ECStrategyFactory::CTCStrategy(ECStrategyFactory::ClToClDis 
					                  ctcs) 
  const {
  ESCTCDisStrat* disstrat = 0;
  switch(ctcs){
  case ECStrategyFactory::Valley:
    if( valstrat==0) valstrat = new ESCTCValley();
    disstrat = valstrat;
    break;
  default:
    if( valstrat==0) valstrat = new ESCTCValley();
    disstrat = valstrat;
    break;
  }
  return disstrat;
}

//--
//   Get a new strategy for PID analysis
ESPIDStrategy* ECStrategyFactory::PidStrategy() const{
  return  PidStrategy(currdata.pidtype);
}

//--
//   Get a new strategy for PID analysis
ESPIDStrategy* ECStrategyFactory::PidStrategy(PIDtypes pidt) const{
  ESPIDStrategy* pid = 0;
  switch(pidt){
    case ECStrategyFactory::Pid04:
      if( pid04==0 )     pid04 =  new ESPID04();
      pid = pid04;
      break;
      // case ECStrategyFactory::Pid05:
      //     return new PID05();
      //     break;
    case ECStrategyFactory::Pid3Cell:
      if( pid3c==0 ) pid3c = new ESPIDEnergy3cell();
      pid= pid3c;	  
      break;
    default:
      if( pid3c==0 ) pid3c = new ESPIDEnergy3cell();
      pid= pid3c;	  
    break;
  }
  return  pid;
}


///   Get a new strategy for Energy correction analysis
ESEnCorStrategy* ECStrategyFactory::EnCorStrategy(EnCortypes enct) const{
  ESEnCorStrategy* cors = 0;
  switch(enct){
  case LightYield:
    if( lightyield ==0 ){
      lightyield = new ESLightYieldCor();
    }
    cors = lightyield;
    break;
  case DeadCells:
    if( encor ==0 ){
      //      encor = new ESEnCorStrategy();  !/ to be changed
    }
    cors = encor;
    break;
  default:
    cors = encor;
  }
  return cors;
}
