//                              -*- Mode: C++ -*-
//: EDClusterModifier.C
//
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 4/4/97
// Last Update ........
//


#include "care/EDClusterModifier.hh"
#include "care/EMClusterTree.hh"
#include "care/EMTrackOnEcal.hh"
#include "care/EMCellFraction.hh"

/* 
CLASS
  EDClusterModifier
  Abstract base class for modifications in cluster entries

*/

//////////////////// EDClusterModifier Class Definitions ////////////////

// No functions

//////////////////// EDClusterModifier Class Definitions ////////////////
    
EDClusterModifier::EDClusterModifier(){}
EDClusterModifier::~EDClusterModifier(){}
  

//--
// Cluster Analyzer
void EDClusterModifier::doModify(EMCluster * clus){
  // perform operation on all elements of the cluster
  for(EMClusterMember* child = clus->begin(); 
      child!=clus->end();  child = child->next())
      child->modifyMe(*this);
}

//--
// Cluster Tree Analyzer
void EDClusterModifier::doModify(EMClusterTree *  clus){
  // perform operation on all elements of the cluster
  for(EMClusterMember* child = clus->begin(); 
      child!=clus->end();  child = child->next())
      child->modifyMe(*this);
}

//--
// TrackOnEcal
void EDClusterModifier::doModify(EMTrackOnEcal *  track){
  // perform operation as it where a general ClusterMember
  EMClusterMember* member = track;
  doModify( member );
}


//--
// EMCellFraction
void EDClusterModifier::doModify(EMCellFraction *  cell){
  // perform operation as it where a general EMHitCell
  EMHitCell* hitcell = cell;
  doModify( hitcell );
}




