// 
//  Author:  Rainer Mankel, Mauro Villa        Created:  20-Feb-99
//                                             Modified: 03-Feb-00
//  Email:   mankel@ifh.de,villa@bo.infn.it    HERA-B Collaboration
// 

// function careelid
// perform electron identification
//
// 02.07.01 Added likelihood for OTR-ECAL matching
// callable from Fortran

#include <iostream.h>
#include <math.h>
#include "arte/arte.hh"
#include "arte/ArteMessage.hh"

void DxLikeECAL(double dx, int calo, double & lee, double & leh);

extern "C" {
  void careelid_(){
    
    for (ArteTable<RTRA>::iterator rtIt=ArteTable<RTRA>::begin();
	 rtIt != ArteTable<RTRA>::end(); ++rtIt) {
      
      // initialize
      (*rtIt)->fle = 0;
      (*rtIt)->lee = 0.0;
      (*rtIt)->leh = 0.0;
      (*rtIt)->lemi = 0.0;

      if( (*rtIt)->fit==0 && (*rtIt)->rccl.begin()!=(*rtIt)->rccl.end()){
	
	float invp = (*rtIt)->pf;
	float cv_invp = (*rtIt)->cvf[14];      

	// get last RPNT
	ArtePointer<RPNT> rp;
	for ( ArteRelationIterator<RPNT> rpntIt = (*rtIt)->rpnt.begin();
	      rpntIt != (*rtIt)->rpnt.end(); ++rpntIt ) {
	  if (! rp.index()) {
	    rp = *rpntIt;
	  }
	  else {
	    if (rp->zf < (*rpntIt)->zf)  rp = *rpntIt;
	  }
	}
	if (rp.index()!=0){
	  
	  if (rp->zf < 730.) continue;
	  invp = rp->pf;        
	  cv_invp = rp->cvf[14];
	} else
	  continue;
	
	(*rtIt)->fle = 1;
	for ( ArteRelationIterator<RCCL> rcclIt = (*rtIt)->rccl.begin();
	      rcclIt != (*rtIt)->rccl.end(); ++rcclIt ) {
	  double lhE = 0.5; // here 33% input tracks are elecrons
	  double lhH = 0.5; // and 67% are hadrons (half of which are mips)
	  
	  double eOverP = (*rcclIt)->esum * fabs( invp );
	  
	  // likelihoods for eOverP
	  if( eOverP>5.0 )  eOverP = 3.0; // to avoid problems
	  double mE = 1.;
	  double sE1 = 0.05;
	  double sE2 = 0.2;
	  double fr1 = 0.6;
	  double lhEeop = 
	    fr1* exp(-0.5 * (eOverP - mE)*(eOverP - mE) / (sE1 * sE1))
	    / (2.5066 * sE1)+
	    (1.-fr1)*exp(-0.5 * (eOverP - mE)*(eOverP - mE) / (sE2 * sE2))
	    / (2.5066 * sE2)+1.0e-20;
	  double mH = 0.4;
	  double sH = 0.25;
	  double lhHeop = exp(-0.5 * (eOverP - mH)*(eOverP - mH) / (sH * sH))
	    / (2.5066 * sH)+1.0e-20;


          // likelihoods for matching
          double lex=0.5,ley=0.5,lhx=0.5,lhy=0.5;
          double dx = (*rcclIt)->x-rp->xf-rp->txf*((*rcclIt)->z-rp->zf);
          double dy = (*rcclIt)->y-rp->yf-rp->tyf*((*rcclIt)->z-rp->zf);
          int calo = (*rcclIt)->flag & 7;
	  if( (*rtIt)->hin+(*rtIt)->hou>0 )
            DxLikeECAL(dx, calo, lex, lhx);    //Dx only for MTR-x  tracks
	  DxLikeECAL(dy, calo, ley, lhy);
	  
	  // Combining likelihoods
	  lhE *= lhEeop*lex*ley;
	  lhH *= lhHeop*lhx*lhy;
	  double norm = 1./(lhE+lhH);
	  lhE *= norm;
	  lhH *= norm;
	  
	  // likelihood for mips
	  double lhMip = 0.9; // starting value
	  double Emean = 0.33; // MIP peak position
	  double Slope = 0.42; // Somewhat arbitrary ... 
	  if( (*rcclIt)->esum>Emean )
	  lhMip *= exp( -((*rcclIt)->esum-Emean)/Slope); // Use single exp!
	  lhMip *= lhx*lhy;
	  // Update other likelihoods
	  // Suppose lhH+lhE = 1 
	
	  lhH = lhH*(1.-lhMip);
	  lhE = lhE*(1.-lhMip);
	  // Now lhE+lhH+lhMip = 1
	  
	  (*rtIt)->lee = lhE;
	  (*rtIt)->leh = lhH;
	  (*rtIt)->lemi = lhMip;
	  
#ifdef FULL
	  double pullEOPm1 = (eOverP-1)/
	    sqrt((*rcclIt)->cve[14]*invp*invp+
		 cv_invp*(*rcclIt)->esum*(*rcclIt)->esum+0.0004);
	  
	  std::cout << "rcelid: set lh for rt=" << *rtIt 
		    << " eOverP =" << eOverP 
		    << " pullEOPm1 =" << pullEOPm1 
		    << " Cluster energy =" << rcclIt->esum;
	  std::cout<< " lhE=" << lhE
		   << " lhH=" << lhH
		   << " lhMip=" << lhMip 
		   << " ... rccl=" << *rcclIt << std::endl;
#endif
	}
      }
      
      
#ifdef FULL
      for ( ArteTable<RTRA>::iterator rtI=ArteTable<RTRA>::begin();
	    rtI != ArteTable<RTRA>::end(); ++rtI) {
	char cc[13];
	float pmc = 0;
	for ( ArteRelationIterator<MTRA> mtIt=((*rtI)->mtra).begin();
	      mtIt != ((*rtI)->mtra).end(); ++mtIt) {
	  strncpy(cc,(*mtIt)->topp->name,12);
	  cc[12] = 0;
	  pmc = sqrt( (*mtIt)->px * (*mtIt)->px 
		      + (*mtIt)->py * (*mtIt)->py
		      + (*mtIt)->pz * (*mtIt)->pz
		      );
	} 
	std::cout << "rcelid: rt=" << *rtI
	     << " " << cc << " p=" << pmc
	     << " lee=" << (*rtI)->lee
	     << " leh=" << (*rtI)->leh
	     << " lemi=" << (*rtI)->lemi << std::endl;
      }
#endif
    }
  }
}

void DxLikeECAL(double dx, int calo, double & lee, double & leh){
  static double ewidth[] = {0.,1.5,0.9,0.0,0.0,0.4};
  static double hwidth[] = {0.,3.5,2.0,0.0,0.0,1.0};
  if( calo==1 || calo==2 ||calo==5 ){
    double tmp1 = dx/ewidth[calo];
    double tmp2 = dx/hwidth[calo];
    double ne = 1./(ewidth[calo]*ewidth[calo]);
    double nh = 1./(hwidth[calo]*hwidth[calo]);
    lee = (0.0000001+exp(-0.5*tmp1*tmp1))*ne/1.0000001;
    leh = (0.01+exp(-0.5*tmp2*tmp2))*nh/1.01;
    // now normalize in the range 0.25-0.75
    double norm = 0.5*(ne+nh)/(lee+leh);
    lee = 0.25+norm*lee;
    leh = 0.25+norm*leh;
  } else {
    lee = 0.5;
    leh = 0.5;
  }
}

