//                              -*- Mode: C++ -*-
//: ESPIDEnergy2cell.hh
//
//
// Author: Spighi, Villa
// ECAL group 
// HERA-B experiment, Desy
// e-mail: spighi@bo.infn.it, villa@bo.infn.it
//
// Created 27/10/97
// Last Update ........
//

#ifndef ESPIDEnergy3cell_HH_
#define ESPIDEnergy3cell_HH_
#include "care/ESPIDStrategy.hh"

/* 
CLASS
  ESPIDEnergy3cell
   Class for PID (pion/electron discrimination) analysis: 
   it evaluates particular quantities to separate pion and electron
   signals using the different lateral spread of the clusters. 
   In particular the program evaluate te quantity
                               (sum on n cells (Ei)
	   Energy fraction =  -----------------------
                               (sum on total cells Ei)
 
  where n is : 3

  Reference: N.I.M.  A302(1991) 36-46 
*/

class ESPIDEnergy3cell : public ESPIDStrategy {
public:
  //-- Constructors
  // since this is a base class only default constructors and destructors
  // are provided
  ESPIDEnergy3cell();
  virtual ~ESPIDEnergy3cell();
  //--
  // Cluster Analyzer
  virtual void doAnalysis(const EMCluster* clus);;
  //--
  // HitTower Analyzer
  virtual void doAnalysis(const EMHitCell* hitC);
  //--
  //  Start a cluster analysis
  virtual void beginAnalysis(const EMCluster* clus, ArtePointer<RCCL> rccl,
			     const EMHitCell*  hitc);
  //--
  // Evaluation likelihood for electron
  virtual void likeElectron();

  // Evaluation likelihood for hadron
  virtual void likeHadron();

  // set the parameter of likelihood curves for electrons and pions
  static void setparlikeEnergy3cell(int cal,int particle,float likeEnergy[]);

private:
  float  energyTot;
  float  energyfraz;
  int frac_cell;
  float enmax[3];
  int contcell;
  //
  static float eparinner[8];
  static float pparinner[8];
  static float eparmiddle[8];
  static float pparmiddle[8];
  static float eparouter[8];
  static float pparouter[8];
};

#endif // ESPIDEnergy3cell_HH_


































