//                              -*- Mode: C++ -*-
//: ESPIDStrategy.hh
//
//
// Author: Spighi, Villa
// ECAL group 
// HERA-B experiment, Desy
// e-mail: spighi@bo.infn.it, villa@bo.infn.it
//
// Created 3/7/97
// Last Update ........
//

#ifndef ESPIDSTRATEGY_HH_
#define ESPIDSTRATEGY_HH_

#include "care/EAClusterAnalyzer.hh"
#include "care/ECALtypes.hh"

/**
class  ESPIDStrategy.
   Class for PID (pion/electron discrimination) analysis: 
   it evaluates particular quantities to separate pion and electron
   signals using the different lateral spread of the clusters. 
   
*/

class ESPIDStrategy : public EAClusterAnalyzer {
public:
  /** Constructors
   since this is a base class only default constructors and destructors
   are provided. */
  ESPIDStrategy();
  virtual ~ESPIDStrategy();
  //-- 
  /// EMClusterMember Analyzer
  virtual void doAnalysis(const EMClusterMember * clsMem);
  //--
  /// EMHitCell Analyzer
  virtual void doAnalysis(const EMHitCell * hitC);
  //--
  /// EMCellFraction Analyzer  
  inline 
  virtual void doAnalysis(const EMCellFraction * hitT);
  //--
  /// Do a Cluster analysis
  virtual void doAnalysis(const EMCluster* clus);
  //--
  /// EMClusterTree analysis
  virtual void doAnalysis(const EMClusterTree * tree);
  //--
  /// EMTrackOnEcal analysis
  virtual void doAnalysis(const EMTrackOnEcal * Track);

 
  //--
  ///  Start a cluster analysis
  virtual void beginAnalysis(const EMCluster* clus, ArtePointer<RCCL> prccl,
			    const EMHitCell* phitc);
 
  //
  /// Update RCCL bank
  virtual void updatebank(ArtePointer<RCCL> prccl);
  //--
  /// Evaluation likelihood for electron
  virtual void likeElectron()=0;

  /// Evaluation likelihood for hadron
  virtual void likeHadron()=0;

  //--
  ///  Return a constant reference to the private data
  float PIDdata() const { return RPID;}

  ///  Return the likelihood for the electron hypotesis
  float likeElectronData() const { return likeEle;}

  ///  Return the likelihood for the hadron hypotesis
  float likeHadronData() const { return likeHad;}

protected: 
  float RPID;
  float likeEle;
  float likeHad;
  ArtePointer<RCCL> rccl;
  const EMHitCell* hitc;
  float xcog;
  float ycog;
  int typecal;
private:

};

/// EMCellFraction Analyzer  
void  ESPIDStrategy::doAnalysis(const EMCellFraction * hitT){
  doAnalysis((const EMHitCell*) hitT);
};


#endif // ESPIDSTRATEGY_HH_


































