//**********************************************************************/
//
// Function for matching Care and Reconstructed SEGments
//
//**********************************************************************/

#include "marple/marple.hh"
#include "care/ECALDetector.hh"
#include "arte/ArteMessage.hh"
#include "arte/RSEG_cmp.hh"
#include "marple/RcRpntPoint.hh"

std::vector<RcSegPoint* > RangSegList;
bool selectTrack( ArtePointer<RSEG> segm);

void matchRsegCare()
{
  if(RangSegList.size()>0){
    for(std::vector<RcSegPoint* >::iterator iter = RangSegList.begin();
      iter!=RangSegList.end(); iter++){
      delete *iter;
    }
    RangSegList.erase(RangSegList.begin(), RangSegList.end());		
  }

  // RANGER - Care matching
  // create list of tracks from Ranger
  if(ArtePointer<RSEG>::getLink()()==0) return;
 
  for( ArteTable<RSEG>::iterator irseg=ArteTable<RSEG>::begin(); 
       irseg!=ArteTable<RSEG>::end(); ++irseg ){
    
    if( selectTrack(*irseg)){

      if( (*irseg)->ze < 1350.0 ){ // main tracker track
	RangSegList.push_back( new RcSegPoint( *irseg, 1 ));
	RangSegList.back()->setty( (*irseg)->yf/(*irseg)->zf ); // since first
                                            //point is much more precise (O.I.)
      } else { // muon track
	RangSegList.push_back( new RcSegPoint(*irseg, 0 ));
      }
      RangSegList.back()->lineswim(1330.); // to speed up procedure 
    }
  }

  for(std::vector<RcSegPoint* >::iterator iter = RangSegList.begin();
      iter!=RangSegList.end(); iter++){
    // store the pointer to the track segment
    ECALDetector::storeTrack(**iter);
  }
  //arteMsg<<"Matching "<<RangSegList.size()<<" segments ";
  // arteMsg.allp();
}
  
// Selection function for segments to be matched
bool selectTrack( ArtePointer<RSEG> segm){
  bool selq = ( segm->fit == 0                   // good track 
	       && !(segm->cmp & Rsegc::bitsign)   // not killed
	       && (segm->cmp & (Rsegc::patt | Rsegc::trit |Rsegc::muon) )
	       // PC+TC+Muon track selection
	       // no downstream segment
	       && segm->ze > 730.0);            // behind magnet
  if( selq && (segm->cmp & Rsegc::muon) 
      && (segm->rbeg.begin()!=segm->rbeg.end()) && segm->zf > 1380.0) {
    // muon track with an initial segment: discard it, if previous segment
    // is not too far from calorimeter (I would like only TC tracks!)
    selq = selq && (*(segm->rbeg.begin()))->ze<950.0; 
  } else if( selq && (segm->cmp & Rsegc::patt) 
	     && (segm->rend.begin()!=segm->rend.end())){
    selq = selq && (*(segm->rend.begin()))->zf>1380.0;
  }
  
  return selq;
}


// ********************************************************************
//
//  Routine: matchRtraCare
//  Author:  Mauro Villa
//
// Description: Function for matching Care and Reconstructed RTRA's
// (VDS-tracker and tracker-only)
//
//
//  Modification:
//  - renamed to avoid crash with old reco chain  -- Patricia Conde
//  - check on rtra bitsign and patt bit             Wouter Hulsbergen
//    replacement of ty by y/z
//    also match rtras without TC segment (!)
// ********************************************************************

ArtePointer<RPNT> selectTrack( ArtePointer<RTRA> rtra);

void matchRtraCare(){

  if(RangSegList.size()>0){
    for(std::vector<RcSegPoint* >::iterator iter = RangSegList.begin();
      iter!=RangSegList.end(); iter++){
      delete *iter;
    }
    RangSegList.erase(RangSegList.begin(), RangSegList.end());		
  }

  if( ArtePointer<RSEG>::getLink()()==0 ) return;

  for( ArteTable<RTRA>::iterator irtra=ArteTable<RTRA>::begin(); 
      irtra!=ArteTable<RTRA>::end(); ++irtra )
    if( !(irtra->cmp & Rsegc::bitsign) && (irtra->cmp & Rsegc::patt) ) {
      ArtePointer<RPNT> rpnt = selectTrack(*irtra);
      if( rpnt.index()>0 ) {

	ArtePointer<RSEG> tracker = 0;
	for( ArteRelationIterator<RSEG>::iterator rseg=irtra->rseg.begin(); 
	     rseg != irtra->rseg.end(); ++rseg ) 
	  if( ((*rseg)->cmp & Rsegc::trit) ||
	      ( ((*rseg)->cmp & Rsegc::patt) && tracker.index()==0) )
	    tracker = (*rseg);
	
	if( tracker.index()>0 ) {
	  RcRpntPoint rpn(rpnt);
	  // this is the line where ty is overwritten 
	  if( !(irtra->cmp & Rsegc::vxd) ) rpn.setty( rpn.y()/rpn.z() ) ;
	  RcSegPoint * rs = new RcSegPoint(rpn,tracker);
	  RangSegList.push_back( rs );
	  RangSegList.back()->lineswim(1330.); // to speed up procedure 
	  //	cout<<" Track "<<rtra<<" pnt:"<<rpnt<<" Segm:"<<tracker<<endl;
	}
      }
    }

  for(std::vector<RcSegPoint* >::iterator iter = RangSegList.begin();
      iter!=RangSegList.end(); iter++){
    // store the pointer to the track segment
    ECALDetector::storeTrack(**iter);
  }
  //  arteMsg<<"Matching "<<RangSegList.size()<<" segments ";
  //  arteMsg.allp();
}
  
// Selection function for tracks to be matched

ArtePointer<RPNT> selectTrack( ArtePointer<RTRA> rtra){
  
  ArtePointer<RPNT> rpnt(0);
  for(ArteRelationIterator<RPNT> irpnt = rtra->rpnt.begin();
      irpnt != rtra->rpnt.end(); ++irpnt )
    if( (*irpnt)->zf<1400. && (rpnt.index()==0 || (*irpnt)->zf>rpnt->zf) )
      rpnt = (*irpnt);
  
  return rpnt;
}
