//  Authors:  Yuri Gorbunov                    Created:  01-Dec-00 ver. 0.01
//            Dmitry Emeliyanov
//            Ivan Kisel                       Modified: 16-Apr-01 ver. 1.01 
//  Email:    emelian@mail.desy.de                       26-Apr-01 ver. 1.02
//            gorbunov@mail.desy.de                      03-May-01 ver. 1.03
//            ikisel@mppmu.mpg.de                        20-May-01 ver. 1.04
//            HERA-B Collaboration                       20-May-01 ver. 2.01 
//                                                       30-May-01 ver. 2.02
//                                                       05-Jul-01 ver. 2.03
//                                                       12-Dec-01 ver. 3.00

  
#include "arte/RSEG_cmp.hh"
#include <map>
#include "cats/ItrCats.hh"
#include "cats/OtrCats.hh"
#include "cats/hbook.h"
#include "cats/higz.h"
#include "arte/Chrono.hh"
#include <iostream.h>
#include "farm/farm.hh"
#include <math.h>
extern "C"{
#include "sltinfo.h"  
}
#include "rang/zfpar.hh"

Chrono zeit_itr_total    ("ItrCats TOTAL ");
Chrono zeit_itr_start    ("Itr START     ");
Chrono zeit_itr_spoint   ("Itr SPOINT    ");
Chrono zeit_itr_neighbors("Itr NEIGHBORS ");
Chrono zeit_itr_collect  ("Itr COLLECT   ");
Chrono zeit_itr_gather   ("Itr GATHER    ");
Chrono zeit_itr_add      ("Itr ADD      ");
Chrono zeit_itr_refit    ("Itr REFIT     ");
Chrono zeit_itr_ghost    ("Itr GHOST     ");
Chrono zeit_itr_rseg     ("Itr RSEG      ");


using namespace std;

typedef struct{
  pSPacepoint psp;
  int propagated,nsl;           
  float xm, ym, dxm, dym;
} PRopspoint, *pPRopspoint;

typedef struct{
  float sin, cos, alpha, sxmn, sxmx, symn, symx, szmn, szmx;
} OTR_SIZE, *pOTR_SIZE;

typedef struct
{
  ArtePointer<GEDE> gede;
  OTR_SIZE gz;
} GEDE_TYPE, *pGEDE_TYPE;

typedef struct 
{
  float y0,x0,yr,xr,xi,xm,zf,ze;
  GEDE_TYPE s[13][100];
  int ngede[13];
} SLGEOM, *pSLGEOM;


int SLT_MODE;

static SLGEOM slg[4]; 

static int ItrCatsRun = 0;
static int MS10shift=0;
static int OTRshift=0;
static const double Bmagnet =   0.65945;
static const double zmagnet =   450.0;

vector<vector<pSPacepoint> > vpneighborSP;
vector<vector<float> >       vchi2neighborSP;
vector<SPacepoint> vspacepSL[3];
ArtePointer<GESL>  gesl_sl[4];
vector<Track> vectrack;

extern "C"{
  float prob_(const float & ccchi2, const int & ndeg);
}

bool ItrPropSPointCompareX(const PRopspoint &a, const PRopspoint &b){
  double ax, bx;
  if (!a.propagated) ax = a.psp->xm-5.0*sqrt(a.psp->cvm[0][0]);
  else               ax = a.xm-a.dxm;
  if (!b.propagated) bx = b.psp->xm-5.0*sqrt(b.psp->cvm[0][0]);
  else               bx = b.xm-b.dxm;
  return (ax < bx);
}
bool ItrHitCompareZ(const pHit a, const pHit b){
  return (a->z < b->z);
}
bool ItrHitCompareX(const pHit a, const pHit b){
  return (a->x < b->x);
}

bool ItrTrackCompareLength(const Track &a, const Track &b){

  /*
  int a_nsp = (a.nclu/9)*9;
  int b_nsp = (b.nclu/9)*9;
  if (a_nsp != b_nsp)
    return (a.nclu > b.nclu);
  else
    //return (a.pf < b.pf);
    return (a.nclu > b.nclu);
  */

  if (a.nclu != b.nclu)
    return (a.nclu > b.nclu);
  else
    return (a.prob > b.prob);

}


extern void vier_dim_inv(double a[4][4]);

float ItrNeighborsChi2(const pSPacepoint sp1, const pSPacepoint sp2);
float ItrNeighborsChi2(const pTrack ptrack1, const pTrack ptrack2);
float ItrNeighborsChi2(const ArtePointer<RSEG> rseg1, const ArtePointer<RSEG> rseg2);
void  ItrFindTrack(const pSPacepoint psp, Track& newtrack, Track currenttrack);
void  ItrFastRefitTrack(Track*);
void  Itr_fill_rseg(vector<Track> *tracktmp,int typevhit);
void  ItrFillRSEG(vector<Track> *tracktmp);
void  ItrGraphIni(void);
void  ItrFitMS(SPacepoint& sp);

void  FillOTrGeometry();
void  ItrOtrProlongation();
void  OtrResolveLR(pTRACK);
void  OtrFastDriftRefitTrack(pTRACK);
bool  OtrHitCompareZ(const pHIT a, const pHIT b);
void  OtrGedeMinMax (float&, float&, float&, float&, float&, float&, const ArtePointer<GEDE>);
void CheckPrevSL(const vector<Track>::iterator trackI, int& nsregions, int& nhits);
void CheckNextSL(const vector<Track>::iterator trackI, int& nsregions, int& nhits);
void SltCatsEvent();

int ITR_DRAW     =  0;

static int Black =  1, 
    DarkGray     =  2, 
    Blue         =  3, 
    Green        =  4, 
    Cyan         =  5, 
    Red          =  6, 
    Magenta      =  7, 
    Brown        =  8, 
    LightBlue    =  9, 
    LightGreen   = 10, 
    LightCyan    = 11, 
    LightRed     = 12, 
    LightMagenta = 13, 
    Yellow       = 14, 
    LightGray    = 15, 
    White        =  0;

static float xpl[100], ypl[100], zpl[100], xpm[100], zpm[100];// ypm[100]
static float x_shift = -25.0;  //ITR 
static float y_shift =  25.0;  //ITR
static int first_call = 1;

static map<int, ArtePointer<GESL> >   geslmap;
static ArtePointer<GEDE>  gede_bank[8][24];
static ArtePointer<GEDE>  bank_gede[4][24];

void ItrCatsEvent(){

  zeit_itr_total.start();

  zeit_itr_start.start();

  int ItrCatsFillGeo = 0;
  ArtePointer<EVHD> evhd = *(ArteTable<EVHD>::begin());

  if (ItrCatsRun == 0){
    ItrCatsRun = evhd->run;
    ItrCatsFillGeo = 1;
  }

  if (evhd->run != ItrCatsRun){
    ItrCatsRun = evhd->run;
    ItrCatsFillGeo = 1;
  }

  if (first_call){
    ItrCatsFillGeo = 1;
  }


  if (first_call){
    first_call = 0;

    cout<<"ItrCats  version 3.00 started"<<endl;
    SLT_MODE=0;
    if(zfpar_.sltmode == 4) SLT_MODE=1;
    if(zfpar_.sltmode == 5) SLT_MODE=2;
    if(zfpar_.sltmode == 6) SLT_MODE=3;

    if(SLT_MODE) cout<<"SLT seeding is ON, sltmode="<<SLT_MODE<<endl;
    else cout<<"SLT seeding is OFF"<<endl;

    if (ITR_DRAW)
      ItrGraphIni();
  }

  if (ItrCatsFillGeo){

    //cout<<"---------------------->"<<endl;
    //cout<<"---------------------->  *** ItrCats new geo filled for run "<<ItrCatsRun<<" ***"<<endl;
    //cout<<"---------------------->"<<endl;

    geslmap.clear();

    FillOTrGeometry();

    char msfi[5],strtmp[5];
    MS10shift=0;
    strcpy(msfi,"MS10");

    for ( ArteTable<GESL>::iterator geslIt = ArteTable<GESL>::begin();
	  geslIt != ArteTable<GESL>::end(); geslIt++ ){
      strtmp[0]=(*geslIt)->name[0];
      strtmp[1]=(*geslIt)->name[1];
      strtmp[2]=(*geslIt)->name[2];
      strtmp[3]=(*geslIt)->name[3];
      strtmp[4]=msfi[4];   
      if(!strcmp(strtmp,msfi)){
	if((*geslIt)->nsl==5) 
	  {
	    MS10shift=0;
	    OTRshift=0;
	  }
	if((*geslIt)->nsl<5)  
	  {
	    MS10shift=3;
	    OTRshift=6;
	  }
      }
    }
  
    for ( ArteTable<GESL>::iterator geslIt = ArteTable<GESL>::begin();
	  geslIt != ArteTable<GESL>::end(); geslIt++ ) {//gesl
      
      if ((*geslIt)->cmp!=3) continue; // not ITR
      if (((*geslIt)->nsl<(5-MS10shift))||((*geslIt)->nsl>(8-MS10shift))) continue; // not PC

      geslmap.insert(pair<int, ArtePointer<GESL> >((*geslIt).index(), *geslIt));
      
      for ( ArteRelationIterator<GEDE> gedeIt = (*geslIt)->gede.begin();
	    gedeIt != (*geslIt)->gede.end(); gedeIt++ ) {
	
	int sec = (*gedeIt)->sec;
	
	int index_Lay=0;
	if((*geslIt)->nsl==(5-MS10shift)) index_Lay=(*gedeIt)->lay - 1;
	if((*geslIt)->nsl==(6-MS10shift)) index_Lay=(*gedeIt)->lay + 7 ;
	if((*geslIt)->nsl==(7-MS10shift)) index_Lay=(*gedeIt)->lay + 11 ;
	if((*geslIt)->nsl==(8-MS10shift)) index_Lay=(*gedeIt)->lay + 15 ; 
	
	if( sec == 1 || sec == 2 ) bank_gede[0][index_Lay]=(*gedeIt);
	if( sec == 3 || sec == 4 ) bank_gede[1][index_Lay]=(*gedeIt);
	if( sec == 5 || sec == 6 ) bank_gede[2][index_Lay]=(*gedeIt);
	if( sec == 7 || sec == 8 ) bank_gede[3][index_Lay]=(*gedeIt);
	
	if( sec==1) gede_bank[(sec-1)][index_Lay]=(*gedeIt);
	if( sec==2) gede_bank[(sec-1)][index_Lay]=(*gedeIt);
	if( sec==3) gede_bank[(sec-1)][index_Lay]=(*gedeIt);
	if( sec==4) gede_bank[(sec-1)][index_Lay]=(*gedeIt);
	if( sec==5) gede_bank[(sec-1)][index_Lay]=(*gedeIt);
	if( sec==6) gede_bank[(sec-1)][index_Lay]=(*gedeIt);
	if( sec==7) gede_bank[(sec-1)][index_Lay]=(*gedeIt);
	if( sec==8) gede_bank[(sec-1)][index_Lay]=(*gedeIt);
	

      }
    }
  }

  if (ITR_DRAW){
    isfaci(White); isplci(Red);

    igbox(680.00, 830.00,-55.00+x_shift, 55.00+x_shift);
    igbox(680.00, 830.00,-55.00+y_shift, 55.00+y_shift);

    isplci(Black); ispmci(Red);
    ismk(3); ismksc(0.5);
  }



  vector<Hit>        hits_bank_z[3][8];
  vector<Hit>        hits_bank_s[3][8];
  vector<pHit>       hits_bank_lay[24];

  
  

  for(int i=0;i<24;i++) hits_bank_lay[i].clear();

  float geslzmin[3], geslzmax[3];
  for(int i=0;i<3;i++){
    for(int j=0;j<8;j++){
      hits_bank_z[i][j].clear();
      hits_bank_s[i][j].clear();
    }
    geslzmin[i] = 0.0;
    geslzmax[i] = 0.0;
  }

  zeit_itr_spoint.start();

  for (map<int, ArtePointer<GESL> >::iterator geslmapIt = geslmap.begin();
	geslmapIt != geslmap.end(); geslmapIt++ ) {
    
    ArtePointer<GESL> gesl = geslmapIt->second;
   
    if(gesl->nsl==(5-MS10shift)) gesl_sl[0]=gesl;
    if(gesl->nsl==(6-MS10shift)) gesl_sl[1]=gesl;
    if(gesl->nsl==(7-MS10shift)) gesl_sl[2]=gesl;
    if(gesl->nsl==(8-MS10shift)) gesl_sl[3]=gesl;

    for ( ArteRelationIterator<GEDE> gedeIt = gesl->gede.begin();
	  gedeIt != gesl->gede.end(); gedeIt++ ) {
      
      float cosa = (*gedeIt)->cos*(*gedeIt)->rot[0] - (*gedeIt)->sin*(*gedeIt)->rot[1];
      float sina = (*gedeIt)->sin*(*gedeIt)->rot[4] - (*gedeIt)->cos*(*gedeIt)->rot[3];

      float alpha = 180.0/3.1415926*atan2(sina, cosa); if (alpha<-10.0) alpha += 360.0;

      if (ITR_DRAW){
	if ( alpha < -2.0)                  isplci(Magenta);
	if ((alpha >=-2.0)&&(alpha <= 2.0)) isplci(Black  ); 
	if ( alpha >  2.0)                  isplci(Cyan   );
	
	zpl[0] = 0.5*((*gedeIt)->zmin + (*gedeIt)->zmax); 
	xpl[0] = (*gedeIt)->sxmn + x_shift; 
	ypl[0] = (*gedeIt)->symn + y_shift; 
	zpl[1] = 0.5*((*gedeIt)->zmin + (*gedeIt)->zmax); 
	xpl[1] = (*gedeIt)->sxmx + x_shift; 
	ypl[1] = (*gedeIt)->symx + y_shift;
	ipl(2,zpl,xpl); ipl(2,zpl,ypl);
      }

      ArteTableIterator<HITB> hitbf=(*gedeIt)->hit1;
      ArteTableIterator<HITB> hitbl=(*gedeIt)->hitl;     

      if (ITR_DRAW){
	switch ((*gedeIt)->sec){
	case 1:  {ispmci(Black     ); break;}
	case 2:  {ispmci(DarkGray  ); break;}
	case 3:  {ispmci(Blue      ); break;}
	case 4:  {ispmci(Red       ); break;}
	case 5:  {ispmci(Green     ); break;}
	case 6:  {ispmci(Magenta   ); break;}
	case 7:  {ispmci(Cyan      ); break;}
	case 8:  {ispmci(Brown     ); break;}
	case 9:  {ispmci(LightBlue ); break;}
	case 10: {ispmci(LightGreen); break;}
	default:  ispmci(DarkGray  );
	}
      }
   
      int nslPoint=0,nslAll=0;
      switch ((gesl->nsl+MS10shift)){
      case 5: {nslAll=0; nslPoint = 0; break;}
      case 6: {nslAll=1; nslPoint = 1; break;}
      case 7: {nslAll=2; nslPoint = 1; break;}
      case 8: {nslAll=3; nslPoint = 2; break;}
      }
      geslzmin[nslPoint] = gesl->z1;
      geslzmax[nslPoint] = gesl->z2;
  
      int sec = (*gedeIt)->sec;
      int lay = (*gedeIt)->lay+(gesl->nsl)*10;
      hitbl++;
      Hit hitc;
      /*
      int index_Lay=0;
      if(nslAll==0) index_Lay=(*gedeIt)->lay - 1;
      if(nslAll==1) index_Lay=(*gedeIt)->lay + 7 ;
      if(nslAll==2) index_Lay=(*gedeIt)->lay + 11 ;
      if(nslAll==3) index_Lay=(*gedeIt)->lay + 15 ; 
      */
      if (distance(hitbf, hitbl)+1 == 0) continue;
      for( ArteTable<HITB>::iterator hitbIt = hitbf ;
	   hitbIt != hitbl; ++hitbIt ){

	hitc.x    = (*hitbIt)->x;
	hitc.z    = (*hitbIt)->z;
	hitc.lay  = lay;
	hitc.used = 0;
	hitc.cosa = cosa;
	hitc.sina = sina;
	hitc.gede = *gedeIt;
	hitc.hitb = (*hitbIt);
	if ((alpha > -1.0)&&(alpha < 1.0))
	  hits_bank_z[nslPoint][sec-1].push_back(hitc);
	else
	  hits_bank_s[nslPoint][sec-1].push_back(hitc);
	
	// hits_bank_lay[index_Lay].push_back(hitc);
	
	if (ITR_DRAW){
	  zpm[0] = (*hitbIt)->z; xpm[0] = (*hitbIt)->x+x_shift; 
	  ipm(1, zpm, xpm);
	}
	
      }
    }
  }

  if (ITR_DRAW)
    iuwk(1,1);

  for(int i=0;i<3;i++){ 
    vspacepSL[i].clear();
    for(int j=0;j<8;j++){
      if (distance(hits_bank_z[i][j].begin(),hits_bank_z[i][j].end()) != 0){
	ItrSPoints(&hits_bank_z[i][j], &hits_bank_s[i][j], &vspacepSL[i], i, geslzmin[i], geslzmax[i],MS10shift);

	for(vector<Hit>::iterator zerHit = hits_bank_z[i][j].begin();
	    zerHit!=hits_bank_z[i][j].end();zerHit++)
	  (*zerHit).used=0;
	for(vector<Hit>::iterator zerHit = hits_bank_s[i][j].begin();
	    zerHit!=hits_bank_s[i][j].end();zerHit++)
	  (*zerHit).used=0;

      }
    }   
  }

  zeit_itr_spoint.stop();
  zeit_itr_start.stop();

  zeit_itr_neighbors.start();

  // matching 
  vpneighborSP.clear();
  vchi2neighborSP.clear();
  int neighborindex = 0;
  for(int i=1;i<3;i++){
    float lgesl_zm = 0.5*(geslzmin[i-1]+geslzmax[i-1]);
    
    vector<PRopspoint> vpropspoint;
    vpropspoint.clear();
    
    for(vector<SPacepoint>::iterator rspIt  = vspacepSL[i].begin();
	rspIt != vspacepSL[i].end(); ++rspIt ){
      
      PRopspoint propspoint;
      propspoint.psp = rspIt;
      propspoint.propagated = 1; // propagate to the left
      propspoint.xm  = rspIt->xm + rspIt->txm*(lgesl_zm - rspIt->zm);
      propspoint.ym  = rspIt->ym + rspIt->tym*(lgesl_zm - rspIt->zm);
      propspoint.dxm = 3.0*sqrt(rspIt->cvm[0][0]+(lgesl_zm-rspIt->zm)*(lgesl_zm-rspIt->zm)*rspIt->cvm[2][2]);
      propspoint.dym = 3.0*sqrt(rspIt->cvm[1][1]+(lgesl_zm-rspIt->zm)*(lgesl_zm-rspIt->zm)*rspIt->cvm[3][3]);      
      vpropspoint.push_back(propspoint);
    }
    
    float rgesl_zm = 0.5*(geslzmin[i]+geslzmax[i]);
    
    for(vector<SPacepoint>::iterator lspIt  = vspacepSL[i-1].begin();
	lspIt != vspacepSL[i-1].end(); ++lspIt ){
      
      PRopspoint propspoint;
      propspoint.psp = lspIt;
      propspoint.propagated = 0; // resl sp, but propagate to the right gesl
      propspoint.xm  = lspIt->xm + lspIt->txm*(rgesl_zm - lspIt->zm);
      propspoint.ym  = lspIt->ym + lspIt->tym*(rgesl_zm - lspIt->zm);
      propspoint.dxm = 3.0*sqrt(lspIt->cvm[0][0]+(rgesl_zm-lspIt->zm)*(rgesl_zm-lspIt->zm)*lspIt->cvm[2][2]);
      propspoint.dym = 3.0*sqrt(lspIt->cvm[1][1]+(rgesl_zm-lspIt->zm)*(rgesl_zm-lspIt->zm)*lspIt->cvm[3][3]);
      vpropspoint.push_back(propspoint);
    }
    
    sort(vpropspoint.begin(), vpropspoint.end(), ItrPropSPointCompareX);
    
    for(vector<PRopspoint>::iterator propspIt  = vpropspoint.begin();
	propspIt != vpropspoint.end(); ++propspIt ){
      
      if (!propspIt->propagated) continue;
      
      float x_top = propspIt->xm+propspIt->dxm;
      
      vector<pSPacepoint> vpnei;
      vector<float>       vchi2nei;
      int neifound = 0;
      
      vector<PRopspoint>::iterator vpropspoint_begin = propspIt;
      vpropspoint_begin++;
      
      for(vector<PRopspoint>::iterator neiIt  = vpropspoint_begin;
	  neiIt != vpropspoint.end(); ++neiIt ){
	
	if (!neiIt->propagated) 
	  if ((neiIt->psp->xm+5.0*sqrt(neiIt->psp->cvm[0][0])) > x_top) break;

	// skip if propagated sp
	if (neiIt->propagated) continue;
	
#ifdef XXX
	//float const_dx = 1.0, const_dy = 5.0;
	//float const_dx = 0.0300, const_dy = 0.1;
	float const_dx = 0.0, const_dy = 0.0;
	
	if (fabs(propspIt->xm - neiIt->psp->xm) > 
	    (propspIt->dxm+3.0*sqrt(neiIt->psp->cvm[0][0])+const_dx)) continue; 
	// out at the left gesl
	if (fabs(propspIt->psp->xm - neiIt->xm) > 
	    (neiIt->dxm+3.0*sqrt(propspIt->psp->cvm[0][0])+const_dx)) continue;    
	// out at the right gesl

	if (fabs(propspIt->ym - neiIt->psp->ym) > 
	    (propspIt->dym+3.0*sqrt(neiIt->psp->cvm[1][1])+const_dy)) continue; 
	// out at the left gesl
	if (fabs(propspIt->psp->ym - neiIt->ym) > 
	    (neiIt->dym+3.0*sqrt(propspIt->psp->cvm[1][1])+const_dy)) continue;    
	// out at the right gesl
#endif

	vector<pHit> vhit;
	vhit.clear();

	for (vector<pHit>::iterator hitIt = neiIt->psp->vhit.begin();
	     hitIt != neiIt->psp->vhit.end(); ++hitIt){
	  vhit.push_back(*hitIt);
	}
	sort(vhit.begin(), vhit.end(), ItrHitCompareZ);

	float zzmin1 = 1000.0, zzmax1 = 0.0;
	float xzmin1 = 0.0,    xzmax1 = 0.0;
	float zpmin1 = 1000.0, zpmax1 = 0.0;
	float xpmin1 = 0.0,    xpmax1 = 0.0;
	float zmmin1 = 1000.0, zmmax1 = 0.0;
	float xmmin1 = 0.0,    xmmax1 = 0.0;

	for (vector<pHit>::iterator hitIt = vhit.begin();
	     hitIt != vhit.end(); ++hitIt){
	  float z = (*hitIt)->z;
	  float x = (*hitIt)->x;
	  float cosa = (*hitIt)->cosa;
	  float sina = (*hitIt)->sina;
	  float alpha = 180.0/3.1415926*atan2(sina, cosa); if (alpha<-10.0) alpha += 360.0;
	  if ((alpha > -1.0)&&(alpha < 1.0)){ // zero projection
	    if (z < zzmin1){zzmin1 = z; xzmin1 = x;}
	    if (z > zzmax1){zzmax1 = z; xzmax1 = x;}
	  }
	  if (alpha < -1.0){ // stereo minus projection
	    if (z < zmmin1){zmmin1 = z; xmmin1 = x;}
	    if (z > zmmax1){zmmax1 = z; xmmax1 = x;}
	  }
	  if (alpha >  1.0){ // stereo plus projection
	    if (z < zpmin1){zpmin1 = z; xpmin1 = x;}
	    if (z > zpmax1){zpmax1 = z; xpmax1 = x;}
	  }
	}

	vhit.clear();

	for (vector<pHit>::iterator hitIt = propspIt->psp->vhit.begin();
	     hitIt != propspIt->psp->vhit.end(); ++hitIt){
	  vhit.push_back(*hitIt);
	}
	sort(vhit.begin(), vhit.end(), ItrHitCompareZ);

	float zzmin2 = 1000.0, zzmax2 = 0.0;
	float xzmin2 = 0.0,    xzmax2 = 0.0;
	float zpmin2 = 1000.0, zpmax2 = 0.0;
	float xpmin2 = 0.0,    xpmax2 = 0.0;
	float zmmin2 = 1000.0, zmmax2 = 0.0;
	float xmmin2 = 0.0,    xmmax2 = 0.0;

	for (vector<pHit>::iterator hitIt = vhit.begin();
	     hitIt != vhit.end(); ++hitIt){
	  float z = (*hitIt)->z;
	  float x = (*hitIt)->x;
	  float cosa = (*hitIt)->cosa;
	  float sina = (*hitIt)->sina;
	  float alpha = 180.0/3.1415926*atan2(sina, cosa); if (alpha<-10.0) alpha += 360.0;
	  if ((alpha > -1.0)&&(alpha < 1.0)){ // zero projection
	    if (z < zzmin2){zzmin2 = z; xzmin2 = x;}
	    if (z > zzmax2){zzmax2 = z; xzmax2 = x;}
	  }
	  if (alpha < -1.0){ // stereo minus projection
	    if (z < zmmin2){zmmin2 = z; xmmin2 = x;}
	    if (z > zmmax2){zmmax2 = z; xmmax2 = x;}
	  }
	  if (alpha >  1.0){ // stereo plus projection
	    if (z < zpmin2){zpmin2 = z; xpmin2 = x;}
	    if (z > zpmax2){zpmax2 = z; xpmax2 = x;}
	  }
	}

	int ok = 1;

	float distmax = 5.0*0.0300;
	if ((zzmin1 != 1000.0)&&(zzmax1 != 0.0)&&(zzmin2 != 1000.0)&&(zzmax2 != 0.0)){
	  float a = (xzmax2-xzmin1)/(zzmax2-zzmin1);
	  float dist1 = (xzmin1 + a*(zzmax1-zzmin1)) - xzmax1;
	  float dist2 = (xzmin1 + a*(zzmin2-zzmin1)) - xzmin2;
	  if ((fabs(dist1) > distmax)||(fabs(dist2) > distmax)) ok = 0;
	}
	if ((zmmin1 != 1000.0)&&(zmmax1 != 0.0)&&(zmmin2 != 1000.0)&&(zmmax2 != 0.0)){
	  float a = (xmmax2-xmmin1)/(zmmax2-zmmin1);
	  float dist1 = (xmmin1 + a*(zmmax1-zmmin1)) - xmmax1;
	  float dist2 = (xmmin1 + a*(zmmin2-zmmin1)) - xmmin2;
	  if ((fabs(dist1) > distmax)||(fabs(dist2) > distmax)) ok = 0;
	}
	if ((zpmin1 != 1000.0)&&(zpmax1 != 0.0)&&(zpmin2 != 1000.0)&&(zpmax2 != 0.0)){
	  float a = (xpmax2-xpmin1)/(zpmax2-zpmin1);
	  float dist1 = (xpmin1 + a*(zpmax1-zpmin1)) - xpmax1;
	  float dist2 = (xpmin1 + a*(zpmin2-zpmin1)) - xpmin2;
	  if ((fabs(dist1) > distmax)||(fabs(dist2) > distmax)) ok = 0;
	}

	if (!ok) continue;


	//ok, probably both are neighbors -> calculate chi2

	SPacepoint sp;
	
	sp.xf  = neiIt->psp->xf;      sp.txf = neiIt->psp->txf;
	sp.yf  = neiIt->psp->yf;      sp.tyf = neiIt->psp->tyf; 
	sp.zf  = neiIt->psp->zf;
	sp.xe  = propspIt->psp->xe;   sp.txe = propspIt->psp->txe;
	sp.ye  = propspIt->psp->ye;   sp.tye = propspIt->psp->tye;
	sp.ze  = propspIt->psp->ze;
	sp.used = 0;                   sp.level = 0; sp.neighborindex = -1;
	
	
	sp.vhit.clear();

	for (vector<pHit>::iterator hitIt = neiIt->psp->vhit.begin();
	     hitIt != neiIt->psp->vhit.end(); ++hitIt){
	  sp.vhit.push_back(*hitIt);
	}
	
	for (vector<pHit>::iterator hitIt = propspIt->psp->vhit.begin();
	     hitIt != propspIt->psp->vhit.end(); ++hitIt){
	  sp.vhit.push_back(*hitIt);
	}

	ItrFitSPoint(sp,0);
		
	if (sp.prob > 0.05){

	  if (neiIt->psp->level == 0)                     neiIt->psp->level++;
	  if (neiIt->psp->level+1 > propspIt->psp->level) propspIt->psp->level = neiIt->psp->level+1;
	  neifound = 1;
	  vpnei.push_back(neiIt->psp);
	  vchi2nei.push_back(sp.chi2);  
	}
      }
      if (neifound){
	propspIt->psp->neighborindex = neighborindex++;
	vpneighborSP.push_back(vpnei);
	vchi2neighborSP.push_back(vchi2nei);
      }
    }
  }
  zeit_itr_neighbors.stop();
  
  zeit_itr_collect.start();
  vector<Track> vectracktmp;
  vectracktmp.clear();
  for ( int level = 3; level >= 0; level-- ) {
   
    vector<Track> vtrackcandidate;
    vtrackcandidate.clear();

    for (int sl_loop=0; sl_loop<3; sl_loop++){
      for(vector<SPacepoint>::iterator spIt  = vspacepSL[sl_loop].begin();
	  spIt != vspacepSL[sl_loop].end(); ++spIt ){
	//if ((spIt->used)&&(spIt->pf>1.0)) continue; // slow track - remove
	//if (spIt->used) continue; 
	
	if (spIt->level == level){
	  
	  int zlay = 0;// check zero layers
	  int slay = 0;// check stereo layers
	  for (vector<pHit>::iterator pcluIt = spIt->vhit.begin();
	       pcluIt != spIt->vhit.end(); ++pcluIt){
	    if (!(*pcluIt)->used){
	      float alpha = 180.0/3.1415926*atan2((*pcluIt)->sina,(*pcluIt)->cosa); 
	      if (alpha<-10.0) alpha += 360.0;
	      if ((alpha > -1.0)&&(alpha < 1.0)) zlay++;
	      else                               slay++;
	    }
	  }
	  if ((zlay <= 1)||(slay <= 1)) spIt->used = 1;// used space point
	  
	  //if ((spIt->used)&&(spIt->pf>1.0)) continue; // again slow track - remove
	  //if (spIt->used) continue; 
	
	  Track newtrack, currenttrack;
	  if(sl_loop==0)
	    newtrack.pspointmap.insert(pair< ArtePointer<GESL>, pSPacepoint >
				       (gesl_sl[0], spIt));
	  if(sl_loop==1)
	    newtrack.pspointmap.insert(pair< ArtePointer<GESL>, pSPacepoint >
				       (gesl_sl[1], spIt));
	  if(sl_loop==2)
	    newtrack.pspointmap.insert(pair< ArtePointer<GESL>, pSPacepoint >
				       (gesl_sl[3], spIt));
	  newtrack.tyf  = spIt->tyf;
	  newtrack.chi2 = spIt->chi2;
	  newtrack.prob = spIt->prob;
	  newtrack.nsp  = 1;
	  newtrack.nclu = distance(spIt->vhit.begin(),spIt->vhit.end()); 
	  newtrack.good = 1;

	  if (level>1){
	    if(sl_loop==0)
	      currenttrack.pspointmap.insert(pair< ArtePointer<GESL>, pSPacepoint >
					     (gesl_sl[0], spIt));
	    if(sl_loop==1)
	      currenttrack.pspointmap.insert(pair< ArtePointer<GESL>, pSPacepoint >
					     (gesl_sl[1], spIt)); 
	    if(sl_loop==2)
	      currenttrack.pspointmap.insert(pair< ArtePointer<GESL>, pSPacepoint >
					     (gesl_sl[3], spIt));
	    currenttrack.tyf  = spIt->tyf;
	    currenttrack.chi2 = spIt->chi2;
	    currenttrack.prob = spIt->prob;
	    currenttrack.nsp  = 1;
	    currenttrack.nclu = distance(spIt->vhit.begin(),spIt->vhit.end()); 
	    currenttrack.good = 1;
	  
  
	   ItrFindTrack(spIt, newtrack, currenttrack);
	  }

	  if (newtrack.good){
	    {
	      float pinv=1.0/(Bmagnet*zmagnet*
			      sqrt(1.0+newtrack.txf*newtrack.txf+newtrack.tyf*newtrack.tyf)/
			      ((newtrack.xf-newtrack.zf*newtrack.txf)*
			       sqrt(1.0+newtrack.txf*newtrack.txf)));
	      int sign1;
	      if(pinv<0.0) sign1=-1;else sign1=1;
	      pinv = fabs(pinv);
	      if (pinv > 0.5) pinv = 2.0*pinv*pinv;
	      newtrack.pf=newtrack.pe=pinv*sign1;
	    }
	    vtrackcandidate.push_back(newtrack); //store track
	    
	  }
	}
      }
    }

    // 1. sort tracks
    sort(vtrackcandidate.begin(), vtrackcandidate.end(), ItrTrackCompareLength);
   
    // 2. remove used hits
    for (vector<Track>::iterator trackIt = vtrackcandidate.begin();
	 trackIt != vtrackcandidate.end(); trackIt++){ //trackIt
      
      //if ((!trackIt->good)&&(trackIt->pf>1.0)) continue;
      //if (!trackIt->good) continue;

      // check the track

      //int length = distance(trackIt->pspointmap.begin(), trackIt->pspointmap.end());

      for (map<ArtePointer<GESL>, pSPacepoint >::iterator spIt = trackIt->pspointmap.begin();
	   spIt != trackIt->pspointmap.end(); spIt++){
	
	if ((spIt->second)->used){ //check space points
	  trackIt->good = 0;
	  break;
	}
	
	int zlay = 0;// check zero layers
	int slay = 0;// check stereo layers
	for (vector<pHit>::iterator pcluIt = (spIt->second)->vhit.begin();
	     pcluIt != (spIt->second)->vhit.end(); ++pcluIt){
	  if (!(*pcluIt)->used){
	    float alpha = 180.0/3.1415926*atan2((*pcluIt)->sina,(*pcluIt)->cosa); 
	    if (alpha<-10.0) alpha += 360.0;
	    if ((alpha > -1.0)&&(alpha < 1.0)) zlay++;
	    else                               slay++;
	  }
	}
	if ((zlay <= 1)||(slay <= 1)){
	  (spIt->second)->used = 1;// used space point
	  trackIt->good = 0;
	  break;
	}
      }

      //if ((!trackIt->good)&&(trackIt->pf>1.0/*0.2*/)) continue; // keep fast tracks, even bad
      //if (!trackIt->good) continue; 
      
      // checking the track finished
      
      Track  track;
      pTrack ptrack = &track;
      
      ptrack->pspointmap.clear();
      for (map<ArtePointer<GESL>, pSPacepoint >::iterator pos = trackIt->pspointmap.begin();
	   pos != trackIt->pspointmap.end(); pos++){
	ptrack->pspointmap.insert(pair<ArtePointer<GESL>, pSPacepoint >
				  (pos->first, pos->second ));
      }
      ptrack->good = 1;//trackIt->good;

      SPacepoint sp;
      pSPacepoint psp, pspf, pspe;
      psp  = &sp;
      pspf = (trackIt->pspointmap.begin())->second;
      pspe = (trackIt->pspointmap.begin())->second;

      psp->vhit.clear();
      for (map<ArtePointer<GESL>, pSPacepoint >::iterator spIt = trackIt->pspointmap.begin();
	   spIt != trackIt->pspointmap.end(); spIt++){
	
	if ((spIt->second)->zf < pspf->zf) pspf = spIt->second;
	if ((spIt->second)->ze > pspe->ze) pspe = spIt->second;

	(spIt->second)->used = 1; // flag clusters as used

	for (vector<pHit>::iterator pcluIt = (spIt->second)->vhit.begin();
	     pcluIt != (spIt->second)->vhit.end(); ++pcluIt){

	  (*pcluIt)->used = 1;

	  psp->vhit.push_back(*pcluIt);
	}
      }
      
      psp->xf  = pspf->xf;   psp->txf = pspf->txf;
      psp->yf  = pspf->yf;   psp->tyf = pspf->tyf; 
      psp->zf  = pspf->zf;
      psp->xe  = pspe->xe;   psp->txe = pspe->txe;
      psp->ye  = pspe->ye;   psp->tye = pspe->tye;
      psp->ze  = pspe->ze;
      psp->used = 0; psp->level = 0; psp->neighborindex = -1;
      
      sort(psp->vhit.begin(), psp->vhit.end(), ItrHitCompareZ);
      
      ItrFitSPoint(*psp,0);

      ptrack->xf   = psp->xf;        ptrack->yf  = psp->yf;
      ptrack->txf  = psp->txf;       ptrack->tyf = psp->tyf;
      ptrack->zf   = psp->zf;        ptrack->ze  = psp->ze;
      ptrack->xe   = psp->xe;        ptrack->ye  = psp->ye;
      ptrack->txe  = psp->txe;       ptrack->tye = psp->tye;
      ptrack->chi2 = psp->chi2;      ptrack->prob = psp->prob;
      ptrack->nsp  = trackIt->nsp;   ptrack->nclu  = trackIt->nclu;

      for (int i=0; i<4; i++) 
	for (int j=0; j<=i; j++){
	  ptrack->cvf[j][i] = psp->cvf[j][i];
	  ptrack->cve[j][i] = psp->cve[j][i];
	}
      float pinv=1.0/(Bmagnet*zmagnet*
		      sqrt(1.0+ptrack->txf*ptrack->txf+ptrack->tyf*ptrack->tyf)/
		      ((ptrack->xf-ptrack->zf*ptrack->txf)*
		       sqrt(1.0+ptrack->txf*ptrack->txf)));
      int sign1;
      if(pinv<0.0) sign1=-1;else sign1=1;
      pinv = fabs(pinv);
      if (pinv > 0.5) pinv = 2.0*pinv*pinv;
      ptrack->pf=ptrack->pe=pinv*sign1;

      for (vector<pHit>::iterator pcluIt = psp->vhit.begin();
	   pcluIt != psp->vhit.end(); ++pcluIt){
	ptrack->vhit.push_back(*pcluIt);
	ptrack->selecthits.push_back((*pcluIt)->hitb);
      }
      
      vectracktmp.push_back(*ptrack); // store track
    }
  }

  //Itr_fill_rseg(&vectracktmp,0);

  zeit_itr_collect.stop();


  // ==========================================hits for hit gathering --

  int nslAll=0;
  for(int i=0;i<3;i++){ 
    for(int j=0;j<8;j++){
      
      for (vector<Hit>::iterator hitIt = hits_bank_z[i][j].begin();
	   hitIt != hits_bank_z[i][j].end(); ++hitIt){

	switch (((*hitIt).gede->gesl->nsl+MS10shift)){
	case 5: {nslAll=0; break;}
	case 6: {nslAll=1; break;}
	case 7: {nslAll=2; break;}
	case 8: {nslAll=3; break;}
	}

	int index_Lay=0;
	if(nslAll==0) index_Lay=(*hitIt).gede->lay - 1;
	if(nslAll==1) index_Lay=(*hitIt).gede->lay + 7 ;
	if(nslAll==2) index_Lay=(*hitIt).gede->lay + 11 ;
	if(nslAll==3) index_Lay=(*hitIt).gede->lay + 15 ; 

	(*hitIt).used = 0;
	hits_bank_lay[index_Lay].push_back(hitIt);
      }
      for (vector<Hit>::iterator hitIt = hits_bank_s[i][j].begin();
	   hitIt != hits_bank_s[i][j].end(); ++hitIt){

	switch (((*hitIt).gede->gesl->nsl+MS10shift)){
	case 5: {nslAll=0; break;}
	case 6: {nslAll=1; break;}
	case 7: {nslAll=2; break;}
	case 8: {nslAll=3; break;}
	}

	int index_Lay=0;
	if(nslAll==0) index_Lay=(*hitIt).gede->lay - 1;
	if(nslAll==1) index_Lay=(*hitIt).gede->lay + 7 ;
	if(nslAll==2) index_Lay=(*hitIt).gede->lay + 11 ;
	if(nslAll==3) index_Lay=(*hitIt).gede->lay + 15 ; 

	(*hitIt).used = 0;
	hits_bank_lay[index_Lay].push_back(hitIt);
      }
    }
  }

  for(int j=0;j<24;j++)
    sort(hits_bank_lay[j].begin(), hits_bank_lay[j].end(),ItrHitCompareX);
 

  //========================================================
  zeit_itr_gather.start();

  vectrack.clear();
  //#ifdef YYY

  for (vector<Track>::iterator trackIt = vectracktmp.begin();
       trackIt != vectracktmp.end(); ++trackIt){ // trackIt
    
    int nclu_total = 0;
    int nclu_free  = 0;
    for (vector<pHit>::iterator hitIt = trackIt->vhit.begin();
	 hitIt != trackIt->vhit.end(); ++hitIt){
      nclu_total++;
      if (!(*hitIt)->used) nclu_free++;
    } 
    
    SPacepoint sp, sp_add;
    pSPacepoint psp;
    psp  = &sp;
    
    sp.xf  = trackIt->xf;   sp.txf = trackIt->txf;
    sp.yf  = trackIt->yf;   sp.tyf = trackIt->tyf; 
    sp.zf  = trackIt->zf;
    sp.xe  = trackIt->xe;   sp.txe = trackIt->txe;
    sp.ye  = trackIt->ye;   sp.tye = trackIt->tye;
    sp.ze  = trackIt->ze;
    sp.used = 0; sp.level = 0; sp.neighborindex = -1;
    sp.nsp = trackIt->nsp;  sp.nclu = trackIt->nclu;
    sp.pf = sp.pe = trackIt->pf;
    
    sp.vhit.clear();
#ifdef XXX // move few lines down
    for (vector<pHit >::iterator hitIt = trackIt->vhit.begin();
	 hitIt != trackIt->vhit.end(); ++hitIt){
      sp.vhit.push_back(*hitIt);
    }
#endif //XXX    

    for (int i=0; i<4; i++) 
      for (int j=0; j<=i; j++){
	sp.cvf[j][i] = trackIt->cvf[j][i];
	sp.cve[j][i] = trackIt->cve[j][i];
      }
    

    // unnecessary !!!!!! 
    //    ItrFitSPoint(sp,0); 

    //------------ try to find hits in other sectors

    int fulllay[24],PointoLay=0;
    int  sectmp=0;

    int FirstLay = -1;
    int LastLay  = 0;
    for( int i=0;i <24;i++) fulllay[i]=0; 
    for (vector<pHit>::iterator hitIt = trackIt->vhit.begin();
	 hitIt != trackIt->vhit.end(); ++hitIt){

      sp.vhit.push_back(*hitIt);//to avoid double looping over hits ^^^^^ see at top

      if((*hitIt)->gede->gesl->nsl==(5-MS10shift)) PointoLay=0;
      if((*hitIt)->gede->gesl->nsl==(6-MS10shift)) PointoLay=8;
      if((*hitIt)->gede->gesl->nsl==(7-MS10shift)) PointoLay=12;
      if((*hitIt)->gede->gesl->nsl==(8-MS10shift)) PointoLay=16;
      
      fulllay[(PointoLay+(*hitIt)->gede->lay)-1]++;
      sectmp=(*hitIt)->gede->sec;

      if (FirstLay < 0){
	if((*hitIt)->gede->gesl->nsl==(5-MS10shift)) FirstLay=0;
	if((*hitIt)->gede->gesl->nsl==(6-MS10shift)) FirstLay=8;
	if((*hitIt)->gede->gesl->nsl==(7-MS10shift)) FirstLay=8;
	if((*hitIt)->gede->gesl->nsl==(8-MS10shift)) FirstLay=16;
      }
      if((*hitIt)->gede->gesl->nsl==(5-MS10shift)) LastLay=7;
      if((*hitIt)->gede->gesl->nsl==(6-MS10shift)) LastLay=15;
      if((*hitIt)->gede->gesl->nsl==(7-MS10shift)) LastLay=15;
      if((*hitIt)->gede->gesl->nsl==(8-MS10shift)) LastLay=23;

    }
    
    vector<pHit> vhit; vhit.clear();

    for (int i=FirstLay;i <=LastLay;i++){
      if (fulllay[i]==0){
	
	int nhits = distance(hits_bank_lay[i].begin(),hits_bank_lay[i].end());
	if (nhits == 0)	continue;

	float z_sgede=0.0;
        if(sectmp==1 || sectmp==2)
          z_sgede=0.5*(bank_gede[0][i]->zmin+bank_gede[0][i]->zmax);
        if(sectmp==3 || sectmp==4)
          z_sgede=0.5*(bank_gede[1][i]->zmin+bank_gede[1][i]->zmax);
        if(sectmp==5 || sectmp==6)
          z_sgede=0.5*(bank_gede[2][i]->zmin+bank_gede[2][i]->zmax);
        if(sectmp==7 || sectmp==8)
          z_sgede=0.5*(bank_gede[3][i]->zmin+bank_gede[3][i]->zmax);


	float x, y, errx2, erry2, errxy2;
	if (fabs(z_sgede-sp.zf) < fabs(z_sgede-sp.ze)){
	  x = sp.xf+sp.txf*(z_sgede-sp.zf);
	  y = sp.yf+sp.tyf*(z_sgede-sp.zf);
	  
	  errx2  = sp.cvf[0][0]+(z_sgede-sp.zf)*(z_sgede-sp.zf)*sp.cvf[2][2];
	  erry2  = sp.cvf[1][1]+(z_sgede-sp.zf)*(z_sgede-sp.zf)*sp.cvf[3][3];
	  errxy2 = sp.cvf[0][1]+(z_sgede-sp.zf)*(z_sgede-sp.zf)*sp.cvf[2][3];
	}
	else{
	  x = sp.xe+sp.txe*(z_sgede-sp.ze);
	  y = sp.ye+sp.tye*(z_sgede-sp.ze);
	  
	  errx2  = sp.cve[0][0]+(z_sgede-sp.ze)*(z_sgede-sp.ze)*sp.cve[2][2];
	  erry2  = sp.cve[1][1]+(z_sgede-sp.ze)*(z_sgede-sp.ze)*sp.cve[3][3];
	  errxy2 = sp.cve[0][1]+(z_sgede-sp.ze)*(z_sgede-sp.ze)*sp.cve[2][3];
	}

	//float errx = sqrt(errx2); //errx *= 3.5;
	float erry = sqrt(erry2); //erry *= 3.5;
	
	vector<pHit>::iterator pcluIt_f =  
	  hits_bank_lay[i].begin();
	
	float cosa = (*pcluIt_f)->cosa; float sina = (*pcluIt_f)->sina;
	float u = x*cosa - y*sina;
	
	float erru = sqrt(errx2*cosa*cosa - 2.0*errxy2*cosa*sina + 
			  erry2*sina*sina);
	
	erru += 0.15;//0.15; // plus error
	
	int kl = 0, kr = nhits-1, k0 = 0,krest=0;
	while((kr-kl) > 1){
	  k0=(kl+kr)>>1;
	  if(u <= (*(pcluIt_f+k0))->x) kr = k0;
	  else                         kl = k0;
	}
	if (((*(pcluIt_f+kr))->x-u) < (u-(*(pcluIt_f+kl))->x)) {k0 = kr; krest=kl;}
	else                                                   {k0 = kl; krest=kr;}
	
	if (fabs(u-(*(pcluIt_f+k0))->x) < erru) { // hit found
	  vector<pHit>::iterator pcluIt_0 = pcluIt_f+k0; 
	  
	  int hit_ok = 0;
	  
	  if(!(*pcluIt_0)->used || (*pcluIt_0)->used )
	    {
	      hit_ok = 1;
	      
	      float  x_convert,y_convert;
	      x_convert =  x*(*pcluIt_0)->cosa - y *(*pcluIt_0)->sina;
	      y_convert =  x*(*pcluIt_0)->sina + y*(*pcluIt_0)->cosa;
	      
	      if (
		  //(x_convert-errx > (*pcluIt_0)->gede->sxmx) ||(x_convert+errx < (*pcluIt_0)->gede->sxmn) || 
		  (y_convert-erry > (*pcluIt_0)->gede->symx) || 
		  (y_convert+erry < (*pcluIt_0)->gede->symn))
		hit_ok = 0;
	    }
	  
	  if (hit_ok){
	    vhit.push_back(*(pcluIt_0));

	    if((*(pcluIt_f+krest))->gede->sec!=(*(pcluIt_f+k0))->gede->sec){
	      
	      if (fabs(u-(*(pcluIt_f+krest))->x) < erru) { // hit found
		vector<pHit>::iterator pcluIt_0 = pcluIt_f+krest; 
		int hit_ok = 0;
		if ( !(*pcluIt_0)->used || (*pcluIt_0)->used )
		  {
		    hit_ok = 1;
		    //inside gede?
		    float  x_convert,y_convert;
		    x_convert =  x*(*pcluIt_0)->cosa - y*(*pcluIt_0)->sina;
		    y_convert =  x*(*pcluIt_0)->sina + y*(*pcluIt_0)->cosa;
		    
		    if (
			//(x_convert-errx > (*pcluIt_0)->gede->sxmx) ||(x_convert+errx < (*pcluIt_0)->gede->sxmn) || 
			(y_convert-erry > (*pcluIt_0)->gede->symx) || 
			(y_convert+erry < (*pcluIt_0)->gede->symn))
		      hit_ok = 0;
		  }
		if (hit_ok)
		  vhit.push_back(*(pcluIt_0));
	      }
	    }
	  }
	}
      }// end loop over hits
    }
    
    if (distance(vhit.begin(), vhit.end()) > 1){ // try to improve space point

      sp_add.vhit.clear();
      for (vector<pHit >::iterator pcluIt = sp.vhit.begin();
	   pcluIt != sp.vhit.end(); ++pcluIt){
	sp_add.vhit.push_back(*pcluIt);
      }
      for (vector<pHit>::iterator pcluIt = vhit.begin();
	   pcluIt != vhit.end(); ++pcluIt){
	sp_add.vhit.push_back(*pcluIt);
      }      
      sort(sp_add.vhit.begin(), sp_add.vhit.end(), ItrHitCompareZ);
      
      float zf = 10000.0;
      float ze = 0.0;
      
      for (vector<pHit>::iterator pcluIt = sp_add.vhit.begin();
	   pcluIt != sp_add.vhit.end(); pcluIt++){
	if ((*pcluIt)->z < zf) zf = (*pcluIt)->z;
	if ((*pcluIt)->z > ze) ze = (*pcluIt)->z;
      }
      
      sp_add.xf  = sp.xf+sp.txf*(zf-sp.zf);   sp_add.txf = sp.txf;
      sp_add.yf  = sp.yf+sp.tyf*(zf-sp.zf);   sp_add.tyf = sp.tyf;
      sp_add.zf  = zf;
      sp_add.xe  = sp.xe+sp.txe*(ze-sp.ze);   sp_add.txe = sp.txe;
      sp_add.ye  = sp.ye+sp.tye*(ze-sp.ze);   sp_add.tye = sp.tye;
      sp_add.ze  = ze;
      sp_add.pf  = sp_add.pe = sp.pf; //!!!!!!!!!!!!!

      sp_add.used = 0; sp_add.level = 0; sp_add.neighborindex = -1;
      
      ItrFitSPoint(sp_add,0);
      if (sp_add.prob > 0.05){
	psp = &sp_add; // ok, 
	//cout<<"Track improved from "<<sp.nclu<<" to "<<sp_add.nclu<<" hits"<<endl;
      }
    }


    //--------------- other sectors

    
    int geslusedmap[3],index_sl=0;
    for (int i=0; i<3; i++)  geslusedmap[i]=0;

    int secfirst,seclast;
    secfirst=seclast=0;

    for (vector<pHit>::iterator hitIt = psp->vhit.begin();
	 hitIt != psp->vhit.end(); hitIt++){

      if(secfirst==0) secfirst=(*hitIt)->gede->sec;
      seclast=(*hitIt)->gede->sec;

      int num_sl = ((*hitIt)->gede->gesl->nsl+MS10shift);
      if(num_sl==5) index_sl=0;
      if(num_sl==6 || num_sl==7) index_sl=1;
      if(num_sl==8) index_sl=2;
      geslusedmap[index_sl]++;
    }
    int start = -1, end = -1;
    
    for (int i=0; i<3; i++){
      if(geslusedmap[i] == 0) continue;
      if(start == -1) start = i;
      end = i;
    }
    for (int i = start-1; i >= 0; i--){//start
      
      vector<pHit> vhit;
      vhit.clear();
      int start_lay=0;     
     
      if(i==0) start_lay=0;       
      if(i==1) start_lay=8;  
      if(i==2) start_lay=16;
      
      for(int j=start_lay; j<(start_lay+8);j++){
	int nhits=distance(hits_bank_lay[j].begin(),hits_bank_lay[j].end());
	if (nhits==0) continue;

	float z_sgede=10.0;
	if(secfirst==1 || secfirst==2) 
          z_sgede=0.5*(bank_gede[0][j]->zmin+bank_gede[0][j]->zmax);
        if(secfirst==3 || secfirst==4)
	    z_sgede=0.5*(bank_gede[1][j]->zmin+bank_gede[1][j]->zmax);
        if(secfirst==5 || secfirst==6) 
          z_sgede=0.5*(bank_gede[2][j]->zmin+bank_gede[2][j]->zmax);
        if(secfirst==7 || secfirst==8) 
          z_sgede=0.5*(bank_gede[3][j]->zmin+bank_gede[3][j]->zmax);

	float Dz2=(z_sgede-psp->zf)*(z_sgede-psp->zf);
	float errx2  = psp->cvf[0][0]+Dz2*psp->cvf[2][2];
	float erry2  = psp->cvf[1][1]+Dz2*psp->cvf[3][3];
	float errxy2 = psp->cvf[0][1]+Dz2*psp->cvf[2][3];
	float erry = 10.0*sqrt(erry2); 
	float cosa,sina,x,y,u;
	cosa= (*hits_bank_lay[j].begin())->cosa;
	sina= (*hits_bank_lay[j].begin())->sina;
	float erru = 3.0*sqrt(errx2*cosa*cosa - 2.0*errxy2*cosa*sina + 
			  erry2*sina*sina);
	erru+=0.03;
	x = psp->xf+psp->txf*(z_sgede-psp->zf);
	y = psp->yf+psp->tyf*(z_sgede-psp->zf);
	u = x*cosa - y*sina;
	float y0,ym,yp;
	y0=x*sina + y*cosa;
	ym=y0-erry;yp=y0+erry;
	vector<pHit>::iterator pcluIt_f = hits_bank_lay[j].begin();
	
	int kl = 0, kr = nhits-1, k0 = 0;
	while((kr-kl) > 1){
	  k0=(kl+kr)>>1;
	  if(u <= (*(pcluIt_f+k0))->x) kr = k0;
	  else                         kl = k0;
	}
	if (((*(pcluIt_f+kr))->x-u) < (u-(*(pcluIt_f+kl))->x)) k0 = kr;
	else                                                   k0 = kl;
	
	vector<pHit>::iterator pcluIt_0 = pcluIt_f+k0; 
	if (fabs(u-(*(pcluIt_0))->x) < erru) { // hit found
	  for (vector<pHit>::iterator pcluIt = pcluIt_0;
	       pcluIt != hits_bank_lay[j].end(); pcluIt++){
	    if (fabs((*pcluIt)->x - u)> erru) 
	      break;
	      
	    if((ym>(*pcluIt)->gede->symx)||(yp<(*pcluIt)->gede->symn))
	      continue;
	      
	    vhit.push_back(*pcluIt);
	  }
	  vector<pHit>::reverse_iterator rpcluIt_f = hits_bank_lay[j].rbegin();
	  vector<pHit>::reverse_iterator rpcluIt_0 = rpcluIt_f+(nhits-k0-1); 
	  for (vector<pHit>::reverse_iterator pcluIt =  (rpcluIt_0+1);
	       pcluIt != hits_bank_lay[j].rend(); pcluIt++){
	    if (fabs((*pcluIt)->x - u) > erru) 
	      break;
	      
	    if((ym>(*pcluIt)->gede->symx)||(yp<(*pcluIt)->gede->symn))
	      continue;
	      
	    vhit.push_back(*pcluIt);
	  }
	    
	} //erru 
      } // loop over lay 

      if (distance(vhit.begin(), vhit.end())<=2) break;      
      float zf = 10000.0;
      float ze = 0.0;
      for (vector<pHit>::iterator pcluIt = vhit.begin();
	   pcluIt != vhit.end(); pcluIt++){
	if ((*pcluIt)->z < zf) zf = (*pcluIt)->z;
	if ((*pcluIt)->z > ze) ze = (*pcluIt)->z;
      }
      
      //refit the track
      
      SPacepoint sptmp;
      
      sptmp.xf  = psp->xf+psp->txf*(zf-psp->zf);   sptmp.txf = psp->txf;
      sptmp.yf  = psp->yf+psp->tyf*(zf-psp->zf);   sptmp.tyf = psp->tyf; 
      sptmp.zf  = zf;
      sptmp.xe  = psp->xe;   sptmp.txe = psp->txe;
      sptmp.ye  = psp->ye;   sptmp.tye = psp->tye;
      sptmp.ze  = psp->ze;
      sptmp.used = 0; sptmp.level = 0; sptmp.neighborindex = -1;
      
      sptmp.vhit.clear();
      for (vector<pHit >::iterator pcluIt = psp->vhit.begin();
	   pcluIt != psp->vhit.end(); ++pcluIt){
	sptmp.vhit.push_back(*pcluIt);
      }
      for (vector<pHit>::iterator pcluIt = vhit.begin();
	   pcluIt != vhit.end(); ++pcluIt){
	sptmp.vhit.push_back(*pcluIt);
      }      
      sort(sptmp.vhit.begin(), sptmp.vhit.end(), ItrHitCompareZ);
      ItrFitMS(sptmp);
      if (sptmp.prob < 0.05) 
	break;
      
      // track is good -> remove clusters and store
      
      psp->xf  = sptmp.xf;   psp->txf = sptmp.txf;
      psp->yf  = sptmp.yf;   psp->tyf = sptmp.tyf; 
      psp->zf  = sptmp.zf;
      psp->xe  = sptmp.xe;   psp->txe = sptmp.txe;
      psp->ye  = sptmp.ye;   psp->tye = sptmp.tye;
      psp->ze  = sptmp.ze;
      psp->used = 0; psp->level = 0; psp->neighborindex = -1; 
      psp->chi2 = sptmp.chi2; psp->prob = sptmp.prob;
      
      psp->vhit.clear();
      for (vector<pHit >::iterator pcluIt = sptmp.vhit.begin();
	   pcluIt != sptmp.vhit.end(); ++pcluIt){
	//	(*pcluIt)->used = 1;
	psp->vhit.push_back(*pcluIt);
      }
      
      psp->nsp++; psp->nclu = distance(psp->vhit.begin(), psp->vhit.end());
      
    }//start

    for (int i = end+1; i < 3; i++){//end
      
      vector<pHit> vhit;
      vhit.clear();
      int start_lay=0;     
      if(i==0) start_lay=0;       
      if(i==1) start_lay=8;  
      if(i==2) start_lay=16;
      
      for(int j=start_lay; j<(start_lay+8);j++){
	int nhits = distance(hits_bank_lay[j].begin(),hits_bank_lay[j].end());
	if (nhits == 0) continue;

	float z_sgede=0;

	if(secfirst==1 || secfirst==2) 
          z_sgede=0.5*(bank_gede[0][j]->zmin+bank_gede[0][j]->zmax);
        if(secfirst==3 || secfirst==4) 
          z_sgede=0.5*(bank_gede[1][j]->zmin+bank_gede[1][j]->zmax);
        if(secfirst==5 || secfirst==6) 
          z_sgede=0.5*(bank_gede[2][j]->zmin+bank_gede[2][j]->zmax);
        if(secfirst==7 || secfirst==8) 
          z_sgede=0.5*(bank_gede[3][j]->zmin+bank_gede[3][j]->zmax);

	float Dz2=(z_sgede-psp->ze)*(z_sgede-psp->ze);
	float errx2  = psp->cve[0][0]+Dz2*psp->cve[2][2];
	float erry2  = psp->cve[1][1]+Dz2*psp->cve[3][3];
	float errxy2 = psp->cve[0][1]+Dz2*psp->cve[2][3];
	float erry = 10.0*sqrt(erry2); 
	float cosa,sina,x,y,u;
	cosa= (*hits_bank_lay[j].begin())->cosa;
	sina= (*hits_bank_lay[j].begin())->sina;
	float erru = 3.0*sqrt(errx2*cosa*cosa - 2.0*errxy2*cosa*sina + 
			  erry2*sina*sina);
	erru+=0.03;
	float y0,ym,yp;
	
	x = psp->xf+psp->txf*(z_sgede-psp->zf);
	y = psp->yf+psp->tyf*(z_sgede-psp->zf);
	u = x*cosa - y*sina;
	y0=x*sina + y*cosa;
	ym=y0-erry;yp=y0+erry;
	vector<pHit>::iterator pcluIt_f = hits_bank_lay[j].begin();
	
	int kl = 0, kr = nhits-1, k0 = 0;
	while((kr-kl) > 1){
	  k0=(kl+kr)>>1;
	  if(u <= (*(pcluIt_f+k0))->x) kr = k0;
	  else                         kl = k0;
	}
	if (((*(pcluIt_f+kr))->x-u) < (u-(*(pcluIt_f+kl))->x)) k0 = kr;
	else                                                   k0 = kl;
	
	vector<pHit>::iterator pcluIt_0 = pcluIt_f+k0; 
	if (fabs(u-(*(pcluIt_0))->x) < erru) { // hit found
	  for (vector<pHit>::iterator pcluIt = pcluIt_0;
	       pcluIt != hits_bank_lay[j].end(); pcluIt++){
	    if (fabs((*pcluIt)->x - u)> erru) 
	      break;
	      
	    if((ym>(*pcluIt)->gede->symx)||(yp<(*pcluIt)->gede->symn))
	      continue;
	      
	    vhit.push_back(*pcluIt);
	  }
	    
	  vector<pHit>::reverse_iterator rpcluIt_f = hits_bank_lay[j].rbegin();
	  vector<pHit>::reverse_iterator rpcluIt_0 = rpcluIt_f+(nhits-k0-1); 
	  for (vector<pHit>::reverse_iterator pcluIt =  (rpcluIt_0+1);
	       pcluIt != hits_bank_lay[j].rend(); pcluIt++){
	    if (fabs((*pcluIt)->x - u) > erru) 
	      break;
	      
	    if((ym>(*pcluIt)->gede->symx)||(yp<(*pcluIt)->gede->symn))
	      continue;
	      
	    vhit.push_back(*pcluIt);
	  }
	    
	} //erru 
      } // loop over lay 

      if (distance(vhit.begin(), vhit.end())<=2) break;
      
      float zf = 10000.0;
      float ze = 0.0;
      for (vector<pHit>::iterator pcluIt = vhit.begin();
	   pcluIt != vhit.end(); pcluIt++){
	if ((*pcluIt)->z < zf) zf = (*pcluIt)->z;
	if ((*pcluIt)->z > ze) ze = (*pcluIt)->z;
      }
      
      //refit the track
      
      SPacepoint sptmp;

      sptmp.xf  = psp->xf;   sptmp.txf = psp->txf;
      sptmp.yf  = psp->yf;   sptmp.tyf = psp->tyf; 
      sptmp.zf  = psp->zf;
      sptmp.xe  = psp->xe+psp->txe*(ze-psp->ze);   sptmp.txe = psp->txe;
      sptmp.ye  = psp->ye+psp->tye*(ze-psp->ze);   sptmp.tye = psp->tye;
      sptmp.ze  = ze;
      sptmp.used = 0; sptmp.level = 0; sptmp.neighborindex = -1;
      
      sptmp.vhit.clear();
      for (vector<pHit >::iterator pcluIt = psp->vhit.begin();
	   pcluIt != psp->vhit.end(); ++pcluIt){
	sptmp.vhit.push_back(*pcluIt);
      }
      for (vector<pHit>::iterator pcluIt = vhit.begin();
	   pcluIt != vhit.end(); ++pcluIt){
	sptmp.vhit.push_back(*pcluIt);
      }      
      sort(sptmp.vhit.begin(), sptmp.vhit.end(), ItrHitCompareZ);
      ItrFitMS(sptmp);
      if (sptmp.prob < 0.05) 
	break;
      // track is good -> remove clusters and store
      
      psp->xf  = sptmp.xf;   psp->txf = sptmp.txf;
      psp->yf  = sptmp.yf;   psp->tyf = sptmp.tyf; 
      psp->zf  = sptmp.zf;
      psp->xe  = sptmp.xe;   psp->txe = sptmp.txe;
      psp->ye  = sptmp.ye;   psp->tye = sptmp.tye;
      psp->ze  = sptmp.ze;
      psp->used = 0; psp->level = 0; psp->neighborindex = -1; 
      psp->chi2 = sptmp.chi2; psp->prob = sptmp.prob;

      psp->vhit.clear();
      for (vector<pHit >::iterator pcluIt = sptmp.vhit.begin();
	   pcluIt != sptmp.vhit.end(); ++pcluIt){
	//	(*pcluIt)->used = 1;
	psp->vhit.push_back(*pcluIt);
      }
      
      psp->nsp++; psp->nclu = distance(psp->vhit.begin(), psp->vhit.end());
      
    }//end
    //!!!!!!!!!!!!!!!!!!!!
    if ((psp->nclu<=6)||(psp->pf>1.0)){ //cut 3x3 at the end anyhow... and 1GeV in SP
      continue;
    }

    ItrFitSPoint(*psp,0);

    //------------ try to find hits in other sectors

    SPacepoint sp_fin;

    FirstLay = -1;
    LastLay  = 0;
    for( int i=0;i <24;i++) fulllay[i]=0; 
    for (vector<pHit>::iterator hitIt = psp->vhit.begin();
	 hitIt != psp->vhit.end(); ++hitIt){

      if((*hitIt)->gede->gesl->nsl==(5-MS10shift)) PointoLay=0;
      if((*hitIt)->gede->gesl->nsl==(6-MS10shift)) PointoLay=8;
      if((*hitIt)->gede->gesl->nsl==(7-MS10shift)) PointoLay=12;
      if((*hitIt)->gede->gesl->nsl==(8-MS10shift)) PointoLay=16;
      
      fulllay[(PointoLay+(*hitIt)->gede->lay)-1]++;
      sectmp=(*hitIt)->gede->sec;

      if (FirstLay < 0){
	if((*hitIt)->gede->gesl->nsl==(5-MS10shift)) FirstLay=0;
	if((*hitIt)->gede->gesl->nsl==(6-MS10shift)) FirstLay=8;
	if((*hitIt)->gede->gesl->nsl==(7-MS10shift)) FirstLay=8;
	if((*hitIt)->gede->gesl->nsl==(8-MS10shift)) FirstLay=16;
      }
      if((*hitIt)->gede->gesl->nsl==(5-MS10shift)) LastLay=7;
      if((*hitIt)->gede->gesl->nsl==(6-MS10shift)) LastLay=15;
      if((*hitIt)->gede->gesl->nsl==(7-MS10shift)) LastLay=15;
      if((*hitIt)->gede->gesl->nsl==(8-MS10shift)) LastLay=23;

    }
    
    vhit.clear();

    for (int i=FirstLay;i <= LastLay;i++){
      if (fulllay[i]==0){
	
	int nhits = distance(hits_bank_lay[i].begin(),hits_bank_lay[i].end());
	if (nhits == 0)	continue;

	float z_sgede=0.0;
        if(sectmp==1 || sectmp==2)
          z_sgede=0.5*(bank_gede[0][i]->zmin+bank_gede[0][i]->zmax);
        if(sectmp==3 || sectmp==4)
          z_sgede=0.5*(bank_gede[1][i]->zmin+bank_gede[1][i]->zmax);
        if(sectmp==5 || sectmp==6)
          z_sgede=0.5*(bank_gede[2][i]->zmin+bank_gede[2][i]->zmax);
        if(sectmp==7 || sectmp==8)
          z_sgede=0.5*(bank_gede[3][i]->zmin+bank_gede[3][i]->zmax);


	float x, y, errx2, erry2, errxy2;
	if (fabs(z_sgede-psp->zf) < fabs(z_sgede-psp->ze)){
	  x = psp->xf+psp->txf*(z_sgede-psp->zf);
	  y = psp->yf+psp->tyf*(z_sgede-psp->zf);
	  
	  errx2  = psp->cvf[0][0]+(z_sgede-psp->zf)*(z_sgede-psp->zf)*psp->cvf[2][2];
	  erry2  = psp->cvf[1][1]+(z_sgede-psp->zf)*(z_sgede-psp->zf)*psp->cvf[3][3];
	  errxy2 = psp->cvf[0][1]+(z_sgede-psp->zf)*(z_sgede-psp->zf)*psp->cvf[2][3];
	}
	else{
	  x = psp->xe+psp->txe*(z_sgede-psp->ze);
	  y = psp->ye+psp->tye*(z_sgede-psp->ze);
	  
	  errx2  = psp->cve[0][0]+(z_sgede-psp->ze)*(z_sgede-psp->ze)*psp->cve[2][2];
	  erry2  = psp->cve[1][1]+(z_sgede-psp->ze)*(z_sgede-psp->ze)*psp->cve[3][3];
	  errxy2 = psp->cve[0][1]+(z_sgede-psp->ze)*(z_sgede-psp->ze)*psp->cve[2][3];
	}

	//float errx = sqrt(errx2); //errx *= 3.5;
	float erry = sqrt(erry2); //erry *= 3.5;
	
	vector<pHit>::iterator pcluIt_f =  
	  hits_bank_lay[i].begin();
	
	float cosa = (*pcluIt_f)->cosa; float sina = (*pcluIt_f)->sina;
	float u = x*cosa - y*sina;
	
	float erru = sqrt(errx2*cosa*cosa - 2.0*errxy2*cosa*sina + 
			  erry2*sina*sina);
	
	erru += 0.15;//0.15; // plus error
	
	int kl = 0, kr = nhits-1, k0 = 0,krest=0;
	while((kr-kl) > 1){
	  k0=(kl+kr)>>1;
	  if(u <= (*(pcluIt_f+k0))->x) kr = k0;
	  else                         kl = k0;
	}
	if (((*(pcluIt_f+kr))->x-u) < (u-(*(pcluIt_f+kl))->x)) {k0 = kr; krest=kl;}
	else                                                   {k0 = kl; krest=kr;}
	
	if (fabs(u-(*(pcluIt_f+k0))->x) < erru) { // hit found
	  vector<pHit>::iterator pcluIt_0 = pcluIt_f+k0; 
	  
	  int hit_ok = 0;
	  
	  if(!(*pcluIt_0)->used || (*pcluIt_0)->used )
	    {
	      hit_ok = 1;
	      
	      float  x_convert,y_convert;
	      x_convert =  x*(*pcluIt_0)->cosa - y *(*pcluIt_0)->sina;
	      y_convert =  x*(*pcluIt_0)->sina + y*(*pcluIt_0)->cosa;
	      
	      if (
		  //(x_convert-errx > (*pcluIt_0)->gede->sxmx) ||(x_convert+errx < (*pcluIt_0)->gede->sxmn) || 
		  (y_convert-erry > (*pcluIt_0)->gede->symx) || 
		  (y_convert+erry < (*pcluIt_0)->gede->symn))
		hit_ok = 0;
	    }
	  
	  if (hit_ok){
	    vhit.push_back(*(pcluIt_0));

	    if((*(pcluIt_f+krest))->gede->sec!=(*(pcluIt_f+k0))->gede->sec){
	      
	      if (fabs(u-(*(pcluIt_f+krest))->x) < erru) { // hit found
		vector<pHit>::iterator pcluIt_0 = pcluIt_f+krest; 
		int hit_ok = 0;
		if ( !(*pcluIt_0)->used || (*pcluIt_0)->used )
		  {
		    hit_ok = 1;
		    //inside gede?
		    float  x_convert,y_convert;
		    x_convert =  x*(*pcluIt_0)->cosa - y*(*pcluIt_0)->sina;
		    y_convert =  x*(*pcluIt_0)->sina + y*(*pcluIt_0)->cosa;
		    
		    if (
			//(x_convert-errx > (*pcluIt_0)->gede->sxmx) ||(x_convert+errx < (*pcluIt_0)->gede->sxmn) || 
			(y_convert-erry > (*pcluIt_0)->gede->symx) || 
			(y_convert+erry < (*pcluIt_0)->gede->symn))
		      hit_ok = 0;
		  }
		if (hit_ok)
		  vhit.push_back(*(pcluIt_0));
	      }
	    }
	  }
	}
      }// end loop over hits
    }
    
    if (distance(vhit.begin(), vhit.end()) > 1){ // try to improve space point

      sp_fin.vhit.clear();
      for (vector<pHit >::iterator pcluIt = psp->vhit.begin();
	   pcluIt != psp->vhit.end(); ++pcluIt){
	sp_fin.vhit.push_back(*pcluIt);
      }
      for (vector<pHit>::iterator pcluIt = vhit.begin();
	   pcluIt != vhit.end(); ++pcluIt){
	sp_fin.vhit.push_back(*pcluIt);
      }      
      sort(sp_fin.vhit.begin(), sp_fin.vhit.end(), ItrHitCompareZ);
      
      float zf = 10000.0;
      float ze = 0.0;
      
      for (vector<pHit>::iterator pcluIt = sp_fin.vhit.begin();
	   pcluIt != sp_fin.vhit.end(); pcluIt++){
	if ((*pcluIt)->z < zf) zf = (*pcluIt)->z;
	if ((*pcluIt)->z > ze) ze = (*pcluIt)->z;
      }
      
      sp_fin.xf  = psp->xf+psp->txf*(zf-psp->zf);   sp_fin.txf = psp->txf;
      sp_fin.yf  = psp->yf+psp->tyf*(zf-psp->zf);   sp_fin.tyf = psp->tyf;
      sp_fin.zf  = zf;
      sp_fin.xe  = psp->xe+psp->txe*(ze-psp->ze);   sp_fin.txe = psp->txe;
      sp_fin.ye  = psp->ye+psp->tye*(ze-psp->ze);   sp_fin.tye = psp->tye;
      sp_fin.ze  = ze;
      sp_fin.pf  = sp_fin.pe = psp->pf; //!!!!!!!!!!!!!

      sp_fin.used = 0; sp_fin.level = 0; sp_fin.neighborindex = -1;
      
      ItrFitSPoint(sp_fin,0);
      if (sp_fin.prob > 0.05){
	//cout<<"TRACK improved from "<<psp->nclu<<" to "<<sp_fin.nclu<<" hits"<<endl;
	psp = &sp_fin; // ok, 
      }
    }


    //-------------------------------

    Track  track;
    pTrack ptrack = &track;
    
    // no sence to keep space points
    ptrack->pspointmap.clear();
    ptrack->good = 1;
    
    ptrack->xf   = psp->xf;        ptrack->yf   = psp->yf;
    ptrack->txf  = psp->txf;       ptrack->tyf  = psp->tyf;
    ptrack->zf   = psp->zf;        ptrack->ze   = psp->ze;
    ptrack->xe   = psp->xe;        ptrack->ye   = psp->ye;
    ptrack->txe  = psp->txe;       ptrack->tye  = psp->tye;
    ptrack->chi2 = psp->chi2;      ptrack->prob = psp->prob;
    ptrack->nsp  = psp->nsp;       ptrack->nclu = psp->nclu;    
    
    for (int i=0; i<4; i++) 
      for (int j=0; j<=i; j++){
 	ptrack->cvf[j][i] = psp->cvf[j][i];
	ptrack->cve[j][i] = psp->cve[j][i];
      }
    
    {
      float pinv=1.0/(Bmagnet*zmagnet*
		      sqrt(1.0+ptrack->txf*ptrack->txf+ptrack->tyf*ptrack->tyf)/
		      ((ptrack->xf-ptrack->zf*ptrack->txf)*
		       sqrt(1.0+ptrack->txf*ptrack->txf)));
      int sign1;
      if(pinv<0.0) sign1=-1;else sign1=1;
      pinv = fabs(pinv);
      if (pinv > 0.5) pinv = 2.0*pinv*pinv;
      ptrack->pf=ptrack->pe=pinv*sign1;
    }

    for (vector<pHit >::iterator hitIt = psp->vhit.begin();
	 hitIt != psp->vhit.end(); ++hitIt){
      ptrack->vhit.push_back(*hitIt);
    }
    sort(ptrack->vhit.begin(), ptrack->vhit.end(),ItrHitCompareZ);

    ItrFastRefitTrack(ptrack);

    if (ptrack->nonmask == 0) continue;
    if ((ptrack->nclu <= 5)&&(ptrack->prob <= 0.5)) continue;

    vectrack.push_back(*ptrack);
    
  } //trackIt

  sort(vectrack.begin(), vectrack.end(), ItrTrackCompareLength);

  zeit_itr_gather.stop();
  //#endif //YYY

  zeit_itr_refit.start();


  //====================search in other sectors========================================

  for(int j=0;j<24;j++){
    for (vector<pHit >::iterator hitIt = hits_bank_lay[j].begin();
	 hitIt != hits_bank_lay[j].end();++hitIt){
      (*hitIt)->used=0;
    }
  }
  vector<Track> vectrackgood;
  vectrackgood.clear();

  int good_tracks = 0;
  for (vector<Track>::iterator trackIt = vectrack.begin();
     trackIt != vectrack.end(); ++trackIt){// trackIt
    // is the track good ?
    int nclu_total = 0;
    int nclu_free  = 0;

    vector<pHit> vphit;
    vector<pHit> vphit_track;
    vphit.clear();
    vphit_track.clear();

    for (vector<pHit>::iterator pcluIt = trackIt->vhit.begin();
	 pcluIt != trackIt->vhit.end(); ++pcluIt){ // over clusters
      nclu_total++;
      if (!(*pcluIt)->used){ 
	vphit_track.push_back(*pcluIt);
	nclu_free++;
	continue;
      }
      int hit_found = 0;
      pHit pclu;

      float z = (*pcluIt)->z;
      float x = trackIt->xf+trackIt->txf*(z-trackIt->zf);
      float y = trackIt->yf+trackIt->tyf*(z-trackIt->zf);

      int start_lay=0;     
      
      if((*pcluIt)->gede->gesl->nsl==(5-MS10shift)) start_lay=-1+(*pcluIt)->gede->lay;       
      if((*pcluIt)->gede->gesl->nsl==(6-MS10shift)) start_lay= 7+(*pcluIt)->gede->lay;  
      if((*pcluIt)->gede->gesl->nsl==(7-MS10shift)) start_lay=11+(*pcluIt)->gede->lay;
      if((*pcluIt)->gede->gesl->nsl==(8-MS10shift)) start_lay=15+(*pcluIt)->gede->lay;

      vector<pHit>::iterator pcluIt_f = hits_bank_lay[start_lay].begin();
      vector<pHit>::reverse_iterator rpcluIt_f = hits_bank_lay[start_lay].rbegin();

      float errx2  = trackIt->cve[0][0]+(z-trackIt->ze)*(z-trackIt->ze)*trackIt->cve[2][2];
      float erry2  = trackIt->cve[1][1]+(z-trackIt->ze)*(z-trackIt->ze)*trackIt->cve[3][3];
      float errxy2 = trackIt->cve[0][1]+(z-trackIt->ze)*(z-trackIt->ze)*trackIt->cve[2][3];
      
      float erry = sqrt(erry2); //erry *= 3.5;
      
      float cosa = (*pcluIt_f)->cosa; float sina = (*pcluIt_f)->sina;
      float u = x*cosa - y*sina;

      float erru = 3.0*sqrt(errx2*cosa*cosa - 2.0*errxy2*cosa*sina + 
			erry2*sina*sina);
      erru += 0.03; // plus error

      int nhits = distance(hits_bank_lay[start_lay].begin(), hits_bank_lay[start_lay].end());

      //==================================

      int kl = 0, kr = nhits-1, k0 = 0;
      while((kr-kl) > 1){
	k0=(kl+kr)>>1;
	if(u <= (*(pcluIt_f+k0))->x) kr = k0;
	else                         kl = k0;
      }
      if (((*(pcluIt_f+kr))->x-u) < (u-(*(pcluIt_f+kl))->x)) k0 = kr;
      else                                                   k0 = kl;
      
      if (fabs(u-(*(pcluIt_f+k0))->x) < erru) { // hit found
	vector<pHit>::iterator pcluIt_0 = pcluIt_f+k0; 
	vector<pHit>::reverse_iterator rpcluIt_0 = rpcluIt_f+(nhits-k0-1); 
	int hit_ok = 0;
	if ( !(*pcluIt_0)->used ){
	  hit_ok = 1;
	  //inside gede?
	  float  x_convert,y_convert;
	  x_convert =  x*(*pcluIt_0)->cosa - y*(*pcluIt_0)->sina;
	  y_convert =  x*(*pcluIt_0)->sina + y*(*pcluIt_0)->cosa;
	  
	  if (
	      (y_convert-10.0*erry > (*pcluIt_0)->gede->symx) || 
	      (y_convert+10.0*erry < (*pcluIt_0)->gede->symn))
	    hit_ok = 0;
	}
	if (hit_ok){
	  vphit.push_back(*pcluIt_0);
	  pclu = *pcluIt_0;
	  hit_found = 1;
	  nclu_free++;
	}
	else{
	  pHit      pclu_best = 0;
	  float    pclu_dist = 1000.0;
	  
	  for (vector<pHit>::iterator pcluIt =  (pcluIt_0+1);
	       pcluIt != hits_bank_lay[start_lay].end(); pcluIt++){
	    if ((*pcluIt)->used) continue;
	    float dist = fabs((*pcluIt)->x - u);
	    if (dist > erru) 
	      break;
	    
	    float x_convert, y_convert;
	    x_convert =  x*(*pcluIt_0)->cosa - y*(*pcluIt_0)->sina;
	    y_convert =  x*(*pcluIt_0)->sina + y*(*pcluIt_0)->cosa;
	    
	    if (
		(y_convert-10.0*erry > (*pcluIt_0)->gede->symx)  ||
		(y_convert+10.0*erry < (*pcluIt_0)->gede->symn))
	      continue;
	    
	    pclu_best = *pcluIt;
	    pclu_dist = fabs((*pcluIt)->x - u);
	    break;
	  }
	  
	  for (vector<pHit>::reverse_iterator pcluIt =  (rpcluIt_0+1);
	       pcluIt != hits_bank_lay[start_lay].rend(); pcluIt++){
	    if ((*pcluIt)->used) continue;
	    float dist = fabs((*pcluIt)->x - u);
	    if (dist > erru) 
	      break;
	    
	    float  x_convert,y_convert;
	    x_convert =  x*(*pcluIt_0)->cosa - y*(*pcluIt_0)->sina;
	    y_convert =  x*(*pcluIt_0)->sina + y*(*pcluIt_0)->cosa;
	    
	    if (
		(y_convert-10.0*erry > (*pcluIt_0)->gede->symx) ||
		(y_convert+10.0*erry < (*pcluIt_0)->gede->symn))
	      continue;
	    
	    if (fabs((*pcluIt)->x - u) < pclu_dist){
	      pclu_best = *pcluIt;
	      pclu_dist = fabs((*pcluIt)->x - u);
	    }
	    break;
	  }
	  
	  if (pclu_dist<1000.0){
	    vphit.push_back(pclu_best);
	    pclu = pclu_best;
	    hit_found = 1;
	    nclu_free++;
	  }
	}
      } //erru 
      

      //====================================

      if (hit_found){
	vphit_track.push_back(pclu);
      }
      else
	vphit_track.push_back(*pcluIt); //which is used ;-[

    }
    
    if ((nclu_free <= 3) || (float(nclu_free)/float(nclu_total) < 0.7)){
      trackIt->good = 0;
      continue; // track is used
    }

    if (distance(vphit.begin(), vphit.end()) == 0){
      
      //no refit needed
     
      vectrackgood.push_back(*trackIt);

      for (vector<pHit>::iterator phitIt = trackIt->vhit.begin();
	   phitIt != trackIt->vhit.end(); ++phitIt){
	(*phitIt)->used = 1;
      }
      good_tracks++;
      continue;
    }
    
    // else create new track
    
    Track track;
    pTrack ptrack = &track;

    // no sense to keep space points
    ptrack->pspointmap.clear();
    ptrack->good = 1;
    
    ptrack->xf   = trackIt->xf;        ptrack->yf  = trackIt->yf;
    ptrack->txf  = trackIt->txf;       ptrack->tyf = trackIt->tyf;
    ptrack->zf   = trackIt->zf;        ptrack->ze  = trackIt->ze;
    ptrack->xe   = trackIt->xe;        ptrack->ye  = trackIt->ye;
    ptrack->txe  = trackIt->txe;       ptrack->tye = trackIt->tye;
    ptrack->chi2 = trackIt->chi2;
    ptrack->nsp  = trackIt->nsp;       ptrack->nclu= trackIt->nclu;    

    for (int i=0; i<4; i++) 
      for (int j=0; j<=i; j++){
	ptrack->cvf[j][i] = trackIt->cvf[j][i];
	ptrack->cve[j][i] = trackIt->cve[j][i];
      }
    
    ptrack->pf = ptrack->pe = trackIt->pf;

    ptrack->vhit.clear();
    for (vector<pHit >::iterator hitIt = vphit_track.begin();
	 hitIt != vphit_track.end(); ++hitIt){
      ptrack->vhit.push_back(*hitIt);
    }
    sort(ptrack->vhit.begin(), ptrack->vhit.end(), ItrHitCompareZ);
    ptrack->nclu = distance(vphit_track.begin(), vphit_track.end());    

    ItrFastRefitTrack(ptrack);
    if (ptrack->nonmask == 0) continue;

    vectrackgood.push_back(*ptrack);
    for (vector<pHit >::iterator pcluIt = ptrack->vhit.begin();
	 pcluIt != ptrack->vhit.end(); ++pcluIt){
      (*pcluIt)->used = 1;
    }
    good_tracks++;
  } //trackit


  //==================================================  

  //clone
  int n_clone = 0;
  
  vector<Track> vecclones;
  for (vector<Track>::iterator ptrack1 = vectrackgood.begin();
       ptrack1 !=  vectrackgood.end(); ++ptrack1){//clone
    n_clone++;
    if (ptrack1->good == 0) continue;
    int n_clone2 = n_clone;

    Track clone;
    pTrack pclone = &clone;
    //float best_prob = 0.0;
    int   best_nclu = 0;
    vector<Track>::iterator ptrack_bad = 0;

    vector<Track>::iterator ptrack_next = ptrack1; ptrack_next++;
    for (vector<Track>::iterator ptrack2 = ptrack_next;
	 ptrack2 != vectrackgood.end(); ++ptrack2){
      n_clone2++;
      if (ptrack2->good == 0) continue;
      vector<Track>::iterator ptrack_f, ptrack_e;
      ptrack_f = ptrack1;
      if (ptrack2->zf < ptrack_f->zf)
	ptrack_f = ptrack2;
      ptrack_e = ptrack1;
      if (ptrack2->ze > ptrack_e->ze)
	ptrack_e = ptrack2;
      
      float ax = (ptrack_e->xe-ptrack_f->xf)/(ptrack_e->ze-ptrack_f->zf);
      float dx = 5.0*0.0200;
      if ((fabs(ptrack_f->xf+ax*(ptrack1->zf-ptrack_f->zf) - ptrack1->xf)/dx > 1.0) ||
	  (fabs(ptrack_f->xf+ax*(ptrack1->ze-ptrack_f->zf) - ptrack1->xe)/dx > 1.0) ||
	  (fabs(ptrack_f->xf+ax*(ptrack2->zf-ptrack_f->zf) - ptrack2->xf)/dx > 1.0) ||
	  (fabs(ptrack_f->xf+ax*(ptrack2->ze-ptrack_f->zf) - ptrack2->xe)/dx > 1.0) ) continue;
      
      ptrack2->good = 0;

      int ok = ((ptrack1->zf-ptrack2->ze)>0.0) || ((ptrack2->zf-ptrack1->ze)>0.0);
      if (!ok) continue;

      ptrack_f = ptrack1;
      ptrack_e = ptrack2;
      if (ptrack1->zf > ptrack2->ze){
	ptrack_f = ptrack2;
	ptrack_e = ptrack1;
      }

      float x1=0.0, dx1=0.0;

      x1  = ptrack_f->xe + ptrack_f->txe*(ptrack_e->zf - ptrack_f->ze);
      dx1 = 5.0*(sqrt(ptrack_f->cve[0][0]+(ptrack_e->zf-ptrack_f->ze)*(ptrack_e->zf-ptrack_f->ze)*
		      ptrack_f->cve[2][2]) + sqrt(ptrack_e->cvf[0][0]) + 0.0200);
      
      float x2=0.0, dx2=0.0;
      
      x2  = ptrack_e->xf + ptrack_e->txf*(ptrack_f->ze - ptrack_e->zf);
      dx2 = 5.0*(sqrt(ptrack_e->cvf[0][0]+(ptrack_f->ze-ptrack_e->zf)*(ptrack_f->ze-ptrack_e->zf)*
		      ptrack_e->cvf[2][2]) + sqrt(ptrack_f->cve[0][0]) + 0.0200);
      
      //if ((fabs((ptrack_e->xf-x1)/dx1)>3.0)||(fabs((ptrack_f->xe-x2)/dx2)>3.0)) continue;


      Track track;
      pTrack ptrack = &track;
      
      ptrack->pspointmap.clear();
      ptrack->good = 1;

      ptrack->xf   = ptrack_f->xf;        ptrack->yf  = ptrack_f->yf;
      ptrack->txf  = ptrack_f->txf;       ptrack->tyf = ptrack_f->tyf;
      ptrack->zf   = ptrack_f->zf;        ptrack->ze  = ptrack_e->ze;
      ptrack->xe   = ptrack_e->xe;        ptrack->ye  = ptrack_e->ye;
      ptrack->txe  = ptrack_e->txe;       ptrack->tye = ptrack_e->tye;
      ptrack->chi2 = ptrack_f->chi2+ptrack_e->chi2;
      ptrack->nsp  = ptrack_f->nsp +ptrack_e->nsp;       
      ptrack->nclu = ptrack_f->nclu+ptrack_e->nclu;    
      
      for (int i=0; i<4; i++) 
	for (int j=0; j<=i; j++){
	  ptrack->cvf[j][i] = ptrack_f->cvf[j][i];
	  ptrack->cve[j][i] = ptrack_e->cve[j][i];
	}
      
      ptrack->pf = ptrack->pe = ptrack_f->pf;
      
      ptrack->vhit.clear();
      for (vector<pHit >::iterator hitIt = ptrack_f->vhit.begin();
	   hitIt != ptrack_f->vhit.end(); ++hitIt){
	ptrack->vhit.push_back(*hitIt);
      }
      for (vector<pHit >::iterator hitIt = ptrack_e->vhit.begin();
	   hitIt != ptrack_e->vhit.end(); ++hitIt){
	ptrack->vhit.push_back(*hitIt);
      }
      sort(ptrack->vhit.begin(), ptrack->vhit.end(), ItrHitCompareZ);

      
      // ItrFastRefitTrack(ptrack);
      // if ((ptrack->nonmask == 0)||(ptrack->prob < 0.1)) continue;
      

      SPacepoint sp;
      
      sp.xf  = ptrack->xf;   sp.txf = ptrack->txf;
      sp.yf  = ptrack->yf;   sp.tyf = ptrack->tyf; 
      sp.zf  = ptrack->zf;
      sp.xe  = ptrack->xe;   sp.txe = ptrack->txe;
      sp.ye  = ptrack->ye;   sp.tye = ptrack->tye;
      sp.ze  = ptrack->ze;
      sp.used = 0; sp.level = 0; sp.neighborindex = -1;
      sp.nsp = ptrack->nsp;  sp.nclu = ptrack->nclu;
      sp.pf = sp.pe = ptrack->pf;
      
      sp.vhit.clear();
      for (vector<pHit >::iterator hitIt = ptrack->vhit.begin();
	   hitIt != ptrack->vhit.end(); ++hitIt){
	sp.vhit.push_back(*hitIt);
      }
      
      for (int i=0; i<4; i++) 
	for (int j=0; j<=i; j++){
	  sp.cvf[j][i] = ptrack->cvf[j][i];
	  sp.cve[j][i] = ptrack->cve[j][i];
	}
      
      ItrFitSPoint(sp,0);
      
      if (sp.prob < 0.1) continue;

      ptrack->xf   = sp.xf;        ptrack->yf  = sp.yf;
      ptrack->txf  = sp.txf;       ptrack->tyf = sp.tyf;
      ptrack->zf   = sp.zf;        ptrack->ze  = sp.ze;
      ptrack->xe   = sp.xe;        ptrack->ye  = sp.ye;
      ptrack->txe  = sp.txe;       ptrack->tye = sp.tye;

      ItrFastRefitTrack(ptrack);
      if ((ptrack->nonmask == 0)||(ptrack->prob < 0.1)) continue;
      

      //if (ptrack->prob > best_prob){ //store clone
      if (ptrack->nclu > best_nclu){ //store clone

	pclone->pspointmap.clear();
	pclone->good = 1;
	
	pclone->xf   = ptrack->xf;        pclone->yf  = ptrack->yf;
	pclone->txf  = ptrack->txf;       pclone->tyf = ptrack->tyf;
	pclone->zf   = ptrack->zf;        pclone->ze  = ptrack->ze;
	pclone->xe   = ptrack->xe;        pclone->ye  = ptrack->ye;
	pclone->txe  = ptrack->txe;       pclone->tye = ptrack->tye;
	pclone->chi2 = ptrack->chi2;      pclone->prob = ptrack->prob;
	pclone->nsp  = ptrack->nsp;       pclone->nclu = ptrack->nclu;
	
	for (int i=0; i<4; i++) 
	  for (int j=0; j<=i; j++){
	    pclone->cvf[j][i] = ptrack->cvf[j][i];
	    pclone->cve[j][i] = ptrack->cve[j][i];
	  }
	
	pclone->pf = pclone->pe = ptrack->pf;

	pclone->vhit.clear();
	for (vector<pHit >::iterator hitIt = ptrack->vhit.begin();
	     hitIt != ptrack->vhit.end(); ++hitIt){
	  pclone->vhit.push_back(*hitIt);
	}

	pclone->vest.clear();
	for (vector<Est>::iterator estIt = ptrack->vest.begin();
	     estIt != ptrack->vest.end(); ++estIt){
	  pclone->vest.push_back(*estIt);
	}

	ptrack_bad = ptrack2;
	//best_prob = ptrack->prob;
	best_nclu = ptrack->nclu;
      }

      ptrack2->good = 0;
    }   

    //if (best_prob > 0.0){
    if (best_nclu > 0.0){
      ptrack1->good = 0;
      ptrack_bad->good = 0;
      vecclones.push_back(*pclone);
    }

  }//clone
  zeit_itr_refit.stop();

#ifdef XXX
  //----------------------------------------------------------------------------------
  //kill ghost !!!
  zeit_itr_ghost.start();
  
  for (vector<Track>::iterator trackIt = vectrackgood.begin();
       trackIt != vectrackgood.end(); ++trackIt){ // trackIt
    if (trackIt->good == 0) continue;
    
    //check whether it is a ghost
    if (trackIt->nclu <= 16){ 
      
      int nsregions_prev = 0, nhits_prev = 0;
      CheckPrevSL(trackIt, nsregions_prev, nhits_prev);
      int nsregions_next = 0, nhits_next = 0;
      CheckNextSL(trackIt, nsregions_next, nhits_next);
      
      int ghost_ok_prev = 0, ghost_ok_next = 0;
      if (nsregions_prev>=4)
	if (float(nhits_prev)/float(nsregions_prev) <= 0.5) ghost_ok_prev = 1;
      if (nsregions_next>=4)
	if (float(nhits_next)/float(nsregions_next) <= 0.5) ghost_ok_next = 1;
      if ( ((ghost_ok_prev) && (ghost_ok_next))       ||
	   ((ghost_ok_prev) && (nsregions_next == 0)) ||
	   ((ghost_ok_next) && (nsregions_prev == 0)))   {
	trackIt->good=0;
      }
    }
  }
  
  zeit_itr_ghost.stop();
#endif //XXX  
  zeit_itr_rseg.start();


  //Itr_fill_rseg(&vectrackgood,1);
  ItrFillRSEG(&vectrackgood);
  ItrFillRSEG(&vecclones);

  if (ITR_DRAW){
    for (vector<Track>::iterator trackIt = vectrackgood.begin();
	 trackIt != vectrackgood.end(); ++trackIt){
      if ((trackIt->nonmask == 0)||(trackIt->good == 0)) continue;
      
      xpl[0] = trackIt->xf+x_shift; ypl[0] = trackIt->yf+y_shift; zpl[0] = trackIt->zf;
      xpl[1] = trackIt->xe+x_shift; ypl[1] = trackIt->ye+y_shift; zpl[1] = trackIt->ze;
      
      isplci(Red);
      ipl(2, zpl, xpl);
      ipl(2, zpl, ypl);
    }
    for (vector<Track>::iterator trackIt = vecclones.begin();
	 trackIt != vecclones.end(); ++trackIt){
      if ((trackIt->nonmask == 0)||(trackIt->good == 0)) continue;
      
      xpl[0] = trackIt->xf+x_shift; ypl[0] = trackIt->yf+y_shift; zpl[0] = trackIt->zf;
      xpl[1] = trackIt->xe+x_shift; ypl[1] = trackIt->ye+y_shift; zpl[1] = trackIt->ze;
      
      isplci(Red);
      ipl(2, zpl, xpl);
      ipl(2, zpl, ypl);
    }
  }
  SltCatsEvent();
  ItrOtrProlongation();
  zeit_itr_rseg.stop();

  if (ITR_DRAW)
    iuwk(1,1);
  zeit_itr_total.stop();

} 


//------------

void CheckNextSL(const vector<Track>::iterator trackI, int& nsregions, int& nhits){
  
  nsregions = 0;
  nhits = 0;

  ArtePointer<GEDE> gede = (*(trackI->vhit.rbegin()))->gede;// find the first gede
  ArtePointer<GESL> gesl = gede->gesl;
  
  int firstsl=(gesl->nsl+MS10shift);
  int firstpl=gede->lay;
  int index_Layf=0;
  if(firstsl==5) index_Layf=firstpl - 1;
  if(firstsl==6) index_Layf=firstpl + 7 ;
  if(firstsl==7) index_Layf=firstpl + 11 ;
  if(firstsl==8) index_Layf=firstpl + 15 ; 
  
  if(index_Layf>15) return;

  for(int gedeL=index_Layf;gedeL<24;gedeL++){
    
    for(int i=0;i<8;i++){  

      ArtePointer<GEDE> gedeT = gede_bank[i][gedeL];
    //trace to the gede z     
    float z_sgede = gedeT->zfw[0];
    float Dz2=(z_sgede-trackI->ze)*(z_sgede-trackI->ze);
    float errx2  = trackI->cve[0][0]+Dz2*trackI->cve[2][2];
    float erry2  = trackI->cve[1][1]+Dz2*trackI->cve[3][3];
    float errxy2 = trackI->cve[0][1]+Dz2*trackI->cve[2][3];
    float erry = 3.0*sqrt(erry2); 
    float cosa = gedeT->cos*gedeT->rot[0] - gedeT->sin*gedeT->rot[1];
    float sina = gedeT->sin*gedeT->rot[4] - gedeT->cos*gedeT->rot[3];
    
    float erru = 3.0*sqrt(errx2*cosa*cosa - 2.0*errxy2*cosa*sina + 
			  erry2*sina*sina);
    erru+=0.03;
    float x = trackI->xe+trackI->txe*(z_sgede-trackI->ze);
    float y = trackI->ye+trackI->tye*(z_sgede-trackI->ze);
    float u = x*cosa - y*sina;
    float y0=x*sina + y*cosa;
    float ym=y0-erry;
    float yp=y0+erry;
    
    if ((ym > gedeT->symn)&&(yp < gedeT->symx)&&
	(u-erru > gedeT->sxmn)&&(u+erru < gedeT->sxmx)){
      
      nsregions++;
      
      int nh = 0;
      ArteTableIterator<HITB> hitbf=gedeT->hit1;
      ArteTableIterator<HITB> hitbl=gedeT->hitl;     
      if (distance(hitbf, hitbl)+1 == 0) continue;
      for( ArteTable<HITB>::iterator hitbIt = hitbf ;
	   hitbIt != hitbl; ++hitbIt ){
	if (fabs(u-(*hitbIt)->x) < erru) nh++;
      }
      if (nh != 0){
	nhits++;
      }
    }//definitely inside
    }//gede
  }
  
}






void CheckPrevSL(const vector<Track>::iterator trackI, int& nsregions, int& nhits){

  nsregions = 0;
  nhits = 0;

  ArtePointer<GEDE> gede = (*(trackI->vhit.begin()))->gede;// find the first gede
  ArtePointer<GESL> gesl = gede->gesl;
  
  int firstsl=(gesl->nsl+MS10shift);
  int firstpl=gede->lay;
  int index_Layf=0;
  if(firstsl==5) index_Layf=firstpl - 1;
  if(firstsl==6) index_Layf=firstpl + 7 ;
  if(firstsl==7) index_Layf=firstpl + 11 ;
  if(firstsl==8) index_Layf=firstpl + 15 ; 

  
  if(index_Layf<8) return;

  for(int gedeL=0;gedeL<index_Layf;gedeL++){
    
    for(int i=0;i<8;i++){  

      ArtePointer<GEDE> gedeT = gede_bank[i][gedeL];
    //trace to the gede z     
    float z_sgede = gedeT->zfw[0];
    float Dz2=(z_sgede-trackI->zf)*(z_sgede-trackI->zf);
    float errx2  = trackI->cvf[0][0]+Dz2*trackI->cvf[2][2];
    float erry2  = trackI->cvf[1][1]+Dz2*trackI->cvf[3][3];
    float errxy2 = trackI->cvf[0][1]+Dz2*trackI->cvf[2][3];
    float erry = 3.0*sqrt(erry2); 
    float cosa = gedeT->cos*gedeT->rot[0] - gedeT->sin*gedeT->rot[1];
    float sina = gedeT->sin*gedeT->rot[4] - gedeT->cos*gedeT->rot[3];
    
    float erru = 3.0*sqrt(errx2*cosa*cosa - 2.0*errxy2*cosa*sina + 
			  erry2*sina*sina);
    erru+=0.03;
    float x = trackI->xf+trackI->txf*(z_sgede-trackI->zf);
    float y = trackI->yf+trackI->tyf*(z_sgede-trackI->zf);
    float u = x*cosa - y*sina;
    float y0=x*sina + y*cosa;
    float ym=y0-erry;
    float yp=y0+erry;
    
    if ((ym > gedeT->symn)&&(yp < gedeT->symx)&&
	(u-erru > gedeT->sxmn)&&(u+erru < gedeT->sxmx)){
      
      nsregions++;
      
      int nh = 0;
      ArteTableIterator<HITB> hitbf=gedeT->hit1;
      ArteTableIterator<HITB> hitbl=gedeT->hitl;     
      if (distance(hitbf, hitbl)+1 == 0) continue;
      for( ArteTable<HITB>::iterator hitbIt = hitbf ;
	   hitbIt != hitbl; ++hitbIt ){
	if (fabs(u-(*hitbIt)->x) < erru) nh++;
      }
      if (nh != 0){
	nhits++;
      }
    }//definitely inside
    }//gede
  }
}








//----------------------------------------------

void ItrFindTrack(const pSPacepoint psp, Track& newtrack, Track currenttrack){
  
  if (psp->level == 1){ // the last sp -> check and store

    if ((currenttrack.nclu > newtrack.nclu)||
	((currenttrack.nclu == newtrack.nclu)&&(currenttrack.prob > newtrack.prob))){

      newtrack.pspointmap.clear();
      for (map<ArtePointer<GESL>, pSPacepoint >::iterator pos = currenttrack.pspointmap.begin();
	   pos != currenttrack.pspointmap.end(); pos++){
	newtrack.pspointmap.insert(pair<ArtePointer<GESL>, pSPacepoint >
				   (pos->first, pos->second ));
      }
      newtrack.tyf  = currenttrack.tyf;
      newtrack.chi2 = currenttrack.chi2;
      newtrack.prob = currenttrack.prob;
      newtrack.nsp  = currenttrack.nsp;
      newtrack.nclu = currenttrack.nclu;
      newtrack.good = currenttrack.good;
    } // store track
  } // hit.level == 1
  else {
    vector<float>::iterator lchiIt = vchi2neighborSP[psp->neighborindex].begin();
    for (vector<pSPacepoint>::iterator plIt = vpneighborSP[psp->neighborindex].begin();
	 plIt != vpneighborSP[psp->neighborindex].end(); ++plIt){

      pSPacepoint pspl = *plIt;


      int zlay = 0, slay = 0;// check zero and stereo layers
      for (vector<pHit>::iterator pcluIt = pspl->vhit.begin();
	   pcluIt != pspl->vhit.end(); ++pcluIt){
	if (!(*pcluIt)->used){
	  float alpha = 180.0/3.1415926*atan2((*pcluIt)->sina,(*pcluIt)->cosa); 
	  if (alpha<-10.0) alpha += 360.0;
	  if ((alpha > -1.0)&&(alpha < 1.0)) zlay++;
	  else                               slay++;
	}
      }
      if ((zlay <= 1)||(slay <= 1)){// used space point

	pspl->used = 1;
	//compare with the original track:
	if ((currenttrack.nclu > newtrack.nclu)||
	    ((currenttrack.nclu == newtrack.nclu)&&(currenttrack.prob > newtrack.prob))){
	  newtrack.pspointmap.clear();
	  for (map<ArtePointer<GESL>, pSPacepoint >::iterator pos = currenttrack.pspointmap.begin();
	       pos != currenttrack.pspointmap.end(); pos++){
	    newtrack.pspointmap.insert(pair< ArtePointer<GESL>, pSPacepoint >
				       (pos->first, pos->second ));
	  }
	  newtrack.tyf  = currenttrack.tyf;
	  newtrack.chi2 = currenttrack.chi2;
	  newtrack.prob = currenttrack.prob;
	  newtrack.nsp  = currenttrack.nsp;
	  newtrack.nclu = currenttrack.nclu;
	  newtrack.good = 1;
	} // store track
      }
      else{//else

	// store the current track for a case of bad prolongation
	Track oldtrack;
	oldtrack.pspointmap.clear();
	for (map<ArtePointer<GESL>, pSPacepoint >::iterator pos = currenttrack.pspointmap.begin();
	     pos != currenttrack.pspointmap.end(); pos++){
	  oldtrack.pspointmap.insert(pair< ArtePointer<GESL>, pSPacepoint >
				     (pos->first, pos->second ));
	}
	oldtrack.tyf  = currenttrack.tyf;
	oldtrack.chi2 = currenttrack.chi2;
	oldtrack.prob = currenttrack.prob;
	oldtrack.nsp  = currenttrack.nsp;
	oldtrack.nclu = currenttrack.nclu;
	oldtrack.good = 1;
	// stored
	
      	currenttrack.pspointmap.insert(pair< ArtePointer<GESL>, pSPacepoint >
				       ((*(pspl->vhit.begin()))->gede->gesl, pspl));
	//currenttrack.chi2 += (*lchiIt);
	
	// fit current track as space point to calculate chi2 and prob      
	SPacepoint sp;
	pSPacepoint psp, pspf, pspe;
	psp  = &sp;
	pspf = (currenttrack.pspointmap.begin())->second;
	pspe = (currenttrack.pspointmap.begin())->second;
	
	psp->vhit.clear();
	for (map<ArtePointer<GESL>, pSPacepoint >::iterator spIt = currenttrack.pspointmap.begin();
	     spIt != currenttrack.pspointmap.end(); spIt++){
	  
	  if ((spIt->second)->zf < pspf->zf) pspf = spIt->second;
	  if ((spIt->second)->ze > pspe->ze) pspe = spIt->second;
	  
	  for (vector<pHit>::iterator pcluIt = (spIt->second)->vhit.begin();
	       pcluIt != (spIt->second)->vhit.end(); ++pcluIt){
	    psp->vhit.push_back(*pcluIt);
	  }
	}
	
	psp->xf  = pspf->xf;   psp->txf = pspf->txf;
	psp->yf  = pspf->yf;   psp->tyf = pspf->tyf; 
	psp->zf  = pspf->zf;
	psp->xe  = pspe->xe;   psp->txe = pspe->txe;
	psp->ye  = pspe->ye;   psp->tye = pspe->tye;
	psp->ze  = pspe->ze;
	psp->used = 0; psp->level = 0; psp->neighborindex = -1;
	
	sort(psp->vhit.begin(), psp->vhit.end(), ItrHitCompareZ);
	
	ItrFitSPoint(*psp,0);
	// fitted
	
	currenttrack.tyf = psp->tyf;
	currenttrack.chi2 = psp->chi2;
	currenttrack.prob = psp->prob;
	currenttrack.nsp  = distance(currenttrack.pspointmap.begin(), currenttrack.pspointmap.end());
	currenttrack.nclu = distance(psp->vhit.begin(), psp->vhit.end());
	
	if (psp->prob > 0.1){
	  ItrFindTrack(pspl, newtrack, currenttrack);
	}
	else{// compare and store old track as new one
	  if ((oldtrack.nclu > newtrack.nclu)||
	      ((oldtrack.nclu == newtrack.nclu)&&(oldtrack.prob > newtrack.prob))){
	    newtrack.pspointmap.clear();
	    for (map<ArtePointer<GESL>, pSPacepoint >::iterator pos = oldtrack.pspointmap.begin();
		 pos != oldtrack.pspointmap.end(); pos++){
	      newtrack.pspointmap.insert(pair< ArtePointer<GESL>, pSPacepoint >
					 (pos->first, pos->second ));
	    }
	    newtrack.tyf  = oldtrack.tyf;
	    newtrack.chi2 = oldtrack.chi2;
	    newtrack.prob = oldtrack.prob;
	    newtrack.nsp  = oldtrack.nsp;
	    newtrack.nclu = oldtrack.nclu;
	    newtrack.good = 1;
	  }
	}
      }
      
      ++lchiIt;
    }
  } 
  return;
}

//====================================================================
float ItrNeighborsChi2(const pSPacepoint sp1, const pSPacepoint sp2){
  float ax, ay, bx, by;
  float G[4][4];
  float chi2 = 0.0;
  float sum  = sp1->zf + sp2->zf;
  float sum2 = sum*sum;

  ax = 0.5*(sp1->txf + sp2->txf); ay = 0.5*(sp1->tyf + sp2->tyf);
  bx = 0.5*(sp1->xf  + sp2->xf) - 0.5*ax*sum;
  by = 0.5*(sp1->yf  + sp2->yf) - 0.5*ay*sum;
  G[2][2] = 0.25*(sp1->cvf[2][2] + sp2->cvf[2][2]);
  G[3][3] = 0.25*(sp1->cvf[3][3] + sp2->cvf[3][3]);
  G[0][0] = 0.25*(sp1->cvf[0][0] + sp2->cvf[0][0]) + 0.25*sum2*G[2][2];
  G[1][1] = 0.25*(sp1->cvf[1][1] + sp2->cvf[1][1]) + 0.25*sum2*G[3][3];
  chi2 += (ax - sp1->txf)*(ax - sp1->txf)/(G[2][2] + sp1->cvf[2][2]);
  chi2 += (ax - sp2->txf)*(ax - sp2->txf)/(G[2][2] + sp2->cvf[2][2]);
  chi2 += (ay - sp1->tyf)*(ay - sp1->tyf)/(G[3][3] + sp1->cvf[3][3]);
  chi2 += (ay - sp2->tyf)*(ay - sp2->tyf)/(G[3][3] + sp2->cvf[3][3]);
  chi2 += (bx + ax*sp1->zf - sp1->xf)*(bx + ax*sp1->zf - sp1->xf)/(G[0][0] + sp1->cvf[0][0]);
  chi2 += (bx + ax*sp2->zf - sp2->xf)*(bx + ax*sp2->zf - sp2->xf)/(G[0][0] + sp2->cvf[0][0]);
  chi2 += (by + ay*sp1->zf - sp1->yf)*(by + ay*sp1->zf - sp1->yf)/(G[1][1] + sp1->cvf[1][1]);
  chi2 += (by + ay*sp2->zf - sp2->yf)*(by + ay*sp2->zf - sp2->yf)/(G[1][1] + sp2->cvf[1][1]);

  return chi2;
}

float ItrNeighborsChi2(const pTrack ptrack1, const pTrack ptrack2){
  float ax, ay, bx, by;
  float G[4][4];
  float chi2 = 0.0;
  float sum  = ptrack1->zf + ptrack2->zf;
  float sum2 = sum*sum;

  ax = 0.5*(ptrack1->txf + ptrack2->txf); ay = 0.5*(ptrack1->tyf + ptrack2->tyf);
  bx = 0.5*(ptrack1->xf  + ptrack2->xf) - 0.5*ax*sum;
  by = 0.5*(ptrack1->yf  + ptrack2->yf) - 0.5*ay*sum;
  G[2][2] = 0.25*(ptrack1->cvf[2][2] + ptrack2->cvf[2][2]);
  G[3][3] = 0.25*(ptrack1->cvf[3][3] + ptrack2->cvf[3][3]);
  G[0][0] = 0.25*(ptrack1->cvf[0][0] + ptrack2->cvf[0][0]) + 0.25*sum2*G[2][2];
  G[1][1] = 0.25*(ptrack1->cvf[1][1] + ptrack2->cvf[1][1]) + 0.25*sum2*G[3][3];
  chi2 += (ax - ptrack1->txf)*(ax - ptrack1->txf)/(G[2][2] + ptrack1->cvf[2][2]);
  chi2 += (ax - ptrack2->txf)*(ax - ptrack2->txf)/(G[2][2] + ptrack2->cvf[2][2]);
  chi2 += (ay - ptrack1->tyf)*(ay - ptrack1->tyf)/(G[3][3] + ptrack1->cvf[3][3]);
  chi2 += (ay - ptrack2->tyf)*(ay - ptrack2->tyf)/(G[3][3] + ptrack2->cvf[3][3]);
  chi2 += (bx + ax*ptrack1->zf - ptrack1->xf)*(bx + ax*ptrack1->zf - ptrack1->xf)/
    (G[0][0] + ptrack1->cvf[0][0]);
  chi2 += (bx + ax*ptrack2->zf - ptrack2->xf)*(bx + ax*ptrack2->zf - ptrack2->xf)/
    (G[0][0] + ptrack2->cvf[0][0]);
  chi2 += (by + ay*ptrack1->zf - ptrack1->yf)*(by + ay*ptrack1->zf - ptrack1->yf)/
    (G[1][1] + ptrack1->cvf[1][1]);
  chi2 += (by + ay*ptrack2->zf - ptrack2->yf)*(by + ay*ptrack2->zf - ptrack2->yf)/
    (G[1][1] + ptrack2->cvf[1][1]);

  return chi2;
}

float ItrNeighborsChi2(const ArtePointer<RSEG> rseg1, const ArtePointer<RSEG> rseg2){
  float ax, ay, bx, by;
  float G[4][4];
  float chi2 = 0.0;
  float sum  = rseg1->zf + rseg2->zf;
  float sum2 = sum*sum;

  ax = 0.5*(rseg1->txf + rseg2->txf); ay = 0.5*(rseg1->tyf + rseg2->tyf);
  bx = 0.5*(rseg1->xf  + rseg2->xf) - 0.5*ax*sum;
  by = 0.5*(rseg1->yf  + rseg2->yf) - 0.5*ay*sum;
  G[2][2] = 0.25*(rseg1->cvf[5] + rseg2->cvf[5]);
  G[3][3] = 0.25*(rseg1->cvf[9] + rseg2->cvf[9]);
  G[0][0] = 0.25*(rseg1->cvf[0] + rseg2->cvf[0]) + 0.25*sum2*G[2][2];
  G[1][1] = 0.25*(rseg1->cvf[2] + rseg2->cvf[2]) + 0.25*sum2*G[3][3];
  chi2 += (ax - rseg1->txf)*(ax - rseg1->txf)/(G[2][2] + rseg1->cvf[5]);
  chi2 += (ax - rseg2->txf)*(ax - rseg2->txf)/(G[2][2] + rseg2->cvf[5]);
  chi2 += (ay - rseg1->tyf)*(ay - rseg1->tyf)/(G[3][3] + rseg1->cvf[9]);
  chi2 += (ay - rseg2->tyf)*(ay - rseg2->tyf)/(G[3][3] + rseg2->cvf[9]);
  chi2 += (bx + ax*rseg1->zf - rseg1->xf)*(bx + ax*rseg1->zf - rseg1->xf)/
    (G[0][0] + rseg1->cvf[0]);
  chi2 += (bx + ax*rseg2->zf - rseg2->xf)*(bx + ax*rseg2->zf - rseg2->xf)/
    (G[0][0] + rseg2->cvf[0]);
  chi2 += (by + ay*rseg1->zf - rseg1->yf)*(by + ay*rseg1->zf - rseg1->yf)/
    (G[1][1] + rseg1->cvf[2]);
  chi2 += (by + ay*rseg2->zf - rseg2->yf)*(by + ay*rseg2->zf - rseg2->yf)/
    (G[1][1] + rseg2->cvf[2]);

  return chi2;
}


//=========================== fill tables ====================
void Itr_fill_rseg(vector<Track> *tracktmp,int typevhit){
  
  RSEG r;
  RHIT rh;
  RPNT rp;


  for(vector<Track>::iterator ptrack =tracktmp->begin();
      ptrack != tracktmp->end(); ++ptrack){
    
    if (ptrack->good == 0) continue;
    
    r.xf  = ptrack->xf;  r.yf  = ptrack->yf;  r.zf  = ptrack->zf; 
    r.txf = ptrack->txf; r.tyf = ptrack->tyf;
    r.xe  = ptrack->xe;  r.ye  = ptrack->ye;  r.ze  = ptrack->ze; 
    r.txe = ptrack->txe; r.tye = ptrack->tye;
    
    r.cvf[ 0] = ptrack->cvf[0][0];      r.cvf[ 1] = ptrack->cvf[0][1];
    r.cvf[ 2] = ptrack->cvf[1][1];      r.cvf[ 3] = ptrack->cvf[0][2];
    r.cvf[ 4] = ptrack->cvf[1][2];      r.cvf[ 5] = ptrack->cvf[2][2];
    r.cvf[ 6] = ptrack->cvf[0][3];      r.cvf[ 7] = ptrack->cvf[1][3];
    r.cvf[ 8] = ptrack->cvf[2][3];      r.cvf[ 9] = ptrack->cvf[3][3];
    
    r.cvf[10] = r.cvf[11] = r.cvf[12] = r.cvf[13] = 0.0;
    r.cvf[14] = 100.0;
    
    r.cve[ 0] = ptrack->cve[0][0];      r.cve[ 1] = ptrack->cve[0][1];
    r.cve[ 2] = ptrack->cve[1][1];      r.cve[ 3] = ptrack->cve[0][2];
    r.cve[ 4] = ptrack->cve[1][2];      r.cve[ 5] = ptrack->cve[2][2];
    r.cve[ 6] = ptrack->cve[0][3];      r.cve[ 7] = ptrack->cve[1][3];
    r.cve[ 8] = ptrack->cve[2][3];      r.cve[ 9] = ptrack->cve[3][3];
    
    r.cve[10] = r.cve[11] = r.cve[12] = r.cve[13] = 0.0;
    r.cve[14] = 100.0;
    
    r.pf = r.pe = ptrack->pf;
      
    r.time = 0.0; r.dtim = 0.0;
    r.chi  = ptrack->chi2;

    r.fit  = 0;
    r.cmp  = Rsegc::patt | Rsegc::msg | Rsegc::bit16;
    
    ArteTable<RSEG>::push_back( r );
    ArtePointer<RSEG> rseg( ArteTable<RSEG>::size() );
    
    if (typevhit == 0){
      for(vector<ArtePointer<HITB> >::reverse_iterator hitbIt = ptrack->selecthits.rbegin();
	  hitbIt != ptrack->selecthits.rend(); ++hitbIt){
	createRelation( *hitbIt, rseg, &RSEG::hitb );
      }
    }
    if (typevhit == 1){
      for (vector<pHit>::iterator pcluIt = ptrack->vhit.begin();
	   pcluIt != ptrack->vhit.end(); ++pcluIt){
	createRelation( (*pcluIt)->hitb, rseg, &RSEG::hitb );
      }
    }
    
    int number_of_rhits = 0;
    for( ArteRelationIterator<HITB> hitbIt = rseg->hitb.begin();
	 hitbIt != rseg->hitb.end(); ++hitbIt ) {
      
      rh.tx = (rseg->xe-rseg->xf)/(rseg->ze-rseg->zf);
      rh.ty = (rseg->ye-rseg->yf)/(rseg->ze-rseg->zf);
      rh.x  = rseg->xf+((*hitbIt)->z-rseg->zf)*rh.tx;
      rh.y  = rseg->yf+((*hitbIt)->z-rseg->zf)*rh.ty;
      rh.p  = rseg->pf;
      rh.q  = (*hitbIt)->q;
      

      ArtePointer<RHIT> rhit = ArteTable<RHIT>::push_back( rh );
      createRelation( *hitbIt, rhit, &HITB::rhit );
      number_of_rhits++;
    }

    rseg->nhit = number_of_rhits; //!!!
   
    if (typevhit == 0){
      for (vector<ArtePointer<HITB> >::reverse_iterator hitbIt = ptrack->selecthits.rbegin();
	   hitbIt != ptrack->selecthits.rend(); ++hitbIt){
	ArtePointer<RHIT> rhit = *((*hitbIt)->rhit.begin());
	createRelation( rseg, rhit, &RSEG::rhit );	  
      }
    }
    if (typevhit == 1){
      for (vector<pHit>::iterator pcluIt = ptrack->vhit.begin();
	   pcluIt != ptrack->vhit.end(); ++pcluIt){
	ArtePointer<RHIT> rhit = *((*pcluIt)->hitb->rhit.begin());
	createRelation( rseg, rhit, &RSEG::rhit );	
      }
    }
    rp.xf  = rseg->xf;  rp.yf  = rseg->yf;  rp.zf = rseg->zf; 
    rp.txf = rseg->txf; rp.tyf = rseg->tyf; rp.pf = rseg->pf;
    for (int i=0; i<15; i++) rp.cvf[i] = rseg->cvf[i];
    
    ArtePointer<RPNT> rpntf = ArteTable<RPNT>::push_back( rp );
    createRelation( rseg, rpntf, &RSEG::rpnt );
    
    //rp.rseg = rseg;
    rp.xf  = rseg->xe;  rp.yf  = rseg->ye;  rp.zf = rseg->ze; 
    rp.txf = rseg->txe; rp.tyf = rseg->tye; rp.pf = rseg->pe;
    for (int i=0; i<15; i++) rp.cvf[i] = rseg->cve[i];
    
    ArtePointer<RPNT> rpnte = ArteTable<RPNT>::push_back( rp );
    createRelation( rseg, rpnte, &RSEG::rpnt ); 
  }
}

//=========================== fill tables ====================
void ItrFillRSEG(vector<Track> *track){  
  RSEG r;
  RHIT rh;
  RPNT rp;
  
  for(vector<Track>::iterator ptrack =track->begin();
      ptrack != track->end(); ++ptrack){
    
    if ((ptrack->nonmask == 0)||(ptrack->good == 0)) continue;

    r.xf  = ptrack->xf;  r.yf  = ptrack->yf;  r.zf  = ptrack->zf; 
    r.txf = ptrack->txf; r.tyf = ptrack->tyf;
    r.xe  = ptrack->xe;  r.ye  = ptrack->ye;  r.ze  = ptrack->ze; 
    r.txe = ptrack->txe; r.tye = ptrack->tye;
    
    r.cvf[ 0] = ptrack->cvf[0][0];      r.cvf[ 1] = ptrack->cvf[0][1];
    r.cvf[ 2] = ptrack->cvf[1][1];      r.cvf[ 3] = ptrack->cvf[0][2];
    r.cvf[ 4] = ptrack->cvf[1][2];      r.cvf[ 5] = ptrack->cvf[2][2];
    r.cvf[ 6] = ptrack->cvf[0][3];      r.cvf[ 7] = ptrack->cvf[1][3];
    r.cvf[ 8] = ptrack->cvf[2][3];      r.cvf[ 9] = ptrack->cvf[3][3];
    
    r.cvf[10] = r.cvf[11] = r.cvf[12] = r.cvf[13] = 0.0;
    r.cvf[14] = 100.0;
    
    r.cve[ 0] = ptrack->cve[0][0];      r.cve[ 1] = ptrack->cve[0][1];
    r.cve[ 2] = ptrack->cve[1][1];      r.cve[ 3] = ptrack->cve[0][2];
    r.cve[ 4] = ptrack->cve[1][2];      r.cve[ 5] = ptrack->cve[2][2];
    r.cve[ 6] = ptrack->cve[0][3];      r.cve[ 7] = ptrack->cve[1][3];
    r.cve[ 8] = ptrack->cve[2][3];      r.cve[ 9] = ptrack->cve[3][3];
    
    r.cve[10] = r.cve[11] = r.cve[12] = r.cve[13] = 0.0;
    r.cve[14] = 100.0;
    
    r.pf = r.pe = ptrack->pf;
    
    r.time = 0.0; r.dtim = 0.0;
    r.chi  = ptrack->chi2;
    
    r.fit  = 0;
    r.cmp  = Rsegc::patt | Rsegc::msg | Rsegc::bit16;
    
    ArteTable<RSEG>::push_back( r );
    ArtePointer<RSEG> rseg( ArteTable<RSEG>::size() );

    int number_of_rhits = 0;
    for (vector<Est>::reverse_iterator estIt = ptrack->vest.rbegin();
	 estIt != ptrack->vest.rend(); ++estIt){
      createRelation((*estIt).phit->hitb, rseg, &RSEG::hitb );
      ArtePointer<RHIT> rhit = ArteTable<RHIT>::push_back(rh);
      createRelation((*estIt).phit->hitb, rhit, &HITB::rhit );
      number_of_rhits++;
      createRelation( rseg, rhit, &RSEG::rhit );
      rhit->tx = (*estIt).tx;
      rhit->ty = (*estIt).ty;
      rhit->x  = (*estIt).x0;
      rhit->y  = (*estIt).y0;
      rhit->p  = rseg->pf;
    }
    rseg->nhit = number_of_rhits; //!!!
    
    //rp.rseg = rseg;
    rp.xf  = rseg->xf;  rp.yf  = rseg->yf;  rp.zf = rseg->zf; 
    rp.txf = rseg->txf; rp.tyf = rseg->tyf; rp.pf = rseg->pf;
    for (int i=0; i<15; i++) rp.cvf[i] = rseg->cvf[i];
    
    ArtePointer<RPNT> rpntf = ArteTable<RPNT>::push_back( rp );
    createRelation( rseg, rpntf, &RSEG::rpnt );
    
    //rp.rseg = rseg;
    rp.xf  = rseg->xe;  rp.yf  = rseg->ye;  rp.zf = rseg->ze; 
    rp.txf = rseg->txe; rp.tyf = rseg->tye; rp.pf = rseg->pe;
    for (int i=0; i<15; i++) rp.cvf[i] = rseg->cve[i];
    
    ArtePointer<RPNT> rpnte = ArteTable<RPNT>::push_back( rp );
    createRelation( rseg, rpnte, &RSEG::rpnt );
  }
}


//===========================================================================

typedef struct{
  float x,z,sina,cosa,sigma_m2,chi2;
  double Gk[4][4];
  double Rk[4];
  int mask;
  vector<pHit>::iterator ihit;
} ITRHIT, *pITRHIT;

void ItrKalmanFilterSmoother(ITRHIT arr[], int numhits, float pinv, float sigma_ms)
{
  double D_z,zprev,sina,cosa,l2sqrt,coeff,q33,q34,q44,zeta_k,Z;
  double h[4],Rk[4],Gk[4][4],Kk[4],L[4][4],CF[4][4],L_1[4][4],A[4][4],dG[4][4];

  for(int i=0;i<4;i++){
    Rk[i]=arr[0].Rk[i];
    for (int j=0;j<=i;j++)
      Gk[j][i]=0.0;
  }
  Gk[0][0]=1.0;Gk[1][1]=4.0;Gk[2][2]=0.1;Gk[3][3]=0.1;
  coeff=(1.0+0.105*0.105*pinv*pinv)*pinv*pinv; // muon mass correction
  l2sqrt=1.0+Rk[2]*Rk[2]+Rk[3]*Rk[3]; 
  l2sqrt=l2sqrt*sqrt(l2sqrt);
  
  q33=sigma_ms*coeff*l2sqrt*(1.0+Rk[2]*Rk[2]); 
  q44=sigma_ms*coeff*l2sqrt*(1.0+Rk[3]*Rk[3]); 
  q34=sigma_ms*coeff*l2sqrt*Rk[2]*Rk[3];
  
  zprev=arr[0].z; // start from the first hit
  for(int k=0; k<numhits; k++){
    sina  = arr[k].sina; 
    cosa  = arr[k].cosa;
    D_z   = arr[k].z-zprev;
    zprev = arr[k].z;
    Rk[0]+=D_z*Rk[2];
    Rk[1]+=D_z*Rk[3];
    Gk[0][0]+=D_z*(2.0*Gk[0][2]+D_z*Gk[2][2]);
    Gk[0][1]+=D_z*(Gk[1][2]+Gk[0][3]+D_z*Gk[2][3]);
    Gk[0][2]+=D_z*Gk[2][2];
    Gk[0][3]+=D_z*Gk[2][3];
    Gk[1][1]+=D_z*(2.0*Gk[1][3]+D_z*Gk[3][3]);
    Gk[1][2]+=D_z*Gk[2][3];
    Gk[1][3]+=D_z*Gk[3][3];
    if(arr[k].mask>0){
      Z=arr[k].sigma_m2+cosa*cosa*Gk[0][0]-2.0*sina*cosa*Gk[0][1]+sina*sina*Gk[1][1];
      zeta_k=arr[k].x-cosa*Rk[0]+sina*Rk[1];
      Kk[0]=cosa*Gk[0][0]-sina*Gk[0][1];
      Kk[1]=cosa*Gk[0][1]-sina*Gk[1][1];
      Kk[2]=cosa*Gk[0][2]-sina*Gk[1][2];
      Kk[3]=cosa*Gk[0][3]-sina*Gk[1][3];
      zeta_k=zeta_k/Z;
      for(int i=0;i<4;i++){
	Rk[i]+=Kk[i]*zeta_k;
	for(int j=0;j<=i;j++)
	  Gk[j][i]-=Kk[i]*Kk[j]/Z;
      }
    }
    //	cout<<k<<" ";
    for(int i=0;i<4;i++){
      arr[k].Rk[i]=Rk[i];
      //	  cout<<Rk[i]<<" ";
      
      for(int j=0;j<=i;j++) 
	arr[k].Gk[j][i]=Gk[j][i];
    }
    //	cout<<" mask="<<arr[k].mask<<endl;
    Gk[2][2]+=q33;Gk[3][3]+=q44;Gk[2][3]+=q34;
  }
  zprev=arr[numhits-1].z;
  
  for(int k=numhits-1;k>=0;k--){
    D_z=zprev-arr[k].z;
    if(k<numhits-1){
      L[0][0]=arr[k].Gk[0][0]+D_z*(2.0*arr[k].Gk[0][1]+
				   D_z*arr[k].Gk[1][1]);
      L[0][1]=arr[k].Gk[0][1]+D_z*arr[k].Gk[1][1];L[1][0]=L[0][1];
      L[0][0]=arr[k].Gk[0][0]+D_z*(2.0*arr[k].Gk[0][2]+
				   arr[k].Gk[2][2]*D_z);
      L[0][1]=arr[k].Gk[0][1]+D_z*(arr[k].Gk[0][3]+arr[k].Gk[1][2]+
				   D_z*arr[k].Gk[2][3]);L[1][0]=L[0][1];
      L[0][2]=arr[k].Gk[0][2]+D_z*arr[k].Gk[2][2];L[2][0]=L[0][2];
      L[0][3]=arr[k].Gk[0][3]+D_z*arr[k].Gk[2][3];L[3][0]=L[0][3];
      L[1][1]=arr[k].Gk[1][1]+D_z*(2.0*arr[k].Gk[1][3]+arr[k].Gk[3][3]*D_z);
      L[1][2]=arr[k].Gk[1][2]+D_z*arr[k].Gk[2][3];L[2][1]=L[1][2];
      L[1][3]=arr[k].Gk[1][3]+D_z*arr[k].Gk[3][3];L[3][1]=L[1][3];
      
      L[2][2]=arr[k].Gk[2][2]+q33;
      L[3][3]=arr[k].Gk[3][3]+q44;
      L[2][3]=arr[k].Gk[2][3]+q34;L[3][2]=L[2][3];
	  
      for(int v=0;v<4;v++) 
	for(int vv=0;vv<4;vv++) 
	  L_1[v][vv]=L[v][vv];
      
      vier_dim_inv(L_1);
      
      CF[0][0]=arr[k].Gk[0][0]+D_z*arr[k].Gk[0][2];
      CF[1][0]=arr[k].Gk[0][1]+D_z*arr[k].Gk[1][2];
      CF[2][0]=arr[k].Gk[0][2]+D_z*arr[k].Gk[2][2];
      CF[3][0]=arr[k].Gk[0][3]+D_z*arr[k].Gk[2][3];
      CF[0][1]=arr[k].Gk[0][1]+D_z*arr[k].Gk[0][3];
      CF[1][1]=arr[k].Gk[1][1]+D_z*arr[k].Gk[1][3];
      CF[2][1]=arr[k].Gk[1][2]+D_z*arr[k].Gk[2][3];
      CF[3][1]=arr[k].Gk[1][3]+D_z*arr[k].Gk[3][3];
      CF[0][2]=arr[k].Gk[0][2];CF[0][3]=arr[k].Gk[0][3];
      CF[1][2]=arr[k].Gk[1][2];CF[1][3]=arr[k].Gk[1][3];
      CF[2][2]=arr[k].Gk[2][2];CF[2][3]=arr[k].Gk[2][3];
      CF[3][2]=arr[k].Gk[2][3];CF[3][3]=arr[k].Gk[3][3];
      for(int i=0;i<4;i++) 
	for(int j=0;j<4;j++){
	  A[i][j]=0.0;
	  for(int m=0;m<4;m++) 
		A[i][j]+=CF[i][m]*L_1[m][j];
	}
      h[0]=arr[k+1].Rk[0]-arr[k].Rk[0]-D_z*arr[k].Rk[2];
      h[1]=arr[k+1].Rk[1]-arr[k].Rk[1]-D_z*arr[k].Rk[3];
      h[2]=arr[k+1].Rk[2]-arr[k].Rk[2];
      h[3]=arr[k+1].Rk[3]-arr[k].Rk[3];
      for(int i=0;i<4;i++)
	for(int j=0;j<4;j++) 
	  arr[k].Rk[i]+=A[i][j]*h[j];
      
      for(int i=0;i<4;i++) 
	for(int j=0;j<=i;j++) 
	  dG[j][i]=arr[k+1].Gk[j][i]-L[j][i];
      
      CF[0][0]=A[0][0]*dG[0][0]+A[0][1]*dG[0][1]+A[0][2]*dG[0][2]+A[0][3]*dG[0][3];
      CF[0][1]=A[0][0]*dG[0][1]+A[0][1]*dG[1][1]+A[0][2]*dG[1][2]+A[0][3]*dG[1][3];
      CF[0][2]=A[0][0]*dG[0][2]+A[0][1]*dG[1][2]+A[0][2]*dG[2][2]+A[0][3]*dG[2][3];
      CF[0][3]=A[0][0]*dG[0][3]+A[0][1]*dG[1][3]+A[0][2]*dG[2][3]+A[0][3]*dG[3][3];
      CF[1][0]=A[1][0]*dG[0][0]+A[1][1]*dG[0][1]+A[1][2]*dG[0][2]+A[1][3]*dG[0][3];
      CF[1][1]=A[1][0]*dG[0][1]+A[1][1]*dG[1][1]+A[1][2]*dG[1][2]+A[1][3]*dG[1][3];
      CF[1][2]=A[1][0]*dG[0][2]+A[1][1]*dG[1][2]+A[1][2]*dG[2][2]+A[1][3]*dG[2][3];
      CF[1][3]=A[1][0]*dG[0][3]+A[1][1]*dG[1][3]+A[1][2]*dG[2][3]+A[1][3]*dG[3][3];
      CF[2][0]=A[2][0]*dG[0][0]+A[2][1]*dG[0][1]+A[2][2]*dG[0][2]+A[2][3]*dG[0][3];
      CF[2][1]=A[2][0]*dG[0][1]+A[2][1]*dG[1][1]+A[2][2]*dG[1][2]+A[2][3]*dG[1][3];
      CF[2][2]=A[2][0]*dG[0][2]+A[2][1]*dG[1][2]+A[2][2]*dG[2][2]+A[2][3]*dG[2][3];
      CF[2][3]=A[2][0]*dG[0][3]+A[2][1]*dG[1][3]+A[2][2]*dG[2][3]+A[2][3]*dG[3][3];
      CF[3][0]=A[3][0]*dG[0][0]+A[3][1]*dG[0][1]+A[3][2]*dG[0][2]+A[3][3]*dG[0][3];
      CF[3][1]=A[3][0]*dG[0][1]+A[3][1]*dG[1][1]+A[3][2]*dG[1][2]+A[3][3]*dG[1][3];
      CF[3][2]=A[3][0]*dG[0][2]+A[3][1]*dG[1][2]+A[3][2]*dG[2][2]+A[3][3]*dG[2][3];
      CF[3][3]=A[3][0]*dG[0][3]+A[3][1]*dG[1][3]+A[3][2]*dG[2][3]+A[3][3]*dG[3][3];
      for(int i=0;i<4;i++) 
	for(int j=0;j<=i;j++)
	  for(int m=0;m<4;m++) 
	    arr[k].Gk[j][i]+=CF[j][m]*A[i][m];
    }
    Z=arr[k].sigma_m2+arr[k].cosa*arr[k].cosa*arr[k].Gk[0][0]-
      2.0*arr[k].sina*arr[k].cosa*arr[k].Gk[0][1]+
      arr[k].sina*arr[k].sina*arr[k].Gk[1][1];
    zeta_k=(arr[k].x-arr[k].cosa*arr[k].Rk[0]+arr[k].sina*arr[k].Rk[1]);
    arr[k].chi2=zeta_k*zeta_k/Z;
    zprev=arr[k].z;
  }
  return;
}


void ItrFastRefitTrack(Track *ptrack){

  const double chi2cut=10.0;
  
  int index,numhits,nonmask,nchanges,nstereo=0,nzero=0;
  double chi2t=0.0,pinv;
  double sigma_ms = 3.5e-6;//3.0e-6; // multiple scattering corrections 
  
  ITRHIT arr[300];
  //  ITRHIT ar;
  
  map<int, int> zerolayermap, stereolayermap;
  
  numhits=0;/*numhits6=0;numhits8=0;*/index=0;nonmask=0;
  nstereo=0;nzero=0;
  for(vector<pHit>::iterator phitIt = ptrack->vhit.begin();
      phitIt != ptrack->vhit.end(); ++phitIt){
    ArtePointer<GEDE> gede = (*phitIt)->gede;
    arr[index].sina=gede->sin*gede->rot[4] - gede->cos*gede->rot[3];
    arr[index].cosa=gede->cos*gede->rot[0] - gede->sin*gede->rot[1];
    arr[index].z=(*phitIt)->z;arr[index].x=(*phitIt)->x;
    arr[index].ihit=phitIt;
    arr[index].mask=1;
    nonmask++;
    if(fabs(arr[index].sina)<0.03) nzero++;
    else nstereo++;
    arr[index].sigma_m2 = (*phitIt)->hitb->r*(*phitIt)->hitb->r*1.2*1.2;
    index++;
  }
  numhits = index;
  //ptrack->nonmask=1;
  /*
  if((nstereo<4)||(nzero<4))
    {
      ar.z=0.0;ar.x=0.0;ar.sina=0.707;ar.cosa=sqrt(1.0-ar.sina*ar.sina);
      ar.mask=10;ar.ihit=NULL;ar.sigma_m2=0.1;
      for(int i=0;i<numhits;i++) arr[i+2]=arr[i];
      arr[0]=ar;
      ar.sina=-ar.sina;
      arr[1]=ar;
      numhits+=2;
    }
  */
  arr[0].Rk[0] = ptrack->xf+ptrack->txf*(arr[0].z-ptrack->zf); 
  arr[0].Rk[1] = ptrack->yf+ptrack->tyf*(arr[0].z-ptrack->zf);
  arr[0].Rk[2] = ptrack->txf;
  arr[0].Rk[3] = ptrack->tyf;
  nchanges=0;
  int nzerolayers = 0, nstereolayers = 0;
  zerolayermap.clear(); stereolayermap.clear();

  pinv=1.0/(Bmagnet*zmagnet*sqrt(1.0+ptrack->txf*ptrack->txf+ptrack->tyf*ptrack->tyf)/((ptrack->xf-ptrack->zf*ptrack->txf)*
						       sqrt(1.0+ptrack->txf*ptrack->txf)));
  int sign1;
  if(pinv<0.0) sign1=-1;else sign1=1;
  pinv = fabs(pinv);
  if (pinv > 0.5) pinv = 2.0*pinv*pinv;
  ptrack->pf=ptrack->pe=pinv*sign1;

  if (nonmask>4){// nonmask
    for(int iter=0;iter<1;iter++){ //iter 2

      ItrKalmanFilterSmoother(arr,numhits,pinv,sigma_ms);
      chi2t=0.0;nonmask=0;nstereo=0;nzero=0;
      zerolayermap.clear(); stereolayermap.clear();
      for(int k=numhits-1;k>=0;k--){
	if(arr[k].mask==10) continue;
	if(arr[k].chi2<chi2cut){
	  chi2t+=arr[k].chi2;(*arr[k].ihit)->mask=1;
	  if(!arr[k].mask) nchanges++;
	  arr[k].mask=1;
	  (*arr[k].ihit)->x0=arr[k].Rk[0];
	  (*arr[k].ihit)->y0=arr[k].Rk[1];
	  (*arr[k].ihit)->tx=arr[k].Rk[2];
	  (*arr[k].ihit)->ty=arr[k].Rk[3];
	  nonmask++;
	  if(fabs(arr[k].sina)<0.03){ 
	    nzero++;
	    int lay = 10*((*arr[k].ihit)->hitb->gede->gesl.index())+(*arr[k].ihit)->hitb->gede->lay;
	    zerolayermap.insert(pair<int, int>(lay, 1));
	  }
	  else{
	    nstereo++;
	    int lay = 10*((*arr[k].ihit)->hitb->gede->gesl.index())+(*arr[k].ihit)->hitb->gede->lay;
	    stereolayermap.insert(pair<int, int>(lay, 1));
	  }
	}
	else{
	  if(arr[k].mask) nchanges++;
	  arr[k].mask=0;(*arr[k].ihit)->mask=0;
	}
      }
      if(!nchanges) break;
    }//iter 2
  }//nonmask

  ptrack->nonmask = 0;
  nzerolayers   = distance(zerolayermap.begin(), zerolayermap.end());
  nstereolayers = distance(stereolayermap.begin(), stereolayermap.end());
  if((nzerolayers>=3) && (nstereolayers>=2)) { //5x5
    ptrack->nonmask=nonmask;
    ptrack->nclu = nzero + nstereo;
    ptrack->chi2 = chi2t;
    if (nzero + nstereo > 4)
      ptrack->prob = prob_(chi2t,(nzero + nstereo - 4));
    else
      ptrack->prob = 0.0001;
    if (((ptrack->nclu<=8)||(ptrack->pf>0.2))&&(ptrack->prob<0.05)) ptrack->nonmask = 0;
    if ((ptrack->nclu <= 8)&&(ptrack->prob <= 0.4)&&(1.0/(ptrack->pf)< 15.0)) ptrack->nonmask = 0; 
    if ((fabs(ptrack->yf/ptrack->zf - ptrack->tyf)> 0.01)&&
	(ptrack->prob <= 0.4)&&(1.0/(ptrack->pf)< 15.0)) ptrack->nonmask = 0; 

    /*
    if ((ptrack->nclu<10)&&
	//((float(numhits6)/float(numhits)>0.7)||(float(numhits8)/float(numhits)>0.7))&&
	(ptrack->prob<0.5)) ptrack->nonmask = 0;
    */
  }
  
  if (ptrack->nonmask != 0){//nonmask 
    int first_flag=1; 
    int last_index=0; 
    for(int k=0; k<numhits; k++) if(arr[k].mask){
      if(arr[k].mask==10) continue;
      last_index=k;
      if(first_flag){
	first_flag=0;
	
	ptrack->xf  = arr[k].Rk[0];          
	ptrack->yf  = arr[k].Rk[1];
	ptrack->zf  = arr[k].z;
	ptrack->txf = arr[k].Rk[2];          
	ptrack->tyf = arr[k].Rk[3];
	
	for(int i=0;i<4;i++) 
	  for(int j=0;j<=i;j++) 
	    ptrack->cvf[j][i] = arr[k].Gk[j][i];    
      }
    }
    ptrack->xe  = arr[last_index].Rk[0];          
    ptrack->ye  = arr[last_index].Rk[1];
    ptrack->ze  = arr[last_index].z;
    ptrack->txe = arr[last_index].Rk[2];          
    ptrack->tye = arr[last_index].Rk[3];
    for(int i=0;i<4;i++) 

      for(int j=0;j<=i;j++) 
	ptrack->cve[j][i] = arr[last_index].Gk[j][i]; 

    if ((ptrack->cvf[0][0]< 0.0)||(ptrack->cvf[1][1]< 0.0)||
	(ptrack->cvf[2][2]< 0.0)||(ptrack->cvf[3][3]< 0.0)){
      cout<<"------------------>"<<endl;
      cout<<"------------------>"<<endl;
      cout<<"------------------> ItrCats bad cvf matrix "<<ptrack->cvf[0][0]<<" "<<
	ptrack->cvf[1][1]<<" "<<ptrack->cvf[2][2]<<" "<<ptrack->cvf[3][3]<<endl;
      cout<<"------------------>"<<endl;
      cout<<"------------------>"<<endl;
    }
    if ((ptrack->cve[0][0]< 0.0)||(ptrack->cve[1][1]< 0.0)||
	(ptrack->cve[2][2]< 0.0)||(ptrack->cve[3][3]< 0.0)){
      cout<<"------------------>"<<endl;
      cout<<"------------------>"<<endl;
      cout<<"------------------> ItrCats bad cve matrix "<<ptrack->cve[0][0]<<" "<<
	ptrack->cve[1][1]<<" "<<ptrack->cve[2][2]<<" "<<ptrack->cve[3][3]<<endl;
      cout<<"------------------>"<<endl;
      cout<<"------------------>"<<endl;
    }
    
    Est est;
    ptrack->vest.clear();
    vector<pHit> htmp;
    htmp.clear();
    for(vector<pHit>::iterator phitIt = ptrack->vhit.begin();
	phitIt != ptrack->vhit.end(); ++phitIt){
      if((*phitIt)->mask>0.0){
	est.phit=(*phitIt);
	est.x0=(*phitIt)->x0;est.y0=(*phitIt)->y0;
	est.tx=(*phitIt)->tx;est.ty=(*phitIt)->ty;
	ptrack->vest.push_back(est);
	htmp.push_back((*phitIt));
      }
    }
    ptrack->vhit.clear();
    for(vector<pHit>::iterator phitIt=htmp.begin();phitIt!=htmp.end(); ++phitIt)
      ptrack->vhit.push_back((*phitIt));
  }
  else{
    for(vector<pHit>::iterator phitIt = ptrack->vhit.begin();
	phitIt != ptrack->vhit.end(); ++phitIt)
      (*phitIt)->mask = 0.0;
  }
  return;   
}

void ItrFitMS(SPacepoint& sp)
{ 
  const double chi2cut=4.0;
  
 
  int index,numhits,nonmask,nstereo=0,nzero=0,nchanges;
  double chi2t=0.0,pinv;
  double sigma_ms = 3.0e-6;// multiple scattering corrections 
  Track  track;
  ITRHIT arr[300];
  pTrack ptrack = &track;

    
  ptrack->pspointmap.clear();
  ptrack->good = 1;
    
  ptrack->xf   = sp.xf;        ptrack->yf   = sp.yf;
  ptrack->txf  = sp.txf;       ptrack->tyf  = sp.tyf;
  ptrack->zf   = sp.zf;        ptrack->ze   = sp.ze;
  ptrack->xe   = sp.xe;        ptrack->ye   = sp.ye;
  ptrack->txe  = sp.txe;       ptrack->tye  = sp.tye;
  ptrack->chi2 = sp.chi2;      ptrack->prob = sp.prob;
  ptrack->nsp  = sp.nsp;       ptrack->nclu = sp.nclu;
  //ptrack->pf=ptrack->pe=0.3;

  for (vector<pHit >::iterator hitIt = sp.vhit.begin();
       hitIt != sp.vhit.end(); ++hitIt){
    ptrack->vhit.push_back(*hitIt);
  }
  sort(ptrack->vhit.begin(), ptrack->vhit.end(),ItrHitCompareZ);

  numhits=0;index=0;nonmask=0;
  nzero=0;nstereo=0;
  for(vector<pHit>::iterator phitIt = ptrack->vhit.begin();
      phitIt != ptrack->vhit.end(); ++phitIt){
    ArtePointer<GEDE> gede = (*phitIt)->gede;
    arr[index].sina=gede->sin*gede->rot[4] - gede->cos*gede->rot[3];
    arr[index].cosa=gede->cos*gede->rot[0] - gede->sin*gede->rot[1];
    arr[index].z=(*phitIt)->z;arr[index].x=(*phitIt)->x;

    if(fabs(arr[index].sina)<0.03) nzero++;
    else nstereo++;

    arr[index].ihit=phitIt;
    arr[index].mask=1;
    nonmask++;
    arr[index].sigma_m2 = (*phitIt)->hitb->r*(*phitIt)->hitb->r*1.2*1.2;
    index++;
  }
  numhits = index;
  nchanges=0;
  pinv=1.0/(Bmagnet*zmagnet*sqrt(1.0+ptrack->txf*ptrack->txf+ptrack->tyf*ptrack->tyf)/
	    ((ptrack->xf-ptrack->zf*ptrack->txf)*sqrt(1.0+ptrack->txf*ptrack->txf)));
  int sign1;
  if(pinv<0.0) sign1=-1;else sign1=1;
  pinv = fabs(pinv);
  if (pinv > 0.5) pinv = 2.0*pinv*pinv;
  ptrack->pf=ptrack->pe=pinv*sign1;//0.3;
  //pinv=1.0;//0.3;//!!!!!!!!!!!!!!!!!!!!
  for(int iter=1;iter<=5;iter++)
    {
      nchanges=0;
      arr[0].Rk[0] = ptrack->xf; arr[0].Rk[1] = ptrack->yf;
      arr[0].Rk[2] = ptrack->txf;arr[0].Rk[3] = ptrack->tyf;
      ItrKalmanFilterSmoother(arr,numhits,pinv,sigma_ms);
      chi2t=0.0;nonmask=0;nstereo=0;nzero=0;
      for(int k=0;k<numhits;k++) if(arr[k].mask!=10)
	{
	  if(arr[k].chi2<chi2cut)
	    {
	      chi2t+=arr[k].chi2;
	      if(arr[k].mask!=1) nchanges++;
	      arr[k].mask=1;
	      if(fabs(arr[k].sina)<0.03) nzero++;
	      else nstereo++;
	    }
	else 
	  {
	    if(arr[k].mask==1) nchanges++;
	    arr[k].mask=0;
	  }
	}
      if(!nchanges) break;
    }
  if((nstereo>=2)&&(nzero>=2))
    {
      for(int iter=1;iter<5;iter++)
	{
	  float best_chi2=100000.0;
	  int best_ind=-1;
	  for(int k=0;k<numhits;k++) if(arr[k].mask==0)
	    {
	      if(arr[k].chi2<best_chi2) 
		{
		  best_ind=k;
		  best_chi2=arr[k].chi2;
		}
	    }
	  if(best_ind!=-1) 
	    {
	      arr[best_ind].mask=1;
	      ItrKalmanFilterSmoother(arr,numhits,pinv,sigma_ms);
	      chi2t=0.0;nonmask=0;nstereo=0;nzero=0;
	      for(int k=0;k<numhits;k++) if(arr[k].mask!=10)
		{
		  if(arr[k].chi2<chi2cut)
		    {
		      chi2t+=arr[k].chi2;
		      arr[k].mask=1;
		      if(fabs(arr[k].sina)<0.03) nzero++;
		      else nstereo++;
		    }
		  else arr[k].mask=0;
		}
	      if(arr[best_ind].mask==0) break;
	    }
	}
    }
  sp.chi2=chi2t;
  //  int  nclu=nzero + nstereo;
  if (nzero + nstereo > 4)
    sp.prob = prob_(chi2t,(nzero + nstereo - 4));
  else
    sp.prob = 0.0001;
  if (nzero + nstereo > 4)
    {
  int first_flag=1;
  int last_index=0;
  for(int k=0; k<numhits; k++) if(arr[k].mask){
    if(arr[k].mask==10) continue;
    last_index=k;
    if(first_flag){
      first_flag=0;
      sp.xf  = arr[k].Rk[0];          
      sp.yf  = arr[k].Rk[1];
      sp.zf  = arr[k].z;
      sp.txf = arr[k].Rk[2];          
      sp.tyf = arr[k].Rk[3];
      for(int i=0;i<4;i++) 
	for(int j=0;j<=i;j++) 
	  sp.cvf[j][i] = arr[k].Gk[j][i];
   }
  }
  sp.xe  = arr[last_index].Rk[0];          
  sp.ye  = arr[last_index].Rk[1];
  sp.ze  = arr[last_index].z;
  sp.txe = arr[last_index].Rk[2];          
  sp.tye = arr[last_index].Rk[3];
  for(int i=0;i<4;i++) 
    for(int j=0;j<=i;j++) 
      sp.cve[j][i] = arr[last_index].Gk[j][i]; 

  vector<pHit> htmp;
  htmp.clear();index=0;
  for(vector<pHit>::iterator phitIt = ptrack->vhit.begin();
      phitIt != ptrack->vhit.end(); ++phitIt)
    {
      if(arr[index].mask==1)
	{
	  htmp.push_back((*phitIt));
	}
      index++;
    }
  sp.vhit.clear();
    for(vector<pHit>::iterator phitIt=htmp.begin();phitIt!=htmp.end(); ++phitIt)
      sp.vhit.push_back((*phitIt));
  
    }
  return;
}



//========================================================


void ItrGraphIni(){

  iginit(0);
  igsse(0, 6);


  iscr(1, Black       ,0.0000, 0.0000, 0.0000);
  iscr(1, DarkGray    ,0.5000, 0.5400, 0.5300);
  iscr(1, Blue        ,0.0000, 0.0000, 1.0000);
  iscr(1, Green       ,0.1333, 0.5451, 0.1333);
  iscr(1, Cyan        ,0.0000, 0.8078, 0.8196);
  iscr(1, Red         ,1.0000, 0.0000, 0.0000);
  iscr(1, Magenta     ,1.0000, 0.0000, 1.0000);
  iscr(1, Brown       ,0.5000, 0.1647, 0.1647);
  iscr(1, LightBlue   ,0.6784, 0.8471, 0.9020);
  iscr(1, LightGreen  ,0.5961, 0.9843, 0.5961);
  iscr(1, LightCyan   ,0.4980, 1.0000, 0.8314);
  iscr(1, LightRed    ,0.9804, 0.5020, 0.4471);
  iscr(1, LightMagenta,0.8549, 0.4392, 0.8392);
  iscr(1, Yellow      ,1.0000, 1.0000, 0.0000);
  iscr(1, LightGray   ,0.8275, 0.8275, 0.8275);
  iscr(1, White       ,1.0000, 1.0000, 1.0000);


  iswn(2, 680.00, 830.00,-55.00, 55.00);  //ITR

  isvp(2,0.0,1.0,0.0,1.0);
  iselnt(2);

  isfais(1);

  isfaci(LightGray);
  isplci(Red);

  igbox(680.00, 830.00,-55.00, 55.00);  //ITR

  iuwk(1,1);

}
void FillOTrGeometry()
{
  int SLid=-1;
  GEDE_TYPE buf;
  
  for ( ArteTable<GESL>::iterator geslIt = ArteTable<GESL>::begin();
	geslIt != ArteTable<GESL>::end(); geslIt++ ) {//gesl
    
    if ((*geslIt)->cmp!=2) continue; // not OTR
    if (((*geslIt)->nsl<8-OTRshift)||((*geslIt)->nsl>11-OTRshift)) 
      continue; // not PC
    SLid++;
    for(int i=0;i<13;i++) slg[SLid].ngede[i]=0;
    for ( ArteRelationIterator<GEDE> gedeIt = (*geslIt)->gede.begin();
	  gedeIt != (*geslIt)->gede.end(); gedeIt++ ) {//gede
      
      float cos = (*gedeIt)->cos*(*gedeIt)->rot[0] - (*gedeIt)->sin*(*gedeIt)->rot[1];
      float sin = (*gedeIt)->sin*(*gedeIt)->rot[4] - (*gedeIt)->cos*(*gedeIt)->rot[3];
      
      float alpha = 180.0/3.1415926*atan2(sin, cos); if (alpha<-10.0) alpha += 360.0;
      
      float sxmn, sxmx, symn, symx, szmn, szmx;
      OtrGedeMinMax(sxmn, sxmx, symn, symx, szmn, szmx, (*gedeIt));
      
      OTR_SIZE gede_size;
      gede_size.alpha = alpha;
      gede_size.sin   = sin;   gede_size.cos   = cos;
      gede_size.sxmn  = sxmn;  gede_size.sxmx  = sxmx;
      gede_size.symn  = symn;  gede_size.symx  = symx;
      gede_size.szmn  = szmn;  gede_size.szmx  = szmx;
      buf.gz=gede_size;
      buf.gede=(*gedeIt);
      
      int sec = (*gedeIt)->sec;
      slg[SLid].s[sec][slg[SLid].ngede[sec]]=buf;
      slg[SLid].ngede[sec]++;
    }
  }
  slg[0].xr=20.5;slg[0].xi=94.1; slg[0].xm=121.3;slg[0].x0=205.2;
  slg[0].yr=20.5;slg[0].y0=136.1;slg[0].zf=704.5;slg[0].ze=733.1;
  slg[1].xr=19.2;slg[1].xi=95.4; slg[1].xm=122.8;slg[1].x0=206.7;
  slg[1].yr=20.5;slg[1].y0=136.1;slg[1].zf=746.5;slg[1].ze=770.6;
  slg[2].xr=19.2;slg[2].xi=95.4; slg[2].xm=121.8;slg[2].x0=206.7;
  slg[2].yr=20.5;slg[2].y0=136.1;slg[2].zf=781.1;slg[2].ze=805.2;
  slg[3].xr=20.5;slg[3].xi=94.1; slg[3].xm=121.3;slg[3].x0=205.2;
  slg[3].yr=20.5;slg[3].y0=136.1;slg[3].zf=821.4;slg[3].ze=850.0;
  /*
    for(int i=0;i<4;i++)
    {
    cout<<"SL "<<i+1<<endl;
    for(int j=1;j<13;j++)
    {
    cout<<"sector "<<j<<" "<<slg[i].ngede[j]<<" GEDEs"<<endl;
    for(int k=0;k<slg[i].ngede[j];k++)
    {
    OTR_SIZE gz=slg[i].s[j][k].gz;
    cout<<slg[i].s[j][k].gede<<" "<<gz.szmn<<":"<<gz.szmx<<" "<<
    gz.sxmn<<":"<<gz.sxmx<<" "<<gz.symn<<":"<<gz.symx<<endl; 
    }
    }
    }
  */
}
typedef struct
{
  ArtePointer<RSEG> rs;
  int SL[4];
  int Nav[4];
  double cve[12];
  float xe,ye,txe,tye,ze,chi2;
  HIT h[200];
  int n,nsl;
} ITR_CAND, *pITR_CAND;


void ItrOtrProlongation()
{
  const float sigma_ms=4e-6;

  ITR_CAND itrc[200];
  int numcand=0,found;
  ITR_CAND buf;
  int SLid,Sec,nGEDE;
  int thits=0;
  OTR_SIZE gz;
  float zprev,xe,ye,zmid,u,v,erru,Dz,u1,u2,sina,cosa,Rk[4],q,zeta_k,Du_k,theta_k,tu_k,h[4],sigma2,Z,f_k,
    q33,q34,q44,coeff,l2sqrt,pinv,zmax;
  double gk[4][4],Gk[4][4],Kk[4];
  ArteTableIterator<HITB> hitbItf,hitbIte;
  HIT hf[100];
  int nfound;

  for (ArteTable<RSEG>::iterator rsegIt = ArteTable<RSEG>::begin();
       rsegIt != ArteTable<RSEG>::end(); rsegIt++)
    { 
      if (!((*rsegIt)->cmp & Rsegc::bit16)) continue; //not cats
      if ((*rsegIt)->cmp & Rsegc::bitsign) continue;  // killed
      if (!((*rsegIt)->cmp & Rsegc::msg)) continue;  //not ITR
      if(fabs((*rsegIt)->pf)>2.0) continue;
      if((*rsegIt)->cve[0]>2.0) continue;
      found=0;
      for(int i=0;i<4;i++) 
	{
	  buf.SL[i]=-1;
	  buf.Nav[i]=0;
	}
      for(int i=0;i<4;i++)
	{
	  float zf=slg[i].zf;
	  float xe,ye;
	  xe=(*rsegIt)->xe+(*rsegIt)->txe*(zf-(*rsegIt)->ze);
	  ye=(*rsegIt)->ye+(*rsegIt)->tye*(zf-(*rsegIt)->ze);
	  if((*rsegIt)->ze>zf) continue;
	  if(((fabs(xe)>slg[i].xr)||(fabs(ye)>slg[i].yr))&&
	     (fabs(xe)<slg[i].x0)&&(fabs(xe)<slg[i].y0))
	    {
	      buf.SL[i]=i;buf.n=0;
	      buf.rs=(*rsegIt);
	      for(int m=0;m<10;m++) buf.cve[m]=(*rsegIt)->cve[m];
	      buf.xe=(*rsegIt)->xe;buf.ye=(*rsegIt)->ye;
	      buf.txe=(*rsegIt)->txe;buf.tye=(*rsegIt)->tye;
	      buf.ze=(*rsegIt)->ze;
	      int sec=-1;
	      if(fabs(xe)<slg[i].xi)
		{
		  if(ye>slg[i].yr) sec=1;if(ye<-slg[i].yr) sec=2;
		  if(fabs(ye)<slg[i].yr)
		    {
		      if(xe<0.0)
			{
			  if(ye>0) sec=5;if(ye<0) sec=6;
			}
		      if(xe>0.0)
			{
			  if(ye>0) sec=3;if(ye<0) sec=4;
			}
		    }
		}
	      if((xe>-slg[i].xm)&&(xe<-slg[i].xi))
		{
		  if(ye>0) sec=9;if(ye<0) sec=10;
		}
	      if((xe<slg[i].xm)&&(xe>slg[i].xi))
		{
		  if(ye>0) sec=7;if(ye<0) sec=8;
		}
	      if(xe>slg[i].xm) sec=11;
	      if(xe<-slg[i].xm) sec=12;
	      buf.Nav[i]=sec;
	      if(sec!=-1) found++;
	    }
	}
      if(found)
	{
	  //	  cout<<"Found..."<<endl;
	  itrc[numcand]=buf;
	  numcand++;
	}
    }
  if(numcand)
    {      
      /*
      cout<<numcand<<" candidates from ITR were found ..."<<endl;
      
      for(int i=0;i<numcand;i++)
	{
	  cout<<i<<endl;
	  for(int j=0;j<4;j++) 
	    if(itrc[i].SL[j]!=-1) cout<<itrc[i].SL[j]<<":"<<
				    itrc[i].Nav[j]<<endl;
	  else cout<<"*:* "<<endl;
	}
      */
    }

  // search for hits

  

  for(int i=0;i<numcand;i++)
    {
       Rk[0]=itrc[i].xe;Rk[1]=itrc[i].ye;
       Rk[2]=itrc[i].txe;Rk[3]=itrc[i].tye;zprev=itrc[i].ze;
       itrc[i].chi2=0.0;itrc[i].nsl=0;
       zmax=-10000.0;
       /*
	 {
	   cout<<i<<" Initial estimates at z="<<zprev<<endl;
	   cout<<Rk[0]<<" "<<Rk[1]<<" "<<Rk[2]<<" "<<Rk[3]<<endl;
	 }
       */
       for(int w=0;w<4;w++) for(int ww=0;ww<4;ww++) Gk[w][ww]=0.0;
       Gk[0][0]=itrc[i].cve[0];Gk[1][1]=itrc[i].cve[2];
       Gk[0][1]=itrc[i].cve[1];Gk[0][2]=itrc[i].cve[3];
       Gk[1][2]=itrc[i].cve[4];Gk[2][2]=itrc[i].cve[5];
       Gk[0][3]=itrc[i].cve[6];Gk[1][3]=itrc[i].cve[7];
       Gk[2][3]=itrc[i].cve[8];Gk[3][3]=itrc[i].cve[9];
       /*
	 {
	   
	   for(int w=0;w<4;w++)
	     {
	       for(int ww=0;ww<4;ww++) cout<<Gk[w][ww]<<" ";
	       cout<<endl;
	     }
	   
	   for(int w=0;w<15;w++) cout<<itrc[i].rs->cve[w]<<" ";
	   cout<<endl;
	   for(int w=0;w<15;w++) cout<<itrc[i].rs->cvf[w]<<" ";
	   cout<<endl;
	   cout<<itrc[i].rs->nhit<<endl;
	 }
       */
       pinv=itrc[i].rs->pe;

       coeff=(1.0+0.105*0.105*pinv*pinv)*pinv*pinv; // muon mass correction
       l2sqrt=1.0+Rk[2]*Rk[2]+Rk[3]*Rk[3]; 
       l2sqrt=l2sqrt*sqrt(l2sqrt);
	
       q33=sigma_ms*coeff*l2sqrt*(1.0+Rk[2]*Rk[2]); 
       q44=sigma_ms*coeff*l2sqrt*(1.0+Rk[3]*Rk[3]); 
       q34=sigma_ms*coeff*l2sqrt*Rk[2]*Rk[3];
       for(int j=0;j<4;j++)
	{
	  if(itrc[i].SL[j]==-1) continue;
	  if(itrc[i].Nav[j]==0) continue;
	  itrc[i].nsl++;
	  SLid=j;Sec=itrc[i].Nav[j];
	  nGEDE=slg[SLid].ngede[Sec];
	  // cout<<"SL="<<SLid<<" Sec="<<Sec<<" nGEDE="<<nGEDE<<endl;
	  nfound=0;
	  for(int k=0;k<nGEDE;k++)
	    {
	      ArtePointer<GEDE> gede=slg[SLid].s[Sec][k].gede;
	      gz=slg[SLid].s[Sec][k].gz;
	      zmid=0.5*(gz.szmx+gz.szmn);
	      Dz=zmid-zprev;
	      xe=Rk[0]+Rk[2]*Dz;ye=Rk[1]+Rk[3]*Dz;
	      gk[0][0]=Gk[0][0]+Dz*(2*Gk[0][2]+Dz*Gk[2][2]);
	      gk[1][1]=Gk[1][1]+Dz*(2*Gk[1][3]+Dz*Gk[3][3]);
	      gk[0][1]=Gk[0][1]+Dz*(Gk[1][2]+Gk[0][3]+Dz*Gk[2][3]);
	      erru=0.2+3.0*sqrt(gk[0][0]*gz.cos*gz.cos+gk[1][1]*gz.sin*gz.sin-2.0*gz.sin*gz.cos*gk[0][1]);

		{
		  //  cout<<"erru="<<erru<<" zmid="<<zmid<<" zprev="<<zprev<<
		  //  " gk00="<<gk[0][0]<<" gk01="<<gk[0][1]<<" gk11="<<gk[1][1]<<endl;

		}
	      u=xe*gz.cos-ye*gz.sin;
	      v=xe*gz.sin+ye*gz.cos;
	      
	      if((gz.sxmx>u)&&(gz.sxmn<u)&&(gz.symx>v)&&(gz.symn<v))
		{
		  //  if(i==7)
		  //  cout<<"GEDE found Fh="<<gede->hit1<<" Lh="<<gede->hitl<<endl;
		  hitbItf = gede->hit1;hitbIte = gede->hitl;
		  if (distance(hitbItf, hitbIte)+1 == 0) continue;
		  hitbIte++;
		  for (ArteTableIterator<HITB> hitbIt = hitbItf; 
		       hitbIt != hitbIte; ++hitbIt ) 
		    {
		      if((*hitbIt)->z<0.0) continue;
		      u1=(*hitbIt)->x-fabs((*hitbIt)->q);
		      u2=(*hitbIt)->x+fabs((*hitbIt)->q);
		      if((fabs(u1-u)>erru)&&(fabs(u2-u)>erru)) continue;
		      if(i==7000) 
			{
			  //  cout<<"hit found "<<erru<<" GEDE found Fh="
			  //   <<gede->hit1<<" Lh="<<gede->hitl<<endl;
			  // cout<<"u="<<u<<" x="<<(*hitbIt)->x<<
			  //  " q="<<(*hitbIt)->q<<endl;
			}
		      HIT hit;
		      hit.gede_index = gede.index();
		      hit.hitb = *hitbIt;
		      hit.x    = (*hitbIt)->x;
		      hit.q    = fabs((*hitbIt)->q);
		      hit.z    = (*hitbIt)->z;
		      hit.r    = (*hitbIt)->r;
		      hit.mask = 1.0;
		      hit.lr   = 0.0;
		      hf[nfound]=hit;
		      nfound++;
		      thits++;
		    }
		}
	    }
	  //	  	  cout<<nfound<<" hits found in SL "<<SLid<<endl;	 
	  for(int k=0;k<nfound;k++)
	    {
	      Dz=hf[k].z-zprev;
	      xe=Rk[0]+Rk[2]*Dz;ye=Rk[1]+Rk[3]*Dz;
	      gk[0][0]=Gk[0][0]+Dz*(2.0*Gk[0][2]+Dz*Gk[2][2]);
	      gk[0][1]=Gk[0][1]+Dz*(Gk[1][2]+Gk[0][3]+Dz*Gk[2][3]);
	      gk[0][2]=Gk[0][2]+Dz*Gk[2][2];
	      gk[0][3]=Gk[0][3]+Dz*Gk[2][3];
	      gk[1][1]=Gk[1][1]+Dz*(2.0*Gk[1][3]+Dz*Gk[3][3]);
	      gk[1][2]=Gk[1][2]+Dz*Gk[2][3];
	      gk[1][3]=Gk[1][3]+Dz*Gk[3][3];
	      gk[2][3]=Gk[2][3];gk[3][3]=Gk[3][3];gk[2][2]=Gk[2][2];

	      ArtePointer<GEDE> gede=hf[k].hitb->gede;
	      sina=gede->sin*gede->rot[4] - gede->cos*gede->rot[3];
	      cosa=gede->cos*gede->rot[0] - gede->sin*gede->rot[1];
	      u=xe*cosa-ye*sina;
	      sigma2=hf[k].hitb->r*hf[k].hitb->r;
	      tu_k=Rk[2]*cosa-Rk[3]*sina;
	      theta_k=sqrt(1.0+tu_k*tu_k);
	      q=(u-hf[k].x)/theta_k;
	      Z=sigma2+cosa*cosa*gk[0][0]-2.0*sina*cosa*gk[0][1]+sina*sina*gk[1][1];
	      zeta_k=hf[k].q-fabs(q);
	      if((zeta_k*zeta_k/Z)>10.0) hf[k].mask=0;
	      else
		{
		  //		   cout<<"filter ..."<<endl;
		  Rk[0]=xe;Rk[1]=ye;
		  hf[k].mask=1.0;
		  hf[k].lr=(q>0.0)?1.0:-1.0;
		  tu_k=Rk[2]*cosa-Rk[3]*sina;
		  theta_k=1.0/sqrt(1.0+tu_k*tu_k);
		  Du_k=Rk[0]*cosa-Rk[1]*sina-hf[k].x;
		  f_k=Du_k*tu_k*theta_k*theta_k;
		  h[0]= cosa*theta_k;
		  h[1]=-sina*theta_k;
		  h[2]=-h[0]*f_k;
		  h[3]= h[1]*f_k;
		  Kk[0]=h[0]*gk[0][0]+h[1]*gk[0][1]+h[2]*gk[0][2]+h[3]*gk[0][3];
		  Kk[1]=h[0]*gk[0][1]+h[1]*gk[1][1]+h[2]*gk[1][2]+h[3]*gk[1][3];
		  Kk[2]=h[0]*gk[0][2]+h[1]*gk[1][2]+h[2]*gk[2][2]+h[3]*gk[2][3];
		  Kk[3]=h[0]*gk[0][3]+h[1]*gk[1][3]+h[2]*gk[2][3]+h[3]*gk[3][3];
		  Z=sigma2;for(int w=0;w<4;w++) Z+=h[w]*Kk[w];
		  zeta_k=hf[k].q*hf[k].lr-Du_k*theta_k;
		  itrc[i].chi2+=zeta_k*zeta_k/Z;
		  Z=1.0/Z;zeta_k=Z*zeta_k;
		  for(int w=0;w<4;w++)
		    {
		      Rk[w]+=Kk[w]*zeta_k;
		      for(int ww=0;ww<=w;ww++) 
			Gk[ww][w]=gk[ww][w]-Z*Kk[ww]*Kk[w];
		    }
		  zprev=hf[k].z;
		  itrc[i].h[itrc[i].n]=hf[k];
		  itrc[i].n++;
		  if(hf[k].z>zmax)
		    {
		      zmax=hf[k].z;
		      itrc[i].ze=hf[k].z;
		      itrc[i].xe=Rk[0];
		      itrc[i].ye=Rk[1];
		      itrc[i].txe=Rk[2];
		      itrc[i].tye=Rk[3];
		    }
		}
	      Gk[2][2]+=q33;Gk[2][3]+=q34;Gk[3][3]+=q44;
	    }
	}
       //              cout<<"Filter passed ..."<<endl;
       if(itrc[i].n!=0) 
	 {
	   itrc[i].chi2=(itrc[i].rs->chi+itrc[i].chi2)/(itrc[i].rs->nhit+itrc[i].n-4.0);
	 }    
    }
  //   cout<<"Search finished ..."<<endl;

  if(thits)
    for(int i=0;i<numcand;i++)
      {
	float mean=itrc[i].n*1.0/itrc[i].nsl;
	if(mean<2.5) 
	  {
	    //	    if(itrc[i].rs->nhit<6)
	    //  itrc[i].rs->cmp = itrc[i].rs->cmp | Rsegc::bitsign; 
	    // kill ITR fragment 
	    // continue;
	  }
	
	TRACK track;
	pTRACK ptrack=&track;
	HIT buf[200];
	int nhit;
	
	track.vphit.clear();
	track.xf=itrc[i].rs->xf;
	track.yf=itrc[i].rs->yf;
	track.txf=itrc[i].rs->txf;
	track.tyf=itrc[i].rs->tyf;
	track.zf=itrc[i].rs->zf;
	track.pf=track.pe=itrc[i].rs->pf;
	
	nhit=0;
	for (ArteRelationIterator<HITB> hitbIt =itrc[i].rs->hitb.begin() ; 
	     hitbIt != itrc[i].rs->hitb.end(); ++hitbIt ) 
	  {
	    HIT hit;
	    hit.gede_index = (*hitbIt)->gede.index();
	    hit.hitb = *hitbIt;
	    hit.x    = (*hitbIt)->x;
	    hit.q    = 0.0;
	    hit.z    = (*hitbIt)->z;
	    hit.r    = (*hitbIt)->r;
	    hit.mask = 1.0;
	    hit.lr   = 1.0;
	    buf[nhit]=hit;
	    nhit++;
	  }
	for(int k=0;k<itrc[i].n;k++)
	  {
	    buf[nhit]=itrc[i].h[k];
	    nhit++;
	  }
	
	for(int k=0;k<nhit;k++)
	  {
	    track.vphit.push_back(&buf[k]);
	  }
	sort(ptrack->vphit.begin(), ptrack->vphit.end(), OtrHitCompareZ);
	//       OtrResolveLR(ptrack);
	ptrack->pspointmap.clear();
	ptrack->good = 1;
	
	ptrack->ze = itrc[i].ze;
	ptrack->xe = itrc[i].xe;ptrack->ye = itrc[i].ye;
	ptrack->txe=itrc[i].txe;ptrack->tye = itrc[i].tye;
	ptrack->vpcluster.clear();
	ptrack->nonmask = 1;
	
	/*
       for(vector<pHIT>::iterator phitIt = ptrack->vphit.begin();
       phitIt != ptrack->vphit.end(); ++phitIt){
       
       ArtePointer<GEDE> gede = (*phitIt)->hitb->gede;
       sina=gede->sin*gede->rot[4] - gede->cos*gede->rot[3];
       cosa=gede->cos*gede->rot[0] - gede->sin*gede->rot[1];
       cout<<"z="<<(*phitIt)->z<<" x="<<(*phitIt)->x<<" q="<<(*phitIt)->q<<endl;
       cout<<"mask="<<(*phitIt)->mask<<" LR="<<(*phitIt)->lr<<" resol="<<(*phitIt)->r<<endl;
       }
       
       
       cout<<"Before refit "<<endl;
       cout<<" zf="<<ptrack->zf<<" xf="<<ptrack->xf<<" yf="<<ptrack->yf<<" txf="<<ptrack->txf<<" tyf="<<ptrack->tyf<<endl;
       cout<<" ze="<<ptrack->ze<<" xe="<<ptrack->xe<<" ye="<<ptrack->ye<<" txe="<<ptrack->txe<<" tye="<<ptrack->tye<<endl;
	*/
	
	OtrFastDriftRefitTrack(ptrack);
	
	//  cout<<"M="<<ptrack->nonmask<<" P="<<ptrack->prob<<endl;
	
	if(ptrack->nonmask==0) continue;
	if(ptrack->prob<0.05) continue;
	itrc[i].rs->cmp = itrc[i].rs->cmp | Rsegc::bitsign; // kill ITR fragment 
	
	/*
	  cout<<"Refit OK: "<<endl;
	  cout<<" zf="<<ptrack->zf<<" xf="<<ptrack->xf<<" yf="<<ptrack->yf<<" txf="<<ptrack->txf<<" tyf="<<ptrack->tyf<<endl;
	  cout<<" ze="<<ptrack->ze<<" xe="<<ptrack->xe<<" ye="<<ptrack->ye<<" txe="<<ptrack->txe<<" tye="<<ptrack->tye<<endl;
	  cout<<"Prob="<<ptrack->prob<<endl;
	*/
	RSEG r;
	RHIT rh;
	RPNT rp;

	r.xf  = ptrack->xf;  r.yf  = ptrack->yf;  r.zf  = ptrack->zf; 
	r.txf = ptrack->txf; r.tyf = ptrack->tyf;
	r.xe  = ptrack->xe;  r.ye  = ptrack->ye;  r.ze  = ptrack->ze; 
	r.txe = ptrack->txe; r.tye = ptrack->tye;
	
       r.cvf[ 0] = ptrack->cvf[0][0];      r.cvf[ 1] = ptrack->cvf[0][1];
       r.cvf[ 2] = ptrack->cvf[1][1];      r.cvf[ 3] = ptrack->cvf[0][2];
       r.cvf[ 4] = ptrack->cvf[1][2];      r.cvf[ 5] = ptrack->cvf[2][2];
       r.cvf[ 6] = ptrack->cvf[0][3];      r.cvf[ 7] = ptrack->cvf[1][3];
       r.cvf[ 8] = ptrack->cvf[2][3];      r.cvf[ 9] = ptrack->cvf[3][3];
       
       r.cvf[10] = r.cvf[11] = r.cvf[12] = r.cvf[13] = 0.0;
       r.cvf[14] = 100.0;
       
       r.cve[ 0] = ptrack->cve[0][0];      r.cve[ 1] = ptrack->cve[0][1];
       r.cve[ 2] = ptrack->cve[1][1];      r.cve[ 3] = ptrack->cve[0][2];
       r.cve[ 4] = ptrack->cve[1][2];      r.cve[ 5] = ptrack->cve[2][2];
       r.cve[ 6] = ptrack->cve[0][3];      r.cve[ 7] = ptrack->cve[1][3];
       r.cve[ 8] = ptrack->cve[2][3];      r.cve[ 9] = ptrack->cve[3][3];

       r.cve[10] = r.cve[11] = r.cve[12] = r.cve[13] = 0.0;
       r.cve[14] = 100.0;
	
       r.pf = r.pe = ptrack->pf;
	
       r.time = 0.0; r.dtim = 0.0;
       r.chi  = ptrack->chi2;

       r.fit  = 0;
       r.cmp  = Rsegc::patt | Rsegc::msg | Rsegc::wch | Rsegc::bit16;
    
       ArteTable<RSEG>::push_back( r );
       ArtePointer<RSEG> rseg( ArteTable<RSEG>::size() );
	
       int number_of_rhits = 0;
       for (vector<EST>::reverse_iterator estIt = ptrack->vest.rbegin();
	    estIt != ptrack->vest.rend(); ++estIt){
	 createRelation((*estIt).phit->hitb, rseg, &RSEG::hitb );
	 ArtePointer<RHIT> rhit = ArteTable<RHIT>::push_back(rh);
	 createRelation((*estIt).phit->hitb, rhit, &HITB::rhit );

	 if((*estIt).phit->hitb->gede->cmp==2)
	   (*estIt).phit->hitb->z*=-1.0;

	 number_of_rhits++;
	 createRelation( rseg, rhit, &RSEG::rhit );
	 rhit->tx = (*estIt).tx;
	 rhit->ty = (*estIt).ty;
	 rhit->x  = (*estIt).x0;
	 rhit->y  = (*estIt).y0;
	 rhit->p  = rseg->pf;
	 rhit->q  = (*estIt).q;
       }
       
       rseg->nhit = number_of_rhits;
       //       cout<<"Total "<<rseg->nhit<<" in joint segment"<<endl;
       rp.xf  = rseg->xf;  rp.yf  = rseg->yf;  rp.zf = rseg->zf; 
       rp.txf = rseg->txf; rp.tyf = rseg->tyf; rp.pf = rseg->pf;
       for (int i=0; i<15; i++) rp.cvf[i] = rseg->cvf[i];
    
       ArtePointer<RPNT> rpntf = ArteTable<RPNT>::push_back( rp );
       createRelation( rseg, rpntf, &RSEG::rpnt );

       rp.xf  = rseg->xe;  rp.yf  = rseg->ye;  rp.zf = rseg->ze; 
       rp.txf = rseg->txe; rp.tyf = rseg->tye; rp.pf = rseg->pe;
       for (int i=0; i<15; i++) rp.cvf[i] = rseg->cve[i];
       
       ArtePointer<RPNT> rpnte = ArteTable<RPNT>::push_back( rp );
       createRelation( rseg, rpnte, &RSEG::rpnt ); 
    }
  /*
  for(int i=0;i<numcand;i++)
    {
      cout<<"Track "<<i<<" chi2="<<itrc[i].chi2<<endl;
      cout<<"Existing hits:"<<endl;
      int flag=1;
      float momentum;
      for(ArteRelationIterator<HITB> hitbIt=itrc[i].rs->hitb.begin();
	  hitbIt!=itrc[i].rs->hitb.end();++hitbIt)
	{
	  cout<<"z="<<(*hitbIt)->z<<" x="<<(*hitbIt)->x<<" ";
	  ArtePointer<GEDE> gede = (*hitbIt)->gede;
	  float sina=gede->sin*gede->rot[4] - gede->cos*gede->rot[3];
	  if(fabs(sina)<0.03) cout<<"z "; else cout<<"s ";
	  cout<<"MTRA :";
	  for(ArteRelationIterator<MIMP> mimpIt=(*hitbIt)->mimp.begin();
	      mimpIt!=(*hitbIt)->mimp.end();++mimpIt)
	    {
	      cout<<(*mimpIt)->mtra<<" "<<(*mimpIt)->tx<<" "<<(*mimpIt)->ty<<" ";
	      if(flag)
		{
		  flag=0;
		  momentum=sqrt((*mimpIt)->mtra->px*(*mimpIt)->mtra->px+
				(*mimpIt)->mtra->py*(*mimpIt)->mtra->py+
				(*mimpIt)->mtra->pz*(*mimpIt)->mtra->pz);
		}
	    }
	  cout<<endl;
	}
      cout<<"Momentum="<<momentum<<endl;
      cout<<"Found hits:"<<endl;
      for(int k=0;k<itrc[i].n;k++)
	{
	  ArtePointer<HITB> hitb=itrc[i].h[k].hitb;
	  cout<<"z="<<hitb->z<<" x="<<hitb->x<<" ";
	  ArtePointer<GEDE> gede = hitb->gede;
	  float sina=gede->sin*gede->rot[4] - gede->cos*gede->rot[3];
	  if(fabs(sina)<0.03) cout<<"z "; else cout<<"s ";
	  cout<<"MTRA :";
	  for(ArteRelationIterator<MIMP> mimpIt=hitb->mimp.begin();
	      mimpIt!=hitb->mimp.end();++mimpIt)
	    {
	      cout<<(*mimpIt)->mtra<<" "<<(*mimpIt)->tx<<" "<<(*mimpIt)->ty<<" ";
	    }
	  cout<<endl;
	}
   }
  */
}
