// ********************************************************************
//
// source:
//
// type:      source code
//
// created:   28. Jan 1997
//
// author:    Thorsten Glebe
//            HERA-B Collaboration
//            Max-Planck-Institut fuer Kernphysik
//            Saupfercheckweg 1
//            69117 Heidelberg
//            Germany
//            E-mail: T.Glebe@mpi-hd.mpg.de
//
// Description: ARTE constructor for RecoVertex
//
// changes:
// 28 Jan 1997 creation by I. Siccama, C. Borgmeier
// 22 Nov 2000 (TG) added namespace CLUE
// 07 Dez 2000 (TG) added initialization of ClueRelation container
// 15 Dec 2000 (TG) bug fix in constructor
// 12 Jan 2001 (TG) removed obsolete include files, changed constructor
// 11 Feb 2001 (TG) modifed constructor
// 09 Jul 2001 (TG) improved CTOR
//
// ********************************************************************

#include "clue/cluearte.hh"
#include "clue/RecoVertex.hh"

#ifdef NAMESPACE
namespace CLUE {
#endif
  
  RecoVertex::RecoVertex( const ArtePointer<RVER>& rver ) :
    out (),
    Vx  ( rver->xv ),
    Vy  ( rver->yv ),
    Vz  ( rver->zv ),
    Chi2 ( rver->chi2 ),
    Ndf  ( rver->ndgf ),
    Mass ( rver->mass ),
    Ntracks( distance(rver->rrel.begin(), rver->rrel.end()) ),
    flag( 0 ) {
    
    // ARTE scheme of cov-matrix filling: 
    // x | 0       so: cov[0] = xx, cov[2] = yy, cov[4] = zy
    // y | 1 2
    // z | 3 4 5
    //   +--------
    //     x y z
    //
    
    for(unsigned int i=0; i<6; ++i) covMatrix[i] = rver->cv[i];

#ifdef XXX
    // get the Vertex chi2
    unsigned short int nf = 0; 
    for( ArteRelationIterator<RREL> rrit = rver->rrel.begin();
	 rrit != rver->rrel.end(); ++rrit ) {
      const ArtePointer<RREL> rrel = *rrit;

      Chi2 += rrel->chi;
      ++nf;
    }

    Ndf = 3*nf - 3;  // in case of a Kalman filter based fit
#endif
  }

#ifdef NAMESPACE
} // namespace CLUE
#endif
