// ********************************************************************
//
// source:
//
// type:      source code
//
// created:   27. Jan 1997
//
// author:    Thorsten Glebe
//            HERA-B Collaboration
//            Max-Planck-Institut fuer Kernphysik
//            Saupfercheckweg 1
//            69117 Heidelberg
//            Germany
//            E-mail: T.Glebe@mpi-hd.mpg.de
//
// Description: ARTE constructor for Topp
//
// changes:
// 27 Jan 1997 creation by C. Borgmeier
// 22 Nov 2000 (TG) added namespace CLUE
// 15 Dec 2000 (TG) cut_blank() added, cuts blanks at end of topp->name
//
// ********************************************************************

#include <string.h>
#include "clue/cluearte.hh"
#include "clue/Topp.hh"

#ifdef NAMESPACE
namespace CLUE {
#endif

  //==============================================================================
  // copy_blank: stop copy at first blank
  //==============================================================================
  static void copy_blank(char* target, char* source, unsigned int n) {
    std::strncpy( target, source, n-1 );
    target[n-1] = '\0'; // termination for clean cout output
    // cut blanks
    for(unsigned int i=0; i<n; ++i) {
      if(target[i] == ' ') {
	target[i] = '\0';
	break;
      }
    }
  }

  //==============================================================================
  // Constructor
  //==============================================================================
  Topp::Topp( const ArtePointer<TOPP>& topp )
    : Mass( topp->mass ),
      Wid ( topp->wid  ),
      Trbw( topp->trbw ),
      Life( topp->life ),
      Chrg( topp->chrg ),
      Colo( topp->colo ),
      Deca( topp->deca ),
      Lund( topp->lund ),
      Gean( topp->gean ),
      Anti( getIDX<TOPP>( topp->anti ) ) {
    
    copy_blank(Name, topp->name, 13);
  }


  //==============================================================================
  // operator =
  //==============================================================================
  const Topp& Topp::operator=( const ArtePointer<TOPP>& topp ) {

    copy_blank(Name, topp->name, 13);

    Mass = topp->mass;
    Wid  = topp->wid ;
    Trbw = topp->trbw;
    Life = topp->life;
    Chrg = topp->chrg;
    Colo = topp->colo;
    Deca = topp->deca;
    Lund = topp->lund;
    Gean = topp->gean;
    Anti = getIDX<TOPP>( topp->anti );
    
    return *this;
  }

#ifdef NAMESPACE
} // namespace CLUE
#endif
