#ifndef __CLONEMAP_HH
#define __CLONEMAP_HH
// *****************************************************************************
//
// source:
//
// type:      header file
//
// created:   02. Jan 2001
//
// author:    Thorsten Glebe
//            HERA-B Collaboration
//            Max-Planck-Institut fuer Kernphysik
//            Saupfercheckweg 1
//            69117 Heidelberg
//            Germany
//            E-mail: T.Glebe@mpi-hd.mpg.de
//
// Description: helper functions to generate TrackClones relations
//
// changes:
// 02 Jan 2001 (TG) creation
// 26 Apr 2001 (TG) end() optimization
//
// *****************************************************************************

#include <multimap.h>

#include "clue/cluearte.hh"
#include "grelation/ClueRelation.hh"
#include "clue/TrackClones.hh"

//==============================================================================
// trait to configure fillConeMap & createCloneRelation
//==============================================================================
template <class T> 
struct R_trait {
  typedef const int                    First_t;
  typedef const ArtePointer<T>         Second_t;
  typedef multimap<First_t, Second_t>  Cl_map;
  typedef Cl_map::value_type           value_type;
  typedef Cl_map::iterator             Cl_mapit;
  typedef Cl_map::const_iterator       Cl_mapcit;
};


//==============================================================================
// fillCloneMap
//==============================================================================
template <class T>
void fillCloneMap(typename R_trait<T>::Cl_map& clonemap,
		  typename R_trait<T>::First_t key,
		  typename R_trait<T>::Second_t ptr) {
  typedef typename R_trait<T>::value_type value_type;

  clonemap.insert(value_type(key,ptr)); 
}


//==============================================================================
// createCloneRelation
//==============================================================================
template <class T> 
void createCloneRelation(typename R_trait<T>::Cl_map& clonemap,
			 unsigned int id) {
  typedef typename R_trait<T>::First_t   First_t;
  typedef typename R_trait<T>::Cl_mapcit Cl_mapcit;

  const Cl_mapcit cend = clonemap.end();
  for(Cl_mapcit it = clonemap.begin(); it != cend;) {
    const First_t key = it->first;
    //    cout << " key: " << key << endl;

    if(clonemap.count(key) > 1) {
//        cout << " ===========> count: " << clonemap.count(key) 
//  	   << " key: " << key
//  	   << endl;

      const ArtePointer<TrackClones> clones
	= ArteTable<TrackClones>::push_back(TrackClones( id ));
      
      it = clonemap.upper_bound(key);

      // loop over clones
      for(Cl_mapcit it2 = clonemap.lower_bound(key);
	  it2 != it; ++it2){
	createRelation(clones, it2->second);
      }
      //      --it;
    } else {
      ++it;
    }
  } // for
} // end of createCloneRelation

#endif
