#ifndef __SLTABLEFILL_HH
#define __SLTABLEFILL_HH
// *****************************************************************************
//
// source:
//
// type:      header file
//
// created:   03. Jan 2001
//
// author:    Thorsten Glebe
//            HERA-B Collaboration
//            Max-Planck-Institut fuer Kernphysik
//            Saupfercheckweg 1
//            69117 Heidelberg
//            Germany
//            E-mail: T.Glebe@mpi-hd.mpg.de
//
// Description: class to fill a BEE event for Single Lepton triggered
//              data
//
// changes:
// 03 Jan 2001 (TG) creation
// 14 Aug 2001 (TG) added FillMisc() declaration
//
// *****************************************************************************

#include "cluearte/TableFill.hh"

/** Class which defines a way to fill a BEE event for a Single Lepton data
    analysis. The TableFill class is used as a base class in order to reuse the
    filling of MC truth information from the \texttt{TableFill} class.

    @memo Single Lepton data BEE event filling
    @author T. Glebe
 */
//============================================================================
// SlTableFill
//============================================================================
class SlTableFill : public TableFill {
public:
  /** @name --- Constructors --- */
  ///
  SlTableFill();
  ///
  virtual ~SlTableFill();

  /** @name --- Functions --- */
  /// call anl\_evselect() from ARTE/analysis
  virtual bool EventSelect()    const;
  /// use anl\_TrSelect()
  virtual bool FillRecoTrack()  const;
  /// do not save RVER's
  virtual bool FillRecoVertex() const;
  /// fill other information
  virtual bool FillMisc()       const;
}; // class SlTableFill
#endif
