//                              -*- Mode: C++ -*-
//: dgcal2.C
//
// Author: Benedetto Giacobbe
// ECAL group
// HERA-B experiment, Desy
// e-mail: giacobbe@bo.infn.it
//
// Created 21/8/97
// Last Update 25/05/01
//
// 25/05/01 Board efficiencies are read from DB
//
// Digitalization routine for level 1
//    Level  Input  Output  Description
//      2    MCAL   DCAL    Fills the Dcal bank for all channels


#include <stdlib.h>
#include <iostream.h>
#include "care/ECdbmc.hh"
#include "arte/ArteMessage.hh"
#include "care/ECALtypes.hh"
#include "care/ECMCcalib.hh"
#include "care/ECCellID.hh"
#include "care/dgcal.hh" 
#include "care/ECDcalIO.hh"
#include "care/MMBank.hh"
#include "care/CmdCareMCSIM.hh"

#define NUMTOW 203
#define DIM_HEADER 4
#define DIM_TRAILER 0

extern "C" {
  void grndm_(float *, const int&);
}

float dgcal_enecalc(int ipl, int ix, int iy);

void dgcal2_(int *){

  int ix = 0, iy = 0, ndim = 0;

  if( ArtePointer<DCAL>::getLink()()!=0 && ArtePointer<DCAL>::nrow()>0 )
    return;
  MMBank<DCAL>::Book(ArtePointer<ECBO>::nrow()*36+10);
  ECMCcalib  mcCalib;          // Calibrator object
  
  ECDcalIO dcal;


  static CmdCareMcSim cmcsim;
  static ECEffBoardArray effarr;
  static int init = 0;
  static int nrun = cmcsim.Nrun();
  if(init == 0){
    
    if(nrun > 0){
        const int id2 = cmcsim.Nvers();
        const char* dbname = cmcsim.DBname();
        int rc;
        rc = effarr.openDB(dbname,"MCROBoards");
        rc += effarr.readFromDB(nrun,id2,0);
        rc += effarr.closeDB();
//        cout<<"**************** DB return code: "<<rc<<" *****************"
//	    <<endl;
    }
        init = 1;
  }

  float ranvec[226];
  if( nrun>0 ) grndm_(ranvec, 226);


  dcal.start();
  // write bank header
  dcal.putBankHeader(0, 0x2000) ;
  unsigned short sfed_id;

  for(ArteTable<ECBO>::iterator ecbo=ArteTable<ECBO>::begin(); 
	  ecbo!=ArteTable<ECBO>::end(); ++ecbo){

    // Header
    dcal.nextFED();
    sfed_id =  (unsigned short) ecbo->fdid;
    dcal.putFedHeader(72,sfed_id,0x1000,0); // not-sparsified data 

    ECEffBoardEntry* efp = &effarr.entry(sfed_id);

    if(ecbo->stat > 0 && (nrun == 0 || efp->eff1 > ranvec[sfed_id-16] )){
      // int ind = ecbo->numb;
      for (int j=0; j<32; j++){
	int cellidx = ecbo->chan[j]; 
	if(cellidx != -1){
	  
	  CellIdx_t cell = cellidx;
	  ECCellID  cellid(cell->ceid);
	  cellid.getModuleIndex(ix, iy, ndim);
	  float ener = dgcal_enecalc(cellid.module(), ix, iy);
	  
	  ener *= mcCalib[ndim];  // MC Energy correction 

	  int adcCh = dgcalech(cellidx, ener);    // digitalized energy
	  dcal.putNextShort(adcCh);
	} else {
	  dcal.putNextShort(0);
	}
      }
    } else {
      for(int j=0; j<32; j++){
	  dcal.putNextShort(0);
      }
    }
    dcal.FEDdone();
  }


}





