/* 
======================================================================== 
= DGITR                                                                =
=                                                                      =
= This routines translates GEANT output MIMP into DIGB and HITB banks  =
= called from DGEVNT                                                   =
=                                                                      =
=  dgitr.C : MSGC - Monte Carlo Program                                = 
=  ------------------------------------                                =  
=       last update : 27.03.2001                                       = 
=       author      : Siegmund Nowak 08/93                             =
=                                                                      = 
=       new version : Torsten Zeuner    03/2001                        =
=       06.03.2002  : changes to allow for keytable steering           =
=                                                                      =
======================================================================== 
*/ 
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <math.h>
#include "arte/arte.hh"
#include "arte/ArteUtil.h"
#include "arte/ArteMessage.hh"
#include <vector>
#include <algorithm>
#include <iostream.h>
#include <fstream.h>
#include "digi/zgdig.hh"
//#include "itrleda.h"

extern "C" 
{  
   void grndm_(float*,int*);
   void granor_(float *,float *);
   void gpoiss_(float *, int *, int &);
   void rnpssn_(float*, int*, int*);
   int  rnorml_(float*, int*);
   void sortzv_(int[], int[], int*, int*, int*, int*);

}

void ITR_DIGI_initialize();
int  setITRDIGI_from_file(char * infile);
int  setITRDIGIALIGN_from_file(char * infile);

extern int   ITR_DIGI_MASK;

int             ITR_DIGI_EFFRES_FLAG = 0;
int             ITR_DIGI_NOISE_FLAG = 0;
int             ITR_DIGI_ALIGN_FLAG = 0;

float itrdigialign[4000][3];
float itrdiginoise[4000];

static int first_dgitr =0;

//=============================================================================

void dgitr(int &ipl) {     
      int   nmim, ifmimp, nmim_f;
      float x_detector[3], xv[3];

      int i0 = 0, i1 = 1, mi1 = -1;
      int *dg_srt = NULL, *hb_srt = NULL, *ind = 0, *ind1 = 0;

      // max number of mimps to be merged to one hit
      static int nmx_merg = 10; 
      // max number of strips in MSGC
      static int mxwire = 768;


//--- initialisation routines at first access ---------------------------------

      if (first_dgitr == 0) { 
        first_dgitr =1;    
        ITR_DIGI_initialize();
      } 

//--- initialisation ----------------------------------------------------------

//    get MIMP information (first MIMP = ifmimp,number of MIMPs nmim) 

      zgdig ZGDIG(ipl);
      nmim = ZGDIG.Get_nmimp(ifmimp);

//    get information about the given plane 

      ArtePointer<GEDE> gede(ipl);
      float gasgap = gede->zmax - gede->zmin;

//    including noise, poisson distributed
//    noise : number of additional noise hits to be generated  

      int noise=0;
      int erro=1;

      float meannois = itrdiginoise[ipl];
      gpoiss_(&meannois,&noise,erro);

      if ( ITR_DIGI_NOISE_FLAG == 0 ) noise = 0;

      if (nmim+noise<=0) {
        ZGDIG.Set_hit_for_nomimp();    
        return;
      }
      nmim_f = nmim+noise;

//    check wether sorting of hits should be increasing or decreasing

      unsigned int lbigr=0; if(fabs(gede->xmax) <= fabs(gede->xmin)) lbigr=1;

      vector<digx> *dg_itr = new vector<digx>;
      vector<hitx> *hb_itr = new vector<hitx>;

      hitx itrhit;
      digx itrdigit;


//----loop over all  mimps ----------------------------------------------------

//    loop over all  mimps - check whether the mimps are really inside the 
//    detector -  rotate mimp coordinates


      for ( int i = ifmimp; i < ifmimp+nmim+noise; i++) { 

        if ( i < ifmimp+nmim ) {

//        generating hits from MIMP
     
          ArtePointer<MIMP> mimp(i);
      
          float deltaz = ( gede->zfw[0] + gasgap/2 ) - mimp->z;

          if (mimp->gede->cmp != 3 ) {
            nmim_f--;
            continue; 
          }

	  if (ITR_DIGI_ALIGN_FLAG == 0) {

            x_detector[0] = mimp->x + mimp->tx*deltaz;          
            x_detector[1] = mimp->y + mimp->ty*deltaz;
            x_detector[2] = 0.;

//          rotate x into wire orientation
            ZGDIG.Rotate2plane_Z(x_detector,xv);
          }

//        apply artifical misalignments if set in KUMAC

          else {

            float dx = itrdigialign[ipl][0];
            float dz = itrdigialign[ipl][1];
            float da = itrdigialign[ipl][2];
            float cosa = gede->rot[0] * cos(da) - gede->rot[1]*sin(da);
            float sina = gede->rot[1] * cos(da) + gede->rot[0]*sin(da);
            
            x_detector[0] = ( mimp->x + dx ) + mimp->tx * ( deltaz + dz );
            x_detector[1] = mimp->y + mimp->ty * ( deltaz + dz );
            x_detector[2] = 0.;

            xv[0] =  x_detector[0] * cosa - x_detector[1] * sina;
            xv[1] =  x_detector[0] * sina + x_detector[1] * cosa;
            xv[2] =  0.;

          }

//        check whether in effective volume

          if ( ( xv[0] < gede->sxmn ) || ( xv[0] > gede->sxmx ) ) { 
            nmim_f--; 
            continue; 
          }
          if ( ( xv[1] < gede->symn ) || ( xv[1] > gede->symx ) ) { 
            nmim_f--; 
            continue; 
          }

//        chamber efficiency
 
          float rndm[1];
          rndm[0]=0.;
          grndm_(rndm, &i1);

          if (rndm[0]>gede->eff) { nmim_f--; continue; }   

//        check for dead regions
      
          for (ArteTableIterator<OTDR> otdrit; otdrit.valid(); otdrit.next()) {
            ArtePointer<OTDR> otdr(*otdrit);
            ArtePointer<GEDE> geda(otdr->gede);
            if ( geda->cmp != 3 ) continue;
            if ( gede == geda ) 
              if ( xv[0] > otdr->xmin && xv[0] < otdr->xmax ) {
                nmim_f--;
	        continue;
	      }
          }

//        ----------------------           
//         ==> MIMP survived !
//        ----------------------

//        fill digit

          int mtrano = mimp->mtra.index();
          int wireno = int((xv[0]-gede->xfw[0])/gede->pit);

          itrdigit.mimp   = i;
          itrdigit.track  = mtrano;         
          itrdigit.strip  = wireno;
          itrdigit.charge = 10;


//        fill hit

          itrhit.x        = xv[0];
          itrhit.mimp     = i;

   	}

        else {

//        treatment of noise 
      
          int uno = 1;
          float rndnum;
           
          grndm_(&rndnum,&uno);    
          float noise_pos = rndnum * mxwire * gede->pit;
          float noise_x = noise_pos + gede->xfw[0];

          if ( ( noise_x < gede->sxmn ) || ( noise_x > gede->sxmx ) ) { 
            nmim_f--; 
            continue; 
          }

//        check for dead regions
      
          for (ArteTableIterator<OTDR> otdrit; otdrit.valid(); otdrit.next()) {
            ArtePointer<OTDR> otdr(*otdrit);
            ArtePointer<GEDE> geda(otdr->gede);
            if ( geda->cmp != 3 ) continue;
            if ( gede == geda ) 
              if ( noise_x > otdr->xmin && noise_x < otdr->xmax ) {
                nmim_f--;
	        continue;
	      }
          }          

//        fill noise digit
         
          itrdigit.mimp   = 0;
          itrdigit.track  = 0;         
          itrdigit.strip  = int(noise_pos);
          itrdigit.charge = 1;

//        fill noise hit

          itrhit.x    = noise_x;
          itrhit.mimp = 0;
        }

        dg_itr->push_back(itrdigit);
        hb_itr->push_back(itrhit);

      } // end of loop over MIMPs

      
      if (nmim_f <= 0) 
	{  ZGDIG.Set_hit_for_nomimp();
	   return;
	}

// ------------- FILL DIGB ---------------------------------------------------- 

     
      int ndigde = 0, iflag = 0;
      int ndigb;

//    number of digits

      ndigde = dg_itr->size();

      if (ndigde <= 0) {
        ZGDIG.Set_hit_for_nomimp();
        return;
      }        

      dg_srt = new int[ndigde];
      ind    = new int[ndigde];

      for (int i=0; i<ndigde; i++) {
        vector<digx>::iterator mu = dg_itr->begin() + i;
        dg_srt[i] = mu->strip;
      }

      if (lbigr) sortzv_(dg_srt, ind, &ndigde, &mi1, &i0, &i0);
      else       sortzv_(dg_srt, ind, &ndigde, &mi1, &i1, &i0);
      
      
      for (int ih = 0; ih < ndigde; ih++) {
        
        vector<digx>::iterator dg = dg_itr->begin() + ind[ih] -1;

        struct DIGB digb;

        digb.id = 10000*dg->strip + ipl;                     
        digb.q  = dg->charge;
        digb.id = dg->strip;

        ArtePointer<DIGB>::append(digb);

        ndigb = ArteTable<DIGB>::size();
        if (iflag == 0) {  // first digit in this detector
          gede->dig1 = ndigb;
          iflag = 1;
        }

//      Create relation between MIMP and DIGB
        if (dg->mimp) 
          createRelation(ArtePointer<MIMP>(dg->mimp), 
                         ArtePointer<DIGB>(ndigb), &MIMP::digb);
        gede->digl = ndigb;

      }

//    clean up

      delete dg_itr;    dg_itr = NULL;
      delete [] dg_srt; dg_srt = NULL;
      delete [] ind;    ind    = NULL; 


// ------------- FILL HITB ---------------------------------------------------- 

      int nhitde, nhitb;

//    number of hits

      nhitde = hb_itr->size();
      if (nhitde <= 0) {
        ZGDIG.Set_hit_for_nomimp();

//      Clean up
        delete hb_itr;       hb_itr = NULL;
        delete [] hb_srt;    hb_srt = NULL;
        delete [] ind;       ind    = NULL;
        delete [] ind1;      ind1   = NULL;

        return;
      }  

      hb_srt = new int[nhitde];
      ind    = new int[nhitde];
      ind1   = new int[nhitde];

      for (int i=0; i<nhitde; i++) {
        vector<hitx>::iterator mh = hb_itr->begin() + i;
        hb_srt[i] = int(10000*mh->x);
      }

      for (int i=0; i<nhitde; i++) { ind[i] = 0; ind1[i] = 0; }

      if (lbigr) sortzv_(hb_srt, ind, &nhitde, &mi1, &i0, &i0);
      else       sortzv_(hb_srt, ind, &nhitde, &mi1, &i1, &i0);
      
      iflag = 0;

      for (int ih = 0; ih < nhitde; ih++) {
        if (ind1[ih] < 0) continue;
        vector<hitx>::iterator hb = hb_itr->begin() + ind[ih] -1;
        
        struct HITB hitb;
 
        hitb.gede = ipl;        
        hitb.z    = gede->zfw[0] + gasgap/2;
        hitb.y    = 0.;
        hitb.chid = 0;

        ArtePointer<HITB>::append(hitb);

        nhitb = ArteTable<HITB>::size();
        ArtePointer<HITB> hit(nhitb);
        if (iflag == 0) {  // first hit in this detector
          gede->hit1 = nhitb;
          iflag = 1;
        }

//      except for the last hit check if next hits are far enough away,
//      if not define the number of hits to be merged and mark them

        int max = 0;

        if (ih != nhitde-1) {
          for (int j=0; j<nhitde; j++) {
            if (ind[j] < 0) continue;
            vector<hitx>::iterator hbb = hb_itr->begin() + ind[j] -1;       
	    if (j!=ih) {
              if ( fabs(hbb->x-hb->x) < gede->dod ) {
                max++;
                ind1[j] = -ind[ih];  
              }
	    }
	  }
	}

//      first : if last or next mimp far enough : create hitb entry

        if (!max) {

          hit->q = 0.;
          hit->r = gede->acc;

          float f,xx;
          granor_(&f,&xx);
          hit->x = hb->x + gede->acc * f;

          if ( hit->x < gede->sxmn || hit->x > gede->sxmx ) {
            hit->x = hb->x;
          }
          
//        Create relation between MIMP and HITB
        
          if (hb->mimp) 
            createRelation(ArtePointer<MIMP>(hb->mimp), 
                           ArtePointer<HITB>(nhitb), &MIMP::hitb);

          gede->hitl = nhitb;
        }

//      merge mimps to one hitb

        else {
          if ( max+1 > nmx_merg) { 
            // cout << "Too many mimps to be merged to one hit ! " << endl;
            // gede->hitl = nhitb;
            // continue;
          }
          max++;
          ind1[ih] = -ind[ih];
//        merge mimps from i to i+max
          float sum = 0., sum2 = 0.;
          for (int j=0; j<nhitde; j++) {
            if (ind1[j] == -ind[ih]) {
              vector<hitx>::iterator hbb = hb_itr->begin() + ind[j] - 1;
              sum  += hbb->x;
	      sum2 += hbb->x * hbb->x;
              if (hbb->mimp)
                createRelation(ArtePointer<MIMP>(hbb->mimp), 
                               ArtePointer<HITB>(nhitb), &MIMP::hitb);
              gede->hitl = nhitb;
	    }
	  }
          sum  = sum/max;
          sum2 = sum2/max;
          hit->r = sqrt((sum2-sum*sum)*(sum2-sum*sum) + gede->acc*gede->acc);
          float f, xx;
          granor_(&f,&xx);  
          hit->x = sum + hit->r * f;

          if ( hit->x < gede->sxmn || hit->x > gede->sxmx )
            hit->x = sum;          

	}

      }

//    Clean up

      delete hb_itr;       hb_itr = NULL;
      delete [] hb_srt;    hb_srt = NULL;
      delete [] ind;       ind    = NULL;
      delete [] ind1;      ind1   = NULL;

      return;
}


//=============================================================================

 void ITR_DIGI_initialize() {


   // initialize ITR digitization

   string name;
   char ITR_hwc[13]="none";

   cout << "   " << endl;
   cout << " ****************************************************** " << endl;
   cout << " ***** ITR DIGITISATION - start of initialisation ***** " << endl;
   cout << " ***** " << endl;


   // add hardware name to gede->hwc

   for ( ArteTableIterator<GEDE> igede; igede.valid(); igede.next() ) {
     ArtePointer<GEDE> gede(*igede);
 
     char ITR_ori[3] = "++";
     char ITR_nam[4] = "M00";

     if (gede->cmp!=3) continue;

     ITR_nam[1] = gede->gesl->name[2];
     ITR_nam[2] = gede->gesl->name[3];     

     // gede->hwc is filled
     if (gede->sec == 3 || gede->sec == 4) 
       ITR_ori[1] = '-';
     if (gede->sec == 6 || gede->sec == 5) { 
       ITR_ori[0] = '-'; 
       ITR_ori[1] = '-'; 
     }
     if (gede->sec == 8 || gede->sec == 7) 
       ITR_ori[0] = '-';

     char ITR_ang[3] = "+0";
     if (gede->rot[3] > 0.05) ITR_ang[1] = '5';
     if (gede->rot[3] < -0.05) { ITR_ang[0] = '-'; ITR_ang[1] = '5';}
     sprintf(ITR_hwc,"%s%s%d%.2s",ITR_nam,
             ITR_ori,gede->lay,ITR_ang);
     strncpy(gede->hwc,ITR_hwc,12);                                                                      
   }

   // check if special files for the ITR EffRes tables or the ITR Alignment
   // tables are set in the kumac

   char * infile;
   infile = getenv("ITR_DIGI_EFFRES_FILE");

   if (infile) setITRDIGI_from_file(infile);

   char * infile1;
   infile1 = getenv("ITR_DIGI_ALIGN_FILE");

   if (infile1 && ITR_DIGI_ALIGN_FLAG) setITRDIGIALIGN_from_file(infile1);


   // if alignment for digitisation is switched off
   // ITR_DIGI_ALIGN_FLAG == 0 : set all values to zero

   if (ITR_DIGI_ALIGN_FLAG == 0) {
 
     cout << " *** No misalignment applied ! " << endl;

     for ( ArteTableIterator<GEDE> igede; igede.valid(); igede.next() ) {
       ArtePointer<GEDE> gede(*igede);
       if (gede->cmp!=3) continue;
       int gedeind = gede.index();
       itrdigialign[gedeind][0]=0.;
       itrdigialign[gedeind][1]=0.;
       itrdigialign[gedeind][2]=0.;
     }
   }

   // if noise generation switched off
   // ITR_DIGI_NOISE_FLAG == 0 : set values to zero 

   if (ITR_DIGI_NOISE_FLAG == 0) {

     cout << " *** No noise generated !" << endl;
     for ( ArteTableIterator<GEDE> igede; igede.valid(); igede.next() ) {
       ArtePointer<GEDE> gede(*igede);
       if (gede->cmp!=3) continue;
       int gedeind = gede.index();
       itrdiginoise[gedeind]=0.;
     }
   }

   // if ideal detector constants should be used (default)
   // ITR_DIGI_EFFRES_FLAG == 0 : use ideal values

   if (ITR_DIGI_EFFRES_FLAG == 0) {

     cout << " *** Ideal detector settings used :" << endl
          << " *** gede->eff = 0.99" << endl
          << " *** gede->acc = 0.01" << endl
          << " *** gede->dod = 0.08"
          << endl;

     for ( ArteTableIterator<GEDE> igede; igede.valid(); igede.next() ) {
       ArtePointer<GEDE> gede(*igede);
       if (gede->cmp!=3) continue;

       gede->eff = 0.99;
       gede->dod = 0.08;
       gede->acc = 0.01;
     }
   }    

   
   cout << " *****" << endl;
   cout << " ***** ITR DIGITISATION - end of initialisation  ****** " << endl;
   cout << " ****************************************************** " << endl;


 }

//=============================================================================

int setITRDIGI_from_file(char * infile) {

  ifstream in;

  float eff,acc,dod,noise;
  string name;
  char buff[100]; 


  if (infile) in.open(infile, ios::in);
  else {
    return -1;
  }

  cout << " *** setITRDIGI_from_file : " << endl
       << " *** gede->eff, gede->acc, gede->dod " << endl
       << " *** and noise level are read from file !" 
       << endl;

   
  while (in>>name) {
    if (name.at(0)!='M') { in.getline(buff,100); }
    else {

      if ((in>>eff>>acc>>dod>>noise)) {

        for ( ArteTableIterator<GEDE> igede; igede.valid(); igede.next() ) {
          ArtePointer<GEDE> gede(*igede);
          if (gede->cmp!=3) continue;  
          const char * ITR_hwc = name.c_str();            
          int cmpname = strncmp(gede->hwc,ITR_hwc,8);
          if (cmpname!=0) continue;

            // update Gede entries

            gede->eff    = eff;
            gede->acc    = acc;
            gede->dod    = dod;

            int gind = gede.index();            
            itrdiginoise[gind]=noise;
	}          

      }
    }
  }
  in.close();

  return 0;
}

//=============================================================================

int setITRDIGIALIGN_from_file(char * infile) {

  ifstream in;

  float du,dz,da;
  string name;
  char buff[100]; 



  if (infile) in.open(infile, ios::in);
  else {
    
    // misalignment switched on in kumac but no file given
    // as default apply gaussian translations to all chambers 
    // no translations in z and no rotations around z
 
    float f,xx;
 
    cout << " ***  Apply misalignment " << endl;
    cout << " ***  normal distributed translations (sigma = gede->acc)" 
         << endl;
 
    for ( ArteTable<GEDE>::iterator gede = ArteTable<GEDE>::begin();
          gede != ArteTable<GEDE>::end(); ++gede ) {
      if ((*gede)->cmp!=3) continue;
      granor_(&f,&xx);
      int gind = (*gede).index();
      itrdigialign[gind][0] = gede->acc * f; 
      itrdigialign[gind][1] = 0.;
      itrdigialign[gind][2] = 0.;
    }

    return 0;
  }

  cout << " *** setITRDIGIALIGN_from_file : " << endl
       << " *** alignment is read from file !" 
       << endl;

   
  while (in>>name) {
    if (name.at(0)!='M') { in.getline(buff,100); }
    else {

      du=dz=da=0.;

      if ((in>>du>>dz>>da)) {

        for ( ArteTableIterator<GEDE> igede; igede.valid(); igede.next() ) {
          ArtePointer<GEDE> gede(*igede);
          if (gede->cmp!=3) continue;    
          const char * ITR_hwc = name.c_str();            
          int cmpname = strncmp(gede->hwc,ITR_hwc,8);
          if (cmpname!=0) continue;

          int gind = gede.index();
  
          cout << gede->hwc << "    : " << du << endl;

          itrdigialign[gind][0] = du; 
          itrdigialign[gind][1] = dz;
          itrdigialign[gind][2] = da;

	}          

      }
    }
  }

  in.close();

  return 0;
}


/************************************************************************
* ITR KUIP
*
* called from  KUIP
***********************************************************************/

extern "C" void kugeti_( int* );
extern "C" void kugetr_( float* );

extern "C" void itrperf_() {

  int number = 0;
  kugeti_( &number);
  if (number > 0 )
    ITR_DIGI_EFFRES_FLAG = 1;
} 

extern "C" void itrmcnoi_() {

  int number = 0;
  kugeti_( &number);
  if (number > 0)
    ITR_DIGI_NOISE_FLAG = 1;
  if (number <= 0)
    ITR_DIGI_NOISE_FLAG = 0;
} 

extern "C" void itralign_() {

  int number = 0;
  kugeti_( &number);
  if (number > 0)
    ITR_DIGI_ALIGN_FLAG = 1;
  if (number <= 0)
    ITR_DIGI_ALIGN_FLAG = 0;
} 
