
// ******************************************************************
//
// A. Gorisek 18/07/2001
//
// Filling of histograms
//
// 05-11-01: (SKAG) - added histograms split into +X and -X PC
//                  - trSelect used for the track selection 
// ******************************************************************

// standard includes
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <vector>   

// ONLINE includes
#include "rhp.h"
#include "event.h"
#include "grr.h"

// ARTE includes
#include "arte/arte.hh"
#include "arte/RSEG_cmp.hh"
#include "dq/DQ.h"
#include "farm/farm.hh"

// externals
extern int FAGRR;

// prototypes
float get_side ( ArtePointer<RSEG> rseg ) ;
int trSelect( vector< ArtePointer<RTRA> > &rtra ) ;

//////////////// Fill GLAL histograms /////////////////////////////
int DQ_CORR_rec_evnt() {
  // show up
  ARTE_mark( "DQ_CORR_rec_evnt 0", 0 );

  vector< ArtePointer<RTRA> > VecRtra;

  int numtras = trSelect (VecRtra) ;    
 
  for ( int it=0; it<int(VecRtra.size()); it++ ) {
    ArtePointer<RTRA> rtra=VecRtra[it] ;      

    float PCseg[6], VDSseg[6], VDStoPCseg[6] ;
    int isvxdseg=0, ispattseg=0 ;
    float side=-10. ;
    
    for ( ArteRelationIterator<RSEG> rsegit = rtra->rseg.begin() ;
	  rsegit != rtra->rseg.end(); rsegit++ ) {
      ArtePointer<RSEG> rseg(rsegit) ;      

      if ( rseg->cmp & Rsegc::vxd ) { //VDS segment
	VDSseg[0]=rseg->ze ;
	VDSseg[1]=rseg->xe ;
	VDSseg[2]=rseg->ye ;
	VDSseg[3]=rseg->txe ;
	VDSseg[4]=rseg->tye ;
	VDSseg[5]=rseg->pe ;
	isvxdseg++ ;
      } else if ( (rseg->cmp & Rsegc::patt) &&
		  (rseg->zf>680.) &&
		  (rseg->ze<860.) ) { //PC segment
	PCseg[0]=rseg->zf ;
	PCseg[1]=rseg->xf ;
	PCseg[2]=rseg->yf ;
	PCseg[3]=rseg->txf ;
	PCseg[4]=rseg->tyf ;
	PCseg[5]=rseg->pf ;

	side = get_side ( rseg ) ;
	
	ispattseg++ ;
      }

    }

    if ( isvxdseg>1 ) cerr <<  "############# more than 1 VDS seg!!!\n" ;
    if ( ispattseg>1 ) cerr << "############# more than 1 PC seg!!!\n" ;

    if ( ispattseg==1 && isvxdseg==1 ) {
      float zX ;
      zX = ((PCseg[1]-PCseg[3]*PCseg[0])-(VDSseg[1]-VDSseg[3]*VDSseg[0]))/
	(VDSseg[3]-PCseg[3]) ;
      rhp_hf1(DQ_ID(COMP_CORR,LVL_REC,1001),zX,1.);
      rhp_hfill(DQ_ID(COMP_CORR,LVL_REC,2001),VDSseg[3]-PCseg[3],zX,1);
      if ( side>0.9 && side<=1.) {
	rhp_hf1(DQ_ID(COMP_CORR,LVL_REC,1011),zX,1.);
	rhp_hfill(DQ_ID(COMP_CORR,LVL_REC,2011),VDSseg[3]-PCseg[3],zX,1);
      } else if ( side<-0.9 && side>=-1.) {
	rhp_hf1(DQ_ID(COMP_CORR,LVL_REC,1012),zX,1.);
	rhp_hfill(DQ_ID(COMP_CORR,LVL_REC,2012),VDSseg[3]-PCseg[3],zX,1);
      }
      

    } 
  }

  ARTE_mark( "DQ_CORR_rec_evnt 1", 0 );

  return 0; // return error code (0 = okay)
}





float get_side ( ArtePointer<RSEG> rseg ) {
  float rc=10. ;

  int pos=0, neg=0, all=0 ;
  for ( ArteRelationIterator<HITB> ithit=rseg->hitb.begin() ;
	ithit!=rseg->hitb.end() ; ++ithit ) {
    if ( (*ithit)->gede->cmp == 2 ) {
      char chside = ((*ithit)->gede->hwc)[0] ;
      (chside=='+') ? pos++ : neg++ ;
    } else if ( (*ithit)->gede->cmp == 3 ) {
      (((*ithit)->gede->xmin+(*ithit)->gede->xmax)>0.) ? pos++ : neg++ ;
    }
    all++ ;
  }
  if ( all != 0 ) rc = float(pos-neg)/float(all) ;
  else rc = 10. ;
  
  return rc ;
}


