// ********************************************************************
// source:    DQ_ITR_hit_evnt.C
// author:    Sonja Wunderlich
// called by: DQ_evnt()
//
// changes:
// 12/21 (SW   rhp_hbook2: 2-dim Occupancy Histograms    ID:5
// 12/21 (SW   rhp_hbook: Dead Helixes                   ID:15
// 12/21 (SW   rhp_hbook: Dead Readout                   ID:16 
// 12/21 (SW   loops for debugging with hbook
// 12/21 (SW)  rhp_hbook: Hits per event per SL          ID:500
// 01/12/01(SW) lookuptable for the chambers
// 01/25 (SW)  rhp_hbook: Hits per event ITR             ID:55             
// 01/27 (SW)  rhp_hbook: not working/dead helix/readout ID:17
// 01/30 (SW) rhp_hbook2: real occupancy per chamber     ID:6
//  2/5  (SW) uses the code DQ_ITR_my_hbook (D_ITR.h) to book rhp and hbook  
//  2/19 (SW) remove MS14 and MS15 out of occupancy-plot
//  4/13 (SW) include 2-dim Bx-spectrum
//  5/17 (SW) fill hitsSL in a 2 dim plot 
//  5/22 (SW) remove the 20 hitsperevent -plots per station 
//  6/25 (SW) coment out not realy necesary staff :)
// 9/4   (SW) take care to use no DQ when there are Monte Carlo's  
// 1/4/02 (JK) Adaption to new geometry (missing magnet stations), history histograms disabled
// ******************************************************************

#include <iostream.h>
#include <map>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "math.h"
#include "vector.h"  

#include "rhp.h"
#include "hbook.h"

#include "arte/arte.hh"
#include "farm/farm.hh"
#include "dq/DQ.h"
#include "dq/DQ_ITR.h"
#include "ITR_marvin.h"

int DQ_ITR_buildIndex();
int DQ_ITR_isKnown(int);
int DQ_ITR_issKnown(string);
int DQ_ITR_chamberIndex(int);
int DQ_ITR_chambersIndex(string);
int DQ_ITR_indexChamber(int);
int DQ_ITR_indexChambers(int);
int DQ_ITR_SL_Chambers (string);
static map<int, int> chambersInOrder;
static map<string, int> chamberssInOrder;
int numChamSL[10]={16, 24, 32, 40, 72, 88, 104, 136, 160, 184};
int numChamSLhalfs[20]={8, 16, 20, 24, 28, 32, 36, 40, 56, 72, 80, 88, 96, 104, 120, 136, 148, 160, 172, 184};

//static int chamberinit = 0;

static int chamber[184] = {112, 114, 115, 117, 122, 124, 125, 127,                 //01
			     132, 134, 135, 137, 142, 144, 145, 147,               //01
			     212, 214, 215, 217, 222, 224, 225, 227,               //03
			     312, 314, 315, 317, 322, 324, 325, 327,               //05
			     412, 414, 415, 417, 422, 424, 425, 427,               //06
			     512, 514, 515, 517, 522, 524, 525, 527,               //10
			     532, 534, 535, 537, 542, 544, 545, 547,               //10
			     552, 554, 555, 557, 562, 564, 565, 567,               //10
			     572, 574, 575, 577, 582, 584, 585, 587,               //10
			     612, 614, 615, 617, 622, 624, 625, 627,               //11
			     632, 634, 635, 637, 642, 644, 645, 647,               //11
			     712, 714, 715, 717, 722, 724, 725, 727,               //12
			     732, 734, 735, 737, 742, 744, 745, 747,               //12
			     812, 814, 815, 817, 822, 824, 825, 827,               //13
			     832, 834, 835, 837, 842, 844, 845, 847,               //13
			     852, 854, 855, 857, 862, 864, 865, 867,               //13
			     872, 874, 875, 877, 882, 884, 885, 887,               //13
			     912, 914, 915, 917, 922, 924, 925, 927,               //14
			     932, 934, 935, 937, 942, 944, 945, 947,               //14
			     952, 954, 955, 957, 962, 964, 965, 967,               //14
			     1012, 1014, 1015, 1017, 1022, 1024, 1025, 1027,        //15
			     1032, 1034, 1035, 1037, 1042, 1044, 1045, 1047,        //15
			     1052, 1054, 1055, 1057, 1062, 1064, 1065, 1067};       //15

static string chambers[184] = {"M01++1", "M01+-1", "M01--1", "M01-+1", 
				 "M01++2", "M01+-2", "M01--2", "M01-+2", 
				 "M01++3", "M01+-3", "M01--3", "M01-+3", 
				 "M01++4", "M01+-4", "M01--4", "M01-+4", 
				 "M03++1", "M03+-1", "M03--1", "M03-+1", 
				 "M03++2", "M03+-2", "M03--2", "M03-+2", 
				 "M05++1", "M05+-1", "M05--1", "M05-+1", 
				 "M05++2", "M05+-2", "M05--2", "M05-+2", 
				 "M06++1", "M06+-1", "M06--1", "M06-+1", 
				 "M06++2", "M06+-2", "M06--2", "M06-+2", 
				 "M10++1", "M10+-1", "M10--1", "M10-+1", 
				 "M10++2", "M10+-2", "M10--2", "M10-+2", 
				 "M10++3", "M10+-3", "M10--3", "M10-+3", 
				 "M10++4", "M10+-4", "M10--4", "M10-+4", 
				 "M10++5", "M10+-5", "M10--5", "M10-+5", 
				 "M10++6", "M10+-6", "M10--6", "M10-+6", 
				 "M10++7", "M10+-7", "M10--7", "M10-+7", 
				 "M10++8", "M10+-8", "M10--8", "M10-+8", 
				 "M11++1", "M11+-1", "M11--1", "M11-+1", 
				 "M11++2", "M11+-2", "M11--2", "M11-+2", 
				 "M11++3", "M11+-3", "M11--3", "M11-+3", 
				 "M11++4", "M11+-4", "M11--4", "M11-+4", 
				 "M12++1", "M12+-1", "M12--1", "M12-+1", 
				 "M12++2", "M12+-2", "M12--2", "M12-+2", 
				 "M12++3", "M12+-3", "M12--3", "M12-+3", 
				 "M12++4", "M12+-4", "M12--4", "M12-+4", 
				 "M13++1", "M13+-1", "M13--1", "M13-+1", 
				 "M13++2", "M13+-2", "M13--2", "M13-+2", 
				 "M13++3", "M13+-3", "M13--3", "M13-+3", 
				 "M13++4", "M13+-4", "M13--4", "M13-+4", 
				 "M13++5", "M13+-5", "M13--5", "M13-+5", 
				 "M13++6", "M13+-6", "M13--6", "M13-+6", 
				 "M13++7", "M13+-7", "M13--7", "M13-+7", 
				 "M13++8", "M13+-8", "M13--8", "M13-+8", 
				 "M14++1", "M14+-1", "M14--1", "M14-+1", 
				 "M14++2", "M14+-2", "M14--2", "M14-+2", 
				 "M14++3", "M14+-3", "M14--3", "M14-+3", 
				 "M14++4", "M14+-4", "M14--4", "M14-+4", 
				 "M14++5", "M14+-5", "M14--5", "M14-+5", 
				 "M14++6", "M14+-6", "M14--6", "M14-+6", 
				 "M15++1", "M15+-1", "M15--1", "M15-+1", 
				 "M15++2", "M15+-2", "M15--2", "M15-+2", 
				 "M15++3", "M15+-3", "M15--3", "M15-+3", 
				 "M15++4", "M15+-4", "M15--4", "M15-+4", 
				 "M15++5", "M15+-5", "M15--5", "M15-+5", 
				 "M15++6", "M15+-6", "M15--6", "M15-+6",};

static int chamberorder[184] = {0, 4, 8, 12, 
				  1, 5, 9, 13, 
				  3, 7, 11, 15, 
				  2, 6, 10, 14,
				  16, 20, 
				  17, 21, 
				  19, 23, 
				  18, 22, 
				  24, 28, 
				  25, 29, 
				  27, 31, 
				  26, 30,
				  32, 36, 
				  33, 37, 
				  35, 39, 
				  34, 38, 
				  40, 44, 48, 52, 56, 60, 64, 68,
				  41, 45, 49, 53, 57, 61, 65, 69, 
				  43, 47, 51, 55, 59, 63, 67, 71, 
				  42, 46, 50, 54, 58, 62, 66, 70, 
				  72, 76, 80, 84, 
				  73, 77, 81, 85,
				  75, 79, 83, 87, 
				  74, 78, 82, 86, 
				  88, 92, 96, 100, 
				  89, 93, 97, 101,
				  91, 95, 99, 103, 
				  90, 94, 98, 102, 
				  104, 108, 112, 116, 120, 124, 128, 132,
				  105, 109, 113, 117, 121, 125, 129, 133,
				  107, 111, 115, 119, 123, 127, 131, 135,
				  106, 110, 114, 118, 122, 126, 130, 134,
				  136, 140, 144, 148, 152, 156,
				  137, 141, 145, 149, 153, 157,
				  139, 143, 147, 151, 155, 159,
				  138, 142, 146, 150, 154, 158, 
				  160, 164, 168, 172, 176, 180,
				  161, 165, 169, 173, 177, 181,
				  163, 167, 171, 175, 179, 183,
				  162, 166, 170, 174, 178, 182};


int DQ_ITR_hit_init(){
#if defined (DQ_ITR_COMPILE)
  cout<<"IIIIIIIIIIIIIII  DQ_ITR_hit_init.C IIIIIIIIIIIIIIIIII"<<endl;
#endif
  ARTE_mark( "DQ_ITR_hit_init", 0 );

  // check (again) if this is Monte Carlo
  if (ArteTable<MTRA>::size() > 0) {
      ARTE_mark( "DQ_ITR_hit_init 0", 0 );  
      return (GRR_OK);
  }


  DQ_ITR_buildIndex();

#if defined (DQ_ITR_CONTROL) 
  DQ_ITR_my_hbook1( h_ITR_targetrate,LVL_HIT, "target rate", 80, -0.5, 39.5);
#endif

 DQ_ITR_my_hbook2(h_HitsPerEventSL ,LVL_HIT,"ITR: hits/EventTR (station)",
		  k_numHalfStation, -0.5, k_numHalfStation -0.5, 100, 0.2, 20.2);
 // DQ_ITR_my_hbook2(h_HitsPerEventSLMeanTenEvent ,LVL_HIT,"ITR: hits/EventTR (history, station)",
 //		  k_numHalfStation, -0.5, k_numHalfStation - 0.5, 1000, 0, 1000);
 
 // DQ_ITR_my_hbook1(h_hitsoverSNcut,LVL_HIT,"ITR_ hits over sn cut",
 //	  2, 0, 2);
 // DQ_ITR_my_hbook1(h_Signalovernoise,LVL_HIT, "ITR: Singal/Noise",
 //	  60, 0,12);
 
 // DQ_ITR_my_hbook1( h_HitsPerEvent, LVL_HIT, "ITR:Hits /TR ", 
 //	   100, 0.2 , 20.2);

 DQ_ITR_my_hbook1(h_event,LVL_HIT, "Events",1 ,0.,1.);
 
 DQ_ITR_my_hbook2(h_Occupancy_bin, LVL_HIT,"ITR: Occ Helixbin (chamber)*",
		  k_numChamber, -0.5, k_numChamber - 0.5, 
		  k_HelixPerChamber, -0.5, k_HelixPerChamber - 0.5);

 // *************************************************************************
 //    for debugging: to get access to full occupancy histogram, replace 
 //     "k_numStrips/4" with "k_numStrips/1"

 DQ_ITR_my_hbook2(h_Occupancy_raw, LVL_HIT,"ITR: Occ raw (chamber)", 
		  k_numChamber, -0.5, k_numChamber - 0.5, 
		   k_numStrips/4, -0.5, k_numStrips/4 - 0.5);

 // *************************************************************************

 DQ_ITR_my_hbook2(h_BXSpectrum, LVL_HIT,"ITR: BX spectrum (chamber)", 
		  k_numChamber, -0.5, k_numChamber - 0.5, 
		  (float)NBUCKET, -0.5, (float)NBUCKET - 0.5);
 DQ_ITR_my_hbook1(h_BXSpectrumEvent, LVL_HIT,"ITR: BX spectrum eventbased (rand trig)", 
		  (float)NBUCKET, -0.5, (float)NBUCKET - 0.5);
//  DQ_ITR_my_hbook2(h_NoiseMeanTenEvent,LVL_HIT,"ITR: Noise (history, chamber)",
// 		  k_numChamber, -0.5, k_numChamber - 0.5,
// 		   1000, 0, 1000);
 //DQ_ITR_my_hbook2(h_numWorkingCham, LVL_HIT,"ITR: working Chamber (history, station)", 
 //		  k_numHalfStation,-.5, k_numHalfStation- 0.5, 1000, 0,1000); 
 DQ_ITR_my_hbook2(h_NoisePerChamber,LVL_HIT,"ITR: Noise (chamber)",
		  k_numChamber, -0.5, k_numChamber - 0.5,
		  300, 0, 15);

 
 ARTE_mark( "DQ_ITR_hit_init 1", 0 );  
 
 return 0; 
}

int DQ_ITR_buildIndex(){
  
#if defined (DQ_ITR_COMPILE)
    cout<< "DQ_ITR_hit_init.C/fill_chamber_index_map.C"<<endl;
#endif
  

  int cnt;
  for ( cnt = 0; cnt < 184; cnt ++) {
    chambersInOrder[chamber[cnt]] = 1;
    
  }
  for ( cnt = 0; cnt < 184; cnt ++) {
    chamberssInOrder[chambers[cnt]] = 1;

  }

  int idx = 1;
  for (map<int,int>::iterator mapIt =  chambersInOrder.begin();
       mapIt != chambersInOrder.end(); mapIt ++, idx ++) {    
    mapIt->second = idx;
  }
  idx = 0;
  for (map<string,int>::iterator mapsIt =  chamberssInOrder.begin();
       mapsIt != chamberssInOrder.end(); mapsIt ++, idx ++) {


    mapsIt->second = chamberorder[idx];

  }

  return (GRR_OK);
}

int DQ_ITR_isKnown(int id){
  
#if defined (DQ_ITR_COMPILE)
      cout<<"DQ_ITR_hit_init.C/isaknownchamber"<<endl;
#endif
    
    if (chambersInOrder.find(id) == chambersInOrder.end())     //not a known chamber
      return(0);
    return(1);
}

int DQ_ITR_issKnown(string id){
  
#if defined (DQ_ITR_COMPILE)
      cout<<"DQ_ITR_hit_init.C/isaknownchamber"<<endl;
#endif
    
    if (chamberssInOrder.find(id) == chamberssInOrder.end()) {//not a known chamber
      //cout << id << endl;
      return(0);
    }
    return(1);
}

int DQ_ITR_chamberIndex(int chamber){
#if defined (DQ_ITR_COMPILE)
    cout<<"DQ_ITR_hit_init.C/get_cnt_from_chamber"<<endl;
#endif

  int cnt = -1;   

  if (chambersInOrder.find(chamber) != chambersInOrder.end())     //not a known chamber
    cnt = chambersInOrder[chamber]; 
  
  return(cnt);
}
int DQ_ITR_chambersIndex(string chamber){
#if defined (DQ_ITR_COMPILE)
    cout<<"DQ_ITR_hit_init.C/get_cnt_from_chamber"<<endl;
#endif

  int cnt = -1;   

  if (chamberssInOrder.find(chamber) != chambersInOrder.end())     //not a known chamber
    cnt = chamberssInOrder[chamber]; 
  
  
  return(cnt);
}

int DQ_ITR_indexChamber (int idx) {
  int rc = -1;
  
  for (map<int,int>::iterator mapIt =  chambersInOrder.begin();
       mapIt != chambersInOrder.end(); mapIt ++) {
    
    if (mapIt->second == idx) { rc = mapIt->first; break; }
  }

  return (rc);
}

int DQ_ITR_indexChambers (int idx) {
  int SL = -1;
  int i,j;
  for (j=0; j<184; j++){
    if ((idx-1) == chamberorder[j])
      break;
  }
  for (i=0; i<20; i++){
   if (j+1 <= numChamSLhalfs[i]){
      SL = i;
      break;
      
    }
  }
  return (SL);
}

char *chamberName_from_hid (int hid) {
  char slname[10][4] = { "01", "03", "05", "06", 
                                "10", "11", "12", "13", 
                                "14", "15" } ;

  char secname[8][3] = { "", "++", "", "+-", "--", "", "-+", "" };
  static char result[16];

  int sl  = hid / 100;
  int lay = (hid % 100) / 10;
  int sec = hid % 10;
  
  sprintf (result, "MS%2s%2s%1d", slname[sl-1], secname[sec-1], lay);

  return(result);
}

int DQ_ITR_SL_Chambers (string chamber){
  int SL = -1;
  int cnt = -1;
  if (chamberssInOrder.find(chamber) != chamberssInOrder.end()){     //not a known chamber
    cnt = chamberssInOrder[chamber]; 
    for (int i=0; i<10; i++){
      if (cnt < numChamSL[i]){
	SL = i + 1;
	//SL = 2 * i + 1;
	break;
      }
    }
//     if ((cnt % 4 == 0 ) || (cnt % 4 == 1))
//       SL++;
    // cout << "chamber id : " << chamber << " chamber nr : " << cnt << " numChamSL[i] : " << numChamSL[check] << " SL number : " << SL << endl;
  }
  return(SL);
}
