
// ******************************************************************
//
// D. Samtleben 10 Aug 1999
//
// Filling of histograms
//
// changes:
// 27 Oct 1999 (AG) use defines from DQ.h
//  1 Nov 1999 (AG) get rid of dq_hid.h
//  4 Nov 1999 (DD) beta=1 radius histogram 
//  5 Nov 1999 (AG) some polishing
// 16 Dec 1999 (RP) likelihood histograms DQ_RICH_rec_pid()
// 30 Mar 2000 (DD) fading histos  
// 11 May 2000 (RP) histograms redefined
// 25 May 2000 (DD) event efficiency monitoring, r2p2 particle id
// 16 Jun 2000 (AG) two new hostograms added sigma(tx/ty) vs momen
// 29 Jun 2000 (DD) DQ_RICH_r2p2: check for vds/otr using hsi/hou
// 10 Jul 2000 (RP) histograms for PID redifined
// 22 Avg 2000 (RP) copy likelihoods from parallel tables 
// 28 Nov 2000 (RP) histograms for track errors redefined
// 21 Dec 2000 (RP) bug removal
// 13 Apr 2001 (RP) some histograms removed
//  8 Apr 2002 (MS) filling of histograms for Rise moved to separate routine
//                  filling of histograms for Riter also called here
//                  ARTE_mark() added
//                  polishing
// ******************************************************************

// standard includes
#include <stdlib.h>
#include <stdio.h>
#include <math.h>

// ONLINE includes
#include "rhp.h"
#include "event.h"
#include "grr.h"

// ARTE includes
#include "arte/arte.hh"
#include "arte/RSEG_cmp.hh"
#include "dq/DQ.h"
#include "dq/DQ_RICH.h"
#include "farm/farm.hh"
#include "rich/rich.h"
#include "rise/RingSearchARTE.h"

// externals
extern int FAGRR;

// prototypes
int RISEhfill();
int DQ_RICH_rec_beta1( ArtePointer<RSEG> rich );
int DQ_RICH_r2p2();
int DQ_RICH_rec_pid();
int DQ_RICH_rec_pid_rpnt();
int  copyRTRItoRTRA( int module );
extern "C"  void riter_hfill_();

//////////////// Fill RICH DQ histograms /////////////////////////////
int DQ_RICH_rec_evnt() {

  ARTE_mark( "DQ_RICH_rec_evnt 0", 0 );

  RISEhfill();
  copyRTRItoRTRA(1);
  riter_hfill_();

  ARTE_mark( "DQ_RICH_rec_evnt 1", 0 );
  return 0; // return error code (0 = okay)
}


int RISEhfill() {
  
  int nrRICHrings=0; 
  for(ArteTable<RSEG>::iterator irseg=ArteTable<RSEG>::begin();
      irseg!=ArteTable<RSEG>::end(); ++irseg) {
    if ((*irseg)->cmp!=Rsegc::rich) continue;
    ArtePointer<RSEG> rich=(*irseg);
    nrRICHrings++;
    
    float tx=rich->txf;
    float ty=rich->tyf;
    rhp_hfill(DQ_ID(COMP_RICH, LVL_REC, 201),tx,ty,1);
    rhp_hf1(DQ_ID(COMP_RICH, LVL_REC, 202),rich->time,1);
    rhp_mult(DQ_ID(COMP_RICH, LVL_REC,2002),.9999);
    rhp_hf1(DQ_ID(COMP_RICH, LVL_REC,2002),rich->time,1);
    if (!DQ_RICH_rec_beta1(rich)) {
      rhp_hf1(DQ_ID(COMP_RICH, LVL_REC, 203),rich->time,1);
      rhp_mult(DQ_ID(COMP_RICH, LVL_REC,2003),.9999);
      rhp_hf1(DQ_ID(COMP_RICH, LVL_REC,2003),rich->time,1);
    }
  }
  rhp_hf1(DQ_ID(COMP_RICH,LVL_REC, 200),nrRICHrings,1);
  
  // 3=too low photon yield
  // 2=too high photon yield
  // 1=no rings but normal photon occupancy
  // 0=rings ok
  if (ArtePointer<EVHI>(5)->hit<25)
    rhp_hf1(DQ_ID(COMP_RICH, LVL_REC, 2004),3.,1.);
  else if (ArtePointer<EVHI>(5)->hit>ArtePointer<RITH>(1)->par[3])
    rhp_hf1(DQ_ID(COMP_RICH, LVL_REC, 2004),2.,1.);
  else if (nrRICHrings==0) 
    rhp_hf1(DQ_ID(COMP_RICH, LVL_REC, 2004),1.,1.);
  else
    rhp_hf1(DQ_ID(COMP_RICH, LVL_REC, 2004),0.,1.);
  
  DQ_RICH_r2p2();
  return 0;
}  


//////////////  Select RICH Beta=1 w/ ECAL ////////////////////////
int DQ_RICH_rec_beta1(ArtePointer<RSEG> rich) {
  static float max_dty=3e-3;  // angle match
  static float min_esum=8;    // minimum cluster energy
  static float max_p=200;     // maximum p
  static float min_p=10;      // minimum p
  for (ArteTable<RCCL>::iterator irccl=ArteTable<RCCL>::begin();
       irccl!=ArteTable<RCCL>::end(); ++irccl) {
    if ((*irccl)->esum<min_esum) continue;
    if (fabs((*irccl)->ty - rich->tyf)>max_dty) continue;
    float p=288/fabs((*irccl)->x - rich->txf*(*irccl)->z);
    if (p<min_p) continue;
    if (p>max_p) continue;
    return 0; // cluster found
  }
  return 1; // cluster not found
}

int DQ_RICH_r2p2() {

  if (!RICH_Search) RICH_Search=new RingSearchARTE;
  for(ArteTable<RTRA>::iterator irtra=ArteTable<RTRA>::begin();
      irtra!=ArteTable<RTRA>::end(); ++irtra) {
    if ((*irtra)->fit!=0) continue;
    if (fabs((*irtra)->pf)>.15) continue;
    if ((*irtra)->hsi<5)  continue;
    if ((*irtra)->hou+(*irtra)->hin<15) continue;
    int richf=0;
    ArtePointer<RSEG> rich;
    for (ArteRelationIterator<RSEG> irseg=(*irtra)->rseg.begin();
	 irseg!=(*irtra)->rseg.end(); ++irseg) {
      if ((*irseg)->cmp==Rsegc::rich)                    {richf=1; rich=(*irseg); break;}
    }
    if (!richf) continue;
    if (rich->ye>0) if (rich->ze/sqrt(rich->ye)<5) continue;

    float rr=rich->time*rich->time;
    float ip=(*irtra)->pf;
    float  p=1/fabs(ip);
    rhp_hfill(DQ_ID(COMP_RICH, LVL_REC, 2005),ip*ip,rr,1);
    RingSearchTrack::ip=ip;
    RICH_Search->makeR2P2lines();
    if (p<8)  rhp_hf1(DQ_ID(COMP_RICH, LVL_REC, 2006),rr-RICH_Search->GetR2P2rad2(0),1);
    if (p<30) rhp_hf1(DQ_ID(COMP_RICH, LVL_REC, 2007),rr-RICH_Search->GetR2P2rad2(2),1);
    if (p<35) rhp_hf1(DQ_ID(COMP_RICH, LVL_REC, 2008),rr-RICH_Search->GetR2P2rad2(3),1);
    if (p<50) rhp_hf1(DQ_ID(COMP_RICH, LVL_REC, 2009),rr-RICH_Search->GetR2P2rad2(4),1);
    
    if (p<8 && fabs(rr-RICH_Search->GetR2P2rad2(0))<1.0e-4) rhp_hf1(DQ_ID(COMP_RICH, LVL_REC, 2010),0,1);
    if (p<30&& fabs(rr-RICH_Search->GetR2P2rad2(2))<1.0e-4) rhp_hf1(DQ_ID(COMP_RICH, LVL_REC, 2010),1,1);
    if (p<35&& fabs(rr-RICH_Search->GetR2P2rad2(3))<1.5e-4) rhp_hf1(DQ_ID(COMP_RICH, LVL_REC, 2010),2,1);
    if (p<50&& fabs(rr-RICH_Search->GetR2P2rad2(4))<2.0e-4) rhp_hf1(DQ_ID(COMP_RICH, LVL_REC, 2010),3,1);    

  }
  return 0;
}





