
// ******************************************************************
//
// M. Braeuer/ D. Samtleben
//
// Declaration of Histograms and Initialization
// Histogram ID:     32 bits
//
// changes:
// 28 Oct 1999 (AG) use defines from DQ.h
//  1 Nov 1999 (AG) get rid of dq_hid.h
// 19 Nov 1999 (AG) show up
// 23 Feb 2000 (MB) Macro bug fixed
// 16 May 2000 (GS) Vertexing histograms added
// 29 May 2000 (GS) Target pictures added
// 30 May 2000 (GS) Profile Hist for residuals added (private use only)
// 28 Jun 2000 (GS) Signal / Noise histograms added
//  6 Jul 2000 (AG) polished printout
// 19 Jul 2000 (GS) Incorrect histo id (203) fixed
//  5 Oct 2000 (GS) Unbiased residual histograms added
// 28 Nov 2000 (GS) Arte_mark included
//  1 Dec 2000 (SM) Several histograms for primary vertices added
//  6 Dec 2000 (GS) Some histograms commented out
//  8 Dec 2000 (GS) Track/evnt histo duplicated for mb and slt events
// 11 Dec 2000 (SM) Vertex/evnt histo duplicated for mb and slt events
// 14 Dec 2000 (GS) Some histogram titles and ranges modified
// 22 Dec 2000 (GS) Histograms for Martin compacted in 3 2-D histograms
// 16 Jan 2001 (GS) Fixed binning of Martins 3 2-D Histograms
// 18 Jan 2001 (SM) Histograms added for the J/psi vertices
// 19 Jan 2001 (SM) Add J/psi-wire
// 13 Jun 2001 (GS) Angular coverage histo back in
//  3 Mar 2002 (GS) Modified to comply to DQ histo naming conventions
// ******************************************************************

// standard includes
#include <iostream.h>
#include <stdlib.h>
#include <stdio.h>

// ONLINE includes
#include "rhp.h"

// Arte include
#include "arte/arte.hh"
#include "farm/farm.hh"
#include "dq/DQ.h"
#include "dq/DQ_SVD.h"
#include "cats/hbook.h"

//
// function code
//
int
DQ_SVD_rec_init()
{
#define VDS_MAXPLANES 128
#define VDS_plane_l  -0.5  // lower bound for plane
#define VDS_plane_h 127.5  // upper bound for plane

char title[100];

  // show up
  ARTE_mark( "DQ_SVD_rec_init 0", 0 ); 
  cout << "DQ_VDS_rec_init" << endl;
  
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,199),"VDS tracks/event*rate mb",
	     200, -0.5, 99.5 );
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,200),"VDS tracks/event*rate slt",
	     200, -0.5, 99.5 );

  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,201),"VDS Hits(tracks)/rate vs plane *",
	     VDS_MAXPLANES,VDS_plane_l,VDS_plane_h);
  rhp_hbook2(DQ_ID(COMP_SVD, LVL_REC,202),"VDS occupancy(tracks)/rate *",
	     VDS_MAXPLANES,VDS_plane_l,VDS_plane_h, 128,0.,1280.);
//  rhp_hbook2(DQ_ID(COMP_SVD, LVL_REC,203),"VDS residuals",
//	     VDS_MAXPLANES,VDS_plane_l,VDS_plane_h,500,-1490,1490);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,204),"VDS total residual",
	     500,-1490,1490);
  rhp_hbook2(DQ_ID(COMP_SVD, LVL_REC,205),"VDS Angular coverage (tracks) *",
	     200,-249.9,249.9, 200,-249.9,249.9);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,206),"VDS hits/track slt",
	     30,-.5, 29.5);

// Unbiased residuals
  rhp_hbook2(DQ_ID(COMP_SVD, LVL_REC,207),"VDS unbiased residuals",
	     VDS_MAXPLANES,VDS_plane_l,VDS_plane_h,1000,-2990,2990);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,208),"VDS total unbiased residual",
	     1000,-2990,2990);

//  for (int i=0; i<128; i++ ) {
//   sprintf(title, "VDS unbias residual Side %d", i);
//    rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,230+i),title,
//	     500,-1490,1490);
//  }

// Profile histogram for residuals 
  hbprof(DQ_ID(COMP_SVD, LVL_REC,209),"VDS residuals",
             VDS_MAXPLANES,VDS_plane_l,VDS_plane_h,-149,149, "");
  hbprof(DQ_ID(COMP_SVD, LVL_REC,210),"VDS unbiased residuals",
             VDS_MAXPLANES,VDS_plane_l,VDS_plane_h,-149,149, "");

// Target spot histograms
//  rhp_hbook2(DQ_ID(COMP_SVD, LVL_REC,211),"VDS Target spot all tracks",200,-1., 1., 200, -1.,1.);
//  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,212),"VDS Target profile in x",200,-1, 1.);
//  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,213),"VDS Target profile in y",200,-1, 1.);  
//  rhp_hbook2(DQ_ID(COMP_SVD, LVL_REC,214),"VDS Sect 1 tracks only",200,-1., 1., 200, -1.,1.);
//  rhp_hbook2(DQ_ID(COMP_SVD, LVL_REC,215),"VDS Sect 2 tracks only",200,-1., 1., 200, -1.,1.);
//  rhp_hbook2(DQ_ID(COMP_SVD, LVL_REC,216),"VDS Sect 3 tracks only",200,-1., 1., 200, -1.,1.);
//  rhp_hbook2(DQ_ID(COMP_SVD, LVL_REC,217),"VDS Sect 4 tracks only",200,-1., 1., 200, -1.,1.);

// Signal / Noise histograms
  rhp_hbook2(DQ_ID(COMP_SVD, LVL_REC,220),"VDS accumulated S/N",
		VDS_MAXPLANES,VDS_plane_l,VDS_plane_h, 128,0.,1280.);
  rhp_hbook2(DQ_ID(COMP_SVD, LVL_REC,221),"VDS accumulated Charge",
		VDS_MAXPLANES,VDS_plane_l,VDS_plane_h, 128,0.,1280.);
  rhp_hbook2(DQ_ID(COMP_SVD, LVL_REC,222),"VDS accumulated Noise",
		VDS_MAXPLANES,VDS_plane_l,VDS_plane_h, 128,0.,1280.);


// Vertexing Histograms
//======================================================================
//      now GEDE-independent booking
//======================================================================

  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,297),"VDS vertex/event mb" ,20, -0.5,19.5);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,298),"VDS vertex/event slt",20, -0.5,19.5);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,299),"VDS vertex/event*rate mb",100, 0.,4.);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,300),"VDS vertex/event*rate slt",100, 0.,4.);
  // OUT: they are redundant --> see number 309
  //rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,301),"VDS vertex/ev wire 1", 20, -0.5,19.5);
  //rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,302),"VDS vertex/ev wire 2", 20, -0.5,19.5);
  //rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,303),"VDS vertex/ev wire 3", 20, -0.5,19.5);
  //rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,304),"VDS vertex/ev wire 4", 20, -0.5,19.5);
  //rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,305),"VDS vertex/ev wire 5", 20, -0.5,19.5);
  //rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,306),"VDS vertex/ev wire 6", 20, -0.5,19.5);
  //rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,307),"VDS vertex/ev wire 7", 20, -0.5,19.5);
  //rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,308),"VDS vertex/ev wire 8", 20, -0.5,19.5);
  rhp_hbook2(DQ_ID(COMP_SVD, LVL_REC,309),"VDS vertex/event vs wire num",
	     9,-0.5,8.5,20, -0.5,19.5);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,340),"VDS tracks/vertex if 1 vtx mb", 50, -0.5,49.5);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,341),"VDS tracks/vertex if 1 vtx slt", 50, -0.5,49.5);

  // SMA: new histograms
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,311),"VDS tracks/vtx wire 1, 1 vtx", 50, -0.5,49.5);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,312),"VDS tracks/vtx wire 2, 1 vtx", 50, -0.5,49.5);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,313),"VDS tracks/vtx wire 3, 1 vtx", 50, -0.5,49.5);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,314),"VDS tracks/vtx wire 4, 1 vtx", 50, -0.5,49.5);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,315),"VDS tracks/vtx wire 5, 1 vtx", 50, -0.5,49.5);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,316),"VDS tracks/vtx wire 6, 1 vtx", 50, -0.5,49.5);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,317),"VDS tracks/vtx wire 7, 1 vtx", 50, -0.5,49.5);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,318),"VDS tracks/vtx wire 8, 1 vtx", 50, -0.5,49.5);

  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,319),"VDS vertex chi2", 100, 0.,20.);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,320),"VDS vertex prob", 100, 0.,1.);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,321),"VDS vertex x", 200, -.2,.8);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,322),"VDS vertex y", 200, -.4,.6);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,323),"VDS vertex z", 200, -6.,0.);
  rhp_hbook2(DQ_ID(COMP_SVD, LVL_REC,324),"VDS vertex dy vs dx", 100, 0.,.06, 100, 0.,.06);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,325),"VDS vertex dz", 100, 0.,.12);

  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,326),"VDS frac tracks used in 1 vertex mb", 100, 0.,1.);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,327),"VDS frac tracks used in 1 vertex slt", 100, 0.,1.);

  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,328),"VDS target rate", 200, 0.,50.);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,329),"VDS GTAR used", 10, -0.5,9.5);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,330),"VDS RTAR found", 10, -0.5,9.5);

  // Histograms for Martin
  rhp_hbook2(DQ_ID(COMP_SVD, LVL_REC,500),"VDS vtx wire 1-8 x-y",
		16,0.5,16.5, 200, -.4, .8);
  rhp_hbook2(DQ_ID(COMP_SVD, LVL_REC,501),"VDS vtx wire 1-8 z",
		8,0.5,8.5, 200, -6.0, 0.);
  rhp_hbook2(DQ_ID(COMP_SVD, LVL_REC,502),"VDS vtx wire 1-8 dx-dy-dz",
		24,0.5,24.5, 100, 0., .12);

  // Histograms for the Jpsi->mumu
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,600),"Jpsi type", 4, 0.,4000.);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,601),"Jpsi mumu mass", 125, 2.,6.);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,602),"Jpsi mumu x",200, -.2,.8 );
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,603),"Jpsi mumu y",200, -.4,.6 );
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,604),"Jpsi mumu z",200, -6.,0.);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,605),"Jpsi mumu prob 1c",100, 0.,1.);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,606),"Jpsi mumu z, prob.ge.0.05",200, -6.,0.);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,607),"Jpsi mumu deltaz",200, -3.,3.);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,608),"Jpsi mumu deltaz sigma",200, -10.,10.);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,611),"Jpsi ee mass", 240, 0.,12.);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,612),"Jpsi ee x",200, -.2,.8 );
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,613),"Jpsi ee y",200, -.4,.6 );
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,614),"Jpsi ee z",200, -6.,0.);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,615),"Jpsi ee prob 1c",100, 0.,1.);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,616),"Jpsi ee z, prob.ge.0.05",200, -6.,0.);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,617),"Jpsi ee deltaz",200, -3.,3.);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,618),"Jpsi ee deltaz sigma",200, -10.,10.);

/*
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,500),"VDS vertex wire 1 x", 200, -.2,.8);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,501),"VDS vertex wire 1 y", 200, -.4,.6);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,502),"VDS vertex wire 1 z", 200, -6.,0.);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,503),"VDS vertex wire 1 dx", 100, 0.,.06);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,504),"VDS vertex wire 1 dy", 100, 0.,.06);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,505),"VDS vertex wire 1 dz", 100, 0.,.12);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,510),"VDS vertex wire 2 x", 200, -.2,.8);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,511),"VDS vertex wire 2 y", 200, -.4,.6);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,512),"VDS vertex wire 2 z", 200, -6.,0.);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,513),"VDS vertex wire 2 dx", 100, 0.,.06);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,514),"VDS vertex wire 2 dy", 100, 0.,.06);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,515),"VDS vertex wire 2 dz", 100, 0.,.12);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,520),"VDS vertex wire 3 x", 200, -.2,.8);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,521),"VDS vertex wire 3 y", 200, -.4,.6);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,522),"VDS vertex wire 3 z", 200, -6.,0.);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,523),"VDS vertex wire 3 dx", 100, 0.,.06);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,524),"VDS vertex wire 3 dy", 100, 0.,.06);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,525),"VDS vertex wire 3 dz", 100, 0.,.12);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,530),"VDS vertex wire 4 x", 200, -.2,.8);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,531),"VDS vertex wire 4 y", 200, -.4,.6);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,532),"VDS vertex wire 4 z", 200, -6.,0.);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,533),"VDS vertex wire 4 dx", 100, 0.,.06);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,534),"VDS vertex wire 4 dy", 100, 0.,.06);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,535),"VDS vertex wire 4 dz", 100, 0.,.12);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,540),"VDS vertex wire 5 x", 200, -.2,.8);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,541),"VDS vertex wire 5 y", 200, -.4,.6);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,542),"VDS vertex wire 5 z", 200, -6.,0.);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,543),"VDS vertex wire 5 dx", 100, 0.,.06);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,544),"VDS vertex wire 5 dy", 100, 0.,.06);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,545),"VDS vertex wire 5 dz", 100, 0.,.12);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,550),"VDS vertex wire 6 x", 200, -.2,.8);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,551),"VDS vertex wire 6 y", 200, -.4,.6);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,552),"VDS vertex wire 6 z", 200, -6.,0.);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,553),"VDS vertex wire 6 dx", 100, 0.,.06);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,554),"VDS vertex wire 6 dy", 100, 0.,.06);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,555),"VDS vertex wire 6 dz", 100, 0.,.12);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,560),"VDS vertex wire 7 x", 200, -.2,.8);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,561),"VDS vertex wire 7 y", 200, -.4,.6);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,562),"VDS vertex wire 7 z", 200, -6.,0.);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,563),"VDS vertex wire 7 dx", 100, 0.,.06);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,564),"VDS vertex wire 7 dy", 100, 0.,.06);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,565),"VDS vertex wire 7 dz", 100, 0.,.12);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,570),"VDS vertex wire 8 x", 200, -.2,.8);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,571),"VDS vertex wire 8 y", 200, -.4,.6);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,572),"VDS vertex wire 8 z", 200, -6.,0.);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,573),"VDS vertex wire 8 dx", 100, 0.,.06);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,574),"VDS vertex wire 8 dy", 100, 0.,.06);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,575),"VDS vertex wire 8 dz", 100, 0.,.12);
  // end histograms for Martin
*/
  /*
    ATTENTION: here the histogram numbers do not match any more
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,330),"VDS GTAR-RTAR x, wire 1", 200, -.1, .1);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,331),"VDS GTAR-RTAR y, wire 1", 200, -.1, .1);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,332),"VDS GTAR-RTAR z, wire 1", 200, -.1, .1);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,340),"VDS GTAR-RTAR x, wire 2", 200, -.1, .1);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,341),"VDS GTAR-RTAR y, wire 2", 200, -.1, .1);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,342),"VDS GTAR-RTAR z, wire 2", 200, -.1, .1);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,350),"VDS GTAR-RTAR x, wire 3", 200, -.1, .1);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,351),"VDS GTAR-RTAR y, wire 3", 200, -.1, .1);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,352),"VDS GTAR-RTAR z, wire 3", 200, -.1, .1);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,360),"VDS GTAR-RTAR x, wire 4", 200, -.1, .1);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,361),"VDS GTAR-RTAR y, wire 4", 200, -.1, .1);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,362),"VDS GTAR-RTAR z, wire 4", 200, -.1, .1);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,370),"VDS GTAR-RTAR x, wire 5", 200, -.1, .1);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,371),"VDS GTAR-RTAR y, wire 5", 200, -.1, .1);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,372),"VDS GTAR-RTAR z, wire 5", 200, -.1, .1);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,380),"VDS GTAR-RTAR x, wire 6", 200, -.1, .1);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,381),"VDS GTAR-RTAR y, wire 6", 200, -.1, .1);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,382),"VDS GTAR-RTAR z, wire 6", 200, -.1, .1);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,390),"VDS GTAR-RTAR x, wire 7", 200, -.1, .1);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,391),"VDS GTAR-RTAR y, wire 7", 200, -.1, .1);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,392),"VDS GTAR-RTAR z, wire 7", 200, -.1, .1);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,400),"VDS GTAR-RTAR x, wire 8", 200, -.1, .1);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,401),"VDS GTAR-RTAR y, wire 8", 200, -.1, .1);
  rhp_hbook1(DQ_ID(COMP_SVD, LVL_REC,402),"VDS GTAR-RTAR z, wire 8", 200, -.1, .1);
  */

  // show up
  ARTE_mark( "DQ_SVD_rec_init 1", 0 ); 

  return 0; // return error code (0 = okay)
}
