
// *****************************************************************************
//
// source:    faCheckRhp.C
//
// type:      source code
//
// created:   30 Jun 2000
//	
// author:    Dr. Andreas Gellrich
//            HERA-B Collaboration
//            Humboldt-University Berlin
//            Physics Department
//            Invalidenstr. 110
//            10115 Berlin
//            Germany
//            E-mail: Andreas.Gellrich@desy.de
//
// symbol:    faCheckRhp
//
// called by: anywhere
//
// purpose:   check for rhp dispatch messages if rpm/rhp is initialized
//
// hints:     uses global flag RPMINIT
//
// changes:
// 30 Jun 2000 (AG) creation
// 11 Jul 2000 (AG) use RHPPUB
// 12 Jul 2000 (AG) still use rhp_dispatch
// 13 Jul 2000 (AG) reset histograms after publishing
// 14 Jul 2000 (AG) improved printout
// 27 Jul 2000 (AG) corrected bug
// 07 Aug 2000 (AG) improved printout
// 09 Aug 2000 (AG) introduced print flag
// 15 Sep 2000 (AG) improved printout; increased number of histograms
// 13 Nov 2001 (JH) use dynamic allocation for hid
// 16 Nov 2001 (JH) publish and reset histograms after CnA key or run Nr get changed 
//  4 Mar 2002 (JH) publish only when RHPPUB>0
//
// *****************************************************************************

// standard includes
#include <iostream.h>

// ONLINE includes
#include "rpm.h"
#include "rhp.h"
#include "grr.h"

// ARTE includes
#include "arte/ArteFlag.h"
#include "farm/farm.hh"

// externals
extern int  RPMINIT;   // rpm/rhp initialized
extern int  RHPPUB;    // use rhp_publish
extern int  FADEB;     // debug flag
extern char *DQ_HBOOK; // HBOOK file
//
// function code
//
int
faCheckRhp(int print)
{
  // rpm/rhp initialized?
  if( !RPMINIT ) return 0;

  // show up
  ARTE_mark( "faCheckRhp 0", 0 );

  // rhp mode
  int rc = GRR_OK, force_pub = 0;
  
  // publish histograms
  if( RHPPUB>0 && !DQ_HBOOK ){

    static int old = -1, oldcnakey=-1, oldrunnr=0;
    // check whether run or CnA key changed to force publication and reset
    if ( (arteflag_.numrun != oldrunnr) || (faEVHDdbmajor() != oldcnakey) ) {
      oldcnakey = faEVHDdbmajor();
      oldrunnr  = arteflag_.numrun;
      force_pub = 1;
    } 

    if( (!(arteflag_.nevents%RHPPUB) || force_pub) && ( arteflag_.nevents != old ) ){
      if (force_pub && FADEB)  ARTE_error(GRR_SINFO, "Either CnA key or Run Number changed. "
				 "Publishing and reseting RHP histograms.", 0);
      old = arteflag_.nevents; 
      force_pub = 0;

      // rhp_publish
      ARTE_mark( "rhp_publish", 0 );
      if( ( rc = rhp_publish( 0 ) ) == GRR_OK ){
	if( print ) cout << " RHP: rhp_publish nevt=" << arteflag_.nevents << endl;
	if (FADEB) ARTE_error( GRR_SINFO, "rhp_publish succesful.", rc );
	
	// loop over IDs and reset them
	int max = 0, *hid = 0;
	rhp_hid( 0, NULL, 0, &max ); // get # of rhp objects
	if (max) {
	  hid = new int[max];
	  rhp_hid( 0, hid, max, &max ); // get all IDs
	  if ( print ) cout << " RHP: reset rhp_hid max=" << max << endl;
	  for( int i=0; i<max; i++ ){
	    rc = rhp_reset( hid[i] ); // reset all IDs
	  }
	  delete[] hid;
	}
      }
      // publish error
      else {
	cout << "RHP: ERROR rhp_publish FAILED. rc=" << rc << endl;
	ARTE_error( GRR_SERROR, "rhp_publish FAILED.", rc );
      }
    }
  }
  
  // show up
  ARTE_mark( "faCheckRhp 1", 0 );
  
  return rc;
}
