/************************************************************************
* CGGESL 
* written by Siegmund Nowak 01/97
* updated 31.01.2001 by B.Fominykh: MU01 - MU04 are added
*
* This routines creates GEANT superlayer banks from ARTE NESL bank
*
*       called from every geometry routine
*
* Arguments  kind         description
*-----------------------------------------------------------------------
*   CSL      in           name of the superlayer
*   MUTT     in           mother name into which it should be positioned
*                         for the moment are foreseen: MOTH,MAGN,RPOT,MUON
*                         RPOT is equivalent to MOTH except that the medium
*                         vaccuum is used instead of air
*   IR       in           overall rotation matrix number (gean)
************************************************************************/

#include "arte/arte.hh"
#include "arte/ArteMessage.hh"
#include "gean/gclink.h"
#include "gean/gcflag.h"
#include "gean/ZGMOTH.h"
#include "gean/ZGESL.h"

extern "C"{
    void cggesl_(char*, char*, int*);
    void gsvolu_(char*, char*, int*, float [], int*, int*);
    void gsatt_(char*,char*,int*);
    int  mednum_(int&,const char*,int);
    void gspos_(char*,int*,char*,float*,float*,float*,int*,char*);
}

void cggesl_(char *csl,char *mutt,int *ir) {
    int i0=0, i1=1, i2=2, isl=0, imed=0, ivol=0, npar=0; 
    float par[5], xmean=0.,ymean=0.,zmean=0., x00,y00,z00;
    char sl_name[5], muttc[5], shape[4];      

    for(int i = 0; i<4; i++) sl_name[i] = toupper(csl[i]);
    sl_name[4] ='\0'; 
    ArteTableIterator<NESL> ns;
    for(; ns.valid(); ns.next()){
       if(!strncmp(sl_name, ns->name,4)) { 
          isl = ns.index();         break; 
       }
    }
    if(!isl) {
       arteMsg << "cggesl- superlayer not found: " << sl_name;
       arteMsg.error();
    }
    if(ns->shp == 1) {
       par[0] = .5*(ns->x2 - ns->x1);
       par[1] = .5*(ns->y2 - ns->y1);
       par[2] = .5*(ns->z2 - ns->z1);
       xmean  = .5*(ns->x2 + ns->x1);
       ymean  = .5*(ns->y2 + ns->y1);
       zmean  = .5*(ns->z2 + ns->z1);
       strncpy(shape,"BOX ",4);
       npar   = 3;
    }
    else if(ns->shp == 3) {
       par[0] = .5*(ns->x2 - ns->x1);
       par[1] =  par[0] + ns->dx;
       par[2] = .5*(ns->y2 - ns->y1);
       par[3] =  par[2] + ns->dy;
       par[4] = .5*(ns->z2 - ns->z1);
       xmean  = .5*(ns->x2 + ns->x1);
       ymean  = .5*(ns->y2 + ns->y1);
       zmean  = .5*(ns->z2 + ns->z1);
       strncpy(shape,"TRD2",4);
       npar   = 5;
    }
    else if(ns->shp == 5) {
       par[0] = ns->x1;
       par[1] = ns->x2;
       par[2] = .5*(ns->z2 - ns->z1);
       xmean  = ns->y1;
       ymean  = ns->y2;
       zmean  = .5*(ns->z2 + ns->z1);
       strncpy(shape,"TUBE",4);
       npar   = 3;
    }
    else {
       arteMsg << "cggesl:other shapes then box/trd2/tube not foreseen"; 
       arteMsg.error();
    }

//    check for the mother ( moth, magn, rpot, muon, trd, blank )

    for(int i=0; i<4; i++) muttc[i] = toupper(mutt[i]);
    muttc[4] = '\0';

    if(!strcmp(muttc,"MOTH")) {
       x00 = xmean;
       y00 = ymean;
       z00 = zgmoth_.ZZERO + zmean;
       if(!strncmp(csl,"MAGN",4)) imed = mednum_(i2,"AIR",3);
       else { 
          if(ns->cmp == 1 && ns->z2 < ArtePointer<NESL>(zgmoth_.MAGNSL)->z1)   
	        imed = mednum_(i1,"VACUUM",6);
          else  imed = mednum_(i1,"AIR",3);
       }
    }
    else if(!strcmp(muttc,"RPOT")) {
       ArtePointer<NESL> nms(zgmoth_.RPOTSL);
       x00 = -.5*(nms->x1 + nms->x2) + xmean;
       y00 = -.5*(nms->y1 + nms->y2) + ymean;
       z00 = -.5*(nms->z1 + nms->z2) + zmean;
       imed = mednum_(i1,"AIR",3);
    }
    else if(!strcmp(muttc,"MAGN")) {
       ArtePointer<NESL> nms(zgmoth_.MAGNSL);
       x00 = -.5*(nms->x1 + nms->x2) + xmean;
       y00 = -.5*(nms->y1 + nms->y2) + ymean;
       z00 = -zgmoth_.ZMAGM + zmean;
       imed = mednum_(i2,"AIR",3);
    }
    else if(!strncmp(muttc,"MU",2)) {
      int mucont = 0;
      for(ArteTableIterator<NESL> muon; muon.valid(); muon.next()){
       if(!strncmp(muttc, muon->name,4)) { 
         ArtePointer<NESL> nms(muon.index());
         x00 = -.5*(nms->x1 + nms->x2) + xmean;
         y00 = -.5*(nms->y1 + nms->y2) + ymean;
         z00 = -.5*(nms->z1 + nms->z2) + zmean;
         imed = mednum_(i1,"AIR",3);
         mucont=1;
         break;
       }
      }
      if(mucont == 0){  
        arteMsg << "cggesl - mother muon superlayer " << muttc << " not found";
        arteMsg.error();
      }
    }
    else if(!strcmp(muttc,"TRSL")) {
       ArtePointer<NESL> nms(zgmoth_.TRDSL);
       x00 = -.5*(nms->x1 + nms->x2) + xmean;
       y00 = -.5*(nms->y1 + nms->y2) + ymean;
       z00 = -.5*(nms->z1 + nms->z2) + zmean;
       imed = mednum_(i1,"AIR",3);
    }
    else if(!strcmp(muttc,"    ")) imed = mednum_(i1,"AIR",3);
    else {  
       arteMsg << "cggesl- a wrong mother has been installed: " << muttc;
       arteMsg.error();
    }

    gsvolu_(csl,shape,&imed,par,&npar,&ivol);
    gsatt_(csl,"SEEN",&i0); 

//    posit the superlayer
    if(strcmp(muttc,"    ")) gspos_(csl,&isl,muttc,&x00,&y00,&z00,ir,"MANY");

    if(gcflag_.idebug) {
      arteMsg << "Superlayer " << sl_name << " implemented - isl = " << isl;   
      arteMsg.warn();
    }
}

