#ifndef _NOARTE_

//----------------------------------------------------------------------------
//
//  Subroutines to fill HITB Table from ITR_marvin's structures
//  ===========================================================
//
//  10/11/2000 add kumac steering for hitresolution, z-shift and rotations for
//             alignment constants   TZ 
//
//  14/12/2000 add ITR_HCUT : cut on maximal number of hits/chamber/event
//
//  18/12/2000 add changes needed for TAXX 
//             (hitb->y = cluster->cog, hitb->q = cluster->sn)
//
//  20.12.2000 acReadITR calls ITR_arte_updateGEDE in case of key change
//             -> apply changes to GEDE 
//
//  05.02.2002 obsolete routine ITR_alignment removed  TZ
//
//  06.03.2002 two routines for ITR MC digitisation added TZ
//----------------------------------------------------------------------------

// include files
#include  <stdio.h>
#include  <stdlib.h>
#include  <string.h>
#include  <math.h>
#include  <fstream.h>

// Arte banks,  C++ version
#include "arte/arte.hh"
#include "grr.h"

#include "aframelib.h"
#include "itrleda.h"
#include "ITR_marvin.h"
#include "ITR_arte.h"
#include "ITR_steer.h"


extern "C" {

/* ------------------------------------------------------------------------
 *
 *  Initialize aframe.
 *
 *  To be called before everything else.
 *  Done is (among others):
 *    - Some initialization
 *
 * ------------------------------------------------------------------------ */
int ITR_arte_param_init()

{
  char *routine = "ITR_arte_param_init";

  aframe_mess(routine,GRR_SINFO,"ITR_arte started");

  return GRR_OK;

}

}  /* end of extern C */  

static int ITR_gedeindex[10][10][8];

// ---------------------------------------------------------------------------
//  Fill HITB table from ITR_Marvin_Sevt_t structure 
//  last change 15/08/2000  TZ
//  bug found by IG removed
//
//  Returns:   GRR_OK    in case of no problems
//             GRR_FISHY otherwise
// ---------------------------------------------------------------------------

int ITR_fillHITBfromSPARSEVT()
{
  static int messagewrite = 1;
  static int errordetect = 0;
  char *routine = "ITR_fillHITBfromSPARSEVT";
  ArtePointer<GEDE> oldgede = 0;

  // First of all get event from ITR_marvin

  ITR_Marvin_Sevt_t *sevt = ITR_marvin_fetch_sevt();

  if ( sevt == NULL ) {
    aframe_mess(routine,GRR_SERROR,"NULL pointer to sparsified information!");
    return GRR_FISHY;
  }

  // check reasonable number of clusters in format 3

  ITR_Marvin_Header_t *header = ITR_marvin_sevtHeader( (char*) sevt->data );

  if ( header->format == 3 && header->ncl > 0x7FFF / 8 ) {
    aframe_mess(routine,GRR_SERROR,"Too many clusters in event --> skipping it!");
    return GRR_FISHY;
  }
 
  float cl_ch[10][8][4];
  int sect=-1;
  
  for (int g=0; g<10; g++)
    for (int h=0; h<8; h++)
      for (int j=0; j<4; j++)
        cl_ch[g][h][j]=0;

  // loop over clusters and count number of clusters
  
  ITR_Marvin_Cluster_t *cluster = ITR_marvin_sevtCluster( (char*) sevt->data, -1 );
   
  while ( cluster ) {

    // get GEDE id from database

    LedaPointer<class ITR_Chamber> chamber = ITR_Chamber->get->key( cluster->chamber );
    LedaPointer<class ITR_nede> nede = chamber->tonede;

    if ( !chamber ) {
      errordetect=1;
      if (messagewrite)
        aframe_mess(routine,GRR_SERROR,"ITR chamber %d not in database");
      cluster = ITR_marvin_sevtCluster(NULL, -1);
      continue;
    }


    ArtePointer<GEDE> gede, gede1;

    int gederow = ITR_arte_returnGEDE( (int) nede->superl_db, (int) nede->lay_db, (int) nede->sec_db[0] );

    if (nede->sec_db[0] == 5 || nede->sec_db[0] == 7) {
      gede = ArtePointer<GEDE>( gederow );
      gede1 = ArtePointer<GEDE> (gederow+1);
    }
    else if (nede->sec_db[0] == 2 || nede->sec_db[0] == 4) {
      gede1 = ArtePointer<GEDE>( gederow );
      gede = ArtePointer<GEDE> (gederow+1);
    }                    
    
    switch (nede->sec_db[0]) {
      case 2 : sect=0;
      break;
      case 4 : sect=1;
      break;
      case 5 : sect=2;
      break;
      case 7 : sect=3;
      break;
    }
   
    cl_ch[nede->superl_db-1][nede->lay_db-1][sect]++;
   
    // get next cluster
    cluster = ITR_marvin_sevtCluster(NULL, -1);

  }
 
  // loop over clusters to fill HITB bank
  
  cluster = ITR_marvin_sevtCluster( (char*) sevt->data, -1 );
  
  while ( cluster ) {

    // get GEDE id from database

    LedaPointer<class ITR_Chamber> chamber = ITR_Chamber->get->key( cluster->chamber );
    LedaPointer<class ITR_nede> nede = chamber->tonede;

    if ( !chamber ) {
      errordetect=1;
      if (messagewrite)
        aframe_mess(routine,GRR_SERROR,"ITR chamber %d not in database");
      cluster = ITR_marvin_sevtCluster(NULL, -1);
      continue;
    }


    ArtePointer<GEDE> gede, gede1;

    int gederow = ITR_arte_returnGEDE( (int) nede->superl_db, (int) nede->lay_db, (int) nede->sec_db[0] );
   
    if (nede->sec_db[0] == 5 || nede->sec_db[0] == 7) {
      gede = ArtePointer<GEDE>( gederow );
      gede1 = ArtePointer<GEDE> (gederow+1);
    }
    else if (nede->sec_db[0] == 2 || nede->sec_db[0] == 4) {
      gede1 = ArtePointer<GEDE>( gederow );
      gede = ArtePointer<GEDE> (gederow+1);
    }      

    switch (nede->sec_db[0]) {
      case 2 : sect=0;
      break;
      case 4 : sect=1;
      break;
      case 5 : sect=2;
      break;
      case 7 : sect=3;
      break;
    }
   
    if (cl_ch[nede->superl_db-1][nede->lay_db-1][sect]<ITR_HCUT) {

      int found = 1;

      if (gede == 0 || gede1 == 0 ) {
        errordetect=1;
        if (messagewrite)
          aframe_mess(routine,GRR_SERROR,"GEDE row is non existing for [%d,%d,%d] !",
  	  	    (int) nede->superl_db, (int) nede->lay_db, (int) nede->sec_db[0]);
        cluster = ITR_marvin_sevtCluster(NULL,-1);
        found = 0;
        continue;
      }

      if ( gede->cmp != 3 || gede1->cmp != 3 ) {
        errordetect=1;
        if (messagewrite)
          aframe_mess(routine,GRR_SERROR,"GEDE row is not an ITR GEDE table");
        cluster = ITR_marvin_sevtCluster(NULL, -1);
        found = 0;
        continue;
      }

      // cluster coordinates -> HITB
      //
      // strip counting in ITR_marvin starts always at the beam pipe
      // (0..767) in Arte first wire is always at the right corner of
      // the corresponding gede plane 
      // for type I+II 8 (III 5)  wires at both sides are not connected 
      // therefor corresponding Arte gede planes have only 752 (758) wires 

      HITB hit;
      int ITR_hitoutofsens = 0;
      float relpos=0;
      float gasgap = gede->zmax-gede->zmin;

      hit.z    = gede->zfw[0]+gasgap/2;

      if ( (int) chamber->type == 3 && ((int) nede->sec_db[0] == 5 || (int) nede->sec_db[0] == 7 )) {
        // -x side Typ III 
        if (cluster->cog>4 && cluster->cog<763) {
          relpos = gede->nwi[0]+gede1->nwi[0]+4-cluster->cog;
          hit.x    = relpos*gede->pit + gede->xfw[0];
        }
        else ITR_hitoutofsens = 1;
      }  

      if ( (int) chamber->type == 3 && ((int) nede->sec_db[0] == 2 || (int) nede->sec_db[0] == 4 )) {
        // +x side Typ III
        if (cluster->cog>4 && cluster->cog<763) {
          relpos = cluster->cog-5;
          hit.x    = relpos*gede->pit + gede->xfw[0];
        }
        else ITR_hitoutofsens = 1;
      }      

      if ( (int) chamber->type < 3 && ((int) nede->sec_db[0] == 5 || (int) nede->sec_db[0] == 7 )) {
        // -x side Typ I+II
        if (cluster->cog>7 && cluster->cog<760) {
          relpos = gede->nwi[0]+gede1->nwi[0]+7-cluster->cog;
          hit.x    = relpos*gede->pit + gede->xfw[0];
        }
        else ITR_hitoutofsens = 1;
      }

      if ( (int) chamber->type < 3 && ((int) nede->sec_db[0] == 2 || (int) nede->sec_db[0] == 4 )) {
        // +x side Typ I+II
        if (cluster->cog>7 && cluster->cog<760) {
          relpos = cluster->cog-8;
          hit.x    = relpos*gede->pit + gede->xfw[0];
        }
        else ITR_hitoutofsens = 1;
      }

      if (!((int) nede->sec_db[0]== 2 || (int) nede->sec_db[0]== 4 || (int) nede->sec_db[0]== 5 || (int) nede->sec_db[0]== 7 )) {
        aframe_mess(routine,GRR_SWARNING,"Wrong sector number in DB detected!");
      }  
 
      if (!ITR_hitoutofsens) {

        if (relpos > gede->nwi[0]) 
          hit.gede = gede1;
        else 
          hit.gede = gede;

        hit.chid = cluster->width;
        hit.q    = cluster->sn;
        hit.y    = cluster->cog;

        // hit resolution default value 0.01 
        // can be changed by kumac

        hit.r    = ITR_HITRES;
    
        ArtePointer<HITB> hitb = ArteTable<HITB>::push_back( hit );

        // update GEDE bank: hit1 only , if GEDEnr has changed,
        //                   hitl always (every additional hit overwrites it)
        // be carefull : 1 ITR-Chamber corresponds to 2 Gede planes

        if ( hitb->gede != oldgede ) {
          if (relpos <= gede->nwi[0]) gede->hit1 = hitb;
          else gede1->hit1 = hitb;
          oldgede = hitb->gede;
        }
    
        if (relpos <= gede->nwi[0]) gede->hitl = hitb;
        else gede1->hitl = hitb;
       
        if ( found == 0 ) {
          aframe_mess(routine,GRR_SERROR,"GEDE row is non-existent");
          cluster = ITR_marvin_sevtCluster(NULL, -1);
          continue;
        }
      } // of (!ITR_hitoutofsens)
    }

    // get next cluster
    cluster = ITR_marvin_sevtCluster(NULL, -1);
  } 
  if (errordetect) messagewrite=0;
  return GRR_OK;
}



// ---------------------------------------------------------------------------
//
//  update GEDE from NEDE + ALIGNMENT
//
// ---------------------------------------------------------------------------


int ITR_arte_updateGEDE( ) {

  // routine for update of GEDE using Alignment information
  // called by acReadITR

  char *routine = "ITR_arte_updateGEDE : ";

  aframe_mess(routine,GRR_SINFO,"GEDE table is now updated from NEDE + Alignment !!");  


  // DB access

  for ( LedaTableIterator<class ITR_Chamber> ledachamber; ledachamber.valid(); ledachamber.next() ) {
    LedaPointer<class ITR_nede> ledanede = ledachamber->tonede;
    LedaPointer<class ITR_Align> ledaalign = ledanede->toalign;

    /* 
       [ wg, 28/jul/00 ]
       if there is no alignment table found for this chamber, create an emtpy one
       (zero shifts, zero rotations).  This prevents core dumps in case ITR_Chamber
       and ITR_Align have different ideas about the number of installed chambers ...
    */

    if (ledaalign == 0) {
      ledaalign = ITR_Align->add->key ((unsigned short) ledachamber->id);
      for (int j = 0; j < 3; j++) {
        ledaalign->ang[j] = 0.;
        ledaalign->eang[j] = 0.;
        ledaalign->delt[j] = 0.;
        ledaalign->edelt[j] = 0.;
        ledaalign->sloff[j] = 0.;
        ledaalign->esloff[j] = 0.;
      }
      ledaalign->redres = 0.;

      /* add align->tonede to ITR_nede */
      RelateRows (ITR_Align, tonede, key ((unsigned short) ledaalign->id),
                  ITR_nede, toalign, key (ledanede->superl_db, ledanede->lay_db,
                                          ledanede->sec_db[0]));
      cout << __PRETTY_FUNCTION__ << ": [WARNING]  Alignment for chamber 0x" 
	   << hex << ledachamber->id << " not found in database" << endl  ;
    }

    int found = 0;

    // loop over ARTE GEDE table to get corresponding GEDE row

    for ( ArteTableIterator<GEDE> igede; igede.valid(); igede.next() ) {
      ArtePointer<GEDE> gede(*igede);
      ArtePointer<GEDE> gede1;
      ArtePointer<GESL> gesl(gede->gesl);
      
      if ( (gede->cmp == 3)
        && (gesl->nsl == (int) ledanede->superl_db)
        && (gede->lay == (int) ledanede->lay_db)
        && (gede->sec == (int) ledanede->sec_db[0]) ) {

        if (ledanede->sec_db[0] == 5 || ledanede->sec_db[0] == 7)
          gede1 = ArtePointer<GEDE> (gede.index()+1);
        else if (ledanede->sec_db[0] == 2 || ledanede->sec_db[0] == 4)
          gede1 = ArtePointer<GEDE>( gede.index()+1 );                
              
        // loop over NEDE to find corresponding NEDE table
      
        for ( ArteTableIterator<NEDE> inede; inede.valid(); inede.next() ) {
          ArtePointer<NEDE> nede(*inede);
          ArtePointer<NEDE> nede1;
          ArtePointer<NESL> nesl(nede->nesl);
      
          if ( (nede->cmp == 3)
            && (nesl->nsl == (int) ledanede->superl_db)
            && (nede->lay == (int) ledanede->lay_db)
            && (nede->sec == (int) ledanede->sec_db[0]) ) {

            if (ledanede->sec_db[0] == 5 || ledanede->sec_db[0] == 7)
              nede1 = ArtePointer<NEDE> (nede.index()+1);
            else if (ledanede->sec_db[0] == 2 || ledanede->sec_db[0] == 4)
              nede1 = ArtePointer<NEDE>( nede.index()+1 );                
        
            found = 1;      

            // update Gede entries
            // physical chamber is divided into two gede planes (gede and gede1)

            gede->sxmn    = nede->sxmn  + ledaalign->delt[0];
            gede1->sxmn   = nede1->sxmn + ledaalign->delt[0];
            gede->sxmx    = nede->sxmx  + ledaalign->delt[0];
            gede1->sxmx   = nede1->sxmx + ledaalign->delt[0]; 

            gede->xfw[0]  = nede->xfw[0]  + ledaalign->delt[0];
            gede1->xfw[0] = nede1->xfw[0] + ledaalign->delt[0];
            gede->xmax    = nede->xmax    + ledaalign->delt[0];
            gede1->xmax   = nede1->xmax   + ledaalign->delt[0];
            gede->xmin    = nede->xmin    + ledaalign->delt[0];
            gede1->xmin   = nede1->xmin   + ledaalign->delt[0];

            gede->zmax    = nede->zmax    + ledaalign -> delt[2];
            gede->zmin    = nede->zmin    + ledaalign -> delt[2];
            gede->zfw[0]  = nede->zfw[0]  + ledaalign -> delt[2];
            gede1->zmax   = nede1->zmax   + ledaalign -> delt[2];
            gede1->zmin   = nede1->zmin   + ledaalign -> delt[2];
            gede1->zfw[0] = nede1->zfw[0] + ledaalign -> delt[2];

            gede->rot[0] = nede->rot[0] * cos(ledaalign -> ang[0]) - nede->rot[1]*sin(ledaalign -> ang[0]);
            gede->rot[1] = nede->rot[1] * cos(ledaalign -> ang[0]) + nede->rot[0]*sin(ledaalign -> ang[0]);
            gede->rot[3] = - gede->rot[1]; 
            gede->rot[4] = gede->rot[0];

            gede1->rot[0] = gede->rot[0];
            gede1->rot[1] = gede->rot[1];
            gede1->rot[3] = gede->rot[3];
            gede1->rot[4] = gede->rot[4];

          }
	} // end of loop over NEDE
      }
    } // end of loop over GEDE 
  }
  return 0;
}

// ---------------------------------------------------------------------------
//
//  fill gedeindex
//
// --------------------------------------------------------------------------- 

int ITR_fillgedeindex() {

  // routine to fill ITR_gedeindex 
  // allows faster access to gede table 
  // gesl->nsl, gede->lay, gede->sec  -> gede.index

  int error = 0;
  char ITR_hwc[13] = "none";
  char *routine = "ITR_fillITRgedeindex";

  // loop over CHAMBER entries in LEDA

  for ( LedaTableIterator<class ITR_Chamber> chamber; chamber.valid(); chamber.next() ) {
    LedaPointer<class ITR_nede> nede = chamber->tonede;

    // loop over ARTE GEDE table to get corresponding GEDE row
    int found = 0;

    for ( ArteTableIterator<GEDE> igede; igede.valid(); igede.next() ) {
      ArtePointer<GEDE> gede(*igede);
      ArtePointer<GEDE> gede1;
      ArtePointer<GESL> gesl(gede->gesl);
      
      if (   (gede->cmp == 3)
          && (gesl->nsl == (int) nede->superl_db)
          && (gede->lay == (int) nede->lay_db)
          && (gede->sec == (int) nede->sec_db[0]) ) {

        if (nede->sec_db[0] == 5 || nede->sec_db[0] == 7)
          gede1 = ArtePointer<GEDE> (gede.index()+1);
        else if (nede->sec_db[0] == 2 || nede->sec_db[0] == 4)
          gede1 = ArtePointer<GEDE>( gede.index()+1 );                
        // fill gedeindex array

        if (    (int) nede->superl_db  >= 1 && (int) nede->superl_db  <= 10
                && (int) nede->lay_db >= 1 && (int) nede->lay_db <= 10
		&& (int) nede->sec_db[0] >= 1 && (int) nede->sec_db[0] <= 8 ) {

          char ITR_ori[3] = "++";
          char ITR_nam[4] = "M00";
          char ITR_cham[5] = "K000";

          ITR_nam[1] = gesl->name[2];
          ITR_nam[2] = gesl->name[3];

          ITR_gedeindex[ (int) nede->superl_db-1 ][ (int) nede->lay_db-1 ][ (int) nede->sec_db[0]-1 ] = gede.index();

          // gede->hwc is filled 

          if (nede->sec_db[0]== 3 || nede->sec_db[0]== 4) ITR_ori[1] = '-';
          if (nede->sec_db[0]== 6 || nede->sec_db[0]== 5) { ITR_ori[0] = '-'; ITR_ori[1] = '-'; }
          if (nede->sec_db[0]== 8 || nede->sec_db[0]== 7) ITR_ori[0] = '-';

          char ITR_ang[3] = "+0";

          if (gede->rot[3] > 0.05) ITR_ang[1] = '5';
          if (gede->rot[3] < -0.05) { ITR_ang[0] = '-'; ITR_ang[1] = '5';}

          ITR_cham[1] = chamber->name[1];
          ITR_cham[2] = chamber->name[2];
          ITR_cham[3] = chamber->name[3];

          sprintf(ITR_hwc,"%s%s%d%.2s%s",ITR_nam,ITR_ori,nede->lay_db,ITR_ang,ITR_cham);

          strncpy(gede->hwc,ITR_hwc,13);
          strncpy(gede1->hwc,ITR_hwc,13);
        }
        else {
          aframe_mess(routine,GRR_SWARNING,"GEDE");
        }
        found = 1;
     
      }
    }   // end loop over ARTE GEDE rows

    // complain in case no corresponding ARTE GEDE row was found

    if ( !found ) {
      aframe_mess(routine,GRR_SWARNING,"ITR chamber <%04x> not found in GEDE table!", 
		  chamber->id);
      error++;
    }

  }
  if ( error )
    return GRR_FISHY;

  return GRR_OK;

}


// ---------------------------------------------------------------------------
//
//  return GEDE row from SL name, layer, and sector numbers
//
// ---------------------------------------------------------------------------

int ITR_arte_returnGEDE( int sl, int lay, int sec )
{
  
  if ( sl >= 1 && sl <= 10 && lay >= 1 && lay <= 10 && sec >= 1 && sec <= 8 ) {
    return ITR_gedeindex[ sl-1 ][ lay-1 ][ sec-1 ];
  }
  else
    return 0;

}



//=============================================================================

int setITRDIGI_from_DB() {

  extern float itrdiginoise[4000];

  char *routine = "setITRDIGI_from_DB : ";

  aframe_mess(routine,GRR_SINFO,"Performance parameters for ITR Digitisation are read from DB !");  

  // DB access

  for ( LedaTableIterator<class ITR_Chamber> ledachamber; ledachamber.valid(); ledachamber.next() ) {
    LedaPointer<class ITR_nede> ledanede = ledachamber->tonede;
    LedaPointer<class ITR_EffRes> theEffRes = ITR_EffRes->get->key(ledachamber->id);

    if (theEffRes == 0) {
       theEffRes = ITR_EffRes->add->key ((unsigned short) ledachamber->id);
       theEffRes->eff = 0.;
       theEffRes->res = 0.;
      for (int j = 0; j < 14; j++) {
        theEffRes->spare[j] = 0.;
      };
    }

    int found = 0;

    // loop over ARTE GEDE table to get corresponding GEDE row

    for ( ArteTableIterator<GEDE> igede; igede.valid(); igede.next() ) {
      ArtePointer<GEDE> gede(*igede);
      ArtePointer<GEDE> gede1;
      ArtePointer<GESL> gesl(gede->gesl);
      
      if ( (gede->cmp == 3)
        && (gesl->nsl == (int) ledanede->superl_db)
        && (gede->lay == (int) ledanede->lay_db)
        && (gede->sec == (int) ledanede->sec_db[0]) ) {

        if (ledanede->sec_db[0] == 5 || ledanede->sec_db[0] == 7)
          gede1 = ArtePointer<GEDE> (gede.index()+1);
        else if (ledanede->sec_db[0] == 2 || ledanede->sec_db[0] == 4)
          gede1 = ArtePointer<GEDE>( gede.index()+1 );                
              
             found = 1;      

            // update Gede entries
            
            if (theEffRes->eff != -1) {
              gede->eff    = theEffRes->eff;
              gede1->eff   = theEffRes->eff;
            }
            if (theEffRes->res !=-1) {
              gede->acc    = theEffRes->res;
              gede1->acc   = theEffRes->res; 
            }
            if (theEffRes->spare[0] !=0) {
              gede->dod    = theEffRes->spare[0];
              gede1->dod   = theEffRes->spare[0];             
            }
            if (theEffRes->spare[1] !=0) {
              int gind = gede.index();            
              itrdiginoise[gind]=theEffRes->spare[1];
              gind = gede1.index();            
              itrdiginoise[gind]=theEffRes->spare[1];
            } 
      }
    } // end of loop over GEDE 
  }
  return 0;
}

//=============================================================================

int setITRDIGIALIGN_from_DB() {

  extern float itrdigialign[4000][3];

  char *routine = "setITRDIGIALIGN_from_DB : ";

  aframe_mess(routine,GRR_SINFO,"Alignment constants for ITR Digitisation are read from DB !");  

  // DB access

  for ( LedaTableIterator<class ITR_Chamber> ledachamber; ledachamber.valid(); ledachamber.next() ) {
    LedaPointer<class ITR_nede> ledanede = ledachamber->tonede;
    LedaPointer<class ITR_Align> ledaalign = ledanede->toalign;

    if (ledaalign == 0) {
      ledaalign = ITR_Align->add->key ((unsigned short) ledachamber->id);
      for (int j = 0; j < 3; j++) {
        ledaalign->ang[j] = 0.;
        ledaalign->eang[j] = 0.;
        ledaalign->delt[j] = 0.;
        ledaalign->edelt[j] = 0.;
        ledaalign->sloff[j] = 0.;
        ledaalign->esloff[j] = 0.;
      }
      ledaalign->redres = 0.;

      /* add align->tonede to ITR_nede */
      RelateRows (ITR_Align, tonede, key ((unsigned short) ledaalign->id),
                  ITR_nede, toalign, key (ledanede->superl_db, ledanede->lay_db,
                                          ledanede->sec_db[0]));
      cout << __PRETTY_FUNCTION__ << ": [WARNING]  Alignment for chamber 0x" 
	   << hex << ledachamber->id << " not found in database" << endl  ;
    }
 
    int found = 0;

    // loop over ARTE GEDE table to get corresponding GEDE row

    for ( ArteTableIterator<GEDE> igede; igede.valid(); igede.next() ) {
      ArtePointer<GEDE> gede(*igede);
      ArtePointer<GEDE> gede1;
      ArtePointer<GESL> gesl(gede->gesl);
      
      if ( (gede->cmp == 3)
        && (gesl->nsl == (int) ledanede->superl_db)
        && (gede->lay == (int) ledanede->lay_db)
        && (gede->sec == (int) ledanede->sec_db[0]) ) {

        if (ledanede->sec_db[0] == 5 || ledanede->sec_db[0] == 7)
          gede1 = ArtePointer<GEDE> (gede.index()+1);
        else if (ledanede->sec_db[0] == 2 || ledanede->sec_db[0] == 4)
          gede1 = ArtePointer<GEDE>( gede.index()+1 );                
              
            found = 1;      

            int gind = gede.index();            
            itrdigialign[gind][0]=ledaalign->delt[0];
            itrdigialign[gind][1]=ledaalign->delt[2];
            itrdigialign[gind][2]=ledaalign->ang[0];
            gind = gede1.index();            
            itrdigialign[gind][0]=ledaalign->delt[0];
            itrdigialign[gind][1]=ledaalign->delt[2];
            itrdigialign[gind][2]=ledaalign->ang[0];                  
           
      }
    } // end of loop over GEDE 
  }
  return 0;
}

//=============================================================================

#endif   /* of #ifndef _NOARTE_ */


