*CMZ :  1.00/00 21/10/94  14.58.02  by  Dominik Ressing
*CMZ :  0.01/02 23/08/94  19.31.16  by  Dominik Ressing
*-- Author :    Dominik Ressing   11/08/94
************************************************************************
*
* L1HIREG
* written by Dominik Ressing, 11/08/94 15.38.55
*
* This routines action:
* searches for cells in the detector DetSpec in the region of rGeoInfo.
* The sequential detector identifier is mandantory -- all others may be
* set to zero to get Cells for all values. Up to nMax cell numbers are reported
* in the arrays iDetSpecF,iCells, the actual number of Cell numbers in N. The detector
* specifier for Cell k is in iDetSpeF(,k) and the Cell number in iCells(k).
*
* Arguments  kind         description
*          (in,update,out)
*-----------------------------------------------------------------------
* iDetSpe     in          detector specifier
* rGeoInf     in          field to search Cells in
* nMax        in          maximal number of Cells to report
* N           out         actual number of Cells found (or nMax+1 if exceeded)
* iDetSpeF    out         array of detector specifiers for found Cells
* iCells       out         array of found Cell numbers
*
************************************************************************
c
      SUBROUTINE L1HIREG(iDetSpe,rGeoInf,nMax,N,iDetSpeF,iCells)
c
      IMPLICIT NONE
c
#include "gean/gcbank.fi"
#include "l1si/l1param.fi"
#include "l1si/l1geodf.fi"
#include "l1si/l1detdf.fi"
c
      INTEGER iDetSpe(L1DETSPElen),iCells(*),iDetSpeF(L1DETSPElen,*),
     &        nMax,N
      REAL    rGeoInf(L1GEOINFlen)
c
      INTEGER jDetGeo,                ! outline of detctor
     &        jDetInf,                ! detector information
     &        jCellGe                 ! geometry of first cell
      REAL    Point(L1POINTlen),      ! Point structure
     &        x,x0,y,dy,dx
      INTEGER iDetCur(L1DETSPElen),   ! specification of current detector
     &        iDetInf(L1DETINFlen),i,i1,i2,k,iLa,Ise,
     &        iFiSe,iLaSe,iFiLa,iLaLa,iNfirst
      LOGICAL allsec,alllay,LastCol,LastRow
c
      allsec = iDetSpe(L1DETSPEsection).eq.0
      alllay = iDetSpe(L1DETSPElayer).eq.0
      N = 0
      iFiSe = iDetSpe(L1DETSPEsection)
      iLaSe = iFiSe
      iFiLa = iDetSpe(L1DETSPElayer)
      iLaLa = iFiLa
c
      IF (allsec) iDetSpe(L1DETSPEsection) = 1
      IF (alllay) iDetSpe(L1DETSPElayer) = 1
      CALL l1detif(iDetSpe,jDetGeo,jDetInf,jCellGe)  ! get detector geometry
      IF ( jDetGeo.le.0 ) THEN                       ! access failed !!
         N = 0
         RETURN
      END IF
c
      IF ( alllay ) THEN
         iFiLa = 1
         iLaLA = IQ(jDetInf+L1DETINFnlay)
      END IF
      IF ( allsec ) THEN
         iFiSe = 1
         iLaSe = IQ(jDetInf+L1DETINFnsec)
      END IF
      Point(L1POINTdx) = 0
      Point(L1POINTdy) = 0
      Point(L1POINTz0) = rGeoInf(L1GEOINFz)
c
      CALL L1DETCP(iDetSpe,iDetCur)
      DO iLa = iFiLa,iLaLa
      DO iSe = iFiSe,iLaSe
c
         x = rGeoInf(L1GEOINFx0) ! upper left corner Cell
         y = rGeoInf(L1GEOINFy0) !
         x0 = x
c
         iDetCur(L1DETSPElayer)   = iLa
         iDetCur(L1DETSPEsection) = iSe
         CALL l1detif(iDetCur,jDetGeo,jDetInf,jCellGe)
         IF ( jDetGeo.le.0 ) THEN           ! access failed !!
            N = 0
            RETURN
         END IF
         iNfirst = N+1                ! position to store first Cell for iDetCur
         LastCol = .false.
         LastRow = .false.
c
         DO WHILE (.true.)                         ! loop till break ==> go to 555
            Point(L1POINTx0) = x
            Point(L1POINTy0) = y
            CALL l1pohit(iDetCur,Point,i1,i2)      ! calculate cell numbers
            IF ( i1.gt.0 ) THEN                    ! cell found ??
               DO i=i1,i2                          ! store all cells
               DO k=iNfirst,N                ! loop over all cells for detector
                  IF ( iCells(k).eq.i ) THEN  ! cell already stored ?
                     LastCol = .true.        ! possible only in last column or last row
                     GO TO 333
                  END IF
               END DO
               N = N + 1
               IF ( N.gt.nMax ) RETURN             ! Cell storage exeeded
               iCells(N) = i                        ! store cell number
               CALL L1DETCP(iDetCur,iDetSpeF(1,N)) ! copy detector specifier
               END DO
            END IF
 333        CONTINUE
            IF ( LastCol ) THEN                    ! last x value processed ?
               IF ( LastRow ) GO TO 555            ! last y value processed -> break
               dy = y
               y = y + Q(jDetInf+L1DETINFyspace)     ! increment y
               IF ( y.gt.rGeoInf(L1GEOINFy0)+rGeoInf(L1GEOINFsy)) ! y limit ?
     $              THEN
                  LastRow = .true.                                 ! we are in last row
                  y = rGeoInf(L1GEOINFy0)+rGeoInf(L1GEOINFsy)  ! use limit value
               END IF
               dy = y - dy                         ! this is the actual y shift
               x0 = rGeoInf(L1GEOINFx0)+dy*rGeoInf(L1GEOINFsin_al)/
     &                                  rGeoInf(L1GEOINFcos_al)! new x start
               x = x0
               LastCol = .false.
            ELSE
               dx = FLOAT(i2-i1+1)*Q(jDetInf+L1DETINFxspace) ! x increment
               x = x + dx                                    ! increment x
               IF ( x.gt.x0+rGeoInf(L1GEOINFsx)) THEN      ! x limit reached ??
                  LastCol = .true.                           ! last column reached
                  x = x0+rGeoInf(L1GEOINFsx)               ! x = bound of region
               ELSE
                  LastCol = .false.
               END IF
            END IF
         END DO
 555     CONTINUE
      END DO
      END DO
c
c
      RETURN
c
      END
