//---------------------------------------------------------------------
// Created: 21.04.98 by   Olya Igonkina
// implementation of MCfunction
//---------------------------------------------------------------------



#include "marple/RcMcFunction.hh"

// return 1 if MC particle can be used as a reference one for match
int RcMtraReference( const ArtePointer<MTRA> & mtra )
{
  if( !( mtra != 0 ) ) return 0;

  // vertex should be close to primary vertex
  if( mtra->bver->z > 100 ) return 0;
  
  float momentum = sqrt( mtra->px*mtra->px +
			 mtra->py*mtra->py +
			 mtra->pz*mtra->pz );

  // momentum of particle should be bigger than 1 GeV/c
  if( momentum < 1 ) return 0;

  return 1;  
}

int RcMtraReferenceOk( const std::vector<ArtePointer<MTRA> > &
		       mtraList1, 
		       const std::vector<ArtePointer<MTRA> > &
		       mtraList2 )
{
  ArtePointer<MTRA> imtra = RcMtraPointer( mtraList1, mtraList2 );
  if( imtra != 0  && RcMtraReference( imtra ) ) 
    return 1;
  return 0;
}

int RcMtraReferenceOk( ArtePointer<MTRA>  & mtraEntry, 
		       const std::vector<ArtePointer<MTRA> > & mtraList )
{
  if( ! RcMtraReference(mtraEntry) ) return 0;
  if( RcMtraOk( mtraEntry, mtraList ) ) 
    return 1;
  return 0;
}
//------------ gamma----------------------------

int RcGamma(const ArtePointer<MTRA> mtra )
{
  if( !( mtra != 0 ) ) return 0;
  if( abs(mtra->topp->lund) == 22 )       return 1;
  return 0;
}

std::vector<ArtePointer<MTRA> > RcGamma( const std::vector<ArtePointer<MTRA> > mtraList )
{
  std::vector<ArtePointer<MTRA> > gammas;
  for( unsigned int iMtra = 0;  iMtra < mtraList.size(); ++iMtra )
    if( RcGamma(mtraList[iMtra]) ) 
      gammas.push_back( mtraList[iMtra]);
  
  return  gammas;
}

//-----------bremstralung------------------------
ArtePointer<MTRA> RcOrigOfBrem( const ArtePointer<MTRA> mtra, 
				float beforeZ )
{
  if( !( mtra != 0 ) ) return 0;
  if( abs(mtra->topp->lund) != 22 ) 
    {
      return 0; 
    }else{
      // cout << " origOfBrem: given particle is not  gamma"
    }
  if( mtra->bver->z > beforeZ )   return 0;
  ArtePointer<MTRA> origBrem  = mtra->btra;
  if( !(origBrem != 0 ) )  return 0;

  int iLund = origBrem->topp->lund;
  if( abs(iLund) == 11  )  return origBrem;
  else                     return 0;

}

// add only bremstralung origin to mtraList
std::vector<ArtePointer<MTRA> > 
RcMtraListOnlyBrem( const std::vector<ArtePointer<MTRA> >   mtraList , 
		    float beforeZ )
{
  std::vector<ArtePointer<MTRA> > mtraListOnlyBrem;
  for( unsigned int iMtra = 0;  iMtra < mtraList.size(); ++iMtra )
    {
      ArtePointer<MTRA> brem = RcOrigOfBrem( mtraList[iMtra], beforeZ );
      if( brem != 0  && !RcMtraOk(brem, mtraList) )
	mtraListOnlyBrem.push_back(brem) ; 
    }
  return mtraListOnlyBrem;
}

// add all bremstralung origin to mtraList
std::vector<ArtePointer<MTRA> > 
RcMtraListWithBrem( const std::vector<ArtePointer<MTRA> >   mtraList,
		    float beforeZ  )
{
  std::vector<ArtePointer<MTRA> > mtraListWithBrem=mtraList;
  for( unsigned int iMtra = 0;  iMtra < mtraList.size(); ++iMtra )
    {
      ArtePointer<MTRA> brem = RcOrigOfBrem( mtraList[iMtra], beforeZ );
      if( brem != 0 && !RcMtraOk(brem, mtraList) )
	mtraListWithBrem.push_back(brem) ; 
    }
  return mtraListWithBrem;
}

//---------------elepton-------------------------
int RcElepton(const ArtePointer<MTRA> mtra )
{
  if( !( mtra != 0 )  ) return 0;
  if( abs(mtra->topp->lund) == 11 ) 
    {
      return 1;
    }
  return 0;
}

std::vector<ArtePointer<MTRA> > RcElepton( const std::vector<ArtePointer<MTRA> > 
				      mtraList )
{
  std::vector<ArtePointer<MTRA> > eleptons;
  for( unsigned int iMtra = 0; iMtra < mtraList.size(); ++iMtra )
    {
      if( RcElepton(mtraList[iMtra]) ) eleptons.push_back( mtraList[iMtra]);
    }
  return  eleptons;
}

//---------------elepton-from-Jpsi-----------------------

int RcEleptonFromJpsi(const ArtePointer<MTRA> mtra )
{
  if( !(mtra != 0) ) return 0;
  if( abs(ArtePointer<TOPP>(mtra->topp)->lund) == 11 ) 
    {
      ArtePointer<MTRA> btra = mtra->btra;
      if( !(btra != 0) ) return 0;
      if( abs(btra->topp->lund) == 443 )	return 1;  
    }
  return 0;
}


std::vector<ArtePointer<MTRA> > 
RcEleptonFromJpsi( const std::vector<ArtePointer<MTRA> >   mtraList )
{
  std::vector<ArtePointer<MTRA> > eleptons;
  for( unsigned int iMtra = 0;  iMtra < mtraList.size(); ++iMtra )
    {
      if( RcEleptonFromJpsi(mtraList[iMtra]) ) 
	eleptons.push_back( mtraList[iMtra]);
    }
  return  eleptons;
}
//---------------electron-------------------------
int RcElectron(const ArtePointer<MTRA> mtra )
{
  if( !( mtra != 0 ) ) return 0;
  if( mtra->topp->lund == 11 ) 
    {
      return 1;
    }
  return 0;
}

std::vector<ArtePointer<MTRA> > 
RcElectron( const std::vector<ArtePointer<MTRA> >   mtraList )
{
  std::vector<ArtePointer<MTRA> > electrons;
  for( unsigned int iMtra = 0; iMtra < mtraList.size(); ++iMtra )
    {
      if( RcElectron(mtraList[iMtra]) ) electrons.push_back( mtraList[iMtra]);
    }
  return  electrons;
}

//---------------electron-from-Jpsi-----------------------

int RcElectronFromJpsi(const ArtePointer<MTRA> mtra )
{
  if( !(mtra != 0 ) ) return 0;
  if( mtra->topp->lund == 11 ) 
    {
      ArtePointer<MTRA> btra( mtra->btra);
      if( !(btra != 0 ) ) return 0;
      if( abs(btra->topp->lund) == 443 )	return 1;  
    }
  return 0;
}


std::vector<ArtePointer<MTRA> > 
RcElectronFromJpsi( const std::vector<ArtePointer<MTRA> >  mtraList )
{
  std::vector<ArtePointer<MTRA> > electrons;
  for( unsigned int iMtra = 0;   iMtra < mtraList.size(); ++iMtra )
    {
      if( RcElectronFromJpsi(mtraList[iMtra]) ) 
	electrons.push_back( mtraList[iMtra]);
    }
  return  electrons;
}

//---------------positron-------------------------
int RcPositron(const ArtePointer<MTRA> mtra )
{
  if( !(mtra != 0)  ) return 0;
  if( mtra->topp->lund == -11 ) 
    {
      return 1;
    }
  return 0;
}

std::vector<ArtePointer<MTRA> > 
RcPositron( const std::vector<ArtePointer<MTRA> >  mtraList )
{
  std::vector<ArtePointer<MTRA> > positrons;
  for( unsigned int iMtra = 0; iMtra < mtraList.size(); ++iMtra )
    {
      if( RcPositron(mtraList[iMtra]) ) positrons.push_back( mtraList[iMtra]);
    }
  return  positrons;
}
//---------------positron-from-Jpsi-----------------------

int RcPositronFromJpsi(const ArtePointer<MTRA> mtra )
{
  if( !(mtra != 0 ) ) return 0;
  if( mtra->topp->lund == -11 ) 
    {
      ArtePointer<MTRA> btra(mtra->btra);
      if( !(btra != 0 )) return 0;
      if( abs(btra->topp->lund) == 443 )	return 1;  
    }
  return 0;
}


std::vector<ArtePointer<MTRA> > 
RcPositronFromJpsi( const std::vector<ArtePointer<MTRA> >   mtraList )
{
  std::vector<ArtePointer<MTRA> > positrons;
  for( unsigned int iMtra = 0; iMtra < mtraList.size(); ++iMtra )
    {
      if( RcPositronFromJpsi(mtraList[iMtra]) ) 
	positrons.push_back( mtraList[iMtra]);
    }
  return  positrons;
}

//----------------J/psi----------------------------

int RcFromJpsi(const ArtePointer<MTRA> mtra )
{
  if( !(mtra != 0 ) ) return 0;
  int iLund = mtra->topp->lund;
  if( abs(iLund) == 11 || abs(iLund) == 13 )
    {
      ArtePointer<MTRA> btra( mtra->btra);
      if( !(btra != 0) ) return 0;
      iLund = btra->topp->lund;
      if( abs(iLund) == 443 )
	{  return 1;  }
    }
  return 0;
}

int RcFromJpsi(const std::vector<ArtePointer<MTRA> > mtraList )
{
  for( unsigned int iMtra = 0; iMtra < mtraList.size(); ++iMtra )
    {
      if( RcFromJpsi(mtraList[iMtra]) ) return 1;
    }
  return 0;
}
