#ifndef _RCMATCHFLAG_H_
#define _RCMATCHFLAG_H_
//---------------------------------------------------------------------
// Created: 24.03.99 by   Olya Igonkina  for MARPLE

// MARPLE class which defines type of matching

//---------------------------------------------------------------------

#include <iostream.h>

/** {\bf MARPLE flag}
    The parameters used by matching are defined by the {\bf flag} : \\
    
    \begin{tabular}{lcccccc}
    flag = & 1  & 1  & 1  & 1  & 1  & 1  \\
           & !  & !  & !  & !  & !  & pe \\
           & !  & !  & !  & !  & ty &    \\
           & !  & !  & !  & tx &    &    \\
           & !  & !  & y  &    &    &    \\
           & !  & x  &    &    &    &    \\
           & z  &    &    &    &    &    \\
	   \end{tabular}

   @memo flag for Matching
*/


class RcMatchFlag
{
public:
  enum VARIABLE { MA_Z  = 32,  // not used in matching but could be usefull
		  MA_X  = 16, 
		  MA_Y  = 8,
		  MA_TX = 4, 
		  MA_TY = 2, 
		  MA_PE = 1};
  
  RcMatchFlag();  // default constructor make flag = 11111;
  RcMatchFlag(int iflag);

  operator int() const;  // conversion of integer
  RcMatchFlag & operator = ( int iflag);
  RcMatchFlag & operator = ( const RcMatchFlag & flag);


  int Nvariables() const;

  inline bool ifmarked_Z()  const;
  inline bool ifmarked_X()  const;
  inline bool ifmarked_Y()  const;
  inline bool ifmarked_TX() const;
  inline bool ifmarked_TY() const;
  inline bool ifmarked_PE() const;
  
  inline void mark_Z();
  inline void mark_X();
  inline void mark_Y();
  inline void mark_TX();
  inline void mark_TY();
  inline void mark_PE();
  
  inline int bitvalue() const;  // using of this function could be confusing

private:
  int _bitvalue; // the internal presentation is done by bits 
  
};

ostream & operator << (ostream &, const RcMatchFlag &);

#include "marple/RcMatchFlag.icc"

#endif //_RCMATCHFLAG
