
// Boris Fominykh 5.01.2000
// allignment correction of the muon detector geometry

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <math.h>
#include <iostream.h>
#include <iomanip.h>

#include "arte/arte.hh"
#include "arte/ArteMessage.hh"

#include "murec/MuDbAlign.h"

extern "C" {
  void MuNede2Gede();
  int MuNameDecode(char*,int*,int*,int*,int*,int*,int*);
}

extern "C" {
int MuAlignment(int KTstatus,int run,char* DBname)
{
  MuNede2Gede();

  MuDbAlign Align;
  Align.setDBname(DBname);
  Align.setRun(run);
  Align.setMinor(0);
  Align.setKTstatus(KTstatus);
  int ret=Align.Read();
  if(ret != 0) {
    arteMsg << "MuAlignment: alignment constants are not read, ret= " << ret << endl;
    arteMsg.warn();
    return -1;
  }
  int Nframe=Align.getNframe();
  if(Nframe <= 0) {
    arteMsg << "MuAlignment: no action, Nframe= " << Nframe << endl;
    arteMsg.warn();
    Align.Release();
    return -2;
  }

// loops over frames
  MuAlign AC;
  for(int i=0; i<Nframe; i++) {
    int nfr = i;
    ret=Align.Get(&nfr,&AC);
    if(ret != 0) {
      arteMsg << "MuAlignment: no data for frame " << nfr << " ret= " << ret << endl;
      arteMsg.warn();
      break;
    }
//    AC.Print();
// apply the corrections, loops over GESL, GEDE
    for ( ArteTableIterator<GESL> isl = ArteTable<GESL>::begin(); 
	  isl != ArteTable<GESL>::end();
	  ++isl ) {
      int cmp  = (*isl)->cmp;
      if((cmp == 8 && AC.type == 1) || (cmp == 9 && AC.type == 2)) {
        int msl  = (*isl)->nsl;  // # muon superlayer
        if(msl == AC.msl) {
          int iv1 = (*isl)->fpl;
          int iv2 = iv1 + (*isl)->npl - 1;
          if(iv2 >= iv1) {
// loops over muon gede
            for ( int iv=iv1; iv<=iv2; iv++) { 
              ArtePointer<GEDE> gede(iv);
// chamber name
              char name[10];
              strncpy(name,gede->hwc,9);
// decode the chamber name
              int type  = 0;
              int msld  = 0;
              int lay  = 0;
              int ud   = 0;
              int nx    = 0;
              int ncab  = 0;
              int ret=MuNameDecode(&name[0],&type,&msld,&lay,&ud,&nx,&ncab);
              if(ret == 0 && msl == msld) {
                lay-=1;
                if(type == 4) lay=0;
// check nx
                int check=0;
                if(ud == 0 && lay < AC.nlay) {
                  if(nx >= AC.nx1u[lay] && nx <= AC.nx2u[lay]) check=1;
                  if(nx == 99 && AC.nx99u[lay] == 1) check=1;
                }
                if(ud == 1 && lay < AC.nlay) {
                  if(nx >= AC.nx1d[lay] && nx <= AC.nx2d[lay]) check=1;
                  if(nx == 99 && AC.nx99d[lay] == 1) check=1;
                }
// apply corrections
                if(check == 1) {
// update the angle
                  float angl = atan2(gede->sin,gede->cos);
                  float anglc= angl + AC.angl/57.29578;
                  gede->sin = sin(anglc);
                  gede->cos = cos(anglc);
// update the coordinates
                  float dx = AC.dx*(gede->cos) - AC.dy*(gede->sin);
                  gede->xmin += dx;
                  gede->sxmn += dx;
                  gede->xmax += dx;
                  gede->sxmx += dx;

                  float dy = AC.dx*(gede->sin) + AC.dy*(gede->cos);
                  gede->ymin += dy;
                  gede->symn += dy;
                  gede->ymax += dy;
                  gede->symx += dy;

                  gede->zmin += AC.dz;
                  gede->zmax += AC.dz;
                }
              }
            }
          }
        }
      }
    }

  }  
  Align.Release();
  if(KTstatus > 0)  return 1;
  else              return 2;
}
}


