
//## Boris Fominykh  26.11.2001
// member functions of class MuGetPadEff

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <fstream.h>
#include <iostream.h>
#include <iomanip.h>

#include "arte/arte.hh"
#include "arte/ArteMessage.hh"

#include "murec/MuGetPadEff.h"

extern "C" int MuNameDecode(char*,int*,int*,int*,int*,int*,int*);

//** Constructor */
MuGetPadEff::MuGetPadEff()
{
  Ready = -1;
  clear();
}

//** Destructor */
MuGetPadEff::~MuGetPadEff()
{
}

//## Methods

void MuGetPadEff::clear()
{
  for(int sl=0; sl<NslG; sl++)
  {
    for(int ud=0; ud<NudG; ud++)
    {
      for(int nch=0; nch<NchG; nch++)
      {
        EffP[sl][ud][nch] = -1.;
        for(int sd=0; sd<NsdG; sd++)
        {
          EffT[sl][ud][nch][sd] = -1.;
        }
      }
    }
  }
  return;
}


// get pad efficiency
float MuGetPadEff::getP(char* name)
{
// decode the chamber name
  int type  = 0;
  int sl  = 0;
  int lay  = 0;
  int ud   = 0;
  int nx    = 0;
  int ncab  = 0;
  int ret=MuNameDecode(name,&type,&sl,&lay,&ud,&nx,&ncab);
  if(ret == 0 && nx == 99) nx=NchG;
  if(ret != 0 || sl < 3 || sl > 4 || ud < 0 || ud > 1 || nx < 1 || nx > NchG)
  {
    // cout << "MuGetPadEff::getP: ret= " << ret << " sl= " << sl << " ud= " << ud;
    // cout << " nx= " << nx << " effP= " << EffP[sl-3][ud][nx-1] << endl;  
    return -1.;   
  }
  if(EffP[sl-3][ud][nx-1] < 0.)
  {
    // cout << "MuGetPadEff::getP: " << " sl= " << sl << " ud= " << ud;
    // cout << " nx= " << nx << " effP= " << EffP[sl-3][ud][nx-1] << endl;  
    return -2.;   
  }
  return EffP[sl-3][ud][nx-1];
}


// get tube efficiency
float MuGetPadEff::getT(char* name,int sd)
{
// decode the chamber name
  int type  = 0;
  int sl  = 0;
  int lay  = 0;
  int ud   = 0;
  int nx    = 0;
  int ncab  = 0;
  int ret=MuNameDecode(name,&type,&sl,&lay,&ud,&nx,&ncab);
  if(ret == 0 && nx == 99) nx=NchG;
  if(ret != 0 || sl < 3 || sl > 4 || ud < 0 || ud > 1 || nx < 1 || nx > NchG || sd < 0 || sd > 1)
  {
    // cout << "MuGetPadEff::getT: ret= " << ret << " sl= " << sl << " ud= " << ud;
    // cout << " nx= " << nx << " sd= " << sd << endl;  
    return -1.;   
  }
  if(EffT[sl-3][ud][nx-1][sd] < 0.)
  {
    // cout << "MuGetPadEff::getT: " << " sl= " << sl << " ud= " << ud;
    // cout << " nx= " << nx << " sd " << sd << " effT= " << EffT[sl-3][ud][nx-1][sd] << endl;  
    return -2.;   
  }
  return EffT[sl-3][ud][nx-1][sd];
}


// print the contents of class
void MuGetPadEff::print()
{
  char* fname = "MuGetPadEff.out";
  ofstream fout(fname);
  if(!fout) 
  {
    arteMsg << "MuGetPadEff::Print: cannot open file " << fname << endl;
    arteMsg.warn();
    return;
  }
  fout << "MuGetPadEff::Print:" << endl; 

  for(int sl=0; sl<NslG; sl++) 
  {
    for(int ud=0; ud<NudG; ud++) 
    {
      fout << endl;
      fout << "Nsl " << dec << sl+3 << " Ud " << ud << endl;
      for(int nch=0; nch<NchG; nch++)
      { 
        int nchpr = nch+1;
        if(nch == NchG-1) nchpr=99;
        if(EffP[sl][ud][nch] >= 0.)
	{
          fout << " Nch " << nchpr << " padEff " << EffP[sl][ud][nch];
          for(int sd=0; sd<NsdG; sd++) 
          {
            fout << " sd " << sd << " tubeEff " << EffT[sl][ud][nch][sd];
          }
          fout << endl;
        }
      }
    }
  }
  fout << endl;
  fout << "END" << endl;
  return;
}


// read the file
int MuGetPadEff::read(char* filename)
{
  FILE *hl = fopen(filename, "r");
  if(!hl) {
    arteMsg << "MuGetPadEff::read - no input file " << filename << endl;
    arteMsg.warn();
    return 1;
  }
// loops over lines: max 10000 lines
  char line[128];
  for(int il=0; il<10000; il++) 
  {
// read the line
    if(fgets(line,sizeof(line),hl) == NULL) goto goodexit;
// end of file
    if(line[0] == 'E' & line[1] == 'N' & line[2] == 'D') goto goodexit;
// pass the comments
    if(line[0] == 'M')
    {
      float efft1;
      float efft2;
      float padeff;
      char name[20];
      if (sscanf(line,"%s PAD %f TUBE1 %f TUBE2 %f",name,&padeff,&efft1,&efft2) == 4)
      {
        name[8] = '\0';
// decode the chamber name
        int type  = -1;
        int sl    = -1;
        int lay   = -1;
        int ud    = -1;
        int nx    = -1;
        int ncab  = -1;
        int ret=MuNameDecode(name,&type,&sl,&lay,&ud,&nx,&ncab);
        if(nx == 99) nx=NchG;
        if(ret != 0 || sl < 3 || sl > 4 || ud < 0 || ud > 1 || nx < 1 || nx > NchG)
        {
	  arteMsg << "MuGetPadEff::read: not decorded " << name << endl;
          arteMsg.warn();
          return 2;   
        }
        EffP[sl-3][ud][nx-1]    = padeff;
        EffT[sl-3][ud][nx-1][0] = efft1;
        EffT[sl-3][ud][nx-1][1] = efft2;
      }
      else goto goodexit;
    }
  }
goodexit:
  fclose(hl);
  Ready = 1;
  return 0;
}
