/************************************************************************
*
* MuMCmask.C
* 
* first author : Jing-Ge Shiu 
* purpose      : C++/SLT code for masking MUON MC hits 
* called from  : dgmuoo.F, dgmupix.icc
* includ file  : MuHitInfo.h, MUDbAll.h
* return value : 0 = masked (channel deactivated),  1 = keep it (channel is activated)
* start date   : May. 01, 2000
* 
* 
* release note : 2000, Oct. 17 physjg
*                so far, it works only with the manually set mask file in the kumac file.
*                the user has to set the following two parameters in the kumac file :
*
*                SETENV MUON_BADFILE [maskfilename]
*                SETENV MUON_CONFIG  $HBROOT/MUON/db/Mu.config
*
*                the examplea for the maskfile format can be found at $HBROOT/MUON/db/Mu.mask*
*
*
*-----------------------------------------------------------------------
*
* ver 0.00     : 0505, 2000 physjg
*                It works for tube/pad readout now, but 99pad chambers need more check
*                Notice the cable maskig mapping from the DbPad file to the readout channels 
*                is not same as for the Nbit1 cable mapping 
* ver 0.01     : 0514, 2000 physjg                 
*                Including the mask for pixel chambers.
*
* ver 0.02     : 1014, 2000 physjg
*                check the result of initialization of MuDbAllIni.  
*                there will be no mask if it failed.
*
*
*
*
*
*
*
**************************************************************************/
#include <iostream.h>
#include <iomanip.h>
#include "arte/arte.hh"
#include "arte/ArteMessage.hh"

#include "murec/MUDbAll.h"
#include "murec/MuHitInfo.h"

static MUDbAll muAllConfig;
static MuHitInfo muhit;

// initialization of the objects
int MuDbAllIni(int KTstatus,int run,char* DBname) 
{
  static int ret=0;
  static short started = 0;

  if (started==0) {
    ret = muAllConfig.ini();
    started++;
    if (ret!=0)  { 
       arteMsg << "MuMCmask : muAllConfig initialization failed, no mask applied." << endl; 
       arteMsg.warn(); 
    }
  }

  return ret;
}



 
extern "C" {
  
void mumcmask2_(ArtePointer<GEDE> pgede, int& rtype, int& channelno, int &chmask) { 

// updated 3.11.2000 by B.Fominykh
  chmask=1;

  if((getenv("MUON_BADFILE") == NULL) ||
     (getenv("MUON_CONFIG")  == NULL)) return;

  char* Dbname = "/MUON_DB";
  MuDbAllIni(-3,0,Dbname);

// updated 3.11.2000 by B.Fominykh

  int tst1=muAllConfig.MuDbTube::Ivalid;
  int tst2=muAllConfig.MuDbPad::Ivalid;
  int tst3=muAllConfig.MuDbPixel::Ivalid;
  if(tst1 != 1 || tst2 != 1 || tst3 != 1) return;

  if ( muhit.putmuhit(pgede, rtype, channelno)>MAXOKFLAG ) {
     chmask = 0;
     return;
  }
  
  Word mask = 0, mskb = 0;
  Word sl  = muhit.superl() - 1;
  Word lay = muhit.dsubl() - 1;
// commented 18.10.2000 by B.Fominykh
//  Word sd  = muhit.frontback();
  Word ud  = muhit.topbottom();
  Word dno = muhit.chamberid();
  Word nc  = ((muhit.readtype()==0)||(muhit.readtype()==2))?muhit.sec():
             ( ~( muhit.sec()+0x02*(((dno<19)&&(ud==1))||((dno>18)&&(ud==0))) ) )&0x03;
  if ((muhit.readtype()==1)&&(dno==99)) nc>>=1;   
	   /* ******************************************************** 
             for the masking parameters :
             -x-y, +x+y : readout cable A/B/C/D = Pad DB cable 1/0/3/2 
             -x+y, +x-y : readout cable A/B/C/D = Pad DB cable 3/2/1/0
      
             chamber 99 : A/C = 1/0 (  (3/1)>>1 = (1/0) ) 
	   ******************************************************** */
  Word nx  = ( (dno!=99)?dno:
               ( (sl>1)?muAllConfig.NchambP[sl-2][ud]
                       :muAllConfig.NchambT[sl][lay][ud] ) ) - 1;
  Word rno = muhit.channelid();
    
  switch (muhit.readtype()) {
  case 0 : // tube  readout : mask mapping from 0 to 15
           mask = (sl<2)?muAllConfig.MaskT[sl][lay][ud][nx][nc]
                        :muAllConfig.MaskP[sl-2][ud][nx][nc];
           mskb = ( muhit.channelid()>16?rno-16:rno ) - 1;
           break;

  case 1 : // pad   reacout : mask mapping from 0 to 15
           mask = muAllConfig.MaskPP[sl-2][ud][nx][nc];
           mskb = ( muhit.channelid()>16?rno-16:rno ) - 1;
           break;

  case 2 : // pixel readout : mask mapping from 0 to 15
           mask = muAllConfig.MaskX[sl][dno-1][nc-1];
           mskb = rno - 1;
           break;
  }

  chmask = mask & (0x8000 >> mskb);

  return;
}



void mumcmask_(int& ipl, int& rtype, int& channelno, int &chmask) { 

// updated 3.11.2000 by B.Fominykh
  chmask=1;
  if((getenv("MUON_BADFILE") == NULL) ||
     (getenv("MUON_CONFIG")  == NULL)) return;

  ArtePointer<GEDE> pgede = ArtePointer<GEDE>(ipl);
  mumcmask2_(pgede, rtype, channelno, chmask);

  return;

}



}








