// 1) The class BAD_NOISY_Cells_Marker defined here allows to create "menu"
// which reads the "hit map" High-Pt file and marks the cells as Ok/BAD/NOISY.
// Typical usage can be found in HPT_DAQ_Abstract.C
//
// 2) To write the "hit-map" file in the same format one can use
// write_cell_states(AnySetup& any_setup, ostream& os);
// or write_cell_states(Detector& det, const string& setup_name, ostream& os);
// 

// Author: Vladislav Balagura (balagura@hera-b.desy.de, 03.11.1999)

#ifndef _io_cell_states_hh
#define _io_cell_states_hh 1

#include "ptset/detector.hh"
#include "ptset/high_pt.hh"
#include "ptset/delimit.hh"
#include <iostream>
#include <iomanip.h>

template <class AnyISection>
class BAD_NOISY_Cells_Marker {
  Fired_Cell::State state;
public:
  BAD_NOISY_Cells_Marker(Fired_Cell::State fState) : state(fState) {
    if ( state != Fired_Cell::BAD && state != Fired_Cell::NOISY ) {
      string tmp;
      tmp << "Wrong state is given: " << int(state) 
	  << " (neither BAD nor NOISY)";
      global_error()(tmp);
      global_error() << "BAD_NOISY_Cells_Marker: program error\n";
    }
  }
  void operator()(istream* is, AnyISection isection) {
    Index ind;
    typedef typename iterator_traits<AnyISection>::value_type AnySection;
    AnySection& section = (*isection);
    typedef typename AnySection::ICell AnyICell;
    AnyICell icell;
    while (1) {
      *is >> ind;
      if ( is->eof() ) return;
      else if ( !(*is) ) { global_error()("can not read Index"); return; }
      if ( (icell = section.find(ind)) == section.end() ) {
	string tmp;
	tmp << "there is no cell with Index " << ind;
	global_error()(tmp);
	return;
      }
      (*icell).second.state = state;
    }	
  }
};

// For one setup
template <class AnySetup>
inline void write_cell_states(AnySetup& setup, ostream& os) {
  for (typename AnySetup::IChamber ich=setup.begin(); ich!=setup.end();
       ++ich) {
    typename AnySetup::Chamber& chamber = (*ich).second;
    os << "  /chamber{ " << (*ich).first << '\n';
    for (typename AnySetup::ISection is=chamber.begin(); is!=chamber.end();
	 ++is) {
      typename AnySetup::Section& section = (*is);
      os << "    /section{ " << is - chamber.begin() << '\n';
      {
	os << "      /BAD{   ";
	Delimiter delim("  ", 5, "\n              ");
	for (typename AnySetup::ICell ic=section.begin(); ic!=section.end();
	     ++ic) {
	  typename AnySetup::Cell& cell = (*ic).second;
	  if (cell.state == Fired_Cell::BAD) {
	    Index ind = (*ic).first;
	    os << '[' << setw(3) << ind.ix << ',' << setw(3) << ind.iy
	       << ']' << delim;
	  }
	}
	os << "}\n\n";
      }
      {
	os << "      /NOISY{ ";
	Delimiter delim("  ", 5, "\n              ");
	for (typename AnySetup::ICell ic=section.begin(); ic!=section.end();
	     ++ic) {
	  typename AnySetup::Cell& cell = (*ic).second;
	  if (cell.state == Fired_Cell::NOISY) {
	    Index ind = (*ic).first;
	    os << '[' << setw(3) << ind.ix << ',' << setw(3) << ind.iy
	       << ']' << delim;
	  }
	}
	os << "}\n";
      }
      os << "    }\n";
    }
    os << "  }\n";
  }
}

// For all setups in the detector 
void write_cell_states(Detector& det, const string& setup, ostream& os);

#endif
