/* This file is analogous to istream_menu.hh. It defines IstreamMenu_1Par:
   InteractiveMenu_1Par + parser.

    IstreamMenu_1Par {
      PtrWrapper<InteractiveParser> parser;
      void run_one_command(istream* is, const Par1& p1);
      void operator()(istream* is, const Par1& p1);
    };
   To start IstreamMenu_1Par menu; use operator()(istream* is, const Par1& p1);
   it goes up to the end of the menu. If parser returns EOF_MESSAGE in
   global_error() - global_error() is cleared. 
   To run only one command: use run_one_command(istream* is, const Par1& p1);
*/
   
// Author V.Balagura (balagura@mail.desy.de)   (23.04.98).

#ifndef _istream_menu_1par_hh
#define _istream_menu_1par_hh 1

#include "ptset/interactive_1par.hh"
#include "ptset/interactive_parser.hh"

/* -------------------- IstreamMenu_1Par -------------------- */
template <class Par1>
class IstreamMenu_1Par : public InteractiveMenu_1Par<Par1> {
 public:
  PtrWrapper<InteractiveParser> parser;
  void run_one_command(istream* is, const Par1& p1);
  void operator()(istream* is, const Par1& p1);
  /* gcc 2.7.2.3.f.1 compiler bug:
     void operator()(istream* is, const Par1& p1) shadows 
     PtrWrapper<InteractiveItem_1Par<Par1> >& operator()(const string& s)
     from parent class InteractiveMenu_1Par<Par1>. 
     To get around this define it once more: */
  PtrWrapper<InteractiveItem_1Par<Par1> >& operator()(const string& s) { 
    return InteractiveMenu_1Par<Par1>::operator()(s); 
  }
};

template <class Par1>
inline void IstreamMenu_1Par<Par1>::run_one_command(istream* is, 
						    const Par1& p1) {
  InteractiveParser::Output input = parser()->get_input(is);
  if ( global_error() ) return;
  InteractiveMenu_1Par<Par1>::operator()(*input.commands, input.arguments, p1);
}
template <class Par1>
inline void IstreamMenu_1Par<Par1>::operator()(istream* is, const Par1& p1) {
  do { run_one_command(is, p1); } while ( !global_error() );
  if ( global_error()() == InteractiveParser::EOF_MESSAGE )
    global_error().clear();
}

#endif
