/* Some examples of functions defining the figures. All of them are placed
   for convinience as static functions in the empty class 
   SetupFigureExamples.

   Now includes:
   rectangle ....................... Rectangle
   rectangle_with_hole               Rectangle outer, Rectangle inner
   rectangle_center_at_0 ........... Point p_max
   rectangle_with_hole_center_at_0   Point upper_right_hole_corner,
				     Point upper_right_section_corner

   rectangle_with_hole_center_at_0_bite_at_bottom - 
                                     Point upper_right_hole_corner,
				     Point upper_right_section_corner,
				     double xmin_bite, double xmax_bite, 
				     double ymax_bite
   cross ........................... Rectangle inner_corners
   cross_center_at_0                 Point p_corner
   plus  ........................... Rectangle inner_corners, 
                                     Rectangle bounding_box
   plus_center_at_0 ................ Point inner_corner, 
                                     Point bounding_box_corner
   plus_with_hole_center_at_0 ...... Point hole_corner,
				     Point inner_plus_corner, 
				     Point bounding_box_corner
   rotated_H_with_hole_center_at_0.. Point hole_corner,
				     Point inner_H_corner, 
				     Point bounding_box_corner
   symmetric_plus_center_at_0 ...... double inner_corner_x, 
				     double bounding_box_corner_x
   up_down_symmetric_rectangles_center_at_0
                                     double corner_x, 
				     double low_y, double high_y

   C_half_at_0                       Rectangle rect, 
                                     double holeX, 
				     double holeY1, double holeY2
   mirror_C_half_at_0 .............. Rectangle rect, 
                                     double holeX, 
				     double holeY1, double holeY2
   */

// Author V.Balagura (balagura@mail.desy.de)   (09.04.98).

#ifndef _setup_figure_examples_hh
#define _setup_figure_examples_hh 1

#include "ptset/setup_figure.hh"

struct SetupFigureExamples {
  static void rectangle(Rectangle rect, Point p, bool *is_inside) {
    *is_inside = rect.contains(p);
  }
  static void rectangle_with_hole(Rectangle outer, Rectangle inner,
				  Point p, bool* is_inside) {
    *is_inside = outer.contains(p) && !inner.contains(p);
  }
  static void rectangle_center_at_0(Point p_max, Point p, bool* is_inside) {
    *is_inside = x_and_y_less(abs(p), p_max);
  }
  static void rectangle_with_hole_center_at_0
  (Point upper_right_hole_corner,
   Point upper_right_section_corner,
   Point p, bool* is_inside) {
    p = abs(p);
    *is_inside =  x_and_y_less(p, upper_right_section_corner) &&
      !x_and_y_less(p, upper_right_hole_corner);
  }
  static void rectangle_with_hole_center_at_0_bite_at_bottom
  (Point upper_right_hole_corner,
   Point upper_right_section_corner,
   double xmin_bite, double xmax_bite, double ymax_bite,
   Point p, bool* is_inside) {
    Point pabs = abs(p);
    *is_inside =  
      x_and_y_less(pabs, upper_right_section_corner) &&
      !x_and_y_less(pabs, upper_right_hole_corner) &&
      (p.x < xmin_bite || p.x > xmax_bite || p.y > ymax_bite);
  }
  static void cross(Rectangle inner_corners, Point p, bool* is_inside) {
    *is_inside =
      (inner_corners.a.x < p.x && p.x < inner_corners.b.x) ||
      (inner_corners.a.y < p.y && p.y < inner_corners.b.y);
  }
  static void cross_center_at_0(Point p_corner, Point p, bool* is_inside) {
    p = abs(p);
    *is_inside = !x_and_y_less(p_corner,  p);
  }
  static void plus(Rectangle inner_corners, Rectangle bounding_box,
		   Point p, bool* is_inside) {
    *is_inside = bounding_box.contains(p) &&
      ((inner_corners.a.x < p.x && p.x < inner_corners.b.x) ||
       (inner_corners.a.y < p.y && p.y < inner_corners.b.y));
  }
  static void plus_center_at_0(Point inner_corner, 
			       Point bounding_box_corner,
			       Point p, bool* is_inside) {
    p = abs(p);
    *is_inside = x_and_y_less(p, bounding_box_corner) &&
      !x_and_y_less(inner_corner, p);
  }
  static void plus_with_hole_center_at_0(Point hole_corner,
					 Point inner_plus_corner, 
					 Point bounding_box_corner,
					 Point p, bool* is_inside) {
    p = abs(p);
    *is_inside = 
      x_and_y_less(p, bounding_box_corner) &&
      !x_and_y_less(inner_plus_corner, p) &&
      !x_and_y_less(p, hole_corner);
  }
  static void rotated_H_with_hole_center_at_0(Point hole_corner,
					      Point inner_H_corner, 
					      Point bounding_box_corner,
					      Point p, bool* is_inside) {
    p = abs(p);
    *is_inside = 
      x_and_y_less(p, bounding_box_corner) &&
      (p.x < inner_H_corner.x || p.y > inner_H_corner.y) &&
      !x_and_y_less(p, hole_corner);
  }
  static void symmetric_plus_center_at_0(double inner_corner_x, 
					 double bounding_box_corner_x,
					 Point p, bool* is_inside) {
    p = abs(p);
    if ( p.x > p.y )
      *is_inside = p.x < bounding_box_corner_x && p.y < inner_corner_x;
    else 
      *is_inside = p.y < bounding_box_corner_x && p.x < inner_corner_x;
  }
  static void up_down_symmetric_rectangles_center_at_0(double corner_x, 
						       double low_y, 
						       double high_y,
						       Point p, 
						       bool* is_inside) {
    p = abs(p);
    *is_inside = p.x < corner_x && p.y > low_y && p.y < high_y;
  }
  static void C_half_at_0(Rectangle rect, double holeX, double holeY1, 
			  double holeY2, Point p, bool* is_inside) {
    *is_inside = rect.contains(p) &&
      ( p.x < holeX ||  p.y < holeY1 || p.y > holeY2 );
  }
  static void mirror_C_half_at_0(Rectangle rect, 
				 double holeX, double holeY1,
				 double holeY2, Point p, bool* is_inside) {
    *is_inside = rect.contains(p) &&
      ( p.x > holeX ||  p.y < holeY1 || p.y > holeY2 );
  }
};

#endif
