// 
//  Author:  Wouter Hulsbergen                 Created:  14-Aug-00
// 
// Description: rangerHit class for rich rings
//
// Modification:
// - Added plane routine which finds `dummy' rich plane needed 04/07/01 S.Lanyov
//   for proper insertion of walls. (See also ECClus.C)
//

#include <iostream>
#include <math.h>
#include <arte/arte.hh>
#include <arte/ArteMessage.hh>
#include "rang/StateP.hh"
#include "rang/StatePI.hh"
#include "rang/StateL.hh"
#include "rang/StateLI.hh"
#include "rang/RichRing.hh"
#include "rang/ScatThick.hh"
#include "gean/ZGESL.h"

//-- Constructor
rangerRichRing::rangerRichRing() : fRseg(0), fRadThick(0) 
{
  cout << "This is the rangerRichRing default constructor" << endl;
}

rangerRichRing::rangerRichRing(ArtePointer<RSEG> rseg,rangerFitMode fm,
			       const float radTh) 
  : rangerHitBase(rseg,fm,0), fRseg(rseg), fRadThick(radTh)
{
#ifdef FULL
  cout << "This is the rangerRichRing(rseg) constructor, rseg=" << rseg << endl ;
#endif
  m = new HepVector(dim());
  (*m)(1) = rseg->txf ;  // rich tx
  (*m)(2) = rseg->tyf ;  // rich ty
  //(*m)(3) = rseg->time ; // rich cherenkov angle
   V = new HepSymMatrix(dim());
  (*V)(1,1) = rseg->cvf[5];
  (*V)(1,2) = rseg->cvf[8];
  //(*V)(1,3) = rseg->cvf[12];
  (*V)(2,2) = rseg->cvf[9];
  //(*V)(2,3) = rseg->cvf[13];  
  //(*V)(3,3) = rseg->cvf[14];
}

rangerRichRing::~rangerRichRing()
{
  delete m;
  delete V;
}

void rangerRichRing::projectP(HepVector& resid,
			      HepMatrix& HH,rangerStateP* s) 
{
#ifdef FULL
  cout << "This is rangerRichRing::projectP" << endl;
#endif
  resid(1) = (*m)(1) - s->tx();
  resid(2) = (*m)(2) - s->ty();
  //resid(3) = 0 ;

  HH(1,1) = 0;
  HH(1,2) = 0;
  HH(1,3) = 1;
  HH(1,4) = 0;
  HH(1,5) = 0;

  HH(2,1) = 0;
  HH(2,2) = 0;
  HH(2,3) = 0;
  HH(2,4) = 1;
  HH(2,5) = 0;
}

void rangerRichRing::projectPI(HepVector& resid,
			     HepMatrix& HH,rangerStatePI* s) 
{
#ifdef FULL
  cout << "This is rangerRichRing::projectP" << endl;
#endif
  resid(1) = (*m)(1) - s->tx();
  resid(2) = (*m)(2) - s->ty();
  //resid(3) = 0 ;

  HH(1,1) = 0;
  HH(1,2) = 0;
  HH(1,3) = 1;
  HH(1,4) = 0;
  HH(1,5) = 0;
  HH(1,6) = 0;

  HH(2,1) = 0;
  HH(2,2) = 0;
  HH(2,3) = 0;
  HH(2,4) = 1;
  HH(2,5) = 0;
  HH(2,6) = 0;
}


void rangerRichRing::projectL(HepVector& resid,
			     HepMatrix& HH,rangerStateL* s)
{
#ifdef FULL
  cout << "This is rangerRichRing::projectL" << endl;
#endif
  resid(1) = (*m)(1) - s->x();
  resid(2) = (*m)(2) - s->y();
  //resid(3) = 0;

  HH(1,1) = 0;
  HH(1,2) = 0;
  HH(1,3) = 1;
  HH(1,4) = 0;

  HH(2,1) = 0;
  HH(2,2) = 0;
  HH(2,3) = 0;
  HH(2,4) = 1;
}



void rangerRichRing::projectLI(HepVector& resid,
			     HepMatrix& HH,rangerStateLI* s) 
{
#ifdef FULL
  cout << "This is rangerRichRing::projectLI" << endl;
#endif
  resid(1) = (*m)(1) - s->x();
  resid(2) = (*m)(2) - s->y();
  //resid(3) = 0;

  HH(1,1) = 0;
  HH(1,2) = 0;
  HH(1,3) = 1;
  HH(1,4) = 0;
  HH(1,5) = 0;

  HH(2,1) = 0;
  HH(2,2) = 0;
  HH(2,3) = 0;
  HH(2,4) = 1;
  HH(2,5) = 0;
}

int rangerRichRing::plane()
{
  // There is no gede/gwal related to an RICH cluster. Still we would
  // like to have a plane entry, since rangerTrack::insertWalls needs
  // this to find in-between-planes. Therefore, I invented yet another
  // `hack': Just return the first gwal related to the RICH:
  static int thisplane=0;
  if( thisplane==0 ) {
    for(ArteTable<GESL>::iterator igesl = ArteTable<GESL>::begin();
        igesl != ArteTable<GESL>::end() && thisplane==0; ++igesl ) 
      if( igesl->cmp == LGESL_RIC ) {
        ArtePointer<GWAL> firstwall = ArtePointer<GWAL>(igesl->fwl);
        thisplane = -firstwall.index();
      }
  }
  return thisplane;
}
