// 
//  Author:  Rainer Mankel                     Created:  29-Sep-98
//           Humboldt University Berlin        Modified:  
//  Email:   mankel@ifh.de                     HERA-B Collaboration
// 
// Modifications:
// - Adapted to `withinBox' scheme      16-Jul-01 Wouter Hulsbergen

#include <iostream>
#include <algorithm>
#include <math.h>
#include "rang/VolBox.hh"
#include "rang/Arrow.hh"

//-- Constructor
rangerVolBox::rangerVolBox(){};

//-- Constructor
rangerVolBox::rangerVolBox( rangerGeWa* gewa ) : rangerVolume( gewa ) 
{
  // initialize coordinates in ACS
  // calculate the reference coordinates in ACS and PCS
  // (tx, ty not used)
  float zP = 0.5*(Gewa->zmin() + Gewa->zmax()) ;
  float xP = 0.5*(Gewa->xmin() + Gewa->xmax()) ;
  float yP = 0.5*(Gewa->ymin() + Gewa->ymax()) ;
  rangerArrow a(zP,xP,yP,0.001,0.001) ;
  rotateArrow(a,-1) ;
  zA = a.z ;

  // we take the four corners, rotate them and determine the extreme
  // coordinates in the ACS
  xA1=yA1= 1.e9 ;
  xA2=yA2=-1.e9 ;
  for(int i=0; i<4; ++i) {
    xP = (i/2) ? Gewa->xmin() : Gewa->xmax() ;
    yP = (i%2) ? Gewa->ymin() : Gewa->ymax() ;
    rangerArrow a(zP,xP,yP,0.001,0.001) ;
    rotateArrow(a,-1) ;
    xA1 = min(xA1,float(a.x)) ;
    xA2 = max(xA2,float(a.x)) ;
    yA1 = min(yA1,float(a.y)) ;
    yA2 = max(yA2,float(a.y)) ;
  }
//   cout << "rangerVolBox: initialize: " << zA << " "
//        << xA1 << " " << xA2 << " " << yA1 << " " << yA2 << endl ;
}


//-- Destructor
rangerVolBox::~rangerVolBox(){};

float rangerVolBox::sLength(float z,float x,float y,
			    float tx,float ty) const {
  float slen=0 ;
  rangerArrow a(z,x,y,tx,ty);
  rotateArrow(a);
  float zP = 0.5*(Gewa->zmin() + Gewa->zmax()) ;
  float xP = a.x + (zP - a.z) * a.tx;  // coordinates in PCS
  float yP = a.y + (zP - a.z) * a.ty;  // see Hera-B 97-090 p 67
  if( Gewa->xmin() <= xP && xP <= Gewa->xmax() &&
      Gewa->ymin() <= yP && yP <= Gewa->ymax() )
    slen = sqrt(1 + a.tx * a.tx + a.ty * a.ty);
  return slen ;
}

