// 
//  Author:  Rainer Mankel                     Created:  29-Sep-98
//           Humboldt University Berlin        Modified:  
//  Email:   mankel@ifh.de                     HERA-B Collaboration
// 
// Modifications:
// - Adapted to `withinBox' scheme      16-Jul-01 Wouter Hulsbergen

#include <iostream>
#include "rang/VolCone.hh"
#include "rang/Arrow.hh"

//-- Constructor
rangerVolCone::rangerVolCone(){};

//-- Constructor
rangerVolCone::rangerVolCone( rangerGeWa* gewa ) : rangerVolume( gewa ) 
{
  // calculate the reference coordinates in ACS and PCS
  // Ths is certainly not yet correct (WH, July 2001)
  zA = 0.5*(Gewa->zmin() + Gewa->zmax()) ;
  xA1 = yA1 = -1.e5 ;
  xA2 = yA2 =  1.e5 ;
}

//-- Destructor
rangerVolCone::~rangerVolCone(){};

float rangerVolCone::sLength(float z,float x,float y,
			     float tx,float ty) const {
  
  rangerArrow a(z,x,y,tx,ty) ;
  rotateArrow(a) ;

  float zP = 0.5*(Gewa->zmin()+Gewa->zmax()) ;
  float xP = a.x + (zP - a.z) * a.tx - Gewa->ymin();  // coordinates in PCS
  float yP = a.y + (zP - a.z) * a.ty - Gewa->ymax();  // see Hera-B 97-090 p 67

  // this intersection finder is still simple but maybe a start
  float rL2 = xP * xP + yP * yP;

  if (rL2 < (Gewa->xmin()*Gewa->xmin())) return 0;
  if (rL2 > (Gewa->xmax()*Gewa->xmax())) return 0;
  return sqrt(1 + a.tx*a.tx + a.ty*a.ty) ;
}


