// 
//  Author:  Rainer Mankel                     Created:  29-Sep-98
//           Humboldt University Berlin        Modified:  
//  Email:   mankel@ifh.de                     HERA-B Collaboration
// 
// Modifications:
// - Introduced ::filterOhneAbsorb, ::predictGlobal and ::absorbGlobal
//                                         14-Aug-00 Wouter Hulsbergen 

// class rangerHitBase
// abstract node base class for all measurements

#ifndef rangerHitBase_HH
#define rangerHitBase_HH

#ifndef rangerNode_HH
#include "rang/Node.hh"
#endif

class HepMatrix ;
class HepSymMatrix ;
class HepVector ;

class rangerHitBase : public rangerNode {

public:

  rangerHitBase();
  rangerHitBase(ArtePointer<RHIT> rhit,rangerFitMode fm) ;
  rangerHitBase(ArtePointer<HITB> hitb,rangerFitMode fm) ;
  rangerHitBase(ArtePointer<RCCL> rccl,int charge,rangerFitMode fm) ;
  rangerHitBase(ArtePointer<RSEG> rseg,rangerFitMode fm,unsigned char side) ;
  virtual ~rangerHitBase();
  
  virtual float msPlain() const { return 0; }
  virtual float dedx() const { return 0; }

  virtual void processNoise(rangerParticleType pType);  // apply process noise to state of this node
  virtual void predProcessNoise(rangerParticleType pType);  // apply process noise to prediction
  virtual void projectP(HepVector& resid,HepMatrix& HH,rangerStateP* s)=0;
  virtual void projectPI(HepVector& resid,HepMatrix& HH,rangerStatePI* s)=0;
  virtual void projectL(HepVector& resid,HepMatrix& HH,rangerStateL* s)=0;
  virtual void projectLI(HepVector& resid,HepMatrix& HH,rangerStateLI* s)=0;
  virtual int filter();              // filter a node from scratch
  virtual int filter(rangerNode* n, rangerParticleType pType); // filter a node using state from other mode
  int absorb();              // update a node by absorbing predicted state
  virtual void residUpdate();        // update residual and chiSq
  virtual void residUpdSmooth(rangerNode* n,HepMatrix& A);

  int filterOhneAbsorb(const rangerState* state, const double qPExp) ;
  int filterOhneAbsorb(rangerNode* n,rangerParticleType pType = MUON) ;
 
  virtual int dim() const=0;       // dimension of measurement
  virtual const double getChiSq() const { return chiSq; };
  const HepVector* getMeasurement() const { return m ; } ;
  const HepVector* getResidual() const { return r ; } ;
  const HepSymMatrix* getCovResidual() const { return R ; }
  const HepSymMatrix* getCovMeasurement() const { return V ; } ;

  virtual int projectGlobal(const rangerState*) ;
  virtual int absorbGlobal(HepVector*,HepSymMatrix*) const ;

protected:
  HepVector* m;       // measurement
  HepSymMatrix* V;    // cov matrix of measurement
  HepMatrix* H;       // projection matrix
  HepVector* r;       // residual
  HepSymMatrix* R;    // cov matrix of residual
  double chiSq;       // chiSquare contribution

private:

};

#endif
