// 
//  Author:  Rainer Mankel                     Created:  29-Sep-98
//           Humboldt University Berlin        Modified:  
//  Email:   mankel@ifh.de                     HERA-B Collaboration
// 
// Modifications:
// - Introduction of ::makeNode for RichRing/Rseg
// - Correction of smoothing of residuals
// - Prototypes for predictGlobal and absorbGlobal
//                                      14-Aug-00 Wouter Hulsbergen

// class rangerNode
// node base class

#ifndef rangerNode_HH
#define rangerNode_HH

#ifndef __ARTE_HH__
#include "arte/arte.hh"
#endif
#ifndef _Matrix_H_
#include "CLHEP/Matrix/Matrix.h"
#endif
#ifndef rangerState_HH
#include "rang/State.hh"
#endif
#ifndef RANGER_PARTYPE_HH
#include "rang/ParticleType.hh"
#endif
#ifndef RANGER_FMODE_HH
#include "rang/FitMode.hh"
#endif

class HepMatrix ;

class rangerNode {

public:
  rangerNode();
  rangerNode(rangerFitMode fm) ;
  rangerNode(ArtePointer<RHIT> rhit,rangerFitMode fm) ;
  rangerNode(ArtePointer<HITB> hitb,rangerFitMode fm) ;
  rangerNode(ArtePointer<RCCL> rccl,int charge,rangerFitMode fm) ;
  rangerNode(ArtePointer<RSEG> rseg,rangerFitMode fm,unsigned char side) ;
  virtual ~rangerNode();

  static rangerNode* makeNode(ArtePointer<RHIT>& rhit,
			      rangerFitMode fm); // create new node
  static rangerNode* makeNode(ArtePointer<RCCL>& rccl,
			      int charge,rangerFitMode fm);
  static rangerNode* makeNode(ArtePointer<RSEG>& rseg,rangerFitMode fm,
			      unsigned char side);
  static rangerNode* makeNode(const ArtePointer<GEDE>& ge,
			      rangerNode* n,const float path=1); // create new node
  static rangerNode* makeNode(const ArtePointer<GWAL>& gw,
			      rangerNode* n,const float path=1); // create new node

  virtual int filter();              // filter a node from scratch
  virtual int filter(rangerNode* n,
		     rangerParticleType pType); // filter a node 
                                     //using state from other mode

  int smooth(rangerNode*n);  // smooth a node using information of following node
  virtual void residUpdSmooth(rangerNode*, HepMatrix&) {} // update residual
  virtual void residUpdate() {} // update residual
  int predict(const double& zDestination,    // z of prediction plane
	      rangerState*& predState,       // result of prediction
	      HepMatrix*& transpMatrix);    // resulting transport matrix
                             // calculate predicted state at z
                             // (forwarding function to state::predict)
  virtual void processNoise(rangerParticleType){};  
                             // apply process noise to state of this node
  virtual void predProcessNoise(rangerParticleType){};  
                             // apply process noise to prediction
  void list();
  void dump();
  virtual int dim() const { return 0; }  // dimension of measurement
  virtual int plane(){
    cout << "This is just the base plane()" << endl;
    return 0;
  }
  virtual float ms() const {
    cout << "This is just the base ms()" << endl;
    return 0;
  }
  //virtual float dedx() const {
  //  cout << "This is just the base dedx()" << endl;
  //  return 0;
  //}
  virtual float msPlain() const=0;
  virtual float dz(){ return 0; }   // thickness of plane
  float radiationThickness();
  float radiationThickEff();
  void covInit();
  virtual void storeRhit(ArtePointer<RHIT>){}; 
  virtual void mergeRhit(ArtePointer<RHIT>,float){};
  void storeRseg(ArtePointer<RSEG> rseg,int fl) { state->storeRseg(rseg,fl); };
  void storeRtra(ArtePointer<RTRA> rtra,int fl) { state->storeRtra(rtra,fl); };
  ArtePointer<RPNT> storeRpnt(ArtePointer<RPNT> rpnt)
                              { return state->storeRpnt(rpnt); };
  const rangerState* getPrediction() const { return predictedState; };
  const HepMatrix* getF() const { return transportMatrix; };
  const double& zNode() const { return z; };
  rangerState& getState() { return *state; } // accessor for state
  double getQPExp() const { return qPExpand; }
  virtual const double getChiSq() const { return 0; };
  virtual int hasHitb() { return 0;}
  virtual const ArtePointer<HITB> getHitb() const { return 0; };
  virtual void updInternal() {};   // update internal degrees of freedom
  
  virtual int projectGlobal(const rangerState*) { return 0;}
  virtual int absorbGlobal(HepVector*,HepSymMatrix*) const { return 0;} 
  
protected:
  double z;
  double qPExpand;                 // Q/p to be used as expansion point
  //rangerNode* nextNode;
  //rangerNode* previousNode;
  rangerState* state;
  rangerState* predictedState;
  HepMatrix* transportMatrix;

  friend ostream& operator<<(ostream &os,const rangerNode &n);
};

#endif
