// 
//  Author:  Alexander Spiridonov              Created:  25.08.00
//
//  Email:   spiridon@ifh.de                   HERA-B Collaboration
//  changed : 
//  check if track is stored                   17/09/00   A. Spiridonov
//  Only use SLT tracks which are fully accepted 30/01/01 B. Petersen
//  Use SLT tracks from single lepton triggers also 01/10/01 B. Petersen
//
// rgsltget.C
//
//       get parameters of slt tracks    
//
#include <iostream.h>
#include <math.h>
#include <stdio.h>
#include <farm/farm.hh>

#include "arte/RSEG_cmp.hh"
#include "arte/arte.hh"

//#include "rang/rgExtDef.hh"
//#include "rang/zfextse.hh"
//#include "rang/zfparext.hh"
#include "rang/zfpar.hh"


#undef DEBUG

// Prototypes

extern "C" 
{
#include "sltinfo.h"  
// /afs/desy.de/group/hera-b/SLT/slt/Rev0502/include/sltinfo.h
}


extern "C" void rgsltget_(int& Ntrack, float* parslt) 
{
  // initialization

  int n_track = 0;
  Ntrack = 0;

  // Get slt info

  //------------------------------------------------------ Basic SLT info
  int daqcode=faDDAQFCS_tcode();
  
  // Detailed trigger definition is given at:
  // http://www-hera-b.desy.de/subgroup/shiftinfo/sub/DAQSHI.html
  
#ifdef DEBUG
  cout<<"Trigger Info:"<<endl;
  if (daqcode>=8) cout<<"Random (4Hz) triggered event"<<endl;
  if (daqcode==0||daqcode==8) cout<<"FLT triggered event"<<endl;
  if (daqcode==6||daqcode==14) cout<<"''Test triggger'' event"<<endl;
#endif
   
  if (daqcode==9 || SLTReadDSLT()!=SLT_OK) { /* No SLT information in non triggered events */
    
    cout <<"No SLT information found"<<endl;
  } else {
    SLT_STATUS stat;
    SLT_TRACK track;
    SLT_REFITTRACK refit;
    SLT_SEEDTRACK seed;
        
    if (SLTStatus(&stat)!=SLT_OK) {
#ifdef DEBUG
      cout <<"No status information"<<endl;
#endif
      return ;
    }
    if (stat.status&SLTEVENT_BYPASS) {
#ifdef DEBUG
      cout <<"Event was a SLT BYPASS event!"<<endl;
#endif
    }
    if (!(stat.status&SLTEVENT_ACCEPT)) {
#ifdef DEBUG
      cout <<"Event rejected"<<endl;
#endif
      return ;
    }

#ifdef DEBUG
    cout<<"Event accepted with "<<stat.ntracks<<" tracks in ";
    cout<<stat.ntriggers<<" trigger pairs"<<endl;
#endif
    
    if (SLTStatus(&stat)==SLT_OK && (stat.status&SLTEVENT_ACCEPT)) {
      // loop over all tracks in this event and create the seeds
      for(int traId=0;traId<stat.ntracks;traId++) {
	if (SLTTrackInfo(traId,&track)==SLT_OK)  {
	  if(track.status != SLTTRACK_ERROR 
	     && track.status==SLTTRACK_ACCEPT) {
	    
	    if(SLTRefitTrackInfo(traId,&refit)==SLT_OK
	       // && SLTSiliTrackInfo(traId,&sili)==SLT_OK &&
	       // && SLTSiliTrackCovInfo(traId,&cov)==SLT_OK &&
	       && SLTSeedTrackInfo(traId,&seed)==SLT_OK 
	       ) {
	      // Now check the particle IDs
	      if ((( seed.trackID==EPLUS || seed.trackID==EMINUS ) && 
		   ( zfpar_.sltmode == 1 || zfpar_.sltmode == 3 ) ) ||
		  (( seed.trackID==MUPLUS || seed.trackID==MUMINUS ) && 
		   ( zfpar_.sltmode == 2 || zfpar_.sltmode == 3 ) )) {
	      

		int L = n_track * 4;
		parslt[L    ] = refit.bx;
		parslt[L + 1] = refit.by;
		parslt[L + 2] = refit.tx;
		parslt[L + 3] = refit.ty;
		
#ifdef DEBUG
		cout << " rgsltget.C n_track " << n_track 
		     << " x " << parslt[L    ] << " y " << parslt[L + 1]
		     << " tx " << parslt[L + 2] << " ty " << parslt[L + 3]
		     << endl;
#endif
		++n_track;
		Ntrack = n_track;
	      }
	    }
	  }
	}
      }
    }
  } 
}

