** 
**  Author:  A. Spiridonov                     Created:  12-Dec-2001
**  changed  : 
**  check mcmode_zfpar                                03/03/02 A.Spiridonov 
**

      subroutine tfdownstr3(hitmode,qualmin)

*
*     downstream propagation of VDS segments into ms01/mc01
*     input and output from/to RSEG table
*     if propagation is ok then VDS rseg is replaced by new rseg with mc01/ms01
*     clone of tfdownstr for reduced detector geometry

*     **** hitmode = 0 : rghlevnt has already been called (slave mode)
*     ****           1 : rghlevnt should be called by tfdownstr
*     **** qualmin     : minimal quality for candidate to be accepted

*#define TFDOWNSTR_DEBUG

      implicit none

#include "rang/zfmode.fi"
#include "rang/zfpar.fi"
*include "rang/zfpar_tc.fi"
#include "rang/zf.fi"
#include "rang/ztripl.fi"
#include "rang/zcut.fi"
#include "pris/zpstat.fi"
#include "arte/ArteFlag.fi"
#include "arte/Arte.fi"
#include "auto/MIMP.fi"
#include "auto/RSEG.fi"
#include "arte/RSEG_cmp.fi"
#include "rang/descend.fi"

      integer hitmode
      real qualmin

      integer ift,iiseg,ift1,ift2,nca_x,direction,
     +      begin_store,istat,nca_prep,newseg

      logical init

      save init
      data init /.false./

      integer nh,nh_PC,imimp,jaux,imc,nhmimp,imcass,NRSEG_rec
      real fracmc

      integer NRSEG_1

      integer maxflt_mag_zfpar_save
      real whitmin_mag_zfpar_save

#include "arte/Arte.ff"
#include "auto/MIMP.ff"
#include "auto/RSEG.ff"

      maxflt_mag_zfpar_save = maxflt_mag_zfpar
      whitmin_mag_zfpar_save = whitmin_mag_zfpar
      maxflt_mag_zfpar = 3
      whitmin_mag_zfpar = 2.

      l_diag = .false.
      diag_level = 0

      call rginit()
      if (.not.l_rinit) call tfrinit() ! restricted init

      l_term = .false.
      lremzc = .false.

      if (.not.init) then
         if (hitmode.eq.1) call rgdom()      ! build domain structure
         init = .true.
      endif


*     **** elementary checks
      if(jrseg.le.0) go to 999
      if(NRSEG().lt.1) go to 999

      if (hitmode.eq.1) then
***         print *,' tfdownstr: load hits (assume tfmatrip not yet done!!'
         call rghlevnt()
      endif

      NRSEG_rec = NRSEG()

      do 10 iiseg=1,NRSEG_rec     ! loop over reconstructed segments

         if (irseg_fit(iiseg).ne.0) go to 10  ! select charged track

         if (iand(irseg_cmp(iiseg),rsegc_bitsign).ne.0) go to 10
                                              ! discard killed segment

**         if (iand(irseg_cmp(iiseg),rsegc_sil).eq.0) go to 10
                                              ! demand Si hits
         if (frseg_ze(iiseg).gt.208.) go to 10 ! z cut

**         if (prseg_rend(iiseg).gt.0) go to 10 ! already continued

         if(mcmode_zfpar.eq.1.and.
     +        (sqrt(frseg_cve(6,iiseg))*(210.-frseg_ze(iiseg)).lt.0.01)
     +        .and.(sqrt(frseg_cve(10,iiseg))*(210.-frseg_ze(iiseg))
     +        .lt.0.01)) goto 10

         call tfclr()           ! clear p.r. buffer
         begin_ft = 1

         call tfdownse3(iiseg,ift1,ift2,istat)    ! seed

          if(istat.lt.0) then
            print *,'tfdownstr: tftcse for seg  ift1,ift2,istat=',
     +              iiseg,ift1,ift2,istat
          endif

         direction = +1

         nca_prep = ift2        ! store new endpointer

         l_2dx = .false.
         l_2dy = .false.
         l_mag = .true.
         begin_ft = nca_prep + 1
         istat = 0


         call tfpropmn3(ift1,+1,2,3,istat)

         if (istat.ge.0) then
         elseif (istat.le.-100) then
            go to 999           ! no further propagation
         elseif (istat.le.-1) then
            go to 10            ! terminate branch
         endif

         if (diag_level.ge.2) then
            call tfl()
            print *,' after tfpropmn2,before tfbest'
            call tfwait()
         endif
         begin_ft = nca_prep + 1
         call tfbest()
         if (diag_level.ge.3) then
            call tfl()
            print *,' after tfbest'
            call tfwait()
         endif

*        **** store loop
 9       continue

         begin_store = nca_prep + 1
         
         NRSEG_1 = nrseg()
         
         do 40 ift=begin_store,nca_ft

#ifdef TFDOWNSTR_DEBUG
               print *,'= tfdownst: ift=',ift,' aliv ',alive_ft(ift),
     +              ' quality=',quality_ft(ift),
     +              ' cut=',qualmin,
     +              ' nhit=',nhit_ft(ift)
#endif
            if (alive_ft(ift)) then
*               if (quality_ft(ift).lt.quality_TC_min
*     +         .or.nhit_ft(ift).lt.5 ) go to 40

#ifdef TFDOWNSTR_DEBUG
               print *,'= tfdownst: ift=',ift,
     +              ' quality=',quality_ft(ift),
     +              ' cut=',qualmin,
     +              ' nhit=',nhit_ft(ift)
               if (quality_ft(ift).lt.qualmin) then
                  print *,' warning: quality cut removed'
               else
                  print *,' ============== CALL TFSTSEG'
               endif
#endif

               if (quality_ft(ift).lt.qualmin) go to 40

*               call tfauxdft(ift)
               call tfrsegft(iiseg,ift1,ift2)
*               call tfauxdft(ift1)
               call tfjoinft(ift1,ift)
*               call tfauxdft(ift1)

               call tfstseg3(ift1)               ! clone of tfstseg
               newseg = nrseg()
               call rgfit(newseg,0)
               if(newseg.gt.NRSEG_1) then
                  iq(iqrseg_cmp(newseg))=iq(iqrseg_cmp(iiseg))
                  iq(iqrseg_cmp(iiseg))=-iq(iqrseg_cmp(iiseg))
               endif

*              **** set the relation RBEG <----> REND
*               call mmbrel(jrsrs,iiseg,newseg) ! set relation

               if (l_diag) call tfwait()

            endif
 40      continue

 10   continue                  !  end of loop on segments

 999  maxflt_mag_zfpar= maxflt_mag_zfpar_save
      whitmin_mag_zfpar= whitmin_mag_zfpar_save
      end
