
#ifndef _ECLASS_HH
#define _ECLASS_HH

// *****************************************************************************
//
// source:    eclass.hh
//
// type:      header file
//
// created:   08 May 2000
//    
// author:    Dr. Andreas Gellrich
//            HERA-B Collaboration
//            Humboldt-University Berlin
//            Physics Department
//            Invalidenstr. 110
//            10115 Berlin
//            Germany
//            E-mail: Andreas.Gellrich@desy.de
//
// purpose:   define event classification bits (1-32)
//
// hints:     evhd->flagdst is defined as 'int'.
//
// changes:
// 09 May 2000 (AG) introduce class JPSIEE and JPSIMM
// 23 May 2000 (AG) introduce class RNDTRG
// 25 May 2000 (AG) introduce class FLTTRG
// 29 May 2000 (AG) introduce single high-ET ECAL trigger
//  5 Jun 2000 (AG) change ECLASS_FLTTRG to bit 30
//  7 Jun 2000 (AG) leave upper 16 bits open for users
// 14 Jun 2000 (AG) introduce ECLASS_FCNC
// 23 Jun 2000 (AG) introduce names for classes
// 25 Jun 2000 (AG) correct names
// 28 Jun 2000 (AG) added more classes
// 19 Jul 2000 (AG) introduce ECLASS_VZERO
// 20 Jul 2000 (AG) introduce ECLASS_SLTEE/MM/DV and ECLASS_TLT
// 26 Jul 2000 (AG) introduced VZERO2 class
// 27 Jul 2000 (AG) introduced SLTEEB
// 28 Jul 2000 (AG) polished code
// 01 Aug 2000 (AG) polished code
// 02 Aug 2000 (AG) introduced ECLASS_DVTX, ECLASS_SLTHPT
// 03 Aug 2000 (AG) rename SLTHPT to SLTCUT
// 16 Aug 2000 (AG) introduced SLTYPS
// 10 Nov 2000 (AG) introduced RECYPS, RECDV 
// 11 Dec 2000 (AG) introduced DIMUON
// 20 Dec 2000 (AG) introduced ALIGN
// 03 Jul 2001 (MV) introduced DIELEID
//
// *****************************************************************************

// physical classes (reserved)
#define ECLASS_JPSIEE  0x0001 // bit= 1 J/Psi -> e+ e-
#define ECLASS_JPSIMM  0x0002 // bit= 2 J/Psi -> mu+ mu-
#define ECLASS_CAL1HT  0x0004 // bit= 3 single high-ET ECAL cluster trigger
#define ECLASS_FCNC    0x0008 // bit= 4 FCNC event
#define ECLASS_VZERO   0x0010 // bit= 5 V0 (Ignacio)
#define ECLASS_VZERO2  0x0020 // bit= 6 V0 (Denis)
#define ECLASS_DVTX    0x0040 // bit= 7 detached vertex
#define ECLASS_SLTYPS  0x0080 // bit= 8 SLT l+ l- > 8 GeV
#define ECLASS_SLTCUT  0x0100 // bit= 9 SLT J/Psi candidates with cuts
#define ECLASS_SLTEEB  0x0200 // bit=10 SLT e+ e- incl. bremstrahlung
#define ECLASS_SLTEE   0x0400 // bit=11 SLT e+ e-
#define ECLASS_SLTMM   0x0800 // bit=12 SLT Mu+ Mu-
#define ECLASS_SLTDV   0x1000 // bit=13 SLT Detached Vertex
#define ECLASS_TLT     0x2000 // bit=14 TLT
#define ECLASS_FLTTRG  0x4000 // bit=15 FLT trigger    (FCS_tcode = 0 || 8)
#define ECLASS_RNDTRG  0x8000 // bit=16 random trigger (FCS_tcode = 8 || 9)

#define ECLASS_RECYPS  0x10000    // bit=17 di-lepton m>6.0 GeV
#define ECLASS_RECDV   0x20000    // bit=18 detached vertex in event
#define ECLASS_DIMUON  0x40000    // bit=19
#define ECLASS_DIELEID 0x80000    // bit=20 di-elect. identified
#define ECLASS_CLASS21 0x100000   // bit=21
#define ECLASS_CLASS22 0x200000   // bit=22
#define ECLASS_CLASS23 0x400000   // bit=23
#define ECLASS_CLASS24 0x800000   // bit=24
#define ECLASS_CLASS25 0x1000000  // bit=25
#define ECLASS_CLASS26 0x2000000  // bit=26
#define ECLASS_CLASS27 0x4000000  // bit=27
#define ECLASS_CLASS28 0x8000000  // bit=28
#define ECLASS_CLASS29 0x10000000 // bit=29
#define ECLASS_CLASS30 0x20000000 // bit=30
#define ECLASS_CLASS31 0x40000000 // bit=31
#define ECLASS_ALIGN   0x80000000 // bit=32


#define ECLASS_NAME_1  "JPSIEE"
#define ECLASS_NAME_2  "JPSIMM"
#define ECLASS_NAME_3  "CAL1HT"
#define ECLASS_NAME_4  "FCNC"
#define ECLASS_NAME_5  "VZERO"
#define ECLASS_NAME_6  "VZERO2"
#define ECLASS_NAME_7  "DVTX"
#define ECLASS_NAME_8  "SLTYPS"
#define ECLASS_NAME_9  "SLTCUT"
#define ECLASS_NAME_10 "SLTEEB"
#define ECLASS_NAME_11 "SLTEE"
#define ECLASS_NAME_12 "SLTMM"
#define ECLASS_NAME_13 "SLTDV"
#define ECLASS_NAME_14 "TLT"
#define ECLASS_NAME_15 "FLTTRG"
#define ECLASS_NAME_16 "RNDTRG"
#define ECLASS_NAME_17 "RECYPS"
#define ECLASS_NAME_18 "RECDV"
#define ECLASS_NAME_19 "DIMUON"
#define ECLASS_NAME_20 "DIELEID"
#define ECLASS_NAME_21 "CLASS21"
#define ECLASS_NAME_22 "CLASS22"
#define ECLASS_NAME_23 "CLASS23"
#define ECLASS_NAME_24 "CLASS24"
#define ECLASS_NAME_25 "CLASS25"
#define ECLASS_NAME_26 "CLASS26"
#define ECLASS_NAME_27 "CLASS27"
#define ECLASS_NAME_28 "CLASS28"
#define ECLASS_NAME_29 "CLASS29"
#define ECLASS_NAME_30 "CLASS30"
#define ECLASS_NAME_31 "CLASS31"
#define ECLASS_NAME_32 "ALIGN"

// prototypes
char *rcClassName( int bit );

int rcClassJPSIEE(); // ECLASS_JPSIEE           1             (J/Psi->e+ e-)
int rcClassJPSIMM(); // ECLASS_JPSIMM           2             (J/Psi->mu+ mu-)
int rcClassCAL1HT(); // ECLASS_CAL1HT           3             (high-Et cluster)
int rcClassFCNC();   // ECLASS_FCNC             4             (FCNC event)
int rcClassVZERO();  // ECLASS_VZERO            5             (K0, Lambda)
int rcClassVZERO2(); // ECLASS_VZERO            6             (K0s, Lamda, Phi)
// int rcClassDVTX();   // ECLASS_DVTX             7             (detached vertex)
int rcClassSLT();    // ECLASS_SLTEEB,EE,MM,DV  9/10/11/12/13 (SLT)
int rcClassTLT();    // ECLASS_TLT             14             (TLT)
int rcClassRECYPS(); // ECLASS_RECYPS          17             (high ll mass)
int rcClassRECDV();  // ECLASS_RECDV           18             (detached vertex)
int rcClassDIMUON(); // ECLASS_DIMUON          19             (di-muon)
int rcClassDIELEID();// ECLASS_DIELEID         20             (di-elect. id)
int rcClassALIGN();  // ECLASS_ALIGN           32             (alignment)

#endif
