// --*-C++-*--
// *****************************************************************************
//
// source:   Geometrical acceptance
//
// type:      source code
//            - routines that return the number of sensitive planes
//              crossed by a track with parameters {z,x,y,yx,ty,q/p}
//            - usage:
//            #include "reco/rcXgeo.h"
//            double track[]={...};
//            int notr = rcXgeo::num_otr_planes(double* track);
//            int nitr = rcXgeo::num_itr_planes(double* track);
//            int nvds = rcXgeo::num_vds_planes(double* track);
//              ...
//
//
// created:   14 Oct 2000
//	
// author:    Denis Dujmic
//
// changes: 
//
// *****************************************************************************
#ifndef _RCXGEO_H_
#define _RCXGEO_H_
#include "arte/arte.hh"


class rcXgeo {
private:
  /// maximum z for sensitive area
  static float max_z_pc;
  /// minimum z for sensitive area
  static float min_z_pc;
  /// component number
  static int  gesl_cmp;
  /// check for gede hit close to track
  static bool hit_within_pitch(ArtePointer<GEDE> gede, double* track);
  static int  num_otr_hits(double* track);
  /// check track geo acceptance by gede
  static bool track_within_gede(ArtePointer<GEDE> gede, double* track);

public:

  /// Returns number of active OTR planes on track path (z,x,y,tx,ty,q/p)
  static int  num_otr_planes(double* track);

  /// Returns number of active ITR planes on track path (z,x,y,tx,ty,q/p)
  static int  num_itr_planes(double* track);

  /// Returns number of active VDS planes on track path (z,x,y,tx,ty,q/p)
  static int  num_vds_planes(double* track);

  /// Returns number of detected rich photons out of 35 generated
  /// assuming only geometrical losses and beta=1 radius
  static int  num_rich_photons(double *track);

};

#endif
