//--*-C++-*--
#ifndef _RCXSEGMENT_H_
#define _RCXSEGMENT_H_

// *****************************************************************************
//
// source:    
//
// type:      source code
//
// created:   26 Jul 2000
//	
// author:    Andreas Gellrich
//            HERA-B Collaboration
//            Humboldt-University Berlin
//            Physics Department
//            Invalidenstr. 110
//            10115 Berlin
//            Germany
//            E-mail: Andreas.Gellrich@desy.de
//
// changes:
// 26 Jul 2000 (AG) creation (code from Denis)
// 04 Aug 2000 (DD) TrackDiff also checks decays
// 11 Oct 2000 (DD) search and kill old segments in RTRA
// 30 Nov 2000 (DD) check for slt seed
// 10 Mar 2001 (DD) Rsegc::slt added
//
// *****************************************************************************

#include <stdio.h>
#include "arte/arte.hh"
#include "arte/RSEG_cmp.hh"
#include "vector.h"


/// segments in fitted tracks
class segment {
 private:
  static int checkTrackDiff(ArtePointer<RTRA> t1, ArtePointer<RTRA> t2);

 public:
  // -- VDS
  inline static bool vdsf() { return ivds!=ArteTable<RSEG>::end(); } 
  static ArteTable<RSEG>::iterator ivds;

  // -- ITR
  inline static bool itrf() { return iitr!=ArteTable<RSEG>::end(); }
  static ArteTable<RSEG>::iterator iitr;

  // -- OTR
  inline static bool otrf() { return iotr!=ArteTable<RSEG>::end(); }
  static ArteTable<RSEG>::iterator iotr;

  // -- TC
  inline static bool  tcf() { return itc!=ArteTable<RSEG>::end(); }
  static ArteTable<RSEG>::iterator itc;

  // -- RICH
  inline static bool richf() { return irich!=ArteTable<RSEG>::end(); }
  static ArteTable<RSEG>::iterator irich;

  // -- ECAL
  inline static bool ecalf() { return iecal!=ArteTable<RCCL>::end(); }
  static ArteTable<RCCL>::iterator iecal;

  // -- MUON
  inline static bool muonf() { return imuon!=ArteTable<RSEG>::end(); }
  static ArteTable<RSEG>::iterator imuon;

  // -- VDS_RICH
  inline static bool vds_richf() { return ivds_rich!=ArteTable<RSEG>::end(); }
  static ArteTable<RSEG>::iterator ivds_rich;

  // -- VDS_ECAL
  inline static bool vds_ecalf() { return ivds_ecal!=ArteTable<RSEG>::end(); }
  static ArteTable<RSEG>::iterator ivds_ecal;

  // -- RICH_ECAL
  inline static bool rich_ecalf() { return irich_ecal!=ArteTable<RSEG>::end(); }
  static ArteTable<RSEG>::iterator irich_ecal;

  // -- VDS_OTR
  inline static bool vds_otrf() { return ivds_otr!=ArteTable<RSEG>::end(); }
  static ArteTable<RSEG>::iterator ivds_otr;

  // -- VDS_ITR
  inline static bool vds_itrf() { return ivds_itr!=ArteTable<RSEG>::end(); }
  static ArteTable<RSEG>::iterator ivds_itr;

  // -- SLT flag
  inline static bool sltf() { return islt; }
  static bool islt;

  // -- MTRA
  inline static bool mtraf() { return imtra!=ArteTable<MTRA>::end(); }
  static ArteTable<MTRA>::iterator imtra;

  // -- Bremsstrahlung
  inline static bool bremsef() { return ibremse!=ArteTable<RCCL>::end(); }
  static ArteTable<RCCL>::iterator ibremse;

  // -- find segments
  static void  resetSegments();
  static void  resolveCmp(ArteTable<RSEG>::iterator s);
  static void  findSegments(ArtePointer<RTRA> t);
  static void  findSegments(ArtePointer<RSEG> s);
  static void  printSegments();
  static void  printSegments(ArtePointer<RSEG> s);
  static void  printSegments(ArtePointer<RTRA> t);
  static int   track_components;
  
  // -- track difference
  static int   TrackDiff(ArtePointer<RTRA> t1, ArtePointer<RTRA> t2);
  static int   TrackDiff(ArtePointer<RSEG> s1, ArtePointer<RTRA> t2);

  // -- cov. matrix: 2dim -> 1dim
  static int   covij[5][5];

  /// Search and kill segments with fit number and/or topp
  static void  killRTRA(int fitflag, int lund=0);

  /**
     Get all vds segments from RTRA, in case two ITR/OTR are matched
     use better chi2/dof 
  */
  static void  getVdsFromRtra(vector<ArtePointer<RTRA> > &vds_list);

  /// Count tracks per event
  static void countTrackOccupancy();
  static int  nt_vds;
  static int  nt_main;
  static int  nt_rich;
  static int  nt_ecal;
  static int  nt_muon;
  static int  nt_vds_v0;

  /// make float arrays
  static void makeParArray(ArtePointer<RTRA> rtra, float *par);

  /// make float arrays
  static void makeParArray(ArtePointer<RSLA> rtra, float *par);

  /// make directions (nx,ny,nz)
  static void makeDirArray(ArtePointer<RTRA> rtra, float *par);

};

#endif
