/*
  This function removes all segments with cmp flag < 0 from RSEG table.
  The relation table is changed accordingly. The order of links is reverse 
  afterwards.

  created by :                                  Olya Igonkina  10.01.2000
  modified          correct order of links, removing of bad rhit     08/06/00 W. Hulsbergen

 */

#include <vector>
#include "arte/arte.hh"
#include "arte/RSEG_cmp.hh"

struct GoodRHIT
{
  RHIT rhit;
  ArtePointer<RHIT> oldLink;
  ArtePointer<RHIT> newLink;
  ArtePointer<HITB> hitb ;
} ;

struct GoodRSEG
{
  RSEG segment;
  ArtePointer<RSEG> oldLink;
  ArtePointer<RSEG> newLink;

  std::vector<ArtePointer<RTRA> > rtras;
  // std::vector<ArtePointer<RBEG> > rbegs;  we keep only relation to the end its enough
  std::vector<ArtePointer<RCCL> > rccls;
  std::vector<ArtePointer<MTRA> > mtras;
  std::vector<ArtePointer<HITB> > hitbs;
  std::vector<ArtePointer<RPNT> > rpnts;
  std::vector<GoodRSEG * > rends; 
  std::vector<GoodRHIT> rhits ; 
};


int rc99removeBadRseg()
{
  std::vector <GoodRSEG> goodSegments;
  //cout << " Start to remove Bad RSEG \n";  
  int nskip = 0;
  //cout << "**** SKIPPING REMOVEBADRSEG ****" << endl ;
  //return 0;

  for( ArteTable<RSEG>::iterator irseg = ArteTable<RSEG>::begin();
       irseg != ArteTable<RSEG>::end(); ++irseg )
    {
      if( ((*irseg)->cmp & Rsegc::bitsign) != 0 )
	{
	 // cout << " skip " << (*irseg) << endl;
	  nskip++;
	  continue;
	}
      goodSegments.push_back( GoodRSEG() );
      GoodRSEG * ptStruct = & goodSegments.back();
      ptStruct->segment = **irseg;
      
      ptStruct->segment.rtra = ArteManRelationPointer<RTRS,RTRA>();
      ptStruct->segment.rccl = ArteManRelationPointer<RCRS,RCCL>();
      ptStruct->segment.mtra = ArteManRelationPointer<MTRS,MTRA>();
      ptStruct->segment.hitb = ArteInvManRelationPointer<RSHI,HITB>();
      ptStruct->segment.rpnt = ArteInvOneRelationPointer<RSRP,RPNT>();
      ptStruct->segment.rhit = ArteInvOneRelationPointer<RSRH,RHIT>();
      ptStruct->segment.rbeg = ArteManRelationPointer<RSRS,RSEG>();
      ptStruct->segment.rend = ArteInvManRelationPointer<RSRS,RSEG>();

      ptStruct->oldLink = *irseg;     
      
      for( ArteRelationIterator<RTRA> rsRtra = (*irseg)->rtra.begin();
	   rsRtra != (*irseg)->rtra.end(); ++rsRtra )
	ptStruct->rtras.push_back( *rsRtra);

      for( ArteRelationIterator<MTRA> rsMtra = (*irseg)->mtra.begin();
	   rsMtra != (*irseg)->mtra.end(); ++rsMtra )
	ptStruct->mtras.push_back( *rsMtra);

      for( ArteRelationIterator<RCCL> rsRccl = (*irseg)->rccl.begin();
	   rsRccl != (*irseg)->rccl.end(); ++rsRccl )
	ptStruct->rccls.push_back( *rsRccl);

      for( ArteRelationIterator<HITB> rsHitb = (*irseg)->hitb.begin();
	   rsHitb != (*irseg)->hitb.end(); ++rsHitb )
	ptStruct->hitbs.push_back( *rsHitb);

      for( ArteRelationIterator<RPNT> rsRpnt = (*irseg)->rpnt.begin();
	   rsRpnt != (*irseg)->rpnt.end(); ++rsRpnt )
	ptStruct->rpnts.push_back( *rsRpnt);

      for( ArteRelationIterator<RHIT> rsRhit = (*irseg)->rhit.begin();
           rsRhit != (*irseg)->rhit.end(); ++rsRhit ) {
        ptStruct->rhits.push_back( GoodRHIT() ) ;
        GoodRHIT * hitptStruct = & ptStruct->rhits.back();
        hitptStruct->rhit = **rsRhit;
        hitptStruct->rhit.hitb = ArtePointer<HITB>() ;
        hitptStruct->rhit.rseg = ArtePointer<RSEG>() ;
        hitptStruct->hitb = (*rsRhit)->hitb ;
      }


    } // end of loop over RSEGs

 // cout << " We skip " << nskip << " segments in total \n";

  // cout << " prepare RSEG-RBEG relations \n";
  // loop again to fix relations between RSEG-REND 
  for( unsigned int iseg = 0; iseg < goodSegments.size() ; ++iseg )
    {
      ArtePointer<RSEG> tmpRSEG = goodSegments[iseg].oldLink;
      for( ArteRelationIterator<RSEG> rsRend = tmpRSEG->rend.begin();
	   rsRend != tmpRSEG->rend.end(); ++rsRend )
	{
	  if( ((*rsRend)->cmp & Rsegc::bitsign) != 0 ) continue;	  
	  // now we know which is relation and will seach for corresponding 
	  // GoodRSEG
	  for( unsigned int iseg2 = 0; iseg2 < goodSegments.size() ; ++iseg2 )
	    {
	      if ( iseg == iseg2 ) continue;
	      if( goodSegments[iseg2].oldLink != *rsRend ) continue;
	      goodSegments[iseg].rends.push_back(&goodSegments[iseg2]);
	    }
	  
	}
    }
  
  // cout << " now clear eveything \n";
  
  //=============================================================
  ArteTable<RSEG>::clear(); // table itself
  ArteTable<RHIT>::clear(); // rhit table
  // reset pointers in hitb to rseg and rhit (should not be necessary)
  for(ArteTable<HITB>::iterator hitbIt=ArteTable<HITB>::begin() ;
      hitbIt!=ArteTable<HITB>::end() ; ++hitbIt) {
    hitbIt->rhit = ArteInvOneRelationPointer<HIRH,RHIT>() ;
    hitbIt->rseg = ArteManRelationPointer<RSHI,RSEG>() ;
  }
  
  // cout << " make Table again \n";  
  for( unsigned int iseg = 0; iseg < goodSegments.size() ; ++iseg )
    {
      GoodRSEG * ptStruct = &  goodSegments[iseg];
      
      ArtePointer<RSEG> Prseg = ArteTable<RSEG>::push_back(ptStruct->segment);
      ptStruct->newLink = Prseg;

      // create relations (reverse order)

      for( unsigned int irtra = ptStruct->rtras.size(); irtra>0 ; --irtra)
	createRelation(Prseg, ptStruct->rtras[irtra-1], &RSEG::rtra);

      for( unsigned int irccl = ptStruct->rccls.size(); irccl>0 ; --irccl)
	createRelation(Prseg, ptStruct->rccls[irccl-1], &RSEG::rccl);

      for( unsigned int imtra = ptStruct->mtras.size(); imtra>0 ; --imtra)
	createRelation(Prseg, ptStruct->mtras[imtra-1], &RSEG::mtra);

      for( unsigned int ihitb = ptStruct->hitbs.size(); ihitb>0 ; --ihitb)
	createRelation(Prseg, ptStruct->hitbs[ihitb-1], &RSEG::hitb);

      for( unsigned int irpnt = ptStruct->rpnts.size(); irpnt>0 ; --irpnt)
	createRelation(Prseg, ptStruct->rpnts[irpnt-1], &RSEG::rpnt);
      
      for( vector<GoodRHIT>::reverse_iterator 
	     goodrhitIt = ptStruct->rhits.rbegin() ;
           goodrhitIt != ptStruct->rhits.rend() ; ++goodrhitIt) {
        ArtePointer<RHIT> rhit = ArteTable<RHIT>::push_back(goodrhitIt->rhit) ;
	//ArtePointer<RHIT> rhit = goodrhitIt->newlink ;
        createRelation(Prseg, rhit, &RSEG::rhit);
        createRelation(goodrhitIt->hitb,rhit,&HITB::rhit) ;
      }
    }
  
  // cout<< " Fill RSEG-RBEG relations \n";
  // now we fix new relations RSEG-REND  ( note that than RSEG-RBEG follows)
  for( unsigned int iseg = 0; iseg < goodSegments.size() ; ++iseg )
    {
      GoodRSEG * ptStruct = &  goodSegments[iseg];      
      for( unsigned int iseg2 = 0; iseg2 < ptStruct->rends.size(); ++iseg2)
	{
	  mmbrel_( ArteManRelationPointer<RSRS,RSEG>::getLink().getInt(),
		   ptStruct->newLink, ptStruct->rends[iseg2]->newLink );
	}
    }
  // cout << " RSEG table is clean from bad segments \n";
  
  return 0;
}
