
// *****************************************************************************
//
// source:    rcClassDIMUON.C
//
// type:      source code
//
// created:   8 Dec 2000
//	
// author:    Brian Petersen
//            E-mail: aagaard@nbi.dk
//
// symbol:    rcClassDIMUON
//
// called by: rcClassify
//
// purpose:   check for presence of two trigger muons
//
// changes:
//  8 Dec 2000 (BP) creation
// 11 Dec 2000 (AG) moved to ARTE/dev
// 14 Dec 2000 (BP) Only count charge tracks, not J/psi's as muons
// 15 Dec 2000 (AG) added debug printout
// 27 May 2001 (IR) Rsegc::bit9 is substituted by Rsegc::sltmu
//
// *****************************************************************************

// ARTE includes
#include "arte/arte.hh"
#include "farm/farm.hh"
#include "arte/RSEG_cmp.hh"

// externals
extern int FADEB;

//
// function code
//
int
rcClassDIMUON()
{
  ARTE_mark( "rcClassDIMUON 0", 0 );

  // inits
  int rc     = 0;
  int cnt=0;

  // count number of muons.
  for( ArteTable<RTRA>::iterator rtra = ArteTable<RTRA>::begin();
       rtra != ArteTable<RTRA>::end(); 
       ++rtra ){
    if( (*rtra)->cmp&Rsegc::sltmu && (*rtra)->fit==0 ) cnt++;
  }
  
  // print
  if( FADEB ) cout << " DIMUON: SLT-muons=" << cnt << endl;

  // check number
  if( cnt>=2 ) rc = 1;
  
  ARTE_mark( "rcClassDIMUON 1", 0 );
  return rc;
}
