
// *****************************************************************************
//
// source:    rcClassVZERO.C
//
// type:      source code
//
// created:   19 Jul 2000
//	
// author:    Dr. Andreas Gellrich
//            HERA-B Collaboration
//            Humboldt-University Berlin
//            Physics Department
//            Invalidenstr. 110
//            10115 Berlin
//            Germany
//            E-mail: Andreas.Gellrich@desy.de
//
// symbol:    rcClassVZERO
//
// called by: rcClassify
//
// purpose:   class events with V0 candidate
//
// hints:
//
// changes:
// 19 Jul 2000 (AG) creation
// 21 Jul 2000 (AG) fix bug
// 25 Jul 2000 (AG) added rcSelectVZERO() and rcSelectVZERO2()
// 26 Jul 2000 (AG) stick to one VZERO selection function
// 26 Jul 2000 (AG) use seperate function for VZERO2
// 04 Dec 2000 (AG) introduce VZERO finder on Federico's code
// 28 Jun 2001 (FS) call directly rcSelect_K0SLambda.
// 26 Sep 2001 (DS) include path corrected
// 05 Nov 2001 (FS) add protection against but ptrs
//
// *****************************************************************************

// standard includes
#include <iostream.h>

// ARTE includes
#include "arte/arte.hh"
#include "farm/farm.hh"
#include "reco/rcSelect_K0S_Lambda.hh"

// externals
extern int FADEB;

// prototypes
int rcSelectVZERO();

//
// function code
//
int
rcClassVZERO()
{
  ARTE_mark( "rcClassVZERO 0", 0 );

  // ints
  int rc = 0;

  //
  // Ignacio's V0 finder
  //
  //ARTE_mark( "rcSelectVZERO", 0 )
  //int flag = rcSelectVZERO(); // bit 1:K0s 2:Lambda
  //if( flag & 0x1 ) rc |= 0x1;
  //if( flag & 0x2 ) rc |= 0x2;

  //
  // Federico's V0 finder
  //

  if(  K0SLAMBDA::rcSelect_K0S_Lambda() ) {
      for( ArteTable<RVER>::iterator rver = ArteTable<RVER>::begin();
	   rver != ArteTable<RVER>::end();++rver ){
    	// Skip primary vertices.

    	if( rver->type == 1 ) continue;
    
	ArtePointer<RTRA> part = rver->btra;
	ArtePointer<RTRA> frtra = ArteTable<RTRA>::begin();
	ArtePointer<RTRA> lrtra = ArteTable<RTRA>::end();
	
	// Check that the RTRA associated to the vertex is valid.
	
	if( part < frtra || part > lrtra ) continue; 
	
	ArtePointer<TOPP> ftopp = ArteTable<TOPP>::begin();
	ArtePointer<TOPP> ltopp = ArteTable<TOPP>::end();
	ArtePointer<TOPP> topp = part->topp;
	
	// Check that the TOPP associated to the vertex is valid.
	
	if( topp < ftopp || topp > ltopp ) continue; 
	
	int atopp = topp->lund;
	if( atopp ==   310 ) rc |= 0x1;
	if( atopp ==  3122 ) rc |= 0x2;
	if( atopp == -3122 ) rc |= 0x2;
      }
      // print
      if( rc ) cout << " VZERO: rc=0x" << hex << rc << dec << endl;
    }

  ARTE_mark( "rcClassVZERO 1", 0 );
  return rc;
}
