// ********************************************************************
//
//  Routine:      rcRefitRtraBasePatt
//                rcRefitRtraVdsEcal
//                rcRefitRtraVdsOnly
//  Author:       W. Hulsbergen from original routine by 
//                A. Spiridonov (20/01/2001)
//
//  Description:  refit routines for RTRAs in rtra2001 scheme
//
//
// ********************************************************************

#include <arte/arte.hh>
#include <rang/zfpar.hh>
#include <rang/Fit.hh>
#include <reco/rcflags.hh>
#include <arte/RSEG_cmp.hh>
#include <farm/farm.hh>

//********************************************************************
//
//  Routine:      rcRefitRtraBasePatt
//
//  Description: Refits all rtras that contain main tracker segment,
//  except for PC-only and PC-TC-only rtras. RHITs are created if they
//  do not exist yet.
//
// Called from: `new reco chain'
//
//********************************************************************

extern int rcCreateRtraRhit(ArtePointer<RTRA>) ;

int rcRefitRtraBasePatt()
{
  ARTE_mark( "rcRefitRtraBasePatt 0", 0 );
  
  if(rcrefit) {   
      for( ArteTable<RTRA>::iterator rtra = ArteTable<RTRA>::begin();
           rtra != ArteTable<RTRA>::end(); ++rtra )
        if( !(rtra->cmp & Rsegc::bitsign) && 
	    (rtra->cmp & Rsegc::patt) ) {
            
	  // check if MC and TC flag set
	  bool vds     = rtra->cmp & Rsegc::vxd ;
	  bool magnet  = rtra->cmp & Rsegc::magt ;
	  bool trigger = rtra->cmp & Rsegc::trit ;
	  
	  if( vds || ( magnet && zfpar_.fitmagn) ) {
	    rcCreateRtraRhit(rtra) ;
	    rangerFit(rtra);
	    if(zfpar_.fitmagn) rtra->mommask = RCMOMREFIT;
	    else               rtra->mommask = RCMOMREFITNOMAG;
	  } else if( trigger && zfpar_.fit <= 0) {
	    rcCreateRtraRhit(rtra) ;
	    rangerFitL(*rtra);
	  }
	}
  }
  
  ARTE_mark( "rcRefitRtraBasePatt 1", 0 );
  return 0;
}

//********************************************************************
//
//  Routine:      rcRefitRtraVdsEcal
//
//  Description: Refits all rtras that contain vds and ecal, but no
//  main tracker. The cluster is used in the fit.
//
// Called from: `new reco chain'
//
//********************************************************************


int rcRefitRtraVdsEcal()
{
  ARTE_mark( "rcRefitRtraVdsEcal 0", 0 );
 
  if(rcrefit) {
    for( ArteTable<RTRA>::iterator rtra = ArteTable<RTRA>::begin();
	 rtra != ArteTable<RTRA>::end(); ++rtra )
      if( !(rtra->cmp & Rsegc::bitsign) &&
	  (rtra->cmp & Rsegc::ecal) &&
	  (rtra->cmp & Rsegc::vxd) &&
	  !(rtra->cmp & Rsegc::patt) ) 
	{
	  rcCreateRtraRhit(*rtra) ;
	  rangerFit(*rtra,MUON,true,false) ;
	  rtra->mommask = RCMOMREFIT ;
	}
  }
  
  ARTE_mark( "rcRefitRtraVdsEcal 1", 0 );
  return 0;
}


//********************************************************************
//
//  Routine:      rcRefitRtraVdsOnly
//
//  Description: Refits all rtras that contain vds, but no ecal or
//  main tracker.
//
// Called from: `new reco chain'
//
//********************************************************************

int rcRefitRtraVdsOnly()
{
  ARTE_mark( "rcRefitRtraVdsOnly 0", 0 );
 
  // I also tried a version in which the cluster is used in the fit
  // and the momentum is determined as well. Unfortunately the refit
  // doesn't converge then.
 
  if(rcrefit) {
    for( ArteTable<RTRA>::iterator rtra = ArteTable<RTRA>::begin();
	 rtra != ArteTable<RTRA>::end(); ++rtra )
      if( !(rtra->cmp & Rsegc::bitsign) &&
	  !(rtra->cmp & Rsegc::ecal) &&
	  (rtra->cmp & Rsegc::vxd) &&
	  !(rtra->cmp & Rsegc::patt) ) 
	{
	  rcCreateRtraRhit(*rtra) ;
	  rangerFitL(*rtra) ;
	}
  }
  
  ARTE_mark( "rcRefitRtraVdsOnly 1", 0 );
  return 0;
}

