
// *****************************************************************************
//
// source:    rcSelectJPSIMM.C
//
// type:      source code
//
// created:   08 Dec 2000
//	
// author:    Dr. Andreas Gellrich
//            HERA-B Collaboration
//            Humboldt-University Berlin
//            Department of Physics
//            Invalidenstr. 110
//            10115 Berlin
//            Germany
//            E-mail: Andreas.Gellrich@desy.de
//
// symbol:    rcSelectJPSIMM
//
// called by: rcSelectDecays
//
// purpose:   di-lepton candidates
//
// hints:     selection results are stored in appropriate tables (RTRA, RVER)
//            cuts might be set in classification if not defaults are used
//            classification bits are set using entries in the tables
//
// changes:
//  8 Dec 2000 (AG) creation
// 11 Dec 2000 (BP) Added actual code based on rcSelectJPSIEE
// 13 Dec 2000 (BP) Modified to JPSIVertex.C for vertexing
// 15 Dec 2000 (AG) polished code
// 27 May 2001 (IR) Rsegc::bit9 is substituted by Rsegc::sltmu
// 
// *****************************************************************************

// standard includes
#include <iostream.h>

// ONLINE includes

// ARTE includes
#include "farm/farm.hh"
#include "arte/RSEG_cmp.hh"
#include "grover/grover.hh"
#include "reco/rcXkinema.h"
#include "reco/JPSIvertex.hh"

// externals
extern int FADEB;     // debug flag steered by runargs=-debug

// MuonID - currently only looks for Trigger tracks

static int isMuon(ArtePointer<RTRA> rtra) {

  if (rtra->cmp&Rsegc::sltmu&&rtra->fit==0) return 1;
  else return 0;
}

// prototypes

int
JPSIvertex(ArtePointer<RTRA> track1,ArtePointer<RTRA> track2,JPSIVtxType t);


//
// function code
//
int
rcSelectJPSIMM()
{
  ARTE_mark( "rcSelectJPSIMM  0", 0 );

  //==================================================================
  // 1. Cuts
  //==================================================================
  float MassCut = 2.0;
  if( FADEB ) cout << "JPSIMM: MassCut=" << MassCut << endl;

  //==================================================================
  // 
  //==================================================================

  for( ArteTable<RTRA>::iterator rtra1 = ArteTable<RTRA>::begin();
       rtra1 != ArteTable<RTRA>::end(); ++rtra1 ) {
    if (isMuon(*rtra1)) {
      double p1=fabs(1.0/(*rtra1)->pf);
      double pz1 = p1 / sqrt(1.+(*rtra1)->txf*(*rtra1)->txf +
			     (*rtra1)->tyf*(*rtra1)->tyf);
      double px1 = (*rtra1)->txf * pz1;
      double py1 = (*rtra1)->tyf * pz1;
      double E1= sqrt(p1*p1+0.1056*0.1056);
      
      for( ArteTable<RTRA>::iterator rtra2 = rtra1;
	   rtra2 != ArteTable<RTRA>::end(); ++rtra2 ) {
	if (isMuon(*rtra2)&&(*rtra1)->pf*(*rtra2)->pf<0) { // only look at opposite charged muons
	  double p2=fabs(1.0/(*rtra2)->pf);
	  double pz2 = p2 / sqrt(1.+(*rtra2)->txf*(*rtra2)->txf +
				 (*rtra2)->tyf*(*rtra2)->tyf);
          double px2 = (*rtra2)->txf * pz2;
          double py2 = (*rtra2)->tyf * pz2;
	  double E2= sqrt(p2*p2+0.1056*0.1056);
	  double mass = sqrt( 2.*(E1*E2 - px1*px2 - py1*py2 - pz1*pz2));
	  
	  
	  if (mass>MassCut) { // Found a J/psi candidate
	    int rc = JPSIvertex(*rtra1 ,*rtra2, MUONS_ALWAYS);
	    if (rc) {
	      ArtePointer<RTRA> rtra = ArteTable<RTRA>::size();
	      rtra->fit = 2;
	      rtra->lee = mass;
	      rtra->leh = mass;
	      rtra->cmp = (*rtra1)->cmp | (*rtra2)->cmp;
	    }
	  }
	}
      }
    }
  }
  
  ARTE_mark( "rcSelectJPSIMM  1", 0 );
  return 0;
}




