
// *****************************************************************************
//
// source:    
//
// type:      source code
//
// created:   26 Jul 2000
//	
// author:    Andreas Gellrich
//            HERA-B Collaboration
//            Humboldt-University Berlin
//            Physics Department
//            Invalidenstr. 110
//            10115 Berlin
//            Germany
//            E-mail: Andreas.Gellrich@desy.de
//
// changes:
// 26 Jul 2000 (AG) creation (code from Denis)
// 04 Aug 2000 (DD) wire constraint, mass histogram 
// 26 Oct 2000 (DD) MC part removed (set pi.lund and pi.mother instead)
// 23 Oct 2001 (DS) fill RTRA rich likelihoods only if not filled before,
//                  use maximal rich likelihoods instead of RISE likelihoods
//
// *****************************************************************************

#include "reco/rcX.h"
#include "reco/rcXkinema.h"
#include "farm/farm.hh"
#include "rhp.h"
#include "dq/DQ.h"

rcX *rcKs1=0;
rcX *rcKs2=0;

// int copyRTRItoRTRA(int algorithm);

int Ks__Piplus_Piminus_RECO();
int maximizeRTRA();
void rcResetRichRtra(float x=-1);

//
//
//
int
Ks()
{
  Ks__Piplus_Piminus_RECO();
  return 0;
}


//////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////
///////////////////////////  Ks reconstruction   /////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////

int
Ks__Piplus_Piminus_RECO()
{
  int verb=0;
  
  if (verb) cout<<">>>>>>>>>>>>>>>>>>  rec Ks2 <<<<<<<<<<<<<<<<<<"<<endl;
  if (!rcKs2) {
    cout<<"initializing Ks"<<endl;
    rcKs2=new rcX("rec_Ks", "Ks -> Pi+Pi", verb);
    decay ks;
    ks.rlevel        = 2;    // 0=full, 1=semi, 2=mtra
    ks.fit           = 103;  // = long decay 3 + flag 100 = 103
    ks.book_tables   = 1;    // 1/0
    ks.root_output   = 1;    // make root file or not
    ks.verb          = verb; // printout level
    rcKs2->setMother(ks);    // set
    vertexAttrib vks;
    vks.rlevel             = 2;     // 0=no, 1=chi2, 2=kalman, 3=kalman+mass
    vks.topp               = kinema::findTopp(310); // pointer to mother
    vks.max_vertex_chi2    = 10.0;  // limit for vertex acceptance 
    vks.verb               = verb;  // printout level
    vks.min_dz_from_wire   = 5;     // minimum decay distance (cm)
    vks.max_drho_from_wire = .5;    // check is decay points toward wire 
    vks.min_mass           = .4;    // min mass
    vks.max_mass           = .6;    // max mass
    vks.allowed_v0_tracks  = 0;     // pions not from v0
    rcKs2->setVertex(vks);          // set

    attribute pi1;
    pi1.topp         = kinema::findTopp(-211);
    pi1.lund         = -211;
    pi1.mother       = 310;
    pi1.mass         = pi1.topp->mass;
    pi1.minP         = -200;
    pi1.maxP         = 0;
    pi1.minPt        = .15;
    pi1.minLRPI      = .0001;
    pi1.maxLRE       = .01;
    pi1.minHRI       = 1;
    pi1.minHSI       = 6;
    pi1.minHIO       = 20;
    pi1.maxCandidate = 25;
    pi1.useVdsOnce   = 1;
    pi1.useMainOnce  = 1;
    rcKs2->addDaughter(pi1);

    attribute pi2;
    pi2.topp         = kinema::findTopp(211);
    pi2.lund         = 211;
    pi2.mother       = 310;
    pi2.mass         = pi2.topp->mass;
    pi2.minP         = 0;
    pi2.maxP         = 200;
    pi2.minPt        = .15;
    pi2.minLRPI      = .0001;
    pi2.maxLRE       = .01;
    pi2.minHRI       = 1;
    pi2.minHSI       = 6;
    pi1.minHIO       = 20;
    pi2.maxCandidate = 25;
    pi2.useVdsOnce   = 1;
    pi2.useMainOnce  = 1;
    rcKs2->addDaughter(pi2);

    if (verb) rcKs2->printDaughters();
  }
  rcKs2->rc();
  if (verb) rcKs2->printTracks();
  if (verb) cout<<">>>>>>>>>>>>>>>>>>  rec Ks2 <<<<<<<<<<<<<<<<<<"<<endl;
  return 0;
}




//
//
//
int
selectKs()
{
  ARTE_mark( "rcX_Ks 0", 0 );
  static int old_total=0;
  int ret=0;

  //=== check if rich likelihoods in RTRA are filled,
  //  take maximal likelihoods from  RTRI otherwise

  int richpid=1;
  if (ArteTable<RTRA>::size()!=0){
    ArtePointer<RTRA> rtra(1);
    if (rtra->lre<0&&rtra->lrmu<0&&rtra->lrpi<0&&rtra->lrk<0&&rtra->lrp<0){
      int mr=maximizeRTRA();
      richpid=-1;
    }
  }
  //  copyRTRItoRTRA(0); // use ringsearch
  rhp_hf1(DQ_ID( COMP_RECO,LVL_REC, 1030 ), 0., 1.); // new event
  ArteTable<RTRA>::iterator ks1=ArteTable<RTRA>::end();
  Ks();
  ArteTable<RTRA>::iterator ks2=ArteTable<RTRA>::end();
  if (rcKs2->total>old_total) { 
    old_total=rcKs2->total;
    rhp_hf1(DQ_ID( COMP_RECO,LVL_REC, 1030 ), 1., 1.); // Ks found
    // -- fill mass
    for (ArteTable<RTRA>::iterator iks=ks1; iks!=ks2; ++iks) {
      ArtePointer<RVER> rver=(*iks)->ever;
      float mass=rver->mass;
      rhp_hf1(DQ_ID( COMP_RECO,LVL_REC, 1031 ), mass, 1.);
    }
    ret=1; 
  }

  //== reset rich likelihoods in RTRA to default value if no rich likelihood
  //   was filled before calling this routine

  if (richpid<0){
       rcResetRichRtra(-1);
  }

  ARTE_mark( "rcX_Ks 1", 0 );
  //cout<<"selectKs ret="<<ret<<endl;
  return ret;
}

//
//
//
void
stopKs()
{
  if (rcKs1) delete rcKs1;
  if (rcKs2) delete rcKs2;
}
