#ifndef __SINGLEPLANEMIRROR_H__
#define __SINGLEPLANEMIRROR_H__

#include "rich/ricutil.h"
#include "rich/Losses.h"

// ------------------------------------------------------------------------
/// description of a single planar mirror
/**
  The class SinglePlaneMirror describe a single planar mirror. Such
  a mirror is a rectangle, described using the class Rectangle. See there
  for additional information.
*/
class PlaneMirror;

class SinglePlaneMirror : public Rectangle {
friend class PlaneMirror;

Table refl;
int loss;

int    ro,co;
char   ttype[9];
char   nname[21];


  inline int  Loss()            const { return loss; }
  inline int  IsReflected()     const { return (loss==1); }
  inline SinglePlaneMirror() {;}

protected:
  inline SinglePlaneMirror(const Rectangle& rr,Table& rrefl,int rro,int c)
     : Rectangle(rr) {
    refl=rrefl;ro=rro;co=c;loss=0;strcpy(nname,"undefined");
    strcpy(ttype,"undefined");
    }

/// if area is subdivided: returns the row, where the mirror is placed
  inline int  row(double y) const { return Rectangle::row(y); }
/// if area is subdivided: return the column, where the mirror is placed
  inline int  col(double x) const { return Rectangle::col(x);}
  
public:
//@Man: reflexion on the mirror
//@{
/// calculate intersection point
inline int intersection(const StraightLine& l,double *path) {
  if (!Rectangle::intersection(l,path)) return PMI_LOSS_MIRRORGAPS;
  return NO_LOSS;
  }
/// reflexion of a photon 
  int reflect(Photon *p);  
/// reflexion for tracking the track to the detector.
  int reflect(StraightLine *t);
/// calculate point on other side of the mirror with same normal distance as x
  inline Vec3 VirtualPoint(const Vec3& x) const {
    return x-2*((x-x0())*ez())*ez();
    }
//@}
/// returns the reflectivity curve of the mirror
  inline Table& reflectivity()   { return refl; }
/// returns the row, where the mirror is placed
  inline int  row() const { return ro; }
/// return the column, where the mirror is placed
  inline int  col() const { return co;}

/// return the name of the mirror
  inline const char*  name() const { return &nname[0];}
/// return the type of the mirror
  inline const char *type() const { return &ttype[0];}

/// assignment: SingleMirror a,b; a=b;
  inline void operator=(const SinglePlaneMirror& sm) {
    Rectangle::operator=(sm);
    refl=sm.refl;
    loss=sm.loss;
    }
/// print the contents of the class to an ASCII-FILE
  void print(FILE* fp=stdout);
/// constructor reading an ASCII-FILE produced by 'print'
  SinglePlaneMirror(ScriptFile& f,Table& reflectivy);
/// constructor using a record in the ARTE--Table CSPM
  SinglePlaneMirror(ArtePointer<CSPM> a);
/// copy constructor
  SinglePlaneMirror(const SinglePlaneMirror &);
  };

#endif
