//--*-C++-*--
//
//  Created: Apr 04 2001
//
//  Modifications:
//
//
#ifndef _RRINGFIT_H_
#define _RRINGFIT_H_
#include "arte/arte.hh"
#include "rise/RObject.h"

void dummyMinFunction();
void dummySelectROI();

class RRingFit {
 public:
  /// Constructor
  RRingFit();
  /// Destructor
  ~RRingFit() {;}
  /// Fit ring
  void fit(vector<RTrack>::iterator itrack);
  /// Make a global fit of found rings
  void globalfit();
  /// Set N0T=photons/(radiation distance)/(Ch. angle)^2
  inline void setN0T(float n0t) { RICH.N0T=n0t; }
  /// Set beta=1 radius
  inline void setRadius0(float r0) { RICH.radius0=r0; }
  /// Get current ring 
  inline RRing* getRing() { return ring; }
  /// Set current ring
  inline void   setRing(RRing &r) { ring=&r; track=r.itrack; }
  /// Compute likelihood function for current ring
  float likelihood();
  /// Set minimization function
  void setMinFunction( void (*myMinFunction)() ) { minimize=myMinFunction; }
  /// select ROI photons
  void (*selectROI)();
  /// Set ROI selection function
  void setSelectROI( void (*mySelectROI)() ) { selectROI=mySelectROI; }
  /// get hit list pointer
  inline vector<RCorrHit> *getHitList() { return &hit_list; }
  /// Fill probability fields for different hypothesis using erfc
  void  probability();

private:
  /// Current ring fit results
  RRing            *ring;
  /// Current track seed
  RTrack           *track;
  /// list of ROI photons
  vector<RCorrHit> hit_list;
  /// Print listo of ROI photons
  inline void printROI() {
    for (vector<RCorrHit>::iterator icorrhit=hit_list.begin();
	 icorrhit!=hit_list.end(); ++icorrhit) icorrhit->print();
  }
  /// List of initial fit conditions (hypothesis, search areas ...)
  vector<RRing> init_ring_list;
  ///  Pointer at minimization function
  void (*minimize)();
  /// Compute likelihood function
  float L();
  /// Function to compute background flux
  void Rb(bool signal);
  /// Compute Rb for signal and background
  inline void RbAll() {
    Rb(false);
    Rb(true);
  }
  /// Momentum resolution from track seed
  inline float dpp() {
    if (track->cov[14]>0)
      return sqrt(track->cov[14])/fabs(track->ip);
    return 0.02;
  }
  /// Angular X resolution
  inline float dX() {
    if (track->cov[5]<0) return 0.003;
    return sqrt(fabs(track->cov[5]));
  }
  /// Angular Y resolution
  inline float dY() {
    if (track->cov[9]<0) return 0.003;
    return sqrt(fabs(track->cov[9]));
  }  /// Track-ring matching chi2
  float matchingChi2();
  /// RICH parameters
  RRICH RICH;
  /// Use other rings in fitting
  bool globalFit;  
  /// Fit quality
  void fitQuality();
  /// Fit Chi2
  void fitChi2();
  /// Minimum distance from R2P2 line in terms of sigma
  void fitDh();
  /// Distance from hypothetical radius in terms of sigma
  void fitDr();
  /// Compute signal flux through the cell
  float flux(const RCorrHit &h);
  /// Compute flux for all photons
  void fluxAll();
  /// Compute flux due to other rings through the cell
  float beta(const RCorrHit &h);
  /// Compute beta for all photons
  void betaAll();
  /// Check the size of search area
  bool  isSAreaOK(bool signal);
  /// Print fit summary for all hypothesis
  void printFitSummary();
  /// Evaluate the truth table and fill the fit flag
  void fillFitFlag();
};

extern RRingFit *gRingFit;


#endif
