#ifndef CROBUSTMEAN_CC
#define CROBUSTMEAN_CC

#include "silarte/CRobustMeans.h"

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

// Implementation file for the RobustMeans class
// =======================================================================
// Author: Martin Braeuer 
//
//    From: 'Robust Estimation and Hypothesis testing'
//    (R. R. Wilcox, Academic Press, 1997)
//
// Modification history:
//
// MBr  12.01.2001: 1. Implementaion according to 
// MBr  .01.2001:                 
// MBr  00.00.2001:                   
// =======================================================================

#define SQR(X) ((X) * (X))

CRobustMeans::CRobustMeans() {
  CMedian::CMedian();
  //  printf("CRobustMeans: n=%d\n",iNumElements);
  Setall();
}

CRobustMeans::~CRobustMeans() {
  // CMedian::~CMedian;
}

void  CRobustMeans::Reset() {
  CMedian::Reset();
  Setall();
}

void CRobustMeans::Setall() {
  //printf("C: n=%d\n",iNumElements);
  iHuberMean     =   0;
  fHubMean       =  -1.0;
  iMaxHuberIts   = 100;
  dHuberBreak    =   0.0001;

  iHampelMean    =   0;
  fHamMean       = -1.0;
  iMaxHampelIts  = 100;
  dHampelBreak   =   0.0001;

  fHubMeanErr    = -1.0;

  fBiWeiMidVar   = -1.0;
  fMedBendMidVar = -1.0;
}

float CRobustMeans::GetHuberMean(float fCutOffConst = 1.28) {

  // Situation:
  if (iStatDone != 0 && iHuberMean != 0) return(fHubMean);
  if (iStatDone == 0) GetMedianStat();

  // Okay, start to work:
  double d,a,v;
  int               i,b;
  unsigned long int j;
  float  fMADN=fMAD*MADSCALE;
  //  printf("  Med=%8.4f  madn=%8.4f\n",fMedian,fMADN);

  fHubMean = pfStore[0]; // MEAN of one value!
  if (iCurrent > 1) {
    fHubMean = fMedian;
    for (i=0;i<iMaxHuberIts;++i) {
      a=0.0;b=0;
      for (j=0;j<iCurrent;++j) {
	v = (pfStore[j] - fHubMean)/fMADN;
	     if (v >  fCutOffConst) a += fCutOffConst;
	else if (v < -fCutOffConst) a -= fCutOffConst;
	else {
	  a += v;
	  b += 1; // PSI' of huber = 1 here, 0 else
	}
	     //	printf("%3d  %8.4f %4d %8.4f\n",j,a,b,v);
      } // loop over data
      d=0.0;
      if (b!=0) d = fMADN*a/b;
             else printf("Robust-Means-ERROR: 0-participiants found!! i=%d j=%ld,m=%8.4f\n",
			 i,j,fHubMean);
      //      printf("%d  d=%E\n",i,d);
      if (fabs(d) < dHuberBreak) break;
      fHubMean += d;
    } // of its
  } // of if >1
  //  printf("  Reached huber-mean after %d its, on %d points: %8.4f del=%7.2E\n",
  //	 i+1,j,fHubMean,d);
  iHuberMean = 1;
  return(fHubMean);
} // of GetHuberMean



// Method: W-Estimator, cp. Wilcox p. 58 or Agee/Turner
float CRobustMeans::GetHampelMean(const float fValA, 
				  const float fValB, 
				  const float fValC) {
  // already done?
  if (iStatDone != 0 && iHampelMean != 0) return(fHamMean);
  if (iStatDone == 0) GetMedianStat();

  // too bad, we have to use the stack now:
  double fUi,fAbsUi,fWi,fNumSum,fDenSum,fSignUi,fSlope,fHamMeanOld;
  int               i;
  unsigned long int j;
  float  fMADN=fMAD*MADSCALE;

  //  printf("  Med=%8.4f  madn=%8.4f  %8.4f %8.4f\n",
  //  fMedian,fMADN,fMAD,MADSCALE);
  fHamMean = pfStore[0]; // MEAN of one value!
  if (iCurrent > 1) {
    fSlope = fValA / (fValB - fValC);
    fHamMean = fMedian;
    for (i=0;i<iMaxHampelIts;++i) {
      fNumSum = 0.0;
      fDenSum = 0.0;
      for (j=0;j<iCurrent;++j) {
	fUi    = (pfStore[j] - fHamMean)/fMADN;

	// Get the PSI:
	fAbsUi = fabs(fUi);
	if (fUi > 0.0) 
	  fSignUi =  1.0; 
	else 
	  fSignUi = -1.0;
	if (fAbsUi < fValA) fWi = fUi;
	else if (fAbsUi < fValB)  fWi = fValA*fSignUi;
	else if (fAbsUi < fValC)  fWi = fSlope*(fUi - fValC*fSignUi);
	else continue; // fWi=0.0, so no more summs

	// Get the Wi
	if (fUi != 0.0) fWi /= fUi;
 	           else fWi = 1.0;

	fNumSum += fWi * pfStore[j];
	fDenSum += fWi;
        // printf("%3d  %8.4f %4d %8.4f\n",j,a,b,v);
      } // loop over data
      fHamMeanOld = fHamMean;
      fHamMean=0.0;
      if (fDenSum != 0.0) fHamMean = fNumSum / fDenSum;
             else printf("RobustHampelMeanERROR: 0-participiants found!! i=%d j=%ld m=%8.4f\n",
			 i,j,fHamMean);
      //      printf("%d  hamM=%12.6f  d=%8.2E\n",
      //	     i,fHamMean,fabs(fHamMeanOld - fHamMean));
      if ( fabs(fHamMeanOld - fHamMean) < 
	   dHampelBreak * fabs(fHamMean) + EPSILON) break;
    } // of its
  } // of if >1
  //  printf("  Reached Hampel-mean after %d its, on %d points: %8.4f del=%7.2E\n",
  //	 i+1,j,fHamMean,d);
  iHampelMean = 1;
  return(fHamMean);
} // of GetHampelMean




// ----------------------------------------------------------------------------
float CRobustMeans::BiWeightMidVariance(float fScale) {  
  double fYiSq,fSum1=0.0,fSum2=0.0,fNormSq,fHelp;

  fNormSq = 1.0 / SQR(fScale * fMAD);

  //  if (iStatDone != 0 && iHuberMean != 0) return(fMedBendMidVar);
  if (iStatDone == 0) GetMedianStat();
  
  // Biwight MidVar:
  unsigned long int i;
  for (i=0;i<iCurrent;++i) {
    fYiSq = SQR(pfStore[i]-fMedian)*fNormSq;
    if (fYiSq < 1.0) {
      fHelp = 1 - fYiSq;
      fSum1 += SQR(pfStore[i] - fMedian)*SQR(fHelp)*SQR(fHelp);
      fSum2 += fHelp*(1 - 5*fYiSq);
    }
  }
  fBiWeiMidVar = sqrt(iCurrent*fSum1)/fabs(fSum2);
  //  printf("SCALE-BiWeightMidVar: %8.4f \n",fBiWeiMidVar);
  
  return(fBiWeiMidVar);
}

// ----------------------------------------------------------------------------
float CRobustMeans::MedianBendMidvariance(float fScale) {
  int   iValA=0;
  float fSum1 = 0.0,yi;
  float fNorm = fScale*fMAD;
  unsigned long int i;
  //  if (iStatDone != 0 && iHuberMean != 0) return(fMedBendMidVar);
  if (iStatDone == 0) GetMedianStat();

  // Percentage-Bend-Midvarince: (Huber-Psi => NOT NICE without cuts!!)
  for (i=0;i<iCurrent;++i) {
    yi = (pfStore[i] - fMedian)/fNorm;
    if (yi < -1.) yi = -1.;
    if (yi >  1.) yi =  1.;
    fSum1 += SQR(yi);
    if (fabs(yi) < 1.0) ++iValA;
  }
  float fMedBendMidVar = fNorm*sqrt(iCurrent*fSum1)/float(iValA);
  //  printf("SCALE-Med.BendMidvar: %8.4f\n", fMedBendMidVar);
  return(fMedBendMidVar);
}


// ----------------------------------------------------------------------------
// NOT implemented yet!! (much additional work needed..)
float CRobustMeans::GetHuberMeanErr(float fCutOffConst = 1.28) {  
  return(0.0);
} // of GetHuberMeanErr


#endif // end of code

// =========== EOF ===== EOF ===== EOF ===== EOF ===== EOF ===== EOF =============
