#ifndef CLASS_HCheck_ICC
#define CLASS_HCheck_ICC


// implementation of the member functions for HCheck class library
// ===================================================================
// Author: Martin Braeuer 
//
// NOTE: 
//
// Modification history:
// MBr  23.02.2000: Initial derive and testing form L4dqCheck
// MBr  24.03.2000: Added single event HELIX status function       
// MBr  09.01.2001: seH-checks debugged (access to state-table!)
// MBr  24.01.2001: seH-checks debugged (access to state-table!)       
// ==================================================================

// includes :
#include  <stdio.h>
#include  <stdlib.h>
#include  <string.h>
#include  <math.h>
#include  <malloc.h>

// The robust friends:
#include  "silarte/CRobustMeans.h"

// temp and dev-work!:
// #include  "hbook.h"

// include the class's header file:
#include "Class_HCheck.h"


// a private maintain/check routine:
void HCheck_class::dump() {
  for (int j=0;j<MAXHELIX+1;++j) {
    printf("%2d: ",j);
    for (int usid = 0; usid < MAXUNSIN; ++usid) {
      //for (int i=0;i<MAXUNSIN;++i)
      switch (helixstat[usid][j]) {
        case H_LOW:     printf("-");break;
        case H_WORKING: printf("0");break;
        case H_HIGH:    printf("+");break;
	case H_MISSING: printf(" ");break;  
        default:        printf("?");
      }
    }
    printf("\n");
  }
}


/* ==================================================================
 * == HCheck_class constr. : Prepare structures for HCheck         ==
 * == Parameter:                                                   ==
 * ==                                                              ==
 * ================================================================== 
 */
HCheck_class::HCheck_class( ) {

  prepare = 0; // This 'makes' a new class -> NOTHING can be prepared!
  accumulate = 0;
  evaluate = 0;
  Pstore = NULL;

  fPrepare(); // do the reset

  cEVCUT      = EVCUT;  
  cMINCUT     = MINCUT;  
  cMAXCUT_N   = MAXCUT_N;  
  cMAXCUT_P   = MAXCUT_P;  
  cHELIXCUT   = HELIXCUT;  
  cMINMEANHIT = MINMEANHIT; 
  globalmeanhit = 0.;
  iHi = 0;
  return;
}

/* ==================================================================
 * == HCheck_class constr. : Prepare structures for HCheck         ==
 * == Parameter:                                                   ==
 * ==                                                              ==
 * ================================================================== 
 */
HCheck_class::HCheck_class(int (*storef)(int, int, int)) {

  prepare = 0; // This 'makes' a new class -> NOTHING can be prepared!
  accumulate = 0;
  evaluate = 0;
  Pstore = storef;

  fPrepare(); // do the reset

  cEVCUT      = EVCUT;  
  cMINCUT     = MINCUT;  
  cMAXCUT_N   = MAXCUT_N;  
  cMAXCUT_P   = MAXCUT_P;  
  cHELIXCUT   = HELIXCUT;  
  cMINMEANHIT = MINMEANHIT; 
  globalmeanhit = 0.;
  iHi = 0;
  return;
}


/* ==================================================================
 * == HCheck_class destructor : Delete structures for Class_HCheck ==
 * ==                                                              ==
 * ================================================================== 
 */
HCheck_class::~HCheck_class() {
 fClear();
}


/* ==================================================================
 * == fClear : Clear/unbook all allocated structures               ==
 * ==                                                              ==
 * ================================================================== 
 */
void HCheck_class::fClear() {
  //HCeck does evrything internally..
  if (prepare>0) { // only if prepared crrectly..
    //    if (type     !=NULL ) { free((char*) type     );type       = NULL;}
  }   
  prepare    = 0;
  accumulate = 0;
  evaluate   = 0;
  ntrack =0;

  return;
}


void HCheck_class::fError_out(const char *s) {
  /*====================================
     Give output to the various sources
   =====================================
   */

  fprintf(stderr,"VDS_DQ: Class_HCheck: Fatal Error : '%s'!!\n",s);  
  return;
} // =============== END of fOut member func. ========================================


/* ==================================================================
 * == fPrepare : Prepare structures for HCheck class               ==
 * == Parameter: ----                                              ==
 * == needs    : ----                                              ==
 * ==                                                              ==
 * ================================================================== 
 */
int HCheck_class::fPrepare() {
  prepare    = 1;
  accumulate = 0;
  evaluate   = 0;
  // Do this always during init 
  for (int i=0;i<MAXUNSIN;++i) 
    for (int j=0;j<MAXHELIX;++j) {
      ocche[i][j] = 0;
      seHelixStat[i][j] = seINVALID;
      seOccHe[i][j]     = 0;
  }
  seAccumulated = 0; // reset single event hit counter
  seMaxHit  = seMAXHITS;
  seMinAccu = seMINACCU;

  return(1);
}

/* ==================================================================
 * == fSetUnmounted : Set the non-mounted planes list              ==
 * == Parameter: *list : -1 terminated list of unmounted USID's    ==
 * == needs    : a prepared HCut object                            ==
 * == return   : 1 okay, -1 invalid planes                         ==
 * ================================================================== 
 */
int HCheck_class::fSetUnmounted(int *list) {
  if (prepare ==0) return(0);

  for (int i=0;i<MAXUNSIN;mounted[i++] = H_WORKING); // ALL mounted.

  // Now the exeptions:
  for (int i=0;i<MAXUNSIN;++i) {
    if (list[i] == -1) break;
    if ((list[i] < -1) || (list[i]>=MAXUNSIN)) return(0);
    mounted[list[i]] = H_LOW;
  }
  return(1);
}


/* ==================================================================
 * == fSetHelixPar : Set Cuts for helixes                          ==
 * == needs  : --                                                  ==
 * == pars   : evcut     cluster < events/EVCUT => dead!           ==
 * ==          mincut    cluster < MINCUT       => dead!           ==
 * ==          maxn      (MAXN*meanhit) = max no of hits/chip  N   ==
 * ==          maxp      (MAXP*meanhit) = max no of hits/chip  P   ==
 * ==          minchips  >MINCHIPS number of chips need to work    ==
 * ==          minmeanhit > <hits all sides>/minmeanhit must       ==
 * ==                       be on a plane                          ==
 * == return : 1 : OK, 0: not initialised                          ==
 * ==                                                              ==
 * ================================================================== 
 */
int HCheck_class::fSetPar(int evcut, int mincut, float maxn, float maxp,
			     int minchips, float minmeanhit) {
  if (prepare != 1) return(0); // prepare before
  cEVCUT    = evcut;   
  cMINCUT   = mincut;  
  cMAXCUT_N = maxn;    
  cMAXCUT_P = maxp;    
  cHELIXCUT = minchips;
  cMINMEANHIT= minmeanhit;
  return(1);
} // =============== END of fSetHelixPar member func. ========================================


/* ==================================================================
 * == fGetHelixPar : Get Cuts for helixes                          ==
 * == needs  : --                                                  ==
 * == pars   : evcut     cluster < events/EVCUT => dead!           ==
 * ==          mincut    cluster < MINCUT       => dead!           ==
 * ==          maxn      (MAXN*meanhit) = max no of hits/chip  N   ==
 * ==          maxp      (MAXP*meanhit) = max no of hits/chip  P   ==
 * ==          minchips  >MINCHIPS number of chips need to work    ==
 * ==          minmeanhit > <hits all sides>/minmeanhit must       ==
 * ==                       be on a plane                          ==
 * == return : 1 if set correctly                                  ==
 * ==                                                              ==
 * ================================================================== 
 */
int HCheck_class::fGetPar(int *evcut, int *mincut, float *maxn, float *maxp,
			     int *minchips, float *minmeanhit) {
  if (prepare != 1) return(0); // prepare before
  *evcut      = cEVCUT;    
  *mincut     = cMINCUT;   
  *maxn       = cMAXCUT_N; 
  *maxp       = cMAXCUT_P; 
  *minchips   = cHELIXCUT;
  *minmeanhit = cMINMEANHIT;
  return(1);
} // =============== END of fGetHelixPar member func. ========================================
       

/* ====================================================================
 * == fAccuHits: Insert hits into the int. occ. maps                 ==
 * == needs    : Class prepared                                      ==
 * == pars     : side=USID of hit, cog cog of hit, number of entries ==
 * == return   : 1 if OK, 0 if not prepared or invalid side-code     ==
 * == Note     : This works with usid and side id !!!                ==
 * == Note     : Use THIS always to store. A NULL in the contructor  ==
 * ==             will lead to an internal store, a function pointer ==
 * ==             to external storage. (YOU have to copy back then!) ==
 * ==                                                                ==
 * ==================================================================== 
 */
int HCheck_class::fAccuHits(int usid,float cog, int num) {
  if (prepare != 1) return(0); // prepare before
  evaluate = 0; // we have a new hit!

  if ((usid < 0) || (usid >= MAXUNSIN)) return(0); // accept only 0..127 !!
  //printf("  gc%d  %8.3f %d |",gedechan,cog,fHelixid(side,cog));fflush(stdout);

  if (Pstore == NULL) {
    ocche[usid][fUHelixid(usid,cog)]+=num;
    accumulate+=num;            // only in case of internal store !!!
  }
  else
    Pstore(usid,fHelixid(usid,cog),num);
  return(1);
} // =============== END of fAccuHits (side,usid,num) member func. =====================


/* ====================================================================
 * == fAccuHit : Insert a hit into the int. occ. maps                ==
 * == needs    : Class prepared                                      ==
 * == pars     : side=USID of hit, cog cog of hit                    ==
 * == return   : 1 if OK, 0 if not prepared or invalid side-code     ==
 * == Note     : This works with usid and side id !!!                ==
 * == Note     : Use THIS always to store. A NULL in the contructor  ==
 * ==             will lead to an internal store, a function pointer ==
 * ==             to external storage. (YOU have to copy back then!) ==
 * ==                                                                ==
 * ==================================================================== 
 */
int HCheck_class::fAccuHit(int side,float cog) {
  if (prepare != 1) return(0); // prepare before
  evaluate = 0; // we have a new hit!
  int usid=-1;
  if (side>=10000 && side<=80903) {
    usid = fSideUnsin(side);
    //printf("s=%d, us=%3d  cog=%8.2f he=%d\n",side,usid,cog,fHelixid(side,cog));
  } else
    if ((side >-1) && (side < MAXUNSIN)) usid=side;
  if (usid == -1) return(0); // accept only 0..127 !!
  //printf("AH:  %d gc%d  %8.3f %d |",side, usid,cog,fHelixid(usid,cog));fflush(stdout);

  if (Pstore == NULL) {
    ++ocche[usid][fUHelixid(usid,cog)];
    ++accumulate;            // only in case of internal store !!!
  }
  else
    Pstore(usid,fHelixid(usid,cog),1);
  return(1);
} // =============== END of fAccuHit (side,usid,num) member func. =====================



/* ==================================================================
 * == fAcceptHit: check that a hit does not contain bad chips      ==
 * == needs    : preppare has to run before                        ==
 * == pars     : side ID and cog of hit                            ==
 * == return   : 1 hit OK, 0 if not prepared or hit bad            ==
 * ==                                                              ==
 * ================================================================== 
 */
int HCheck_class::fAcceptHit(int side,float cog) {
 return(1);
}


/* ==================================================================
 * == fClearHelixes : reset hit-maps                               ==
 * == needs  : Class prepared                                      ==
 * == pars   : --                                                  ==
 * == return : 0 if not prepared                                   ==
 * ==                                                              ==
 * ================================================================== 
 */
int HCheck_class::fClearHelixes () {
  if (prepare != 1) return(0);
  for (int usid=0;usid<MAXUNSIN;++usid)
    for (int i=0;i<MAXHELIX;++ i)
      ocche[usid][i] = 0;
  accumulate = 0;
  return(1);
}

/* ==================================================================
 * == fEvaluate1: set up a new helixstate matrix                   ==
 * == needs     : fAccuHit at least MIN_ACCU times                 ==
 * == pars      : evdone=number of processed events                ==
 * == return    : 1: no change, 0: chips changed, -1: error        ==
 * ==                                                              ==
 * ================================================================== 
 */

int HCheck_class::fEvaluate1(int evdone) {
  int nhelix=0;

  // Calculate the mean no of chips to find out if we can process:
  float meanhithelix = 0.;
  int   nohelix      = 0; 
  for (int usid=0; usid<MAXUNSIN; ++usid) {
    if (mounted[usid] != H_WORKING) continue; // evaluate only mounted ones..
    if (((usid%4) == 1) || ((usid%4) == 2))
      nhelix=8; // p-side
    else
      nhelix=10; //n-side;
    for (int j=0;j<nhelix;++j) {
      nohelix++;
      meanhithelix+=ocche[usid][j];
    }
  }
  // at least MIN_ACCU hits/chip in tot. seen
  if (accumulate < (MIN_ACCU*meanhithelix/nohelix)) return(NOT_READY);

  // clear the helixstate array:
  for (int i=0;i<MAXUNSIN;++i) 
    for (int j=0;j<MAXHELIX+1;++j) {
      o_helixstat[i][j] = helixstat[i][j];
    }

  for (int usid=0; usid<MAXUNSIN; ++usid) {
    if (mounted[usid] != H_WORKING) continue; // evaluate only mounted ones..

    for (int j=0;j<MAXHELIX+1;++j) {
      helixstat[usid][j] = H_WORKING; // set all existing to WORING
    }
  }
  // evaluate helixes:, loop over all gedes existing
  //printf("  %d\n",helixstat[0][0]);
  //dump();printf("\nd1\n");
  int minok = int(evdone/cEVCUT);
  if (minok < cMINCUT) minok=cMINCUT;
  //printf("  %d\n",helixstat[0][0]);

  if (minok < cMINCUT) minok=cMINCUT;

  for (int usid=0; usid<MAXUNSIN; ++usid) {
    if (mounted[usid] != H_WORKING) continue; // evaluate only mounted ones..

    float meanhit=0.;
    if (((usid%4) == 1) || ((usid%4) == 2))
      nhelix=8; // p-side
    else
      nhelix=10; //n-side;
    
    // the exclude-loops:
    // find to few events:
    int hcount =0;
    // calc underrun, calc mean
    for (int i=0;i<nhelix;++i) {
      if (ocche[usid][i] < minok) 
	helixstat[usid][i] = H_LOW; // to few cluster
      else {
	++hcount;
	helixstat[usid][i] = H_WORKING;
      }
      meanhit += float(ocche[usid][i]);
    }
    if (nhelix == 8) {
      helixstat[usid][8] = H_LOW;
      helixstat[usid][9] = H_LOW;
    }

    int hcount_o=hcount;
    //printf("%d  hc=%2d\n",hgedeP->side,hcount);
    while (1) {
      //calc act. mean value:
      meanhit=0.;
      for (int i=0;i<nhelix;++i) 
	if (helixstat[usid][i] == H_WORKING) meanhit+=float(ocche[usid][i]);
      meanhit/=hcount;

      // flag to death with overrruns:
      hcount = 0;
      for (int i=0;i<nhelix;++i) {
	if (helixstat[usid][i] == H_WORKING)
	  if (nhelix == 10) // n-side!
	    if (ocche[usid][i] > (cMAXCUT_N*meanhit)) 
	      helixstat[usid][i] = H_HIGH; // to much cluster
	    else ++hcount;
	  else 
	    if (ocche[usid][i] > (cMAXCUT_P*meanhit)) 
	      helixstat[usid][i] = H_HIGH; // to much cluster
	    else ++hcount;
      }
      if (hcount_o == hcount) break; // did not find anydody to remove!
      hcount_o=hcount;
    }
    //    printf("   %d  hc=%2d\n",hgedeP->side,hcount);
    

    //total plane to kill:
    if (hcount < cHELIXCUT) {
      //printf("%d  ",hgedeP->side);
      for (int i=0;i<MAXHELIX+1;++i) {
	helixstat[usid][MAXHELIX] = H_LOW;  // dead for align5
	//	printf("%3d  %4d   ",i,ocche[usid][i]);
      }
      //hgedeP->status = 0;                 // dead for tracking
      //printf("\n");
    }
    //printf("  %d\n",helixstat[0][0]);

  } // of gede loop 1

  //dump();printf("\nd2\n");
  //now we build the global mean hit no's:
  globalmeanhit=0.;
  int planecount = 0;
  int totalhelix = 0;
  int hitweight[MAXUNSIN]; // number of participating helixes
  int hitsplane[MAXUNSIN];
  for (int i=0;i<MAXUNSIN;++i) {
    hitsplane[i]=0;
    hitweight[i]=0;
  }

  for (int usid=0; usid<MAXUNSIN; ++usid) {
    if (mounted[usid] != H_WORKING) continue; // evaluate only mounted ones..
    if (((usid%4) == 1) || ((usid%4) == 2))
      nhelix=8; // p-side
    else
      nhelix=10; //n-side;
      
    int hitsonplane = 0;
    if (helixstat[usid][MAXHELIX] == H_WORKING) ++planecount; // only workings
                                           else continue; 
    for (int i=0;i<nhelix;++i)
      if (helixstat[usid][i] == H_WORKING) {
	hitsonplane+=ocche[usid][i]; // take the hits of each working helix
	++hitweight[usid];
	++totalhelix;
      }
    // printf("%3d hw=%2d\n",usid,hitweight[usid]);
    //hitsonplane = int(hitsonplane*10/hitweight[usid]);
    hitsplane[usid] = hitsonplane;
    globalmeanhit += hitsonplane;
  } // of gede loop 2

  //dump();

  globalmeanhit /= planecount;

  for (int i=0;i<MAXUNSIN;++i) {
    //if (hitsplane[i] >0) printf("  %3d : g=%5.0f  act=%5d nh=%2d\n",i,globalmeanhit,hitsplane[i],hitweight[i]);
    if ((hitsplane[i] >0)&&(hitsplane[i] < globalmeanhit/cMINMEANHIT)) { // kill a plane again:
      for (int j=0;j<MAXHELIX+1;++j) 
	//printf("  %d: %4d %4d",j,ocche[i][j],helixstat[i][j]);
      for (int j=0;j<MAXHELIX+1;++j)
	helixstat[i][j] = H_LOW;
      // hgedeP->status = 0;                 // dead for tracking
      //printf("\n");
    }
  }

  have_acc    = accumulate;
  have_events = evdone;
  accumulate = 0; // reset the accumulation flag, to start a new one
  evaluate = 1;   // now we have succesfully evaluated

  // check if plane had identical problems before!
  int retk = 1;
  for (int i=0;i<MAXUNSIN;++i) {
    if ((  helixstat[i][MAXHELIX] != H_WORKING) &&
	(o_helixstat[i][MAXHELIX] != H_WORKING)    ) continue;
    for (int j=0;j<MAXHELIX+1;++j) 
      if (o_helixstat[i][j] != helixstat[i][j]) {
	retk=0;
	//printf("c hange in (%3d,%2d)  o=%d n=%d\n",i,j,o_helixstat[i][j],helixstat[i][j]);
      }
  }
  //printf("return %d\n",retk);
  //dump();
  //int i;
  //printf("enter1: ");scanf("%d",&i);
  return(retk);
}  // =============== END of fEvaluate1 member func. ========================================

/* ==================================================================
 * == fEvaluate2: set up a new helixstate matrix                   ==
 * == needs     : fAccuHit at least MIN_ACCU times                 ==
 * == pars      : evdone=number of processed events                ==
 * == return    : 1: no change, 0: chips changed, -1: error        ==
 * ==            second try for evaluation                         ==
 * ================================================================== 
 */

int HCheck_class::fEvaluate2(int evdone) {

  float highmean =  1.9;
  float lowmean  =  0.1;
  float minh     = -0.7;
  float maxh     =  0.7;
  int   lowhelix =  2;
  int   nhelix   =  0;
  int i,j,usid;

  //printf("AVIATION: HCheck 2: eval nev=%d hi=%d\n",evdone,accumulate);
  //dump(); 

  //  hbook2(800+iHi,"occ, seen",128,-0.5,127.5,11,-0.5,10.5);
  // Calculate the mean no of chips to find out if we can process:
  float meanhithelix = 0.;
  int   nohelix      = 0; 
  for (usid=0; usid<MAXUNSIN; ++usid) {
    if (mounted[usid] != H_WORKING) continue; // evaluate only mounted ones..
    if (((usid%4) == 1) || ((usid%4) == 2))
      nhelix=8; // p-side
    else
      nhelix=10; //n-side;
    for (int j=0;j<nhelix;++j) {
      nohelix++;
      meanhithelix+=ocche[usid][j];
      // printf("n=%d  ",ocche[usid][j]);
      //   hf2(800+iHi,float(usid),float(j),float(ocche[usid][j]));
    }
  }
  printf("minh = %8.4f\n",meanhithelix/nohelix);
  // at least MIN_ACCU hits/chip in tot. seen
  if (accumulate < (MIN_ACCU*meanhithelix/nohelix)) return(NOT_READY);

  // clear the helixstate array:
  for (i=0;i<MAXUNSIN;++i) 
    for (int j=0;j<MAXHELIX+1;++j) {
      o_helixstat[i][j] = helixstat[i][j];
    }

  for (usid=0; usid<MAXUNSIN; ++usid) {
    if (mounted[usid] != H_WORKING) continue; // evaluate only mounted ones..
    for (int j=0;j<MAXHELIX+1;++j) {
      helixstat[usid][j] = H_WORKING; // set all existing to WORING
    }
  }

  int n1c = 0,n2c = 0,p1c = 0;
  float n1mean[MAXHELIX+1];
  float n2mean[MAXHELIX+1];
  float p1mean[MAXHELIX+1];
  float meanplane[MAXUNSIN];
  float nocc[MAXUNSIN][MAXHELIX+1];
  for (i=0;i<MAXHELIX+1;++i) {
    n1mean[i]    = 0.;
    n2mean[i]    = 0.;
    p1mean[i]    = 0.;
  }

  for (i=0;i<MAXUNSIN;meanplane[i++] = 0.);


  for (usid=0; usid<MAXUNSIN; ++usid) {
    if (mounted[usid] != H_WORKING) continue; // evaluate only mounted ones..
    switch (usid%4) {
    case 0 : 
	for (int i=0;i<10;++i) 
	  n1mean[i]+=ocche[usid][i];
	n1c++;
	break;
    case 3 :
      for (int i=0;i<10;++i) 
	n2mean[i]+=ocche[usid][i];
      n2c++;
      break;
    case 1 :
    case 2 :
      for (int i=0;i<8;++i) 
	p1mean[i]+=ocche[usid][i];
      p1c++;
      break;
    }      
  } // of gede loop
  //  hbook1(804+iHi,"means p0,n1,n2, munp0,n1n2",6,0.5,6.5);
  //  hf1(804+iHi,4,p1c);hf1(804,5,n1c);hf1(804,6,n2c);  
  //  hbook1(805+iHi,"means n1",10,-0.5,9.5);
  //  hbook1(806+iHi,"means n2",10,-0.5,9.5);
  //  hbook1(807+iHi,"means p1",10,-0.5,9.5);
  for (i=0;i<10;++i) {
    n1mean[i]/=n1c;// hf1(805+iHi,float(i),n1mean[i]);
    n2mean[i]/=n2c;// hf1(806+iHi,float(i),n2mean[i]);
    p1mean[i]/=p1c;// hf1(807+iHi,float(i),p1mean[i]);
  }

  /*
  printf("\nmeans for HELIX positions (n1hits=%d n2hits=%d phits=%d):\n n1: ",n1c,n2c,p1c);
  for (int i=0;i<10;++i)
    printf("%8.2f  ",n1mean[i]);
  printf("\n n2: ");
  for (int i=0;i<10;++i)
    printf("%8.2f  ",n2mean[i]);
  printf("\n p0: ");
  for (int i=0;i<8;++i)
    printf("%8.2f  ",p1mean[i]);
  printf("\n");
  */

  // now normalise the chip-occupancies:
  int ophelix =0;

  //  hbook2(801+iHi,"n1-check",64,-0.5,63.5,10,-0.5,9.5);int iNK1=0;
  //  hbook2(802+iHi,"p0-check",64,-0.5,63.5, 8,-0.5,7.5);int iNK2=0;
  //  hbook2(803+iHi,"n2-check",64,-0.5,63.5,10,-0.5,9.5);int iNK3=0;
  //  hbook2(821+iHi,"n1 sum",64,-0.5,63.5,10,-0.5,9.5);
  //  hbook2(822+iHi,"p0 sum",64,-0.5,63.5, 8,-0.5,7.5);
  //  hbook2(823+iHi,"n2 sum",64,-0.5,63.5,10,-0.5,9.5);
  for (usid=0; usid<MAXUNSIN; ++usid) {
    if (mounted[usid] != H_WORKING) continue; // evaluate only mounted ones..
    if (((usid%4) == 1) || ((usid%4) == 2))
      nhelix=8; // p-side
    else
      nhelix=10; //n-side;

    ophelix = 0;
    meanplane[usid] = 0.;
    for (i=0;i<nhelix;++i) {
      if ((usid%4) == 0) nocc[usid][i] = ocche[usid][i]/n1mean[i];
      if ((usid%4) == 1) nocc[usid][i] = ocche[usid][i]/p1mean[i];
      if ((usid%4) == 2) nocc[usid][i] = ocche[usid][i]/p1mean[i];
      if ((usid%4) == 3) nocc[usid][i] = ocche[usid][i]/n2mean[i];
      if (nocc[usid][i] < lowmean) 
	nocc[usid][i] = 0.0;
      else
	if (nocc[usid][i] > highmean) 
	  nocc[usid][i] = 10.0;
        else {
	  ++ophelix;
	  meanplane[usid] += nocc[usid][i];
	}
      //      if ((usid%4) == 0) hf2(801+iHi,float(iNK1),float(i),ocche[usid][i]);
      //      if ((usid%4) == 1) hf2(802+iHi,float(iNK2),float(i),ocche[usid][i]);
      // if ((usid%4) == 2) hf2(802+iHi,float(iNK2),float(i),ocche[usid][i]);
      // if ((usid%4) == 3) hf2(803+iHi,float(iNK3),float(i),ocche[usid][i]);

    } //of he-i loop
    if (ophelix==0) meanplane[usid] = 0.0;
               else meanplane[usid] /= ophelix;

    // flag total planes:
    if (ophelix < lowhelix) {
      for (i=0;i<MAXHELIX+1;++i)
	helixstat[usid][i] = H_LOW;
    }

    // mean value of plane:
    if (meanplane[usid] < lowmean) {
      for (i=0;i<MAXHELIX+1;++i)
	helixstat[usid][i] = H_LOW;
    }
    if (meanplane[usid] > highmean) {
      for (i=0;i<MAXHELIX+1;++i)
	helixstat[usid][i] = H_HIGH;
    }

    /*    if ((usid%4) == 0) {
	  ++iNK1;
	  //      hf2(821+iHi,float(iNK1),float(i),nocc[usid][i]);
	  }
	  if ((usid%4) == 1) {
	  ++iNK2;
	  //  hf2(822+iHi,float(iNK2),float(i),nocc[usid][i]);
	  }
	  if ((usid%4) == 2) {
	  ++iNK2;
	  //   hf2(822+iHi,float(iNK2),float(i),nocc[usid][i]);
	  }
	  if ((usid%4) == 3) {
	  ++iNK3;
	  //  hf2(823+iHi,float(iNK3),float(i),nocc[usid][i]);
	  }
    */
  } // of for usid


  //  hbook2(810+iHi,"diff in occ",128,-0.5,127.5,11,-0.5,10.5);
  //  hbook1(811+iHi,"meanplane",128,-0.5,127.5);
  for (usid=0; usid<MAXUNSIN; ++usid) {
    if (mounted[usid] != H_WORKING) continue; // evaluate only mounted ones..
    //  hf1(811+iHi,float(usid),meanplane[usid]);
    //    printf("%8.3f  ",meanplane[usid]);
    if (((usid%4) == 1) || ((usid%4) == 2))
      nhelix=8; // p-side
    else
      nhelix=10; //n-side;

    if (helixstat[usid][MAXHELIX] == H_HIGH) {
      //printf("%3d: Too hot !\n",usid);
      continue;
    }
    if (helixstat[usid][MAXHELIX] == H_LOW) {
      //printf("%3d: Too few hits !\n",usid);
      continue;
    }
    //printf("%3d| ",usid);

    //Calculate the difference to means of current plane:
    for (i=0;i<nhelix;++i) {
      //printf("%5.2f %4d ",nocc[usid][i]-meanplane[usid],ocche[usid][i]);
      float w = nocc[usid][i]-meanplane[usid];
      //    hf2(810+iHi,float(usid),float(i),w);
      if (w>maxh) helixstat[usid][i]=H_HIGH;
      if (w<minh) helixstat[usid][i]=H_LOW;
      //if (w<minh) printf("--  ");
      //if (w>maxh) printf("++  ");
      //if ((w<=maxh) &&  (w>=minh)) printf("    ");
    }
    //    if (nhelix ==8) printf("                ");
    //printf("        tm=%6.2f\n",meanplane[usid]);
  }


  have_acc    = accumulate;
  have_events = evdone;
  accumulate = 0; // reset the accumulation flag, to start a new one
  evaluate = 1;   // now we have succesfully evaluated

  // check if plane had identical problems before!
  int retk = 1;
  for (i=0;i<MAXUNSIN;++i) {
    if ((  helixstat[i][MAXHELIX] != H_WORKING) &&
	(o_helixstat[i][MAXHELIX] != H_WORKING)    ) continue;
    for (int j=0;j<MAXHELIX+1;++j) {
      if (((o_helixstat[i][j] == H_LOW) && (helixstat[i][j] == H_HIGH)) ||
	  ((o_helixstat[i][j] == H_HIGH) && (helixstat[i][j] == H_LOW))    )
	continue; // ill before and also now, but changed state
      if (o_helixstat[i][j] != helixstat[i][j]) {
	retk=0;
	//printf("change in (%3d,%2d)  o=%d n=%d\n",i,j,o_helixstat[i][j],helixstat[i][j]);
      }
    }
  }
  // finally reset the occupancy/helix
  //  hbook2(820+iHi,"occ, seen",128,-0.5,127.5,11,-0.5,10.5);
  for (usid=0;usid<MAXUNSIN;++usid)
    for (j=0;j<MAXHELIX;++j) {
      //  hf2(820+iHi,float(usid),float(j),float(ocche[usid][j]));
      ocche[usid][j] = 0;
    }
  //printf("return %d\n",retk);
  //dump();
  //int i;
  //printf("enter1: ");scanf("%d",&i);
  //  printf("HID=%d+800+i\n",iHi);iHi+=100;
  return(retk);
}  // =============== END of fEvaluate2 member func. ========================================

/* ==================================================================
 * == fEvaluate3: set up a new helixstate matrix                   ==
 * == needs     : fAccuHit at least MIN_ACCU times                 ==
 * == pars      : evdone=number of processed events                ==
 * == return    : 1: no change, 0: chips changed, -1: error        ==
 * ==            second try for evaluation                         ==
 * ================================================================== 
 */

int HCheck_class::fEvaluate3(int evdone) {

  float fCut1High  =  5.0;
  float fCut1Low   = -3.0;
  float fLowPlane  =  0.2; // as a x5 inside 1sig loss 
  float fHighPlane =  5.0; // as a x5 inside 1sig 'gain'
  float fCut2High  =  2.0;
  float fCut2Low   = -2.0;
  float w=0.0,fMeanVal=0.0,fMeanSig=0.0;
  int   lowhelix =  4;
  int   nhelix   =  0;
  int   iKill = 0;
  int i,usid;
  
  // this IS something:
  CRobustMeans cRMeanChips(MAXUNSIN * MAXHELIX);

  //printf("AVIATION: HCheck 3: eval nev=%d hi=%d\n",evdone,accumulate);
  //dump(); 

  //  hbook2(800+iHi,"occ, seen",128,-0.5,127.5,11,-0.5,10.5);
  // Calculate the mean no of chips to find out if we can process:
  float meanhithelix = 0.;
  int   nohelix      = 0; 
  cRMeanChips.Reset();
  for (usid=0; usid<MAXUNSIN; ++usid) {
    if (mounted[usid] != H_WORKING) continue; // evaluate only mounted ones..
    if (((usid%4) == 1) || ((usid%4) == 2))
      nhelix=8; // p-side
    else
      nhelix=10; //n-side;
    for (int j=0;j<nhelix;++j) {
      if (ocche[usid][j] == 0) continue;  // NEW!!
      nohelix++;
      cRMeanChips.Insert(float(ocche[usid][j]));
      //  hf2(800+iHi,float(usid),float(j),float(ocche[usid][j]));
    }
  }
  meanhithelix =  cRMeanChips.GetHampelMean();
  //  printf("minh = %8.4f n=%d (maxw=%d)\n",meanhithelix,nohelix,MAXHELIX*MAXUNSIN);
  // at least MIN_ACCU hits/chip in tot. seen
  if (accumulate < MIN_ACCU*meanhithelix) return(NOT_READY);

  // copy the old helixstate array:
  for (i=0;i<MAXUNSIN;++i) 
    for (int j=0;j<MAXHELIX+1;++j) {
      o_helixstat[i][j] = helixstat[i][j];
    }

  for (usid=0; usid<MAXUNSIN; ++usid) {
    if (mounted[usid] != H_WORKING) continue; // evaluate only mounted ones..
    for (int j=0;j<MAXHELIX+1;++j) {
      helixstat[usid][j] = H_WORKING; // set all existing to WORING
    }
  }

  // Missing: r-normalising?

  // count usid's:
  int n1c = 0,n2c = 0,p1c = 0; // counters for ID
  for (usid=0; usid<MAXUNSIN; ++usid) {
    if (mounted[usid] != H_WORKING) continue; // evaluate only mounted ones..
    if (usid%4 == 0) n1c++;
                else if (usid%3 == 0) n2c++;
                                 else p1c++;
  }
  //  hbook1(804+iHi,"means p0,n1,n2, munp0,n1n2",6,0.5,6.5); // save numbers!
  //  hf1(804+iHi,4,p1c);hf1(804,5,n1c);hf1(804,6,n2c);

  /*   hbook1(805+iHi,"ref n1",10,-0.5,9.5);
       hbook1(806+iHi,"ref n2",10,-0.5,9.5);
       hbook1(807+iHi,"ref p1",10,-0.5,9.5);
       hbook1(825+iHi,"sig-ref n1",10,-0.5,9.5);
       hbook1(826+iHi,"sig-ref n2",10,-0.5,9.5);
       hbook1(827+iHi,"sig-ref p1",10,-0.5,9.5);
  */
  float n1mean[MAXHELIX+1],n1sig[MAXHELIX+1];
  float n2mean[MAXHELIX+1],n2sig[MAXHELIX+1];
  float p1mean[MAXHELIX+1],p1sig[MAXHELIX+1];

  CRobustMeans cRMeanSide(MAXUNSIN);
  
  // get robust n1-means:
  for (i=0;i<10;i++) { // N : 10 chips
    cRMeanSide.Reset();
    for (usid=0; usid<MAXUNSIN; ++usid) {
      if ((mounted[usid]  != H_WORKING) || 
	  (usid%4         != 0        ) ||   // n1-sides
	  (ocche[usid][i] == 0        )    ) continue;// NO zero contrib.
      cRMeanSide.Insert(float(ocche[usid][i]));
    }
    n1mean[i] = cRMeanSide.GetHampelMean();
    n1sig[i]  = cRMeanSide.BiWeightMidVariance();
    //    printf("n1m(%2d)=%8.1f  ",i,n1mean[i]);
    //    hf1(805+iHi,float(i),n1mean[i]);
    //    hf1(825+iHi,float(i),n1sig[i]);
  }
  //  printf("\n");
  for (i=0;i<10;i++) { // N : 10 chips
    cRMeanSide.Reset();
    for (usid=0; usid<MAXUNSIN; ++usid) {
      if ((mounted[usid]  != H_WORKING) || 
	  (usid%4         != 3        ) ||   // n2-sides
	  (ocche[usid][i] == 0        )    ) continue;// NO zero contrib.
      cRMeanSide.Insert(float(ocche[usid][i]));
    }
    n2mean[i] = cRMeanSide.GetHampelMean();
    n2sig[i]  = cRMeanSide.BiWeightMidVariance();
    //    printf("n2m(%2d)=%8.1f  ",i,n2mean[i]);
    //    hf1(806+iHi,float(i),n2mean[i]);
    //    hf1(826+iHi,float(i),n2sig[i]);
  }
  //  printf("\n");
  for (i=0;i<8;i++) { // P : 8 chips
    cRMeanSide.Reset();
    for (usid=0; usid<MAXUNSIN; ++usid) {
      if ((mounted[usid]  != H_WORKING) || 
	  (usid%4         == 0        ) ||  // p-sides
	  (usid%4         == 3        ) || 
	  (ocche[usid][i] == 0        )    ) continue;// NO zero contrib.
      cRMeanSide.Insert(float(ocche[usid][i]));
    }
    p1mean[i] = cRMeanSide.GetHampelMean();
    p1sig[i]  = cRMeanSide.BiWeightMidVariance();
    //    printf("p1m(%2d)=%8.1f  ",i,p1mean[i]);
    //    hf1(807+iHi,float(i),p1mean[i]);
    //    hf1(827+iHi,float(i),p1sig[i]);
  }
  //  printf("\n");

  float meanplane[MAXUNSIN];        
  float nocc[MAXUNSIN][MAXHELIX+1]; // normalised occupancy

  // now normalise the chip-occupancies:
  int ophelix =0;

  /*   hbook2(801+iHi,"n1-check",64,-0.5,63.5,10,-0.5,9.5);int iNK1=0; // nnp-sorted occupancy
       hbook2(802+iHi,"p0-check",64,-0.5,63.5, 8,-0.5,7.5);int iNK2=0;
       hbook2(803+iHi,"n2-check",64,-0.5,63.5,10,-0.5,9.5);int iNK3=0;
       hbook2(821+iHi,"n1 sum",64,-0.5,63.5,10,-0.5,9.5);              // norm-occ, nnp-sorted
       hbook2(822+iHi,"p0 sum",64,-0.5,63.5, 8,-0.5,7.5);
       hbook2(823+iHi,"n2 sum",64,-0.5,63.5,10,-0.5,9.5);
       hbook2(808+iHi,"ref-normed vals",128,-0.5,127.5,10,-0.5,9.5);  
       for (usid=0;usid<128;++usid) 
       for (i=0;i<MAXHELIX;++i) hf2(808,float(usid),float(i), -100.0);
       hbook1(809+iHi,"ref-normed distrib",100,-9.9,9.9);
       hbook1(811+iHi,"meanplane meas",128,-0.5,127.5);
       hbook1(812+iHi,"meanplane errs",128,-0.5,127.5);
       hbook1(813+iHi,"meanplane norm",128,-0.5,127.5);
       hbook1(814+iHi,"meanplane norm-dist",100,-1.7,1.7);
  */
  for (usid=0; usid<MAXUNSIN; ++usid) {
    if (mounted[usid] != H_WORKING) continue; // evaluate only mounted ones..
    if (((usid%4) == 1) || ((usid%4) == 2))
      nhelix=8; // p-side
    else
      nhelix=10; //n-side;

    ophelix = 0;
    cRMeanSide.Reset();
    for (i=0;i<nhelix;++i) {
      if ((usid%4) == 0) {
	//	nocc[usid][i] = ocche[usid][i]/n1mean[i];
	//	hf2(801+iHi,float(iNK1),float(i),ocche[usid][i]);
	w = (ocche[usid][i] - n1mean[i])/n1sig[i]; 
      }
      if (((usid%4) == 1) || ((usid%4) == 2)) {
	//	nocc[usid][i] = ocche[usid][i]/p1mean[i];
	//	hf2(802+iHi,float(iNK2),float(i),ocche[usid][i]);
	w = (ocche[usid][i] - p1mean[i])/p1sig[i]; 
      }
      if ((usid%4) == 3) {
	//	nocc[usid][i] = ocche[usid][i]/n2mean[i];
	//	hf2(803+iHi,float(iNK3),float(i),ocche[usid][i]);
	w = (ocche[usid][i] - n2mean[i])/n2sig[i]; 
      }
      nocc[usid][i] = w;
      //      hf2(808+iHi,float(usid),float(i),w+100.0);
      //      hf1(809+iHi,w,1.0);
      
      if (w < fCut1Low) {                        // 1. soft cut LOWER
	helixstat[usid][i]=H_LOW;
	//	printf("1 Cold-cut: usid=%3d ch=%2d: s=%8.4f\n",usid,i,w);
	nocc[usid][i] = -100.0;
      } else {                                   // 1. soft cut UPPER
	if (w > fCut1High) { 
	  helixstat[usid][i]=H_HIGH;
	  //	  printf("1 Hot-cut : usid=%3d ch=%2d: s=%8.4f\n",usid,i,w);
	  nocc[usid][i] = 100.0;
	} else {
	  ++ophelix;
	  if ((usid%4) == 0) 
	    cRMeanSide.Insert(float(ocche[usid][i])/n1mean[i]);
	  if (((usid%4) == 1) || ((usid%4) == 2))
	    cRMeanSide.Insert(float(ocche[usid][i])/p1mean[i]);
	  if ((usid%4) == 3) 
	    cRMeanSide.Insert(float(ocche[usid][i])/n2mean[i]);
	}
      }
    } //of the-i loop
    
    // flag total planes:
    iKill = 0; // no kill of plane..
    if (ophelix < lowhelix) {
      iKill = -1;
      meanplane[usid] = 0.0; // normalise OUT !
      w = -100; // significance below all limits
    } else {
      fMeanVal = cRMeanSide.GetHampelMean();
      fMeanSig = cRMeanSide.BiWeightMidVariance();
      meanplane[usid] = fMeanVal; // store for normalisation!
      w = fMeanVal - fMeanSig;
      /*
	  hf1(811+iHi,float(usid),fMeanVal);      
	  hf1(812+iHi,float(usid),fMeanSig);
	  hf1(813+iHi,float(usid),w);
	  hf1(814+iHi,w,1.0);
      */
    }

    // flag total planes:
    if (ophelix < lowhelix)                  iKill = -1;
    if ((fMeanVal - fMeanSig) < fLowPlane )  iKill = -1;
    if ((fMeanVal + fMeanSig) > fHighPlane)  iKill =  1;
    
    if (iKill == 1) {
      for(i=0;i<MAXHELIX + 1;++i)
	helixstat[usid][i] = H_HIGH;
      meanplane[usid] = -100.0;
    } else if (iKill == -1) {
      for(i=0;i<MAXHELIX + 1;++i)
	helixstat[usid][i] = H_LOW;
      meanplane[usid] = -100.0;
    }      
    //   if (iKill != 0) printf("KR1: p=%3d k=%2d\n",usid,iKill);
    
    /*    // sorting:
	  if ((usid%4) == 0) {
	  for (i=0;i<10;++i) hf2(821+iHi,float(iNK1),float(i),nocc[usid][i]);
	  ++iNK1;
	  }
	  if (((usid%4) == 1) || ((usid%4) == 2)){
	  for (i=0;i<8;++i) hf2(822+iHi,float(iNK2),float(i),nocc[usid][i]);
	  ++iNK2;
	  }
	  if ((usid%4) == 3) {
	  for (i=0;i<10;++i) hf2(823+iHi,float(iNK3),float(i),nocc[usid][i]);
	  ++iNK3;
	  }    
    */
  } // of for usid // -----------------------------------------------------------
  
  //  hbook2(817+iHi,"plane - normed vals    " , 128, -0.5, 127.5, 10, -0.5,  9.5);
  //  for (usid=0;usid<128;++usid) 
    //    for (i=0;i<MAXHELIX;++i) hf2(817,float(usid),float(i), -100.0);
  //  hbook1(818+iHi,"plane - normed distrib"  , 100, fCut1Low, fCut1High);
  //  hbook1(819+iHi,"plane - normed occupancy", 128, -0.5, 127.5);

  for (usid=0; usid<MAXUNSIN; ++usid) { // ------------- 2. 2. 2. 2. -----------
    if ((mounted[usid] != H_WORKING)          || 
	(helixstat[usid][MAXHELIX] == H_HIGH) || 
	(helixstat[usid][MAXHELIX] == H_LOW)     ) continue;
    
    if (((usid%4) == 1) || ((usid%4) == 2)) nhelix=8; // p-side
    else nhelix=10; //n-side;
     
    //hard cut, normalised groups:
    ophelix = 0;
    for (i=0;i<nhelix;++i) {
      if (helixstat[usid][i] != H_WORKING) continue;
      
      // new norms according to groups with plane-norms:
      if (meanplane[usid] <= 0.0) 
	w = -100.;
      else {
	if ((usid%4) == 0)
	  w = ((ocche[usid][i]/meanplane[usid]) - n1mean[i])/n1sig[i];       
	if (((usid%4) == 1) || ((usid%4) == 2)) 
	  w = ((ocche[usid][i]/meanplane[usid]) - p1mean[i])/p1sig[i]; 
	if ((usid%4) == 3)
	  w = ((ocche[usid][i]/meanplane[usid]) - n2mean[i])/n2sig[i]; 
	//	hf1(819+iHi,float(usid),ocche[usid][i] / meanplane[usid]);
      }
      //      hf2(817+iHi,float(usid),float(i),w+100.0);
      //      hf1(818+iHi,w,1.0);

      if (w <fCut2Low) {                       // -------- 2. hard cut LOWER
	helixstat[usid][i]=H_LOW;
	//	  printf("2 Cold-cut: usid=%3d ch=%2d: s=%8.4f\n",usid,i,w);
      } else 
	if (w > fCut2High) {                   // -------- 2. hard cut UPPER
	  helixstat[usid][i]=H_HIGH;
	  //	  printf("2 Hot-cut : usid=%3d ch=%2d: s=%8.4f\n",usid,i,w);
	} else
	  ++ophelix;  
    } // chip-loop
    if (ophelix < lowhelix) {
      //      printf("PLANE low chipno: p=%3d: nc=%2d\n",usid,ophelix);
      for (i=0;i<MAXHELIX+1;++i)
      	helixstat[usid][i] = H_LOW;
    }
  } // usid-loop
  
  
  have_acc    = accumulate;
  have_events = evdone;
  accumulate = 0; // reset the accumulation flag, to start a new one
  evaluate = 1;   // now we have succesfully evaluated
  
  // check if plane had identical problems before!
  int retk = 1;
  for (i=0;i<MAXUNSIN;++i) {
    if ((  helixstat[i][MAXHELIX] != H_WORKING) &&
	(o_helixstat[i][MAXHELIX] != H_WORKING)    ) continue;
    for (int j=0;j<MAXHELIX+1;++j) {
      if (((o_helixstat[i][j] == H_LOW) && (helixstat[i][j] == H_HIGH)) ||
	  ((o_helixstat[i][j] == H_HIGH) && (helixstat[i][j] == H_LOW))    )
	continue; // ill before and also now, but changed state
      if (o_helixstat[i][j] != helixstat[i][j]) {
	retk=0;
	//printf("change in (%3d,%2d)  o=%d n=%d\n",i,j,o_helixstat[i][j],helixstat[i][j]);
      }
    }
  }
  
  //  printf("HID = %d+i\n",iHi+800);iHi+=100;
  return(retk);
}  // =============== END of fEvaluate3 member func. ========================================
  

/* ===================================================================
 * == fCorrectGeometry: scale the occupancy matrix of a side to     ==
 * ==                   correct for special geometries (50x!!)      ==
 * == needs     : fAccuHit has to have been run, called before eval ==
 * == iSide     : ID of side to correct                             ==
 * == fFact     : Arrey of factros to correct with (NO check done!) ==
 * == return    : 1: OK, 0: wrong side                              ==
 * ==                                                               ==
 * =================================================================== 
 */
int HCheck_class::fCorrectGeometry(      int   iSide,
				   const float fFact[MAXHELIX] ) {
  if (iSide<0 || iSide>MAXUNSIN) return(0);
  for (int i=0;i<MAXHELIX;++i) 
    ocche[iSide][i] = int(ocche[iSide][i] * fFact[i]);
  
  return(1);
} // =============== END of fCorrectGeometry member func. ===================================



/* ===================================================================
 * == fComputeRefs: Get a histo+sigmas of each helix-pos in a set of==
 * ==               Superlayer                                      ==
 * == needs     : fAccuHit has to have been run                     ==
 * == iSuperlayer1 : ID of first SL (1..8)                          ==
 * == iSuperlayer2 : ID of first SL (1..8)                          ==
 * == fValues      : Values found (order: N1, N2, P1, MAXHELIX each)==
 * == fSigmas      : corresponing sigmas                            ==
 * == return       : 1: OK, 0: wrong SL's                           ==
 * ==                                                               ==
 * =================================================================== 
 */
int HCheck_class::fComputeRefs(int   iSuperlayer1,
			       int   iSuperlayer2,
			       float fValues[3*MAXHELIX],
			       float fSigmas[3*MAXHELIX] ) {
  --iSuperlayer1;
  if (iSuperlayer1 < 0 || iSuperlayer1 >= MAXUNSIN/16 ||
      iSuperlayer2 < 1 || iSuperlayer2 >  MAXUNSIN/16    ) return(0); // SL invalid 

  int iSideStart = 16*iSuperlayer1;
  int iSideStop  = 16*iSuperlayer2;
  int usid,i;
  CRobustMeans cRMeanSide(MAXUNSIN);
  
  //  printf("SL-analysis: SL=%2-d%2d side=%3d to %3d\n");

  // get robust n1-means:
  for (i=0;i<MAXHELIX;i++) { // N : 10 chips
    cRMeanSide.Reset();
    fValues[i] = 0.0;
    fSigmas[i] = 0.0;
    for (usid=iSideStart; usid<iSideStop; ++usid) {
      if ((mounted[usid]  != H_WORKING) || 
	  (usid%4         != 0        ) ||   // n1-sides
	  (ocche[usid][i] == 0        )    ) continue;// NO zero contrib.
      cRMeanSide.Insert(float(ocche[usid][i]));
    }
    fValues[i] = cRMeanSide.GetHampelMean();
    fSigmas[i] = cRMeanSide.BiWeightMidVariance();
  }

  for (i=0;i<MAXHELIX;i++) { // N : 10 chips
    cRMeanSide.Reset();
    fValues[i+ MAXHELIX] = 0.0;
    fSigmas[i+ MAXHELIX] = 0.0;
    for (usid=iSideStart; usid<iSideStop; ++usid) {
      if ((mounted[usid]  != H_WORKING) || 
	  (usid%4         != 3        ) ||   // n2-sides
	  (ocche[usid][i] == 0        )    ) continue;// NO zero contrib.
      cRMeanSide.Insert(float(ocche[usid][i]));
    }
    fValues[i + MAXHELIX] = cRMeanSide.GetHampelMean();
    fSigmas[i + MAXHELIX] = cRMeanSide.BiWeightMidVariance();
  }
 
  for (i=0;i<MAXHELIX;i++) { // P : 8 chips
    cRMeanSide.Reset();
    fValues[i + MAXHELIX + MAXHELIX] = 0.0;
    fSigmas[i + MAXHELIX + MAXHELIX] = 0.0;
    if (i>7) break;
    for (usid=iSideStart; usid<iSideStop; ++usid) {
      if ((mounted[usid]  != H_WORKING) || 
	  (usid%4         == 0        ) ||  // p-sides
	  (usid%4         == 3        ) || 
	  (ocche[usid][i] == 0        )    ) continue;// NO zero contrib.
      cRMeanSide.Insert(float(ocche[usid][i]));
    }
    fValues[i + MAXHELIX + MAXHELIX] = cRMeanSide.GetHampelMean();
    fSigmas[i + MAXHELIX + MAXHELIX] = cRMeanSide.BiWeightMidVariance();
  }
  //printf("  mh=%d  v1=%8.4f v2=%8.4f \n",MAXHELIX,fValues[28],fValues[29]);
  return(1);
} // =============== END of fComputeRefs member func. =======================================
 

  
/* ==================================================================
 * == fResult  : Textoutput of result                              ==
 * == needs    : fEvaluate1 has to run before                      ==
 * == pars     :                                                   ==
 * == return   : 1 if OK, 0 if not evaluated before                ==
 * ==                                                              ==
 * ================================================================== 
 */
int HCheck_class::fResult() {
 int nc,nhelix,i;
 if (evaluate != 1) return(0); // evaluate before

 printf("\nList of planes with suspicious properties:\n");
 /*
 printf(" used cuts (for %d accumulated hits in %d events) : \n",have_acc,have_events);
 printf("  EVCUT      (no of clus/chip < events/EVCUT)                 : %3d\n",cEVCUT);
 printf("  MINCUT     (no of clus/chip < MINCUT)                       : %3d\n",cMINCUT);
 printf("  MAXCUT_N   (no of clus/chip > MAXCUT_N*meanhits_on_n_plane) : %6.2f\n",cMAXCUT_N);
 printf("  MAXCUT_P   (no of clus/chip > MAXCUT_P*meanhits_on_p_plane) : %6.2f\n",cMAXCUT_P);
 printf("  MINMEANHIT (each plane must have > meanhit/minmeanhit clus) : %6.2f\n",cMINMEANHIT);
 printf("  HELIXCUT   ( > HELIXCUT chips need to work )                : %d\n\n",cHELIXCUT);
 */

 // loop whole usid struct:
 for (int usid=0; usid<MAXUNSIN; ++usid) {
   if (((usid%4) == 1) || ((usid%4) == 2))
     nhelix=8; // p-side
   else
     nhelix=10; //n-side;

   if (mounted[usid] != H_WORKING) continue; // evaluate ony mounted ones..

   if (helixstat[usid][MAXHELIX] != H_WORKING) {
     printf("%3d : Entire Plane dead! ",usid);
     if (helixstat[usid][MAXHELIX] == H_LOW) 
       printf("Too few hits.\n");
     else 
       printf("Too hot.\n");
   } else {
     // check for changes:
     nc = 0;
     for (i=0;i<nhelix;++i)
       if (helixstat[usid][i] != H_WORKING) {
	 nc=1;break;
       }
     if (nc == 1) {
       printf("%3d : Helixes dead     : ",usid);
       for (i=0;i<nhelix;++i) {
	 printf("  %d",i);
	 if (helixstat[usid][i] == H_LOW)     printf("--");
	 if (helixstat[usid][i] == H_WORKING) printf("  ");
	 if (helixstat[usid][i] == H_HIGH)    printf("++");
       }
       printf("\n");
     }
   }
 }
 return(1);
} // =============== END of fResult member func. ========================================

/* ==================================================================
 * == fGetWorking : returns the number of working helixes          ==
 * == needs    : fEvaluate1/2 has to run before                    ==
 * == pars     :                                                   ==
 * == return   : -1: if error, number otherwise                    ==
 * ==                                                              ==
 * ================================================================== 
 */
int HCheck_class::fGetWorking() {
 int nhelix;
 
 if (evaluate != 1) return(-1); // evaluate before

 int working = 0;

 // loop whole usid struct:
 for (int usid=0; usid<MAXUNSIN; ++usid) {
   if (((usid%4) == 1) || ((usid%4) == 2))
     nhelix=8; // p-side
   else
     nhelix=10; //n-side;

   if (mounted[usid] != H_WORKING) continue; // evaluate only mounted ones..
   for (int j=0;j<nhelix;++j)
     if (helixstat[usid][j] == H_WORKING) ++working; 
 }
 return(working);
} // =============== END of fGetWorking member func. ========================================

/* ==================================================================
 * == fGetNonWorking : returns the number of not working helixes   ==
 * == needs    : fEvaluate1/2 has to run before                    ==
 * == pars     :                                                   ==
 * == return   : -1: if error, number otherwise                    ==
 * ==                                                              ==
 * ================================================================== 
 */
int HCheck_class::fGetNotWorking() {
 int nhelix;
 
 if (evaluate != 1) return(-1); // evaluate before

 int nworking = 0;

 // loop whole usid struct:
 for (int usid=0; usid<MAXUNSIN; ++usid) {
   if (mounted[usid] != H_WORKING) continue; // evaluate only mounted ones..

   if (((usid%4) == 1) || ((usid%4) == 2))
     nhelix=8; // p-side
   else
     nhelix=10; //n-side;

   for (int j=0;j<nhelix;++j)
     if (helixstat[usid][j] != H_WORKING) ++nworking; 
 }
 return(nworking);
} // =============== END of fGetNotWorking member func. ============================


/*
  ==================================================================================
  ==================================================================================
  ===       SINGLE EVENT:
  ==================================================================================
  ==================================================================================
*/
       
/* ==================================================================
 * == fseReset : Reset the single event cut structs                ==
 * == needs    : --                                                ==
 * == pars     : --                                                ==
 * == return   : --                                                ==
 * ==                                                              ==
 * ================================================================== 
 */
void HCheck_class::fseReset() {
  for (int usid=0;usid<MAXUNSIN;++usid)
    for (int he=0;he<MAXHELIX;++he) {
      seHelixStat[usid][he] = seINVALID;
      seOccHe[usid][he]      = 0;
      seOccHeWrong[usid][he] = 0;
    }
  return;
}

       
/* ==================================================================
 * == fseSetPars : Set the actual parameter for the se-functs      ==
 * == needs    : --                                                ==
 * == pars     : maxhitChip : Max num. of       hits/chip to accept==
 * ==            maxwronghit: Max num. of wrong-hits/chip to accept==
 * ==            minhits    : Min number of hits/event to process  ==
 * == return   : --                                                ==
 * ==                                                              ==
 * ================================================================== 
 */
void HCheck_class::fseSetPars(int maxhitChip, int maxwronghit, int minhits) {
  seMaxHit      = maxhitChip;
  seMaxWrongHit = maxwronghit;
  seMinAccu     = minhits;
  return;
}

       
/* ==================================================================
 * == fseGetPars : Get the actual parameter for the se-functs      ==
 * == needs    : --                                                ==
 * == pars     : maxhitChip* : Max num. of hits/chip to accept     ==
 * ==            maxwronghit*: Max num. of wrong-hits/chip to accept=
 * ==            minhits*    : Min number of hits/event to process ==
 * == return   : --                                                ==
 * ==                                                              ==
 * ================================================================== 
 */
void HCheck_class::fseGetPars(int* maxhitChip, int *maxwronghit, int* minhits) {
  *maxhitChip = seMaxHit;
  *maxwronghit= seMaxWrongHit;
  *minhits    = seMinAccu;
  return;
}

       
/* ==================================================================
 * == fseAccuHit : Accumulate a hit                                ==
 * == needs    : class inited, Reset done in time                  ==
 * == pars     : side (USID or side ID), cog of hit                ==
 * ==            iSign: 1 if right sign hit, -1 else. Default: 1   ==
 * == return   : 1 if stored                                       ==
 * ==                                                              ==
 * ================================================================== 
 */
int HCheck_class::fseAccuHit(int side,float cog, int iSign = 1) {
  //printf("s=%3d, cog=%8.2f %d\n",side,cog,fHelixid(side,cog));
  if (seAccumulated == 0) fseReset(); // new event!

  if (side>-1 && side<MAXHELIX)
    // ATTENTION: Use fUHelix ID since we have USID
    if (iSign>0) 
      ++seOccHe[     side][fUHelixid(side,cog)];
    else
      ++seOccHeWrong[side][fUHelixid(side,cog)];      
  else
    if (side>=10000 && side<=80903) {
      // ATTENTION: Use fHHelix ID since we have SideId
      //printf("s=%3d, cog=%8.2f %d\n",side,cog,fHelixid(side,cog));
      if (iSign>0) 
	++seOccHe[fSideUnsin(side)][     fHelixid(side,cog)];
      else
	++seOccHeWrong[fSideUnsin(side)][fHelixid(side,cog)];      
    }
    else
      return(0); // invalid side ID
  ++seAccumulated;          
  return(1);
}

/* ==================================================================
 * == fseAccuHit : Accumulate a hit                                ==
 * == needs    : class inited, Reset done in time                  ==
 * == pars     : superlayer, sector, layer, cog                    ==
 * ==            iSign: 1 if right sign hit, -1 else. Default: 1   ==
 * == return   : 1 if stored                                       ==
 * ==                                                              ==
 * ================================================================== 
 */
int HCheck_class::fseAccuHit(int sl, int sec, int lay, float cog, int iSign = 1) {
  int usid = fUnsin(sl,sec,lay);
  if (seAccumulated == 0) fseReset(); // new event!
  if (usid>-1 && usid<128)
    if (iSign>0) 
      ++seOccHe[usid][     fUHelixid(usid,cog)];
    else
      ++seOccHeWrong[usid][fUHelixid(usid,cog)];      
  else
    return(0); // invalid side ID
  ++seAccumulated;  
  return(1);
}

// The public state table: 
// 0=seBAD : BAD, 1=seOK : OK,  -1 : seINVALID
//signed byte seHelixStat[MAXUNSIN][MAXHELIX];

/* ==================================================================
 * == fseEvaluate : See if event is OK, reset done                 ==
 * == needs    : at least seMinHits accumulated hits               ==
 * == pars     : --                                                ==
 * == return   : -1 if not enough statistic,                       ==
 * ==            or number of bad guys                             ==
 * ==                                                              ==
 * ================================================================== 
 */
int HCheck_class::fseEvaluate() {
  int nChips = 0;
  if (seAccumulated < seMinAccu) return(-1);
  for (int usid=0;usid<MAXUNSIN;++usid)
    for (int he=0;he<MAXHELIX;++he)     {
      seHelixStat[usid][he] = seBAD;
      if ((     seOccHe[usid][he] < seMaxHit      ) && 
	  (seOccHeWrong[usid][he] < seMaxWrongHit )    )  // found a good guy!
	seHelixStat[usid][he] = seOK;
      else 	
	++nChips;
      // everything is done, reset the current chip:
      //      seOccHe[usid][he] = 0; 
      //      seOccHeWrong[usid][he] = 0; 
    }  
  seAccumulated = 0;
  return(nChips);
} // of fseEvaluate


/* ==================================================================
 * == fseHitOkay : See if hit is OK, reset done                    ==
 * == needs    : NO accumulated hits                               ==
 * == pars     : side (USID or side ID), cog of hit                ==
 * == return   : -1 if no evaluation availible,                    ==
 * ==            -2 if resulting layer is incorrect                ==
 * ==             1 if Hit is OK, 0 else                           ==
 * ==                                                              ==
 * ================================================================== 
 */
int HCheck_class::fseHitOkay(int side,float cog) {
  //printf("s=%3d, cog=%8.2f %d\n",side,cog,fHelixid(side,cog));
  if (seAccumulated != 0) return(-1);

  if (side>-1 && side<MAXUNSIN) {
    if (seHelixStat[side][fUHelixid(side,cog)] == seOK)
      return(1);
    else
      return(0);
  } else {
    if (side>=10000 && side<=80903) {
      //printf("s=%3d, cog=%8.2f %d\n",side,cog,fHelixid(side,cog));
      if (seHelixStat[fSideUnsin(side)][fHelixid(side,cog)] == seOK)
	return(1);
      else
	return(0);
    }
  }
  return(-2);
} // of fseHitOkay(1)

int HCheck_class::fseHitOkay(int iUsid,int iChip) {
  //printf("s=%3d, chip=%d \n",side,iChip);
  if (seAccumulated != 0) return(-1);
  if (iChip<0 || iChip>=MAXHELIX) return(-1);

  if (iUsid>-1 && iUsid<MAXUNSIN) {
    if (seHelixStat[iUsid][iChip] == seOK)
      return(1);
    else
      return(0);
  } 
  return(-2);
} // of fseHitOkay(1)


/* ==================================================================
 * == fseHitOkay : See if hit is OK, reset done                    ==
 * == needs    : NO accumulated hits                               ==
 * == pars     : superlayer, sector, layer, cog                    ==
 * == return   : -1 if no evaluation availible,                    ==
 * ==            -2 if resulting layer is incorrect                ==
 * ==             1 if Hit is OK, 0 else                           ==
 * ==                                                              ==
 * ================================================================== 
 */
int HCheck_class::fseHitOkay(int sl, int sec, int lay, float cog) {
  if (seAccumulated != 0) return(-1);

  int usid = fUnsin(sl,sec,lay);
  if (usid<0 || usid>=MAXHELIX)   return(-2);

  // look in table (simple?!!!):
  if (seHelixStat[usid][fUHelixid(usid,cog)] == seOK)  return(1);

  return(0);
} // of fseHitOkay(2)

#endif
// ============ EOF ====== EOF ====== EOF ====== EOF ====== EOF ====== EOF
