// *****************************************************************************
//
// source:    kasltsim.C
//
// type:      source code
//
// created:   16 Mar 2001
//
// author:    Lars Soezueer
//            email: soezueer@mail.desy.de
//
// purpose:   treat KUIP commands for sltsim from sltarte
//
// changes:   03-aug-2001 (BP) Changed default SLT executable to summer 2000
//            15-oct-2001 (LS) Commands for SLT parameters
//            06-nov-2001 (PK) SLT parameters -ECALminor, -ECALmajor, -ECALDB
//            07-nov-2001 (LS) removed SLT parameter -ECALCALIB:
//            07-nov-2001 (BP) Use $HBROOT instead of /afs/desy.de/
//                             to find default executable
//
// *****************************************************************************
// standard includes
#include <iostream.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>

// Arte includes
#include "farm/farm.hh"
#include "sltarte/kasltsim.hh"
#include "arte/ArteMessage.hh"

extern "C"
{
  char* ku_gets();
  void kugeti_(int *);
  int ku_npar(void);
}

kasltsim* kasltsim::m_instance = 0;

kasltsim* kasltsim::Instance()
{
  if (m_instance == 0) {
    m_instance = new kasltsim();
  }
  return m_instance;
}

kasltsim::kasltsim():
 m_edcfile("sltsim2000"),
 m_debuglevel(0),
 m_runnumber()
{
  char* hbroot=getenv("HBROOT");
  if (! hbroot) {
    hbroot="/afs/desy.de/group/hera-b";
    arteMsg << "kasltsim.C: $HBROOT not set, assume " << hbroot;
    arteMsg.warn();
  }
  char* path=new char[256];
  path[0]=0;
  strcpy(path,hbroot);
  path=strcat(path,"/SLT/Rev0604/Linux_intel_glibc/bin/");
 
  m_edcpath=path;
 m_SLT_params["-SMCPNM:"]          = "~/RC/top";
 m_SLT_params["-SMCSTT:"]          = "10";
 m_SLT_params["-SLPevc:"]          = "MON_0";
 m_SLT_params["-Process:"]         = "SLP/_0";
 m_SLT_params["-EVAfile:"]         = "";
 m_SLT_params["-SLPerrpost:"]      = "0";
 m_SLT_params["-SLPnodist:"]       = "1";
 m_SLT_params["-SLPseglen:"]       = "200000";
 m_SLT_params["-EVAdiag:"]         = "0";
 m_SLT_params["-EVAformat:"]       = "raw";
 m_SLT_params["-EDCnoRND"]         = "";
 m_SLT_params["-EDCtarg"]          = "";
 m_SLT_params["-EDCreadpipe"]      = "";
 m_SLT_params["-SLPscn:"]          = "scn";
 m_SLT_params["+P"]                = "vds-itr.iol";
 m_SLT_params["-SLPNoSMC"]         = "";
 m_SLT_params["-SLPNoSCN"]         = "";
 m_SLT_params["-EDCmc:"]           = "2";
 m_SLT_params["-CARE_noHotCl:"]    = "0";
 m_SLT_params["-SLT_EcalIniMode:"] = "1";
 m_SLT_params["-SLT_geoIniMode:"]  = "3";
 m_SLT_params["-TGStrigMode:"]     = "3";
 m_SLT_params["-EDCpipeout:"]      = "1";
 m_SLT_params["-ECALDB:"]          = "/ECAL_CALIB";
 locked_SLT_params["-Domain:"]     = "";
 locked_SLT_params["-SLPdbg:"]     = "";
 locked_SLT_params["-PlbTrigType:"]= "";
 locked_SLT_params["-ECALminor "]  = "";
 locked_SLT_params["-ECALmajor "]  = "";
};

bool kasltsim::set_SLT_param(char* param, char* value, bool force) {
    //returns true if parameter is successfully changed
    SLT_params_t::iterator curl;
    if (! force) curl = Instance()->locked_SLT_params.find(param);
    if (force || curl==Instance()->locked_SLT_params.end()) {
      SLT_params_t::iterator cur = Instance()->m_SLT_params.find(param);
      if (cur==Instance()->m_SLT_params.end()) {
        cout << "Setting new SLT parameter " << param << value << endl;
      } else {
        cout << "Changing SLT parameter " << param << cur->second
	     << " to " << param << value << endl;
      }
      Instance()->m_SLT_params[param] = value;
      return true;
    } else {
      cout << "SLT parameter " << param << " is locked!" << endl;
      return false;
    }
}

bool kasltsim::unset_SLT_param(char* param) {
    //returns true if parameter is unset successfully
    SLT_params_t::iterator curl = Instance()->locked_SLT_params.find(param);
    if ( curl==Instance()->locked_SLT_params.end()) {
      SLT_params_t::iterator cur = Instance()->m_SLT_params.find(param);
      if (cur==Instance()->m_SLT_params.end()) {
        cout << "SLT parameter " << param << " was not set!" << endl;
        return false;
      } else {
        cout << "Unsetting SLT parameter " << param  << endl;
        Instance()->m_SLT_params.erase(param);
        return true;
      }
    } else {
      cout << "SLT parameter " << param << " is locked!" << endl;
      return false;
    }
}

char*const* kasltsim::get_SLT_params() {
    char** argv;
    argv = new char*[Instance()->m_SLT_params.size() + 2];
    argv[0]=get_edcfile();
    if (get_debuglevel()>1)
      cout << "get_SLT_params: argc should be " << Instance()->m_SLT_params.size()+1
           << endl << "get_SLT_params: ";
    unsigned int count = 1;
    for ( SLT_params_t::iterator cur=Instance()->m_SLT_params.begin();
          cur != Instance()->m_SLT_params.end(); cur++, count++ ) {
      string s(cur->first);
      s += cur->second;
      argv[count]=new char[s.size()+1];
      argv[count]=strcpy(argv[count],s.c_str());
      if (get_debuglevel()>1)
        cout << "[" << count << "]" << argv[count] << " ";
    }
    argv[count]=NULL;
    if (get_debuglevel()>1)
      cout << "[" << count << "]NULL"  << endl;
    return argv;
}

void kasltsim::dump_SLT_param() {
    cout << "Current setting of SLT parameters:" << endl;
    for ( SLT_params_t::iterator cur=Instance()->m_SLT_params.begin();
          cur != Instance()->m_SLT_params.end(); cur++ ) {
      cout << "   " << cur->first << cur->second << endl;
    }
    for ( SLT_params_t::iterator cur=Instance()->locked_SLT_params.begin();
          cur != Instance()->locked_SLT_params.end(); cur++ ) {
      cout << "   " << cur->first << "<set dynamically>" << endl;
    }
    cout << "End of SLT parameter listing." << endl;
  }


extern "C"
void slt_sim_path()
{
  ARTE_mark( "slt_sim_path 0", 0);

  // read in path
  char *path=(char*) malloc(255);
  strncpy(path, ku_gets(),254);

  if (path[strlen(path)-1] != '/') 
    path = strcat(path,"/");

  kasltsim::set_edcpath(path);

  ARTE_mark( "slt_sim_path 1", 0);
}

extern "C"
void slt_sim_file()
{
  ARTE_mark( "slt_sim_file 0", 0);

  // read in file
  char *file = new char[255];
  strncpy(file, ku_gets(),255);
  kasltsim::set_edcfile(file);

  ARTE_mark( "slt_sim_file 1", 0);
}
extern "C"
void slt_sim_runnumber()
{
  ARTE_mark( "slt_sim_runnumber 0", 0);

  // read in runnumber 
  int arg;
  kugeti_(&arg);
  kasltsim::set_runnumber(arg);

  ARTE_mark( "slt_sim_runnumber 1", 0);
}

extern "C"
void slt_sim_debuglevel()
{
  ARTE_mark( "slt_sim_debuglevel 0", 0);

  // read in debuglevel  
  int arg;
  kugeti_(&arg);
  kasltsim::set_debuglevel(arg);

  ARTE_mark( "slt_sim_debuglevel 1", 0);
}

extern "C"
void slt_sim_setpar()
{
  ARTE_mark( "slt_sim_setpar 0", 0);

  // read in param
  char *param = new char[255];
  strncpy(param, ku_gets(),255);

  // read in optional value
  char *value;
  if (ku_npar()>1) {
    value = new char[255];
    strncpy(value, ku_gets(),255);
  } else value="";

  kasltsim::set_SLT_param(param,value,false);

  ARTE_mark( "slt_sim_setpar 1", 0);
}

extern "C"
void slt_sim_unsetpar()
{
  ARTE_mark( "slt_sim_unsetpar 0", 0);

  // read in param
  char *param = new char[255];
  strncpy(param, ku_gets(),255);

  kasltsim::unset_SLT_param(param);

  ARTE_mark( "slt_sim_unsetpar 1", 0);
}

extern "C"
void slt_sim_showpar()
{
  ARTE_mark( "slt_sim_showpar 0", 0);

  kasltsim::dump_SLT_param();

  ARTE_mark( "slt_sim_showpar 1", 0);
}
