!-----------------------------------------------------------------------
! fitting routines:  straigth line in 2D
! M. Staric, jan-2001
!
!-----------------------------------------------------------------------


      SUBROUTINE TEMA_FIT_HOR

! fitting lines in horizontal projection using alfa=0 hits only
! externals: SORTZV (from CERN library)

      IMPLICIT NONE
#include "tema/TEMA_HITS.fi"
#include "tema/TEMA_HSEG.fi"
#include "tema/TEMA_LAY.fi"
      INTEGER ILAY(NLAY)

      integer NSIZ
      PARAMETER (NSIZ=1024)
      REAL X(NSIZ), Z(NSIZ), SIG(NSIZ)

      INTEGER I,J,K,N,LAYER
      REAL A,B,VAR,FI
      INTEGER IER1, IER2
      SAVE IER1, IER2

      CALL SORTZV(IK,INDIK,NHIT(1),-1,0,0)
      NN_HS=0
      DO I=0,MAXHS
         IP_HS(I)=0
      ENDDO
      DO I=1,NHIT(1)
         K=INDIK(I)
         NN_HS=MAX(IK(K),0)
         IF(NN_HS.GT.MAXHS) THEN
            CALL MEPRIC(IER1,10,'TEMA: tema_fit_hor_, NN_HS > MAXHS')
            NN_HS=MAXHS
            GOTO 1
         ENDIF
         IP_HS(NN_HS)=I
      ENDDO
 1    CONTINUE
      DO I=1,NN_HS
         IF(IP_HS(I).EQ.0) IP_HS(I)=IP_HS(I-1)
      ENDDO

      DO J=1,NN_HS
         N=0
         CALL VZERO(ILAY,NLAY)
         DO I=IP_HS(J-1)+1,IP_HS(J)
            N=N+1
            IF(N.LE.NSIZ) THEN 
               K=INDIK(I)
               X(N)=XHIT(K)
               Z(N)=ZHIT(K)
               SIG(N)=SHIT(1,K)
               LAYER=LAY(K)
               ILAY(LAYER)=1
            ENDIF
         ENDDO
         IF(N.GT.NSIZ) THEN
            CALL MEPRIC(IER2,10,'TEMA: tema_fit_hor_, N > NSIZ')
         ENDIF
         N=MIN(N,NSIZ)
         NL_HS(J)=0
         DO I=1,NLAY
            NL_HS(J)=NL_HS(J)+ILAY(I)
         ENDDO
         FIT_HS(J)=NL_HS(J).GT.2
         IF(FIT_HS(J)) THEN
            CALL TEMA_FIT2DLINE(Z,X,SIG,N,AA_HS(J),BB_HS(J),COV_HS(1,J),
     &           HI2_HS(J),NDF_HS(J),FIT_HS(J))
         ENDIF
         IF(FIT_HS(J)) THEN
            ZMI_HS(J)=Z(1)
            ZMA_HS(J)=Z(1)
            DO I=2,N
               ZMI_HS(J)=MIN(ZMI_HS(J),Z(I))
               ZMA_HS(J)=MAX(ZMA_HS(J),Z(I))
            ENDDO               
            R0_HS(1,J)=BB_HS(J)
            R0_HS(2,J)=0.
            R0_HS(3,J)=0.
            FI=ATAN(AA_HS(J))
            VN_HS(1,J)=COS(FI)
            VN_HS(2,J)=0.
            VN_HS(3,J)=-SIN(FI)
         ENDIF
      ENDDO

      RETURN
      END



      SUBROUTINE TEMA_FIT_VERT

! fitting lines in vertical planes (defined by 2D segments) using alfa<>0 hits
! externals: SORTZV, LFITW (from CERN library)

      IMPLICIT NONE
#include "tema/TEMA_HITS.fi"
#include "tema/TEMA_RSEG.fi"
#include "tema/TEMA_HSEG.fi"
#include "tema/TEMA_LAY.fi"
      INTEGER ILAY(NLAY)

      integer NSIZ
      PARAMETER (NSIZ=1024)
      REAL Y(NSIZ), Z(NSIZ), SIG(NSIZ)
      REAL R1(3)
      LOGICAL OK
      INTEGER I,J,K,L,N,LAYER
      REAL A,B,COV(3),HI2
      INTEGER NDF
      INTEGER IER1, IER2
      SAVE IER1, IER2

      CALL SORTZV(IK,INDIK,NHIT(2),-1,0,0)
      NSEG=0
      DO I=0,MAXSEG
         IP_SEG(I)=0
      ENDDO
      DO I=1,NHIT(2)
         K=INDIK(I)
         NSEG=MAX(IK(K),0)
         IF(NSEG.GT.MAXSEG) THEN
            CALL MEPRIC(IER1,10,'TEMA: tema_fit_vert_, NSEG > MAXSEG')
            NSEG=MAXSEG
            GOTO 1
         ENDIF
         IP_SEG(NSEG)=I
      ENDDO
 1    CONTINUE
      DO I=1,NSEG
         IF(IP_SEG(I).EQ.0) IP_SEG(I)=IP_SEG(I-1)
      ENDDO

      DO J=1,NSEG
         DO K=1,3
            NL_SEG(K,J)=0
         ENDDO
         L=LHS_SEG(J)
         FIT_SEG(J)=FIT_HS(L)
         IF(.NOT.FIT_SEG(J)) GOTO 2
         N=0
         CALL VZERO(ILAY,NLAY)
         DO I=IP_SEG(J-1)+1,IP_SEG(J)
            K=INDIK(I)
            LAYER=LAY(K)
            ILAY(LAYER)=1
            IF(K.GT.NHIT(1)) THEN
               CALL TEMA_CROSS_IX(R0_HS(1,L),VN_HS(1,L),K,R1,OK)
               IF(OK) THEN
                  N=N+1
                  IF(N.LE.NSIZ) THEN
                     Y(N)=R1(2) 
                     Z(N)=R1(3)
                     SIG(N)=SHIT(1,K)/ABS(S_ALFA(K))
                  ENDIF
               ENDIF
            ENDIF
         ENDDO
         IF(N.GT.NSIZ) THEN
            CALL MEPRIC(IER2,10,'TEMA: tema_fit_vert_, N > NSIZ')
         ENDIF
         N=MIN(N,NSIZ)
         DO I=1,NLAY
            K=KOT(I)+2
            NL_SEG(K,J)=NL_SEG(K,J)+ILAY(I)
         ENDDO
         FIT_SEG(J)=NL_SEG(2,J).GT.2.AND.(NL_SEG(1,J)+NL_SEG(3,J)).GT.2
         IF(FIT_SEG(J)) THEN
            CALL TEMA_FIT2DLINE(Z,Y,SIG,N,A,B,COV,HI2,NDF,FIT_SEG(J))
            IF(FIT_SEG(J)) THEN
               SEG_R(1,J)=R0_HS(1,L)
               SEG_R(2,J)=B
               SEG_R(3,J)=R0_HS(3,L)
               SEG_S(1,J)=AA_HS(L)
               SEG_S(2,J)=A
               SEG_S(3,J)=1.
               SEG_CV(1,J)=COV_HS(1,L)
               SEG_CV(2,J)=0.
               SEG_CV(3,J)=COV(1)
               SEG_CV(4,J)=COV_HS(2,L)
               SEG_CV(5,J)=0.
               SEG_CV(6,J)=COV_HS(3,L)
               SEG_CV(7,J)=0.
               SEG_CV(8,J)=COV(2)
               SEG_CV(9,J)=0.
               SEG_CV(10,J)=COV(3)
               SEG_HI2(J)=HI2+HI2_HS(L)
               NDF_SEG(J)=NDF+NDF_HS(L)
               SEG_ZMI(J)=ZMI_HS(L)
               SEG_ZMA(J)=ZMA_HS(L)
               DO I=1,N
                  SEG_ZMI(J)=MIN(SEG_ZMI(J),Z(I))
                  SEG_ZMA(J)=MAX(SEG_ZMA(J),Z(I))
               ENDDO
            ENDIF
         ENDIF
 2       CONTINUE
      ENDDO

      RETURN
      END



      SUBROUTINE TEMA_CROSS_IX(R0,VN,K,R1,OK)

      REAL R0(3), VN(3)
      INTEGER K
      REAL R1(3)
      LOGICAL OK

#include "tema/TEMA_HITS.fi"
      REAL R(3), DIR(3)
      REAL DL /7./
      SAVE DL

      OK=.FALSE.
      CALL TEMA_WIRE_VEC(K,R,DIR)
      SN=S_PROD(DIR,VN)
      IF(SN.EQ.0) RETURN
      PATH=0.
      DO I=1,3
         PATH=PATH+(R0(I)-R(I))*VN(I)
      ENDDO
      PATH=PATH/SN
      IF(PATH.LT.0.OR.PATH.GT.1) RETURN
      DO I=1,3
         R1(I)=R(I)+PATH*DIR(I)
      ENDDO
      OK=.TRUE.

      RETURN
      END




      SUBROUTINE TEMA_FIT2DLINE(X,Y,EY,NT,A,B,COV,HI2,NDF,OK)

! 2D line fit: y=a*x+b

      REAL X(*), Y(*), EY(*)    ! points, errors [in]
      INTEGER NT                ! number of points [in]
      REAL A                    ! slope [out]
      REAL B                    ! constant [out]
      REAL COV(3)               ! covariance matrix of parameters [out]
      REAL HI2                  ! chi**2 [out]
      INTEGER NDF               ! n.d.f [out]
      LOGICAL OK                ! =T for sucsessfull fit

      HI2=0.
      NDF=-2
      A11=0.
      A21=0.
      A22=0.
      G1=0.
      G2=0.
      DO I=1,NT
         IF(EY(I).NE.0) THEN
            WT=1./EY(I)**2
            A11=A11+WT
            A21=A21+WT*X(I)
            A22=A22+WT*X(I)**2
            G1=G1+WT*Y(I)
            G2=G2+WT*Y(I)*X(I)
         ENDIF
      ENDDO
      D=A11*A22-A21**2
      OK=D.NE.0
      IF(.NOT.OK) RETURN
      COV(1)=A22/D              ! V11
      COV(2)=-A21/D             ! V21=V12
      COV(3)=A11/D              ! V22
      B=COV(1)*G1+COV(2)*G2
      A=COV(2)*G1+COV(3)*G2
      DO I=1,NT
         IF(EY(I).NE.0) THEN
            HI2=HI2+((A*X(I)+B-Y(I))/EY(I))**2
            NDF=NDF+1
         ENDIF
      ENDDO
      
      RETURN
      END



      SUBROUTINE TEMA_PRO2DLINE(A,B,COV,X,Y,EY2)

! 2D line prolongation to x with error estimation

      REAL A                    ! slope [in]
      REAL B                    ! constant [in]
      REAL COV(3)               ! covariance matrix of parameters [in]
      REAL X                    ! x [in]
      REAL Y, EY2               ! y, error_y**2 [out]

      Y=A*X+B
      EY2=COV(1)+2*COV(2)*X+COV(3)*X**2

      RETURN
      END



