// *****************************************************************************
//
// source:    L3_MakeRhits.C
//
// type:      source code
//
// created:   06 Dec 1999
//
// author:    Stefan Scharein, Ullrich Schwanke
//            HERA-B Collaboration
//            DESY Zeuthen
//            Platanenallee 6
//            D-15738 Zeuthen
//            Germany
//            email: scharein@ifh.de, schwanke@ifh.de
//
// symbol:    
//
// changes:
//
// *****************************************************************************
#include <vector>
#include "arte/arte.hh"

// Create RHITs for a segment, which has only HITBs
void L3_MakeRhits( ArtePointer<RSEG> rseg ) {
  vector<ArtePointer<RHIT> > rhits;
  for( ArteRelationIterator<HITB> hitb = rseg->hitb.begin();
       hitb != rseg->hitb.end(); ++hitb ) {
    RHIT rh;
    rh.x = rseg->xf+((*hitb)->z-rseg->zf)*rseg->txf;
    rh.y = rseg->yf+((*hitb)->z-rseg->zf)*rseg->tyf;
    rh.tx = rseg->txf;
    rh.ty = rseg->tyf;
    rh.p = 0;
    rh.q = 0;
    ArteTable<RHIT>::push_back(rh);
    ArtePointer<RHIT> rhit( ArteTable<RHIT>::nrow() );
    rhits.push_back( rhit );
    createRelation( *hitb, rhit, &HITB::rhit );
  }
  for( vector<ArtePointer<RHIT> >::reverse_iterator rhit
         = rhits.rbegin(); rhit!=rhits.rend(); ++rhit ) {
    createRelation( rseg, *rhit, &RSEG::rhit );
  }
}

void L3_MakeSegTra(ArtePointer<RSEG> rseg, ArtePointer<RTRA> rtra) {
  createRelation( rtra, rseg, &RTRA::rseg );
}

void L3_MakeRpntSegTra(vector<ArtePointer<RPNT> > rpnts,
		       ArtePointer<RSEG> rseg, ArtePointer<RTRA> rtra) {
  for( vector<ArtePointer<RPNT> >::iterator rpnt
         = rpnts.begin(); rpnt!=rpnts.end(); ++rpnt ) {
    createRelation( rtra, *rpnt, &RTRA::rpnt );
    createRelation( rseg, *rpnt, &RSEG::rpnt );
  }
}

void L3_MakeHitb(ArtePointer<HITB> hitb, ArtePointer<RSEG> rseg) {
  createRelation( rseg, hitb, &RSEG::hitb);
} 

void L3_MakeRsegHitb(ArtePointer<HITB> hitb, ArtePointer<RSEG> rseg) {
  createRelation( rseg, hitb, &RSEG::hitb);
}

void L3_MakeRtraHitb(ArtePointer<HITB> hitb, ArtePointer<RTRA> rtra) {
  createRelation( rtra, hitb, &RTRA::hitb);
}




