*CMZ :          20/05/96  12.58.00  by  Unknown
*-- Author :   T.Lohse
c
c-------------------------------------------------------------
c
c       subroutine:   VTPRIM(itB,flag)
c
c       input:    itB  track no for a B             Int
c                 reconstructed in VTPSIK from
c                 a J/Psi Ks decay
c                 = 0 if no such decay exists
c
c       output:   flag (<0 if fail)                 Int
c
c       action:   loop over the wires and reconstruct
c                 primary vertices.
c                 If itB>0 a B vertex is assumed to be there.
c                 This means that VTPSIK has to be called first.
c
c-------------------------------------------------------------
c
      subroutine  VTPRIM(itB,flag)

      implicit none

      integer flag, itB

#include "vt/vtdat.fi"
#include "vt/vtwire.fi"
#include "vt/vtpatt.fi"

      integer iw,niv,it,itk,i,j,k,l,iv,flag1,jmin,imax
      double precision 
     1  cp,sp,ct,st,ax,ay,bx,by,z,term,term1
      double precision 
     1  ch2min, ch2i, ch2max
      double precision 
     1  x(3), cov(3,3)
      double precision 
     1  xl(ntrtot)
      integer itw(ntrtot), cut(ntrtot)
      integer ivbuf(nvtmax)
      double precision 
     1  VTDVT
      double precision 
     1  scale 

      DATA scale /2.d0/

      flag = 0

c
c---> prepare wire buffers and wire-track association
c

      call VTWINI(0)
      if (itB.le.0) then
        ch2max = ch2wtr
        call VTIDWI(1,trmult,ch2max)
      else
        ch2max = ch2wB
        call VTIDWI(itB,itB,ch2max)
        ch2max = ch2wtr
        if (itB.gt.1)      call VTIDWI(1,itB-1,ch2max)
        if (itB.lt.trmult) call VTIDWI(itB+1,trmult,ch2max)
      endif

      nvprim = 0

c
c---> loop over all wires
c
      do  iw = 1, nw

        do  i = 1, 3
          x(i) = xw(i,iw)
          do  j = 1, 3
c
c---> constrain the wire, use a sigma of "scale" times its diameter
c
            cov(i,j) = scale**2*Cw(i,j,iw)
          enddo
        enddo

        cp = cos(phiw(iw))
        sp = sin(phiw(iw))
        ct = cos(thew(iw))
        st = sin(thew(iw))
c
c--->  get linear position of track along wire (in wire plane)
c
        do  i = 1, ntrwi(iw)
          it = itrwi(iw,i)
          ax = p(1,it)
          ay = p(2,it)
          bx = p(3,it)
          by = p(4,it)
          z = ( ( (bx-xw(1,iw))*cp + (by-xw(2,iw))*sp )*st
     1                             + xw(3,iw)*ct ) /
     1        ( ct-st*(cp*ax+sp*ay) )
          xl(i) = ( (ax*z+bx-xw(1,iw))*cp + (ay*z+by-xw(2,iw))*sp )*ct
     1            + (z-xw(3,iw))*st
          cut(i) = 0
          itw(i) = it
        enddo
c
c--->  sort the positions in ascending order
c
        do  i = 2, ntrwi(iw)
          j = i
  100     continue
          if (xl(j).lt.xl(j-1)) then
            term     = xl(j-1)
            xl(j-1)  = xl(j)
            xl(j)    = term
            k        = itw(j-1)
            itw(j-1) = itw(j)
            itw(j)   = k
            j = j-1
            if ( j .gt. 1 ) goto 100
          endif
        enddo
c
c--->  cut into clusters allowing for a minimal hit separation
c
        do  i = 1, ntrwi(iw)-1
          if (xl(i+1)-xl(i).gt.xlmin) cut(i) = 1
        enddo
        cut(ntrwi(iw)) = 1

c
c--->  allow for minimal cluster width by cutting longest links
c
        k    = 1
        i    = 1
  200   continue
        if ( cut(i).ne.0 ) then
          if ( xl(i)-xl(k) .gt. xlwid ) then
            term = 0.
            do j = k, i-1
              term1 = xl(j+1)-xl(j)
              if (term1.gt.term) then
                term = term1
                l = j
              endif
            enddo
            cut(l) = 2
            i = l
            goto 200
          else
            k=i+1
          endif
        endif
        i=i+1
        if (i.le.ntrwi(iw))  goto 200

c
c--->  merge all cluster pairs compatible with minimal width
c
  300   continue
        k=1
        l=0
        j=0
        term = xl(ntrwi(iw))-xl(1)+1.
        do  i = 1, ntrwi(iw)
          if ( cut(i).ne.0 ) then
            if (l.ne.0) then
              term1 = xl(i)-xl(k)
              if (term1.lt.term) then
                j=l
                term=term1
              endif
            endif
            if (cut(i).eq.2) then
              if (l.ne.0) k=l+1
              l=i
            else
              l=0
              k=i+1
            endif
          endif
        enddo
        if ( j.ne.0 .and. term.lt.xlwid ) then
          cut(j) = 0
          goto 300
        endif

c
c---> add these clusters as primary vertices
c---> if they have at least 3 tracks
c
        k = 1
        niv = 0
        do  i = 1, ntrwi(iw)
          if (cut(i).ne.0) then
            if (i-k+1.ge.3) then
              call VTINIT(x,cov,iv,flag1)
              if (flag1.lt.0) then
c---> buffer overflow, too many vertices
c---> vertex reconstruction hopeless
                goto 999
              endif
              do  j = k, i
                call VTADD(iv,itw(j),flag1)
                if (flag1.lt.0) then
c---> buffer overflow, too many tracks on this vertex
c---> leave the rest unassigned
                  flag = -2
                endif
              enddo
c
c---> make sure that wire is used as initial vertex estimate
c--->   smooth(iv) = 1   (higher convergence speed)
c
              smooth(iv) = 1
              call VTFIT(iv,0,flag1)
              if (flag1.lt.0) then
                flag = -3
c---> give up on this vertex, try to keep the others
                call VTDEL(iv,flag1)
              else
                niv = niv + 1
                ivbuf(niv) = iv
              endif
              k = i+1
            endif
          endif
        enddo
c
c---> compute smoothed chi**2 of all tracks on a vertex
c---> take out extreme outliers
c
        do  j = 1, niv
          iv = ivbuf(j)
  500     continue
          if ( ntr(iv) .lt. 2 ) goto 600
          call VTDIS(iv,flag1)
          if (flag1.lt.0) then
            flag = -3
            goto 600
          endif
          imax = 0
          ch2max = -1.d0
          do  itk = 1, ntr(iv)
            ch2i = chi2s(iv,itk)
            if ( ch2i .gt. ch2max ) then
              ch2max = ch2i
              imax = itk
            endif
          enddo
          if (imax.gt.0.and.ch2max.gt.ch2vtr) then
            call VTKILL(iv,imax,flag1)
            if (flag1.lt.0) then
              flag=-3
              goto 600
            endif
            goto 500
          endif
  600     continue
        enddo
c
c---> take out clusters which have less than 3 tracks if
c---> they are not isolated
c
        do  j = 1, niv
          iv = ivbuf(j)
          if (ntr(iv).lt.3.and.ntrwi(iw).gt.ntr(iv)) then
            call VTDEL(iv,flag1)
            if (flag1.eq.0) then
              ivbuf(j) = 0
            else
              flag=-1
            endif
          endif
        enddo
        i = 0
        do  j = 1, niv
          if (ivbuf(j).ne.0) then
            i = i+1
            ivbuf(i)=ivbuf(j)
          endif
        enddo
        niv = i
c
c---> compute chi**2 of all unassociated tracks and
c---> add to cluster if close enough
c
  400   continue
        do  i = 1, ntrwi(iw)
          it = itw(i)
          if ( nv(it) .eq. 0 ) then
            jmin = 0
            ch2min = 100.*ch2vtr
            do  j = 1, niv
              iv = ivbuf(j)
              ch2i = VTDVT(iv,it)
              if (ch2i.lt.ch2min) then
                jmin = iv
                ch2min = ch2i
              endif
            enddo
            if (jmin.gt.0.and.ch2min.lt.ch2vtr) then
              call VTADD(jmin,it,flag1)
              if (flag1.lt.0) then
                flag = -2
              else
                goto 400
              endif
            endif
          endif
        enddo
c
c---> update list of primary vertices
c
        do j = 1, niv
          nvprim = nvprim + 1
          ivprim(nvprim) = ivbuf(j)
        enddo
      enddo
c
c---> add every unassociated track as a vertex
c
      do  it = 1, trmult
        iw = iwitr(it)
        if ( nv(it).eq.0 .and. iw.gt.0 ) then
          do  i = 1, 3
            x(i) = xw(i,iw)
            do  j = 1, 3
              cov(i,j) = scale**2*Cw(i,j,iw)
            enddo
          enddo
          call VTINIT(x,cov,iv,flag1)
          if (flag1.lt.0) then
c---> buffer overflow, too many vertices
            flag=-5
          else
            call VTADD(iv,it,flag1)
            if (flag1.lt.0) then
              flag = -2
              call VTDEL(iv,flag1)
              if (flag1.lt.0) flag=-1
            else
              call VTFIT(iv,0,flag1)
              if (flag1.lt.0) then
                flag = -3
                call VTDEL(iv,flag1)
                if (flag1.lt.0) flag=-1
              else
                nvprim = nvprim+1
                ivprim(nvprim)=iv
              endif
            endif
          endif
        endif
      enddo
c
c---> merge closeby vertices
c
      ch2max = ch2vv
      call VTMERG(ch2max,flag1)
      if (flag1.lt.0) flag=-4
      k=0
      do  i = 1, nvprim
        iv = ivprim(i)
        if (vtset(iv)) then
          k=k+1
          ivprim(k)=ivprim(i)
        endif
      enddo
      nvprim=k

c
c---> check low multiplicity vertices (mostly ghosts)
c
      i=0
  700 continue
      i=i+1
      if (i.gt.nvprim) goto 800

      iv = ivprim(i)
      if ( ntr(iv) .lt. ntmin ) then
c
c---> check that this is not a vertex containing a reconstructed
c     B-->J/Psi Ks. Such a primary vertex is kept in any case,
c     even with a single track (the B)
c
        if (itB.gt.0) then
          if (jv(iv,itB).ne.0) goto 700
        endif
c
c---> try to reassign tracks of low mult vertex to other vertices
c
        do  l = 1, ntr(iv)
          it = ktr(iv,l)
          jmin = 0
          ch2min = 100.*ch2vtr
          do  j = 1, nvprim
            if (j.ne.i) then
              k = ivprim(j)
              ch2i = VTDVT(k,it)
              if (ch2i.lt.ch2min) then
                jmin = k
                ch2min = ch2i
              endif
            endif
          enddo
          if (jmin.gt.0.and.ch2min.lt.ch2vtr) then
            call VTADD(jmin,it,flag1)
            if (flag1.lt.0) flag = -2
          endif
        enddo
c
c---> take out all primary vertices which fall below the
c---> minimal cluster size (ntmin)
c
        call VTDEL(iv,flag1)
        if (flag1.eq.0) then
          do  k = i+1, nvprim
            ivprim(k-1)=ivprim(k)
          enddo
          ivprim(nvprim) = 0
          nvprim = nvprim - 1
          i = i-1
        else
          flag = -1
        endif
      endif
      goto 700
  800 continue
c
c---> identify the primary vertex of the signal B
c
      ivB = 0
      if (itB.gt.0) then
        do  i = 1, nvprim
          if (jv(ivprim(i),itB).ne.0) ivB=ivprim(i)
        enddo
      endif

      return

  999 continue

      flag = -1

      return
      end
