/**********************************************************
 Program for creating OTR Leda database.
 Calibration data will be defined separately.
 Alignment tables are defined with some dummy data (for tests).
 Necessary files can be taken e.g. from ~lanyov/otr_db/

 AL 11.08.99
 **********************************************************/
 
 
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <arte/Arte.h>
#include <arte/ArteUtil.h>
#include <auto/NEDE.h>
#include <auto/NESL.h>
#define LEDA_INITIALIZATION
//#include "ledaC.h"
#include "wax/otr_db.h"
 
void report_memory(char* message) {
 /* Report memory - works at Solaris */
  printf("\n %s \n",message);
//  system("ps -ef -os,vsz,osz,rss,pmem,fname | egrep 'VSZ|geanread'");
  printf("\n");
}

int dummy_align() { return 0; }

int otr_db_file(char* DbFile) {

  /* Main routine to fill OTR Leda database and write to a file. */

  int nnede,i,inede;
  int nnesl,inesl;
  int icalib;
  int j,Nrow,Nsharc,empty;
  char str[500];
  FILE *Table,*tab_list,*sharc_list,*Sharc;
  char *name_qua;
  char *sharc_file;

  MizziDatabase(db);
  LedaTableIterator(OTR_nede,aiter); 
  LedaTableRow(OTR_nede) *nederow;

  LedaTableIterator(OTR_nesl,biter); 
  LedaTableRow(OTR_nesl) *neslrow;

  LedaRelationIterator(nederow->tonesl, nede_nesl_iter);
  LedaRelationIterator(neslrow->tonede, nesl_nede_iter);

  LedaTableIterator(OTR_cab,cabiter); 
  LedaTableRow(OTR_cab) *c;

  LedaTableIterator(OTR_calib,calibiter); 
  LedaTableRow(OTR_calib) *calibrow;

  LedaTableIterator(OTR_sharc,sharciter); 
  LedaTableRow(OTR_sharc) *s;

  LedaTableRow(OTR_align) *alignrow;
  LedaRelationIterator(nederow->toalign, nede_align_iter);
  
  LedaRelationIterator(c->tosharc, cab_sharc_iter);
  LedaRelationIterator(s->tocab, sharc_cab_iter);

  AddStderrStream(stderr,"stderr");
  AddStderrStream(fopen("Otr_db.err","w"),"Otr_db.err");
  AddStdoutStream(stdout,"stdout");
  AddStdoutStream(fopen("Otr_db.log","w"),"Otr_db.log");

//SetLogLevel(10);
  SetLogLevel(100);
 
  report_memory("Start of program for filling OTR database.");
 
 
  /* pre-initialization */
  OTR_nede->initialize();
  OTR_nesl->initialize();
  OTR_cab->initialize();
  OTR_calib->initialize();
  OTR_sharc->initialize();
  OTR_align->initialize();
 
  report_memory("Database initialized.");

  /* Check relation net consistency and complete initialization */
  LedaCheckDD();

  /*------- NESL ------*/

  nnesl=NNESL();

  for(inesl=1;inesl <= nnesl; inesl++){

    if(NESL(inesl).cmp==2){

      char neslname[5];

      /*      printf("name: %i\n",NESL(inesl).pnede);*/

      for(i=0;i<5;i++) neslname[i]=0;
      strncpy(neslname,NESL(inesl).name,4);

      neslrow = OTR_nesl->add->key(neslname); /* set key for string */

      neslrow->inesl= inesl;
      neslrow->cmp  = NESL(inesl).cmp;
      neslrow->shp  = NESL(inesl).shp;
      neslrow->nsl  = NESL(inesl).nsl;
      neslrow->mfl  = NESL(inesl).mfl;
      neslrow->lay  = NESL(inesl).lay;
      neslrow->x1   = NESL(inesl).x1;
      neslrow->x2   = NESL(inesl).x2;
      neslrow->y1   = NESL(inesl).y1;
      neslrow->y2   = NESL(inesl).y2;
      neslrow->z1   = NESL(inesl).z1;
      neslrow->z2   = NESL(inesl).z2;
      neslrow->dx   = NESL(inesl).dx;
      neslrow->dy   = NESL(inesl).dy;
      neslrow->npl  = NESL(inesl).npl;
      neslrow->fpl  = NESL(inesl).fpl;
      neslrow->nwl  = NESL(inesl).nwl;
      neslrow->fwl  = NESL(inesl).fwl;
      neslrow->ltyp = NESL(inesl).ltyp;
      neslrow->pnede= NESL(inesl).pnede;

    }
  }
  
  printf(" How many objects in NESL? %d\n",OTR_nesl->get_nrows());

  report_memory("NESL defined.");

/*-------------------------------- NEDE ----------------------------*/

  nnede = NNEDE(); 

  for(inede=1; inede <= nnede; inede++){
    
    if(NEDE(inede).cmp==2) {
      
      if(NEDE(inede).hwc[0]!='n'){
	char hwc[13];

	for(i=0;i<13;i++) hwc[i]=0;
	strncpy(hwc,NEDE(inede).hwc,12);

	nederow = OTR_nede->add->key(hwc); /* set key for string */

	nederow->inede = inede;
	for(i=0;i<9;i++) 
	  nederow->rot[i] = NEDE(inede).rot[i];

	for(i=0;i<4;i++) {
	  nederow->xfw[i] = NEDE(inede).xfw[i];
	  nederow->zfw[i] = NEDE(inede).zfw[i];
	  nederow->nwi[i] = NEDE(inede).nwi[i];
	  nederow->y1w[i] = NEDE(inede).y1w[i];
	  nederow->y2w[i] = NEDE(inede).y2w[i];
	}
	
	nederow->xmin = NEDE(inede).xmin;
	nederow->xmax = NEDE(inede).xmax;

	nederow->ymin = NEDE(inede).ymin;
	nederow->ymax = NEDE(inede).ymax;
	
	nederow->zmin = NEDE(inede).zmin;
	nederow->zmax = NEDE(inede).zmax;
	
	nederow->sxmn = NEDE(inede).sxmn;
	nederow->sxmx = NEDE(inede).sxmx;
	
	nederow->symn = NEDE(inede).symn;
	nederow->symx = NEDE(inede).symx;
	
	nederow->acc = NEDE(inede).acc;
	nederow->eff = NEDE(inede).eff;
	nederow->dod = NEDE(inede).dod;
	nederow->pit = NEDE(inede).pit;
	nederow->ms  = NEDE(inede).ms;
	
	nederow->cos = NEDE(inede).cos;
	nederow->sin = NEDE(inede).sin;

	nederow->nsl = NEDE(inede).nsl;
	nederow->lay = NEDE(inede).lay;
	nederow->sec = NEDE(inede).sec;
	nederow->shp = NEDE(inede).shp;
	nederow->cmp = NEDE(inede).cmp;

	nederow->nesl = NEDE(inede).onesl;
	nederow->dedx = NEDE(inede).dedx;


	/*------- set nede <--> nesl relation ---------*/ 
	for(biter.begin(); biter.valid(); biter.next()) {
	  neslrow = biter.get();
	  if(neslrow->inesl == NEDE(inede).onesl){
	    RelateRows ( OTR_nede, tonesl, key(hwc),
			 OTR_nesl, tonede, key(neslrow->name));
	    
	  }
	}
	/*---------------------------------------*/
      }
    }
  }

  printf(" How many objects in NEDE? %d\n",OTR_nede->get_nrows());

  report_memory("NEDE defined.");

  /*----------------------------------- test writes ----------------------------------*/
  /*----- get relation nede to nesl ------*/ 
  /*
  printf("\n\n Test of NEDE <--> NESL relation.\n\n");
  for(aiter.begin(); aiter.valid(); aiter.next()) {
    nederow = aiter.get();
    nederow = OTR_nede->get->key(nederow->hwc);
    for (nede_nesl_iter.begin(); nede_nesl_iter.valid(); nede_nesl_iter.next()){
      neslrow = nede_nesl_iter.get();
      printf("NEDE hwc=%s  <--> NESL name=%s\n", nederow->hwc, neslrow->name);
    }
  }
  */

  /*--------------------------------------*/

  /*----- get relation nesl to nede ------*/ 
  /*  for(biter.begin(); biter.valid(); biter.next()) {
      b=biter.get();
      
      printf("NESL NAME %s \n",neslrow->name);
      
      b=nesl->get->key(neslrow->ib);
      
      for (nesl_nede_iter.begin(); nesl_nede_iter.valid(); nesl_nede_iter.next()){
      hwc = nesl_nede_iter.get();
      printf("NEDE HWC %s\n",nederow->hwc);
      }
      }*/
  /*--------------------------------------*/
  /*----------------------------------------------------------------------------------*/

  /*------------------ storing cabling --------------------*/
  tab_list=fopen("tables","r");
  Nrow = 0; /*-- number of strings --*/
  name_qua=malloc(100);
  
  for(i=0;i<500;i++) str[i]=0;
  
  do{
    if(fscanf(tab_list,"%s",name_qua)==EOF) break;
    
    Table=fopen(name_qua,"r");
    
    if(Table==NULL){ 
      printf("\n\nError: Can't open file %s\n\n",name_qua);
      return 0;
    } 
    
    fgets(str,500,Table); /*-------- skip first row in file l*.tab -------*/
    
    do{ /*-----------------String loop---------------------------------------*/
      
      for(i=0;i<500;i++) str[i]=0; 
      
      if(fgets(str, 500, Table)==NULL) break; 
      j=0; 
      if(str[0] != '=' && str[0] != '-') { 
	
	empty = 0;
	for(i=0;i<500;i++) {
	  if(str[i]=='m' || str[i]=='p' || str[i]=='t') empty=1;
	}
	
	if(empty) c=OTR_cab->add->key(++Nrow); 
	
	
	for(i=0;i<500;i++) { 
	  
	  if(str[i]==0) break;
	  
	  if(str[i]=='m' || str[i]=='p' || str[i]=='t'){
	    
	    j++;
	    
	    if(j==1){                                 /*---gencha1---*/
	      strncpy(c->gencha1,str+i-1,15);
	    }  
	    if(j==2){                                 /*---gencha2---*/
	      strncpy(c->gencha2,str+i-1,15);
	    }
	    if(j==3){                                 /*---modcha1---*/
	      strncpy(c->modcha1,str+i-1,15);
	    }
	    if(j==4){                                 /*---modcha2---*/
	      strncpy(c->modcha2,str+i-1,15);
	    }
	    if(j==5){                                 /*----modhv----*/
	      strncpy(c->modhv,str+i-1,15);
	    }
	    if(j==6){                                 /*----ftbpos---*/
	      strncpy(c->ftbpos,str+i-1,15);
	    }
	    if(j==7){                                 /*----ftbasd---*/
	      strncpy(c->ftbasd,str+i-1,15);
	    }
	    if(j==8){                                 /*----ftbhv----*/
	      strncpy(c->ftbhv,str+i-1,15);
	    }
	    if(j==9){                                 /*----ftbcab---*/
	      strncpy(c->ftbcab,str+i-1,15);
	    }
	    if(j==10){                                /*----lvpos----*/
	      strncpy(c->lvpos,str+i-1,15);
	    }
	    if(j==11){                                /*----lvcab----*/
	      strncpy(c->lvcab,str+i-1,15);
	    }
	    if(j==12){                                /*---tdccab----*/
	      strncpy(c->tdccab,str+i-1,15);
	    }
	    if(j==13){                                /*---tdccon----*/
	      strncpy(c->tdccon,str+i-1,15);
	    }
	    if(j==14){                                /*--previos1---*/
	      strncpy(c->previous1,str+i-1,15);
	    }
	    if(j==15){                                /*--previos2---*/
	      strncpy(c->previous2,str+i-1,13);
	    }
	    if(j==16){                                /*---next1-----*/
	      strncpy(c->next1,str+i-1,15);
	    }
	    if(j==17){                                /*---next2-----*/
	      strncpy(c->next2,str+i-1,15);
//	      strncpy(c->status,str+i+14,15);         /*---status----*/
	    }
	  }
	}
      }	
    }while(1);  /*----------end of string loop-------------------------------*/
    fclose(Table); 

  }while(1);
  fclose(tab_list);
  /* --------------------- end of cabling ----------------------------*/

  printf(" How many objects in cabling table? %d\n",OTR_cab->get_nrows());

  report_memory("Cabling defined.");


  /*------- CALIB - now filled with dummy contents ------*/

  for(icalib=1;icalib <= OTR_cab->get_nrows(); icalib++){
    
    calibrow = OTR_calib->add->key(icalib); /* set key for string */
    
    calibrow->icalib = icalib;

    for(i=0;i<16;i++) {
      calibrow->status[i] = 0;
    };

    for(i=0;i<9;i++) {
      calibrow->t0[i] = 0.;
    };
    
    RelateRows ( OTR_calib, tocab  , key(icalib),
		 OTR_cab  , tocalib, key(icalib));
  }
  
  printf(" How many objects in CALIB? %d\n",OTR_calib->get_nrows());

  report_memory("CALIB defined.");


  /*------------------ sharc data base ---------------------------*/
  sharc_list=fopen("sharc","r");
  sharc_file=malloc(100);

  for(i=0;i<500;i++) str[i] = 0; /*clean bufer*/
  
  Nsharc=0;

  do{
    if(fscanf(sharc_list,"%s",sharc_file)==EOF) break;

    Sharc=fopen(sharc_file,"r");

    if(Sharc==NULL){ 
      printf("\n\nError: Can't open file %s\n\n",name_qua);
      return 0;
    } 

    fgets(str,500,Sharc); /*-------- skip first row in file  -------*/

   do{ /*-----------------String loop---------------------------------------*/
      if(fgets(str, 500, Sharc)==NULL) break; 

      if(str[0] != '=' && str[0] != '\n') {

	s=OTR_sharc->add->key(++Nsharc);
	sscanf(str,"%s %i %s %i %i",s->board,&s->fed,s->sharc_cable,&s->sharc_board,&s->sharc_board_pos);

	/*--- set cabling sharc relations ---*/ /* set relation in cable loop its faster*/

	for(cabiter.begin(); cabiter.valid(); cabiter.next()) {
	  c=cabiter.get();
	  if(!strncmp(c->tdccon,s->board,8)) {
	    /*
	    printf("  icab=%4i tdccon=%s  <==>  isharc=%3i Nsharc=%3i board=%s cmp=%i\n",
		   c->icab,c->tdccon,
		   s->isharc,Nsharc,s->board,strncmp(c->tdccon,s->board,8));
		   */
	    RelateRows ( OTR_sharc, tocab, key(Nsharc),
			 OTR_cab, tosharc, key(c->icab));
	  }
	}
      }

    }while(1);

   fclose(Sharc);
   
  }while(1);
  
  fclose(sharc_list);
  
  printf(" How many objects in sharc table? %d\n",OTR_sharc->get_nrows());

  /* end of sharc storing*/

  report_memory("Calling dummy_align");
  dummy_align();
  report_memory("Called dummy_align");

  report_memory("Database defined.");


  /* Test of alignment table relations */
  /*
  for(aiter.begin(); aiter.valid(); aiter.next()) {
    nederow = aiter.get();
    printf(" Getting NEDE=%d, hwc=%s<--> ALIGN relation.\n",nederow->inede,nederow->hwc);
    for (nede_align_iter.begin(); nede_align_iter.valid(); nede_align_iter.next()){
      alignrow = nede_align_iter.get();
      printf("NEDE hwc=%s  <--> ALIGN id=%d \n", nederow->hwc, alignrow->id);
    }
  }
  */

  /* saving to database */
 
  printf("Opening database file %s...\n",DbFile);
  db.open(DbFile,0);
  printf("Database file %s opened.\n",DbFile);
  db.stamp(10,1);
 
  OTR_nede->book(db);
  OTR_nede->store_data(DBeq);
  OTR_nede->store_relations(DBeq);
  report_memory("Table nede stored.");

  OTR_nesl->book(db);
  OTR_nesl->store_data(DBeq);
  OTR_nesl->store_relations(DBeq);
  report_memory("Table nesl stored.");
 
  OTR_cab->book(db);
  OTR_cab->store_data(DBeq);
  OTR_cab->store_relations(DBeq);
  report_memory("Table cab stored.");

  OTR_calib->book(db);
  OTR_calib->store_data(DBeq);
  OTR_calib->store_relations(DBeq);
  report_memory("Table calib stored.");

  OTR_sharc->book(db);
  OTR_sharc->store_data(DBeq);
  OTR_sharc->store_relations(DBeq);
  report_memory("Table sharc stored.");

  OTR_align->book(db);
  OTR_align->store_data(DBeq);
  OTR_align->store_relations(DBeq);
  report_memory("Table align stored.");

  db.close();
 
  report_memory("Database stored in Otr.db .");

  /* deleting from memory */
  OTR_nede->remove->all();
  OTR_nesl->remove->all();
  OTR_cab->remove->all();
  OTR_calib->remove->all();
  OTR_sharc->remove->all();

  free(name_qua);
  free(sharc_file);

  report_memory("Everything removed.");

  return 1;
}

void otr_db() {

  /* KUIP entry */

  char* ku_gets();
  char DbFile[80];
  strcpy(DbFile, ku_gets());
  printf("Calling otr_db_file(%s).\n",DbFile);
  otr_db_file(DbFile);
  printf("Finished otr_db_file(%s).\n",DbFile);
}
